package Modules::BannerAd;
use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Input;
use Modules::BaseDCD;
use ExSite::ReportBuilder;

use vars qw(@ISA);
push(@ISA,"Modules::BaseDCD");

#---------------------------------------------------------------------
sub new {
	my $this = shift;
	my ($obj,$class);
	$obj = {};
	$class = ref($this) || $this;
	bless $obj, $class;
	$class =~ /^(.+?::)+?(.+)$/;
	$obj->{class_name} = $2;
	$obj->{db} = $share{DB};
	$obj->{cgi} = $config{server}{server}.$config{server}{CGIpath};
	$obj->{html} = $config{server}{server}.$config{server}{HTMLpath};
	$obj->read;  # inherits BaseDCD::read()
	$obj->read_conf;
	return $obj;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return "BannerAd";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
        	return "BannerAd module serves banner ads to websites";
	}elsif (/isRestricted/){
		return 1;
	}
	return;
}
sub write{
	my ($this,$options) = @_;
	my (%opt,$action);
	%opt = &DecodeString($options);
	$action = $this->{input}{action} || $opt{action};
	# Process click through
	if ($this->{input}{bannerad_id}){
		return $this->processBannerClick($this->{input}{bannerad_id});
	# Display banners
	}else{
		my $banner_count = $options =~ /\d+/ ? $options : 1;
		return $this->displayBanner($banner_count);
	}
	return undef;
}
#---------------------------------------------------------------------
# Control panel
sub control_panel{
	my $this = shift;
	my $out;	
	my $action = $this->{input}{action};
	# Add a new banner ad
	if($action eq "new_bannerad" && $this->{input}{section_id}){
		$out = $this->newBanner($this->{input}{section_id});
	# Delete a banner ad
	}elsif($action eq "delete_bannerad" && $this->{input}{bannerad_id}){
		$out = $this->deleteBanner($this->{input}{bannerad_id});
	# Copy a banner ad
	}elsif($action eq "copy_bannerad" && $this->{input}{bannerad_id}){
		$out = $this->copyBanner($this->{input}{bannerad_id});
	# Edit a banner ad
	}elsif($action eq "edit_bannerad" && $this->{input}{bannerad_id}){
		$out = $this->editBanner($this->{input}{bannerad_id});
	# View banner ad details
	}elsif($action eq "detail_bannerad" && $this->{input}{bannerad_id}){
		$out = $this->detailBanner($this->{input}{bannerad_id});
	# Export banner click through log to excel
	}elsif($action eq "export_log" && $this->{input}{bannerad_id}){
		$out = $this->exportLog($this->{input}{bannerad_id});
	# View summary reports
	}else{
		$out = $this->viewSummary();
	}
	return $out;
}
#----------------------------------------------------------------------
# Summary report
sub viewSummary{
	my ($this) = @_;
	my @sites = $this->{db}->my_real_sites();
	my $out;
	foreach my $site (@sites){
		$out .= $this->viewBannerReport($site);
	}
	return $out;
}
# Add a new banner ad
sub newBanner{
	my ($this,$sectionid) = @_;
	my $out = "\n<h1>New Banner</h1>\n";
	$out .= $this->{db}->make(table=>"bannerad",data=>{section_id=>$sectionid});
	return $out;
}
# Edit a banner ad
sub editBanner{
	my ($this,$banneradid) = @_;
	my $out = "\n<h1>Edit Banner</h1>\n";
	$out .= $this->{db}->make(table=>"bannerad",record=>$banneradid,extra=>{reply=>$this->myRelink()});
	return $out;
}
# Export banner ad click through log to excel
sub exportLog{
	my ($this,$banneradid) = @_;
	# Report headings
	my %heading = (time=>"Timestamp",viewer_ip=>"Banner viewer's ip",referrer=>"Referrer");
	# Name of the columns that we are going to export
	my @column = keys(%heading);
	my @logs = $this->{db}->fetch_match("bannerad_log",{bannerad_id=>$banneradid},"bannerad_log_id asc");
	for(my $i = 0; $i < @logs; $i++){
		foreach my $col(@column){
			# Format export data
			$logs[$i]{$col} = $this->{db}->show_data("bannerad_log",$col,$logs[$i]{$col},$logs[$i]{bannerad_log_id});
		}
	}
	my $filename = "banner_log.xls";
	print "content-type: application/vnd.ms-excel\ncontent-disposition: attachment; filename=$filename\n\n";
	print $this->{db}->export({table=>"bannerad_log",format=>"excel",data=>\@logs,column=>\@column,heading=>\%heading});
        return;
}
# View banner details
sub detailBanner{
	my ($this,$banneradid) = @_;
	# Click through report
	#
	# Fetch the logs
	my @logs = $this->{db}->fetch_match("bannerad_log",{bannerad_id=>$banneradid},"bannerad_log_id asc");
	# Report heading
	my @lheader = ("Timestamp","Banner viewer's ip","Referrer");
	my @ldata;
	foreach my $log(@logs){
		# Gather data
		push(@ldata,[
			$this->{db}->show_data("bannerad_log","time",$log->{time},$log->{bannerad_log_id}),
			$this->{db}->show_data("bannerad_log","viewer_ip",$log->{viewer_ip},$log->{bannerad_log_id}),
			$this->{db}->show_data("bannerad_log","referrer",$log->{referrer},$log->{bannerad_log_id}),
		]);
	}
	# Rxport to excel link
	my $tool = "<a href=\"".$this->myRelink({bannerad_id=>$banneradid,action=>"export_log"})."\">";
	$tool .= "<img src=\"$this->{html}/_Modules/BannerAd/excel16.gif\" border=\"0\">";
	$tool .= "</a>";
	my %lreport = (
		title=>"Banner Access Logs",
		headers=>\@lheader,
		data=>\@ldata,
		tools=>$tool,
	);
	# Place the click through report in the footer section of the banner details report
	my $foot = $this->reportbuilder(\%lreport);
	# Banner details report
	#
	# Fetch the banner ad
	my %ad = $this->{db}->fetch("bannerad",$banneradid);
	# Define column heading
	my @header = ("Ad link","Link text","Description text","Binary ad data","Height of the ad","Width of the ad","Type",
		"Cost per view/click","Enable logging","Total remaining value of views/clicks","Priority to be given to this ad",
		"Date to stop showing timed ads","Status");
	my @header_col = ("url","adata","ddata","fdata","height","width","bannerad_type","rate","logging","balance","weight","expiry_date","status");
	my @data;
	# Only display the columns defined in @header_col
	foreach my $col(@header_col){
		# format data
		push(@data,$this->{db}->show_data("bannerad",$col,$ad{$col},$ad{bannerad_id}));
	}
	# Define options link info
	my @options = (		
		{id=>"edit",label=>"Edit",url=>$this->myRelink({action=>"edit_bannerad",bannerad_id=>$ad{bannerad_id}}),img=>"$this->{html}/_Modules/BannerAd/edit12.gif"},
		{id=>"delete",label=>"Delete",url=>"javascript:deleteConfirm($ad{bannerad_id})",img=>"$this->{html}/_Modules/BannerAd/rm12.gif"},
	);
	# Options links(edit and delete commands)
	my $option;
	foreach my $opt(@options){
		$option .= "<a href=\"$opt->{url}\">";
		$option .= "<img src=\"$opt->{img}\" hspace=\"2\" border=\"0\" align=\"middle\" title=\"$opt->{label}\" alt=\"$opt->{label}\">";
		$option .= "</a>";
	}
	my %report = (
		headers=>\@header,
		data=>\@data,
		tools=>$option,
		foot=>$foot,
	);
	my $out = $this->javascript();
	$out .= "\n<h1>View Details</h1>\n";
	$out .= $this->reportbuilder(\%report);
	return $out;
}
# Delete a banner ad
sub deleteBanner{
	my ($this,$banneradid) = @_;
	$this->{db}->trash_r("bannerad",$banneradid);
	$this->redirect($this->myRelink());
	return;
}
# Copy a banner ad
sub copyBanner{
	my ($this,$banneradid) = @_;
	my %bannerad = $this->{db}->fetch("bannerad",$banneradid);
	# Remove bannerad_id(primary key)
	delete $bannerad{bannerad_id};
	# Set the balance to 0
	$bannerad{balance} = 0;
	$this->{db}->insert("bannerad",\%bannerad);
	$this->redirect($this->myRelink());	
	return;
}
# Fetch banner ads
sub fetchBannerad{
	my ($this,$sectionid) = @_;
	my $sql = "select * from bannerad where (section_id = $sectionid";
	$sql .= $this->{db}->is_admin() ? " or section_id is null or section_id = 0" : "";
	$sql .= ")";
	my @banners = $this->{db}->custom_query($sql);
	return \@banners;
}
# Summary report
sub viewBannerReport{
	my ($this,$section) = @_;
	# Fetch banner ads
	my @banners = @{$this->fetchBannerad($section->{section_id})};
	# Report heading
	my @header = ("Options","Section ID","Status","Type","Weight","Expiry Date","Ad link","Use Image?","Enable Logging","Rate","Balance");
	my @data;
	foreach my $banner(@banners){
		# Define option link info
		my @options = (
		{id=>"detail",label=>"View Details",url=>$this->myRelink({action=>"detail_bannerad",bannerad_id=>$banner->{bannerad_id}}),img=>"$this->{html}/_Modules/BannerAd/view12.gif"},
		{id=>"edit",label=>"Edit",url=>$this->myRelink({action=>"edit_bannerad",bannerad_id=>$banner->{bannerad_id}}),img=>"$this->{html}/_Modules/BannerAd/edit12.gif"},
		{id=>"copy",label=>"Copy",url=>$this->myRelink({action=>"copy_bannerad",bannerad_id=>$banner->{bannerad_id}}),img=>"$this->{html}/_Modules/BannerAd/copy12.gif"},
		{id=>"delete",label=>"Delete",url=>"javascript:deleteConfirm($banner->{bannerad_id})",img=>"$this->{html}/_Modules/BannerAd/rm12.gif"},
		);
		my $option;
		# Create option links
		foreach my $opt(@options){
			$option .= "<a href=\"$opt->{url}\">";
			$option .= "<img src=\"$opt->{img}\" hspace=\"2\" border=\"0\" align=\"middle\" title=\"$opt->{label}\" alt=\"$opt->{label}\">";
			$option .= "</a>";
		}
		# Gather and format data
		push(@data,[
			$option,
			($banner->{section_id} || "n/a"),
			$this->displayStatus($banner),
			$banner->{bannerad_type},
			$banner->{weight},
			$this->{db}->show_data("bannerad","expiry_date",$banner->{expiry_date},$banner->{bannerad_id}),
			$this->{db}->show_data("bannerad","url",$banner->{url},$banner->{bannerad_id}),			
			$banner->{fdata} ? "Yes" : "No",
			$banner->{logging},			
			$this->{db}->show_data("bannerad","rate",$banner->{rate},$banner->{bannerad_id}),
			$this->{db}->show_data("bannerad","balance",$banner->{balance},$banner->{bannerad_id}),
		]);
	}
	# Tool links
	my $tool = "<a href=\"".$this->myRelink({action=>"new_bannerad",section_id=>$section->{section_id}})."\">";
	$tool .= "<img src=\"$this->{html}/_Modules/BannerAd/new12.gif\" border=\"0\" title=\"New banner\" alt=\"New banner\">";
	$tool .= "</a>";
	my %report = (
		title=>$section->{title},
		headers=>\@header,
		data=>\@data,
		tools=>$tool,
	);
	my $out = $this->javascript();
	$out .= $this->reportbuilder(\%report);
	return $out;
}
# Display the status column on the summary report
sub displayStatus{
	my ($this,$bannerad) = @_;
	my @status;
	# Default status(active or inactive)
	push(@status,$bannerad->{status});
	if ($this->isExpired($bannerad)){
		# expired banner ad
		push(@status,"expired");
	}
	if ($this->isExhausted($bannerad)){
		# exhausted banner ad
		push(@status,"exhausted");
	}
	return join(", ",@status);
}
# Is the banner ad exhausted?
sub isExhausted{
	my ($this,$bannerad) = @_;
	# Only check the pay_per_view or pay_per_click ads
	return 0 if $bannerad->{bannerad_type} !~ /pay/i;
	# Funds available
	return 0 if $bannerad->{balance} > 0;
	return 1;
}
sub getTimeObject{
	my ($this) = @_;
	if (!$this->{time_object}){
		require ExSite::Time;
		$this->{time_object} = new ExSite::Time;
	}
	return $this->{time_object};
}
sub CompareToNow{
	my ($this,$t1,$t2) = @_;
	my $tObj = $this->getTimeObject();
	return $tObj->CompareToNow($t1,$t2);	
}
# Is the banner ad expired?
sub isExpired{
	my ($this,$bannerad) = @_;
	# Only check the timed ads
	return 0 if $bannerad->{bannerad_type} !~ /timed/i;
	# The ad isn't expired
	return 0 if ($this->CompareToNow($bannerad->{expiry_date}) >= 0);
	return 1;
}
# Is the ad active?
sub isActive{
	my ($this,$bannerad) = @_;
	# The ad is considered active when it is not:
	# - expired
	# - exhausted
	return 1 if (!$this->isExpired($bannerad) && !$this->isExhausted($bannerad));
	return 0;
}
# Filter viewable ads by status
sub filterByStatus{
	my ($this,$bannerads) = @_;
	my @newbanners;
	# Construct a list consist of active ads
	foreach my $ad(@$bannerads){
		if ($this->isActive($ad)){
			push(@newbanners,$ad);
		}
	}
	return \@newbanners;
}
# Filter ads by weights
sub filterByWeight{
	my ($this,$bannerads) = @_;
	# Separate ads into:
	# - display_always: weight >= 10
	# - display_random: weight < 10
	my (@display_always,@display_random);
	foreach my $ad(@$bannerads){
		if ($ad->{weight} >= 10){
			push(@display_always,$ad);
		}else{
			# if no weight is specified, the weight is presumed to be 1
			$ad->{weight} = $ad->{weight} || 1;
			push(@display_random,$ad);
		}
	}
	return (\@display_always,\@display_random);
}
# Filter displayed ads
sub filterByDisplayedID{
	my ($this,$bannerads) = @_;
	# Skip the check if no ads are displayed on the page
	return $bannerads if !exists($this->{displayed_id});
	# Get the list of displayed ads from $this->{displayed_id}
	my $idlist = join("|",keys(%{$this->{displayed_id}}));
	my @newlist;
	# Construct a list of ads that has not being displayed on the page
	foreach my $ad(@$bannerads){
		if ($ad->{bannerad_id} !~ /$idlist/){
			push(@newlist,$ad);
		}
	}
	return \@newlist;
}
# Adjust ad's balance
sub adjustBalance{
	# $type: banner ad type
	my ($this,$bannerad,$type) = @_;
	# Only adjust the balance on the ad if it matches the $type
	return if $bannerad->{bannerad_type} !~ /$type/i;
	my $cost = $bannerad->{balance} - $bannerad->{rate};
	$this->{db}->update("bannerad",{balance=>$cost},{bannerad_id=>$bannerad->{bannerad_id}});
	return;
}
# Adjust balance for the pay per view ad
sub adjustBalancePayPerViewAd{
	my ($this,$bannerad) = @_;
	return $this->adjustBalance($bannerad,"pay_per_view");
}
# Adjust balance for the pay per click ad
sub adjustBalancePayPerClickAd{
	my ($this,$bannerad) = @_;
	return $this->adjustBalance($bannerad,"pay_per_click");
}
# Show banners on the page
sub showBanner{
	my ($this,$bannerad) = @_;
	# Adjust balance for pay per view ads
	$this->adjustBalancePayPerViewAd($bannerad);	
	my $link = "";
	# Click through link
	if ($bannerad->{url}){
		$link = "$this->{cgi}/bannerad.cgi?bannerad_id=$bannerad->{bannerad_id}";
	}
	my $out = undef;
	# Use banner image
	if ($bannerad->{fdata}){
		$out .= "<span class=\"BannerAd\">";
		$out .= $link ? "<a href=\"$link\" target=\"bannerad\">" : "";
		$out .= $this->{db}->show_data("bannerad","fdata",$bannerad->{fdata},$bannerad->{bannerad_id});
		$out .= $link ? "</a>" : "";
		$out .= "</span>";
	# Use text
	}else{
		$out = "<table width=\"$bannerad->{width}\" height=\"$bannerad->{height}\">";
		$out .= "<tr>";
		$out .= "<td class=\"BannerAd\">";
		$out .= $link ? "<a href=\"$link\" target=\"bannerad\">" : "";
		$out .= $bannerad->{adata};
		$out .= $link ? "</a>" : "";
		$out .= $bannerad->{ddata} ?  "<br>$bannerad->{ddata}" : "";
		$out .= "</td>";
		$out .= "</tr>";
		$out .= "</table>";
	}
	return $out;
}
# Process banner click through
sub processBannerClick{
	my ($this,$banneradid) = @_;
	# Fetch the ad
	my %ad = $this->{db}->fetch("bannerad",$banneradid);
	# Adjust balance for pay per click ad
	$this->adjustBalancePayPerClickAd(\%ad);
	# log the click through info if the ad has the logging option enabled
	if ($ad{logging} =~ /on/i){
		my %insert = (
			bannerad_id=>$ad{bannerad_id},
			viewer_ip=>$ENV{REMOTE_ADDR},
			referrer=>$ENV{HTTP_REFERER},
		);
		$this->{db}->insert("bannerad_log",\%insert);
	}
	# Fix the ad url
	my $url = $ad{url} =~ /^http:\/\//i ? "" : "http://";
	$url .= $ad{url};
	# Use meta refresh for redirection
	my $out = "content-type: text/html\n\n
<html>
<head>
<meta http-equiv=\"refresh\" content=\"0;url=$url\">
</head>
<body>
</body>
</html>";
	return $out;
}
# Display banners on the screen
sub displayBanner{
	my ($this,$count) = @_;
	# Skip if the page object doesn't exist
	return if !exists($share{Page});
	# Get section_id
	my $sectionid = $share{Page}->id("section");
	# Fetch ads
	my $sql = "select * from bannerad where status = 'active' and (section_id = $sectionid or section_id = 0 or section_id is null)";
	my @bannerads = $this->{db}->custom_query($sql);
	# Remove displayed ads
	@bannerads = @{$this->filterByDisplayedID(\@bannerads)};
	# Remove inactive ads
	@bannerads = @{$this->filterByStatus(\@bannerads)};
	# Separate adds into always displayed ads and randomly displayed ads
	my ($display_always,$display_random) = $this->filterByWeight(\@bannerads);
	my %display_random;
	# Weight total for the random ads
	my $weight_total = 0;
	# Calculate the weight total
	foreach my $ad (@$display_random){
		$display_random{$ad->{bannerad_id}} = $ad;
		$weight_total += $ad->{weight};
	}
	# Put selected banners into @out_banner
	my @out_banner;
	for(my $i = 0; $i < $count; $i++){
		# Always selected ads(weight >= 10)
		if (($i + 1) <= @$display_always){
			# Sort ads by weight then id
			@$display_always = sort{($b->{weight} <=> $a->{weight}) || ($a->{bannerad_id} <=> $b->{bannerad_id})} @$display_always;
			# Select the ads one by one
			for(my $i = 0; $i < $count; $i++){
				# Put the ad into @out_banner
				push(@out_banner,$this->showBanner($display_always->[$i]));
				# Flag the ad as displayed
				$this->{displayed_id}{$display_always->[$i]{bannerad_id}} = 1;
			}
		# Randomly seledcted ads(weight < 10)
		# 
		# Ad selection is done by choosing the position($ad_position) that 
		# the ad locates on the list($weight_total: total number of positions).
		# The number of positions that an ad may take on the list is determined by its weight.
		#
		}else{
			# Generate a random number
			my $random = int(scalar(@bannerads) * rand(10000));
			# Position of the ad that we are going to select
			my $ad_position = ($random % $weight_total) + 1;
			# Track the ad position.
			my $count = 0;
			# Loop through the hash %display_random
			foreach my $id(keys(%display_random)){
				# Accumulate the counter
				$count  = $count + $display_random{$id}{weight};
				# Once the counter reaches the position($count >= $ad_position), the ad is selected.
				if ($count >= $ad_position){
					push(@out_banner,$this->showBanner($display_random{$id}));
					# Remove the weight of the selected ad from $weight_total
					$weight_total = $weight_total - $display_random{$id}{weight};
					# Remove the ad from the hash
					delete $display_random{$id};
					# Flag the ad as displayed
					$this->{displayed_id}{$id} = 1;
					# Exit the loop
					last;
				}
			}
		}
	}
	return join("<br>",@out_banner);
}
#----------------------------------------------------------------------
# Delete confirmation popup
sub javascript{
	my ($this) = @_;
	my $out = "\n<script language=\"javascript\">";
	$out .= "
	function deleteConfirm(id){
		if (confirm('Are you sure want to delete this record?')){
			location.href = '".$this->myRelink({action=>"delete_bannerad"})."&bannerad_id=' + id;
		}
	}";
	$out .= "\n</script>";
	return $out;
}
# Build reports using ExSite::ReportBuilder
sub reportbuilder{
	my ($this,$opt) = @_;
	require ExSite::ReportBuilder;
	$this->{rb} = new ExSite::ReportBuilder() if !$this->{rb};
	my $rb = $this->{rb};
	$rb->init(
		title=>$opt->{title},
		data=>$opt->{data},
		tools=>$opt->{tools},
		headers=>$opt->{headers},
		foot=>$opt->{foot},
		width=>$opt->{width},
	);
 	return $rb->make();
}
# A variation of relink
sub myRelink{
	my ($this,$data) = @_;
	my (@list,$path,@systemQuery);
	$path = $ENV{SCRIPT_NAME};
	$path .= $ENV{PATH_INFO} ?  $ENV{PATH_INFO} : "";
	$path .= "?";
	foreach my $key (keys(%{$this->{input}})){
		next if $key !~ /^_/i;
		push(@systemQuery,"$key=$this->{input}{$key}");
	}
	$path .= join("&",@systemQuery);
	$path .= @systemQuery > 0 ? "&" : "";
	foreach my $key (keys(%$data)){
		push(@list,"$key=$data->{$key}");
	}
	return $path.join("&",@list);
}
# Page redirection
sub redirect{
        my ($this,$reply) = @_;
        my $p = $share{Page} || $share{PageBuf};
        if ($p){
                $p->redirect($reply);
        }else{
                print "location: $reply\n\n";
        }
        return;
}
#----------------------------------------------------------------------
1;
