package Modules::Calendar;

=pod

TODO :
code supports public and members adding events, but dbmap permissions
do not.  Need custom form processing to work around this.

Can we jump around by more than one month at a time?

=cut

use strict;
use ExSite::Config;
use ExSite::Input;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use ExSite::Image;
# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD ExSite::ContentData);

use vars qw(@wdy @month @dayinmonth);

@wdy = qw(Sun Mon Tue Wed Thu Fri Sat);
@month = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
@dayinmonth = qw(31 28 31 30 31 30 31 31 30 31 30 31);

sub write {
    my ($this,$options) = @_;

    my $in = new ExSite::Input;
    my $args = $in->combine();
    my %args = %$args;

    # predefined action
    if ($options =~ /upcoming/) {
	return $this->wrap($this->list_upcoming);
    }
    elsif ($options =~ /next/) {
	return $this->wrap($this->next_event($options));
    }
    elsif ($options =~ /newevent/) {
	# new event action hard-coded in oyster
	if ($this->admin) {
	    if ($in->{caladm} eq "doform") {
		return $this->doform;
	    }
	    return $this->newevent;
	}
	return "New Event: Permission Denied.";
    }
    elsif ($args{sid} && $args{calendar} eq "reg" && $args{evtid}){
    	return $this->wrap($this->registration_form($args{sid},$args{evtid}));
    }
    
    my $db = $share{DB}; 
    my $in = $this->{input};

    # adminstrator action
    if ($this->admin) {
	# new event action selected from control panel
	if ($in->{caladm} eq "new") {
	    return $this->newevent;
	}
	elsif ($in->{caladm} eq "doform") {
	    return $this->doform;
	}
    }

    # fetch selected event
    if ($in->{evtid}) {
	$this->fetch($in->{evtid});
    }

    # select date to show
    if (! $args{date} && $options =~ /thismonth/) {
	my $today = $this->today;
	$today =~ s/-\d{1,2}$//;
	$args{date} = $today;
    }
    my ($y,$m,$d);
    if ($args{date}) {
	# explicit
	($y,$m,$d) = split /-/,$args{date};
    }
    elsif ($this->{event}) {
	# use event date
	($y,$m,$d) = split /-/,$this->{event}{start};
    }
    else {
	# no date - use today
	($y,$m,$d) = $this->today;
    }
    # save date info for other methods
    $this->{date} = [$y,$m,$d];

    # fetch events for the selected month

    #my @events = $db->fetch_match("event",{start=>"$y-$m-%"},"start");
    my @events = $this->fetch_events_by_date($y,$m);

    # show the month

    my $out = $this->show_month(@events);

    # list events

    if ($in->{evtid}) {
	# show selected event
	$out .= $this->show;
    }
    elsif ($args{date}) {
	$out .= $this->list($args{date});
    }
    else {
	$out .= $this->list_upcoming;
    }

    return $this->wrap($out);
}

sub admin {
    my $this = shift;
    return ($config{Calendar}{new_event_access} =~ /public/) ||
	($config{Calendar}{new_event_access} =~ /users|members/ && 
	 $share{DB}->authorize);
}

sub wrap {
    my ($this,$out) = @_;
    return "<div class=\"Calendar\">\n$out\n<\div>\n";
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/ModuleName/) {
	return "Event Calendar";
    }
    elsif (/isRestricted/) {
	return defined $config{Calendar}{isRestricted} ?
	    $config{Calendar}{isRestricted} : 1;
    }
    elsif (/isService/) {
	return 1;
    }
    elsif (/Publish/) {
	return \&publish;
    }
    elsif (/DynContentParameter/) {
	return "<p><b>Option:</b>
<select name=\"param\">
<option value=\"\">(default)</option>
<option value=\"upcoming\">Upcoming Events</option>
<option value=\"next\">Next Event</option>
<option value=\"thismonth\">What's Happening This Month</option>
<option value=\"newevent\">New Event Form</option>
</select></p>\n";
    }
    elsif (/ModuleInfo/) {
	return "The Calendar tool manages and displays an event calendar.
Users can navigate the calendar by month, or visit particular days that
contain events.  Individual events may be visited, to view event details,
view event web pages, or download brochures or other info.";
    }
    return;
}

sub show {
    my $this = shift;
    my $e = $this->{event};
    return if (! $e or $e->{status} eq "hidden");
    my ($out,$img);
    if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Calendar/$e->{type}.gif") {
	$img = "<table align=\"right\" style=\"padding:12pt;\"><tr><td align=\"center\">
<img src=\"$config{server}{HTMLpath}/_Modules/Calendar/$e->{type}.gif\" align=\"middle\"><br>
<span style=\"font-size:8pt;\">$e->{type}</span></td></tr></table>\n";
    }
    $out .= "<h2>$img ".$this->show_data("name")."</h2>\n";
    if ($e->{status} eq "canceled") {
	$out .= "<h3 class=\"error\">THIS EVENT HAS BEEN CANCELED</h3>\n";
    }
    $out .= "<h4>Date: <i>".$this->show_date($e->{start},$e->{end})."</i>\n";
    my $time = $this->show_time($e->{tstart},$e->{tend});
    if ($time) {
	$out .= "<br>Time: <i>$time</i>\n";
    }
    if ($e->{place}) {
	$out .= "<br>Place: <i>".$this->show_data("place")."</i>";
    }
    if ($e->{host}) {
	$out .= "<br>Host: <i>".$this->show_data("host")."</i>";
    }
    $out .= "</h4>\n";
    $out .= "<p>".$this->show_data("description")."</p>\n";
    $out .= "<p>".$this->show_data("body")."</p>\n";
    if ($e->{upload}) {
	$out .= "<p>Brochure: ".$this->show_data("upload")."</p>\n";
    }
    if ($e->{url}) {
	$out .= "<p>More Information: ".$this->show_data("url")."</p>\n";
    }
    if ($e->{photo}) {
	my $img = new ExSite::Image($e->{photo});
	my $filename = $img->name;
	my $imgurl = "$config{server}{HTMLpath}/_Modules/Calendar/$e->{event_id}/$filename";
	if (! -e "$config{server}{HTMLroot}$imgurl") {
	    $imgurl = "$config{server}{CGIpath}/getdata.cgi/$filename?tab=event&col=photo&id=$e->{event_id}";
	}
	$out .= "<p><img src=\"$imgurl\"></p>\n";
    }
    # don't show the 'register...' link for canceled and past events,
    # or for admin view
    if ($this->{admin}) {
	$out .= "<p><i>[registration link will be placed here]</i></p>\n";
    }
    else {
	if ($e->{status} ne "canceled" || !$this->past_event_check($e->{start})){
	    if ($e->{survey_id}){
		$out .= "<p><a href=\"".$this->link(sid=>$e->{survey_id},calendar=>"reg",evtid=>$e->{event_id})."\">Click here to register</a>.</p>\n";
	    }elsif ($e->{reg_url}) {
		$out .= "<p><a href=\"$e->{reg_url}\">Click here to register</a>.</p>\n";
	    }
	}
    }
    return $out;
}

# past event check
sub past_event_check{
    my ($this,$startdate) = @_;
    return 0 if !$startdate;
    return 0 if $startdate gt $this->today();
    return 1;
}

sub show_data {
    my ($this,$col) = @_;
    return undef if (! $this->{event});
    if ($col eq "upload" && $this->{event}{$col}) {
	my $img = new ExSite::Image($this->{event}{upload});
	my $filename = $img->name;
	my $pubpath = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Calendar/$this->{event}{event_id}/$filename";
	if (-e $pubpath) {
	    return "<a href=\"$pubpath\">$filename</a>";
	}
    }
    elsif ($col eq "photo" && $this->{event}{$col}) {
	my $img = new ExSite::Image($this->{event}{photo});
	my $filename = $img->name;
	my $pubpath = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Calendar/$this->{event}{event_id}/$filename";
	if (-e $pubpath) {
	    return "<p><img src=\"$pubpath\"></p>\n";
	}
    }
    return $share{DB}->show_data("event",$col,$this->{event}{$col},
				 $this->{event}{event_id});
}

sub list {
    my $this = shift;
    my $date = shift;
    my (@events,$eventspan);
    my $db = $share{DB}; 
    if (! $date) {
	# no date requested - select whole month
	@events = $this->fetch_events_by_date($this->{date}[0],$this->{date}[1]);
	$eventspan = "month";
    }
    elsif ($date =~ /^\d+-\d+$/) {
	# a whole month was requested
	@events = $this->fetch_events_by_date($this->{date}[0],$this->{date}[1]);
	$eventspan = "month";
    }
    else {
	# specific day was selected
	@events = $this->fetch_events_by_date($this->{date}[0],$this->{date}[1],$this->{date}[2]);
	$eventspan = "date";
    }

    my $out .= "<h1>$config{Calendar}{heading}{calendar}</h1>\n";
    
    if (@events) {
        $out .= "<p>For more details, click on the event link.</p>\n";
	$out .= "<dl>\n";
	foreach my $evt (@events) {
	    $out .= $this->summarize_event($evt);
	}
	$out .= "</dl>\n";
    }
    else {
	$out .= "<p>No events were found for this $eventspan.</p>\n";
    }
    return $out;
}

sub list_upcoming {
    my $this = shift;
    my $db = $share{DB}; 
    my $date = $this->today;
    my $limit = $config{Calendar}{max_upcoming} || 10;

    my $out .= "<h1>$config{Calendar}{heading}{upcoming}</h1>\n";

    my @events = $this->fetch_events($date,$limit);

    #$db->custom_query("select * from event where start >= \"$date\" and status = \"active\" and section_id=".$share{DB}->this_site()->{section_id}." order by start limit $limit");
    
    if (@events) {
	$out .= "<p>For more details, click on the event link.</p>\n";
	$out .= "<dl>\n";
	foreach my $evt (@events) {
	    $out .= $this->summarize_event($evt);
	}
	$out .= "</dl>\n";
    }
    else {
	$out .= "<p>No upcoming events.</p>\n";
    }
    return $out;
}

sub next_event {
    my ($this,$options) = @_;
    my $db = $share{DB}; 
    my $date = $this->today;
    my @events = $this->fetch_events($date,1);

    #my @events = $db->custom_query("select * from event where start >= \"$date\" and status = \"active\" and section_id=".$db->this_site()->{section_id}." order by start limit 1");

    if (@events) {
	$this->{event} = $events[0];
	my $out;
	if ($options !~ /noheader/) {
		$out .= "<h3>$config{Calendar}{heading}{next}</h3>\n";
	}
	$out .= $this->show;
	return $out;
    }
    return "<p>No upcoming events.</p>\n";
}

sub summarize_event {
    my ($this,$evt) = @_;
    my ($out,$img,$url);
    $this->{event} = $evt;
    my $edate = $this->show_date($evt->{start},$evt->{end});
#    if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Calendar/$evt->{type}.gif") {
#	$img = "<img src=\"$config{server}{HTMLpath}/_Modules/Calendar/$evt->{type}.gif\" align=\"right\">";
#    }
    $url = $this->link(evtid=>$evt->{event_id});
    $out .= "<dt><b>$edate</b></dt>\n<dd>$img";
    $out .= "<a href=$url>".$this->show_data("name")."</a><br>";
    if ($evt->{host}) {
	$out .= "Hosted by ".$this->show_data("host").".<br>\n";
    }
    $out .= $this->show_data("description")."<p></dd>\n";
    return $out;
}

# show_month : display calendar month, with links to events

sub show_month {
    my ($this,@events) = @_;
    my ($sec,$min,$hr,$mday,$mon,$year,$wday,$yday,$isdst);
    my ($time,$today);
    my @today = $this->today;

    # pick the date that the calendar should reflect

    if ($this->{date}) {
	# explicit date
	my ($y,$m,$d) = @{$this->{date}};
	if ($y < 1950 || $y > 2050 || $m < 1 || $m > 12) {
	    return "(invalid date)";
	}
	use Time::Local;
	my $day = $d || "1";
	$m--;
	$time = timelocal(0,0,12,$day,$m,$y);
    }
    else {
	# deprecated
	$today = 1;
	$time = time();
    }

    # convert date to localtime() values

    ($sec,$min,$hr,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($time);
    # convert to YYYY-MM-DD format
    $year += 1900;
    my $realmonth = $mon + 1;
    if ($realmonth < 10) { $realmonth = "0$realmonth"; }
    my $realmday = $mday;
    if ($realmday < 10) { $realmday = "0$realmday"; }
    my $leap = ((($year % 4) == 0) && (($year % 100) != 0)) ||
	(($year % 400) == 0);
    if ($leap) { $dayinmonth[1] += 1; }

    # flag the days with events

    my @dayflags;
    foreach my $evt (@events) {
	my ($sy,$sm,$sd) = split /-/, $evt->{start};
	my ($ey,$em,$ed) = split /-/, $evt->{end};
	next if ($sm != $realmonth && $em != $realmonth);
	my $beginday = ($sm == $realmonth ? $sd : 1);
	my $endday = ($em == $realmonth ? $ed : $dayinmonth[$mon]);
	foreach my $eventday ($beginday..$endday) {
	    $dayflags[$eventday] = 1;
	}
    }

    # build the calendar HTML

    my $out;
    # setup links to last month and next month
    my ($lastmonthurl, $nextmonthurl);
    my ($lastyear,$lastmonth) = my ($nextyear,$nextmonth) = ($year,$mon);
    # lastmonth already good, since $mon goes from 0..11
    # must add 2 to next month to get the real next month, tho
    $nextmonth += 2;
    if (! $lastmonth) {
	$lastyear--;
	$lastmonth = 12;
    }
    if ($nextmonth > 12) {
	$nextyear++;
	$nextmonth = 1;
    }
    if ($lastmonth < 10) { $lastmonth = "0$lastmonth"; }
    if ($nextmonth < 10) { $nextmonth = "0$nextmonth"; }
    
    my $lastmonthurl = $this->link(date=>"$lastyear-$lastmonth",evtid=>undef);
    my $nextmonthurl = $this->link(date=>"$nextyear-$nextmonth",evtid=>undef);
    
    my $url = $this->link(evtid=>undef);
    my $monthselector = "<form method=\"POST\" action=\"$url\" name=\"calform\">\n<select name=\"date\" class=\"Calendar\" onchange=\"document.calform.submit()\">\n";
    my $date_span = $config{Calendar}{date_selector_range} || 3;
    my $frommonth = $mon - $date_span;
    my $tomonth = $mon + $date_span;
    for (my $imon = $frommonth; $imon <= $tomonth; $imon++) {
	my $y = $year;
	my $dmon = $imon + 1;
	if ($dmon < 1) { $y--; $dmon += 12; }
	if ($dmon > 12) { $y++; $dmon -= 12; }
	my $tmpmon = $dmon - 1;
	$monthselector .= "<option value=\"$y-$dmon\"";
	if ($imon == $mon) { $monthselector .= " selected"; }
	$monthselector .= ">$month[$tmpmon] $y</option>\n";
    }
    $monthselector .= "</select></form>\n";

    $out .= "<TABLE CELLSPACING=1 CELLPADDING=0 BORDER=0 CLASS=\"Calendar\" align=right>\n<TR><TH COLSPAN=7 BGCOLOR=#669999>$monthselector</TH></TR>\n";
    $out .= "<TR BGCOLOR=#CCCCFF><TH>Su</TH><TH>Mo</TH><TH>Tu</TH><TH>We</TH><TH>Th</TH><TH>Fr</TH><TH>Sa</TH></TR>\n";

    $out .= "<TR BGCOLOR=#C0C0C0>";
    my $first_wday = $wday - ($mday % 7);
    if ($first_wday < 0) { $first_wday += 7; }
    my $weekday = 0;
    if ($first_wday < 6) {  ## kludge
	foreach (0..$first_wday) {
	    $out .= "<TD></TD>";
	    $weekday++;
	}
    }
    foreach my $d (1..$dayinmonth[$mon]) {
	if ($weekday == 7) {
	    $out .= "</TR>\n";
	    $weekday = 0;
	}
	my $day = $d < 10 ? "0$d" : $d;
	if ($weekday == 0) {
	    $out .= "<TR BGCOLOR=#C0C0C0>";
	}
	if ($dayflags[$d]) {
	    # event days are red
	    $out .= "<TD class=\"CalendarEvent\" BGCOLOR=#FFCCCC>";
	}
	elsif ($d == $mday && $today[1]==$this->{date}[1] && $today[2]==$this->{date}[2]) {
	    # today is yellow
	    $out .= "<TD class=\"CalendarToday\" BGCOLOR=#FFFFCC>";
	}
	else {
	    # everything else is grey
	    $out .= "<TD>";
	}
	if ($dayflags[$d]) {
	    my $url = $this->link(date=>"$year-$realmonth-$day",evtid=>undef);
	    $out .= "<a href=$url>$d</a>";
	}
	else {
	    $out .= $d;
	}
	$out .= "</td>\n";

	$weekday++;
    }
    if ($weekday != 7) { 
	foreach my $d ($weekday+1 .. 7) { $out .= "<td></td>"; }
	$out .= "</TR>\n"; 
    }
    $out .= "</TABLE>\n";
    return $out;
}

sub today {
    my $this = shift;
    my (undef,undef,undef,$mday,$mon,$year,undef,undef,undef) = 
	localtime(time());
    $year += 1900;
    $mon += 1;
    if ($mon < 10) { $mon = "0$mon"; }
    if ($mday < 10) { $mday = "0$mday"; }
    if (wantarray) {
	return ($year,$mon,$mday);
    }
    return "$year-$mon-$mday";
}

# compare_date:
#   returns 1 if date1 > date2
#           0 if date1 == date2
#          -1 if date1 < date2

sub compare_date {
    my ($this,$date1,$date2) = @_;
    #($date1,undef) = split / /,$date1;   # for datetime fields
    #($date2,undef) = split / /,$date2;   # ditto
    my ($y1,$m1,$d1) = split (/-/,$date1);
    my ($y2,$m2,$d2) = split (/-/,$date2);
    if    ($y2 > $y1) { return 1; }
    elsif ($y2 == $y1) {
	if ($m2 > $m1) { return 1; }
	elsif ($m2 == $m1) {
	    if ($d2 > $d1) { return 1; }
	    elsif ($d2 == $d1) { return 0; }
	}
    }
    return -1;
}

# show date/time : display a datetime, but with more logic than the default
# method in Report::show_data

sub show_date {
    my ($this,$date1,$date2) = @_;
    return "" if (!$date1);
    my ($year1,$mon1,$day1) = split /-/,$date1;
    if ($date2 && $date2 ne $date1) {
	my ($year2,$mon2,$day2) = split /-/,$date2;
	if ($year1 != $year2 || $mon1 != $mon2) {
	    return "$month[$mon1-1] $day1, $year1 - $month[$mon2-1] $day2, $year2";
	}
	else {
	    return "$month[$mon1-1] $day1-$day2, $year1";
	}
    }
    else {
	return "$month[$mon1-1] $day1, $year1";
    }
}

sub show_time {
    my ($this,$time1,$time2) = @_;
    my $out;
    if ($time1 != /^[\s0]0:00/) {
	$time1 =~ s/^0//;
	my ($hour,$min,undef) = split /:/,$time1;
	my $am;
	if ($hour < 12) { $am = 1; }
	elsif ($hour > 12) { $hour -= 12; }
	my $tstr = "$hour:$min ";
	$tstr .= $am ? "am" : "pm";
	$out = "$tstr";
    }
    if ($time2 != $time1 && $time2 != /^[\s0]0:00/) {
	$time2 =~ s/^0//;
	my ($hour,$min,undef) = split /:/,$time2;
	my $am;
	if ($hour < 12) { $am = 1; }
	elsif ($hour > 12) { $hour -= 12; }
	my $tstr = "$hour:$min ";
	$tstr .= $am ? "am" : "pm";
	$out .= " - $tstr";
    }
    return $out;
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    my $in = $this->{input};
    my $db = $share{DB};
    $this->{admin} = 1;  # set admin flag to add admin tools
    my $url = "$config{server}{CGIpath}/ctrl-panel.cgi/Calendar";
    $_ = $in->{caladm};

    if ($in->{evtid}) {
	$this->fetch($in->{evtid});
	return "<p class=error>Invalid event ID.</p>" if (! $this->{event});
    }

    if (/new/) {
	return $this->newevent;
    }
    elsif (/edit/) {
	my $action = $this->link(caladm=>"doform");
	$out .= "<h1>$config{Calendar}{heading}{edit}</h1>\n<dl>\n";
	$this->calendar_handler();
	$out .= $db->make(table=>"event",
			  record=>$in->{evtid},
			  action=>$action,
			  extra=>{caladm=>"doform"},
			  );
	return $out;
    }
    elsif (/doform/) {
	return $this->doform;
    }
    elsif (/delete/) {
	$share{DB}->trash_key("event",$in->{evtid});
	# errorcheck?
    }
    elsif (/show/) {
	if ($in->{evtid}) {
	    $this->fetch($in->{evtid});
	    if ($this->{event}) {
		$out .= $this->show;
		my $edit = $this->link(caladm=>"edit");
		my $delete = $this->link(caladm=>"delete");
		$out .= "<p><a href=\"$edit\"><img src=\"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{edit}{img}\" border=\"0\"></a> 
<a href=\"$delete\"><img src=\"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{delete}{img}\" border=\"0\"></a></p>\n";
	    }
	}
	else {
	    $out .= "<p class=error>No event selected.</p>\n";
	}
    }
    $out .= "<h1>$config{Calendar}{heading}{myevents}</h1>\n";
    $out .= $this->report;
    return $out;
}

sub newevent {
    my $this = shift;
    my $in = $this->{input};
    my $db = $share{DB};
    my $action = $this->link(caladm=>"doform");
    my $out .= "<h1>$config{Calendar}{heading}{new}</h1>\n<dl>\n";
    $this->calendar_handler();
    $out .= $db->make(table=>"event",
		      action=>$action,
		      data=>{section_id=>$in->{section_id},member_id=>$db->my_uid},
		      extra=>{caladm=>"doform"},
		      );
    return $out;
}

sub doform {
    my $this = shift;
    my $in = $this->{input};
    my $db = $share{DB};
    my $out;
    $db->get_data;
    # tweak the form data
    my @evtid = keys %{$db->{form}{event}};
    my $evtid = $evtid[0] || "_";
    $db->{action} = (exists $db->{form}{event}{$evtid}{event_id} ? 
		     "update" : "insert");
    if (! $db->{form}{event}{$evtid}{status}) {
	# default the event status to active
	$db->{form}{event}{$evtid}{status} = "active";
    }
    if (! $db->{form}{event}{$evtid}{access}) {
	# default the event access to public
	$db->{form}{event}{$evtid}{access} = "public";
    }
    # tie event to current site
    my $site = $in->{section_id} || $db->this_site()->{section_id};
    if ($db->{form}{event}{$evtid}{section_id}) {
	# ownership check
	if (! $db->owns($db->{form}{event}{$evtid}{section_id})) {
	    return $this->error("You do not have permission to add events to this website.");
	}
    }
    elsif (! $db->is_admin) {
	$db->{form}{event}{$evtid}{section_id} = $site;
	$this->warn("Event changed to a local event - you do not have permission to add global events.");
    }

    # tie event to member
    if ($db->{form}{event}{$evtid}{member_id}) {
	if (! $db->is_admin &&
	    $db->{form}{event}{$evtid}{member_id} != $db->my_uid) {
	    $db->{form}{event}{$evtid}{member_id} = $db->my_uid;
	}
    }
    elsif (! $this->admin) {
	$db->{form}{event}{$evtid}{member_id} = $db->my_uid;
    }
    
    # scale photo to a reasonable size
    if ($db->{form}{event}{$evtid}{photo}) {
	my $img = new ExSite::Image($db->{form}{event}{$evtid}{photo});
	$img->scale(400,400,-1);
	$db->{form}{event}{$evtid}{photo} = $img->encode;
    }
    
    # process the form
    $out = $db->do;

    # notification
    my $evt = $db->{form}{event}{$evtid};
    my $whoami = $db->my_name;
    my $message = "
Event added by $whoami:

$evt->{name}
$evt->{start}
$evt->{place}

$evt->{description}

$evt->{body}
";

    my $s = $db->fetch("section",$site);
    my $calendar_mgr = $db->owner_email($s);
    &SendMail($calendar_mgr,$calendar_mgr,
	      "$config{site}{name} - Calendar Update: $evt->{name}",
	      $message);

    if ($out =~ /Done\!/) {
	my $url = $this->link(caladm=>undef,evtid=>undef);
	$out = "<h3>Calendar Updated Successfully</h3>
<p><a href=\"$url\">Click here to return to calendar view</a>.</p>\n";
    }
    return $out;
}

sub report {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    my @sites = $db->my_real_sites;

    foreach my $site (@sites) {
	my %report = ( title=>$site->{title},
		       headers=>[ "Date","Event","Type","Status","Options" ],
		       data=>[],
		       nodata=>"No events found.",
		       );

	if ($db->license("Calendar",$site->{section_id})) {
	    my @e = $db->fetch_child("event",
				     "section",
				     $site->{section_id},
				     {start=>"desc"}
				     );
	    
	    if (@e > 0) {
		foreach my $e (@e) {
		    my @data;
		    $this->{event} = $e;
		    my $edit = $this->link(caladm=>"edit",evtid=>$e->{event_id});
		    my $view = $this->link(caladm=>"show",evtid=>$e->{event_id});
		    my $delete = $this->link(caladm=>"delete",evtid=>$e->{event_id});
		    push @data, $this->show_date($e->{start},$e->{end})." ".$this->show_time($e->{tstart},$e->{tend});
		    push @data, ($this->show_data("name"),
				 $this->show_data("type"),
				 $this->show_data("status"),
				 "<a href=\"$view\"><img src=\"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{view}{img}\" border=\"0\" alt=\"info\" title=\"info\"></a> 
<a href=\"$edit\"><img src=\"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{edit}{img}\" border=\"0\" alt=\"edit\" title=\"edit\"></a> 
<a href=\"$delete\"><img src=\"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{delete}{img}\" border=\"0\" alt=\"delete\" title=\"delete\"></a>\n");
		    push @{$report{data}}, \@data;
		}
	    }
	}
	my $new = $this->link(caladm=>"new",section_id=>$site->{section_id});
	$report{tools} = "<a href=\"$new\"><img src=\"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{new}{img}\" border=\"0\" alt=\"new event\" title=\"new event\"></a>";
	my $r = new ExSite::ReportBuilder(%report);
	$out .= $r->make;
	$out .= "<p>\n";
    }
    return $out;
}

# calendar handler
sub calendar_handler{
	my $this = shift;
	my $db = $share{DB};
	$db->handler("select_foreign_key",\&handler_select_foreign_key);
	return;
}
# select foreign key handler
sub handler_select_foreign_key{
	my $this = shift;
	my $table = shift;
	my $rec = shift;
	my $col = shift;
	my $ftable = shift;
	my $ok = $this->authorize();
	# admin
	#if ($this->is_admin()){
	#	return $this->fetch_match("survey",{type=>"registration"});
	# webmaster
	#}els
	if ($ok == 2){
		my $sql = "select * from survey where type = 'registration' ";
		my @sites = $this->my_real_sites();
		my @siteid;
		foreach my $site(@sites){
			push(@siteid,$site->{section_id});
		}
		$sql .= " and (section_id = ".join(" or section_id = ",@siteid).")";
		# fetch registration forms
		return $this->custom_query($sql);
	}
	return undef;
}
# SimpleRegistration form
sub registration_form{
	my $this = shift;
	my $surveyid = shift;
	my $eventid = shift;
	return "<!--&SimpleRegistration(action=do&sid=$surveyid&evtid=$eventid)-->";
}

### FETCHING EVENTS

sub fetch {
    my ($this,$id) = @_;
    my $db = $share{DB};
    my $e = $db->fetch("event",$id);
    # validate that event belongs to this site
    if ($e->{section_id}) {
	if (! $this->{admin} && 
	    ($e->{section_id} != $share{DB}->this_site()->{section_id})) {
	    return undef; 
	}
    }
    $this->{event} = $e;
    return $e;
}

# fetch a number of events after a certain date

sub fetch_events {
    my ($this,$date,$limit) = @_;
    my $db = $share{DB};
    if ($date !~ /^\d{4}-\d{1,2}-\d{1,2}$/) { $date = $this->today; }
    if (! $limit) { $limit = $config{Calendar}{max_list_size} || 10; }
    my $section_id = $db->this_site()->{section_id};
    my @events = $db->custom_query("select * from event where start >= \"$date\" and status = \"active\" and (section_id=$section_id or section_id=0) order by start limit $limit");
    return @events;
}

# same as fetch_events, but fetches member-owned events only

sub fetch_my_events {
    my ($this,$date,$limit) = @_;
    my $db = $share{DB};
    if ($date !~ /^\d{4}-\d{1,2}-\d{1,2}$/) { $date = $this->today; }
    if (! $limit) { $limit = $config{Calendar}{max_list_size} || 10; }
    my $section_id = $db->this_site()->{section_id};
    my $uid = $db->my_uid;
    my @events = $db->custom_query("select * from event where start >= \"$date\" and status = \"active\" and (section_id=$section_id or section_id=0) and member_id=$uid order by start limit $limit");
    return @events;
}

# fetch_events_by_date : year, month required; day is optional

sub fetch_events_by_date {
    my ($this,$y,$m,$d) = @_;
    my (@events,$st,$end,$cond);
    if ($d) { 
	$st = $end = "$y-$m-$d";
	$cond = "start <= '$st' and end >= '$end'";
    }
    else {
	$st = "$y-$m-01";
	$end = "$y-$m-".$ExSite::Report::month_days[$m];
	$cond = "((start >= '$st' and start <= '$end') or (end >= '$st' and end <= '$end'))";
    }
    my $section_id = $share{DB}->this_site()->{section_id};
    $cond .= " and status=\"active\" and (section_id=$section_id or section_id=0)";
    @events = $share{DB}->custom_query("select * from event where $cond order by start");
    return @events;
}

### PUBLISHING

sub publish {
    my ($this) = @_;
    my $in = new ExSite::Input;
    my $data = $in->query_or_post;
    if ($data->{evt_id}) {
	$this->publish_event($data->{evt_id});
    }
    else {
	my $today = $this->today;
	my @events = $share{DB}->custom_query("select event_id,upload,photo from event where start >= \"$today\"");
	foreach my $evt (@events) {
	    $this->publish_event($evt);
	}
    }
}

sub unpublish {
    my ($this) = @_;
    my $in = new ExSite::Input;
    my $data = $in->query_or_post;
    if ($data->{evt_id}) {
	$this->rmdir("$config{server}{HTMLpath}/_Modules/Calendar/$data->{evt_id}");
    }
    else {
	# don't bother fetching events - just delete every subdir in the
	# calendar directory
	opendir D, "$config{server}{HTMLpath}/_Modules/Calendar";
	my @files = <D>;
	closedir D;
	foreach my $f (@files) {
	    if ($f =~ /^\d+$/ && -d $f) {
		$this->rmdir("$config{server}{HTMLpath}/_Modules/Calendar/$f");
	    }
	}
    }
}

sub publish_event {
    my ($this,$evt) = @_;
    my $db = $share{DB};
    my $event = ref $evt eq "HASH" ? $evt : $this->fetch($evt);
    my $out;
    my $path = "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/Calendar/$event->{event_id}";
    if ($event->{upload} || $event->{photo}) {
	my $mkdir = $this->mkdir($path);
	if (! $mkdir) {
	    return $this->show_diagnostics("error","html");
	}
	if ($event->{upload}) {
	    my $img = new ExSite::Image($event->{upload});
	    my $filename = $img->name;
	    my $pubpath = "$path/$filename";
	    $out .= $this->publish_file($pubpath,$img->get);
	}
	if ($event->{photo}) {
	    my $img = new ExSite::Image($event->{photo});
	    my $filename = $img->name;
	    my $pubpath = "$path/$filename";
	    $out .= $this->publish_file($pubpath,$img->get);
	}
    }
    return $out;
}

sub publish_file {
    my ($this,$path,$imgdata) = @_;
    my $out;
    $path =~ /^(((\/[\w_-]+)+)\/([\w_-]+\.[\w]+))$/;
    my $secure_filename = $1;
    if ($secure_filename) {
	$out .= "Writing $secure_filename... ";
	if (open OUTFILE, ">$secure_filename") {
	    print OUTFILE $imgdata;
	    close OUTFILE;
	    $out .= "succeeded.<br>\n";
	}
	else {
	    $out .= $this->error("Calendar::publish: publish $secure_filename failed: $!");
	}
    }
    else {
	$out .= $this->error("Calendar::publish: invalid filename $path");
    }
    return $out;
}

1;
