package Modules::CatalogPromotions;
#---------------------------------------------------------------------
# Catalog promotions plugins highlists a single product from the 
# Catalog and presents it like a banner ad.
#
# Usage:
#
# <!--&CatalogPromotions(product_id=xxx)-->
#
#---------------------------------------------------------------------
use strict;
use ExSite::Config;
use ExSite::Misc;
use Modules::Catalog;

use vars qw(@ISA);
push(@ISA,"Modules::Catalog");

#---------------------------------------------------------------------
sub new {
	my $this = shift;
	my ($obj,$class);
	$obj = {};
	$class = ref($this) || $this;
	bless $obj, $class;
	$class =~ /^(.+?::)+?(.+)$/;
	$obj->{class_name} = $2;
	$obj->{db} = $share{DB};
	$obj->{cgi} = $config{server}{server}.$config{server}{CGIpath};
	$obj->{html} = $config{server}{server}.$config{server}{HTMLpath};
	$obj->read;  # inherits BaseDCD::read()
	$obj->read_conf("Catalog");
	$obj->init();
	$obj->promotion_mode(1);
	return $obj;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return "CatalogPromotions";
	}elsif (/ModuleInfo/){
        	return "The catalog promotions plug-in highlights a single product from the catalog, and presents it like a banner ad.";
	}elsif (/DynContentParam/){
		return $this->getDynContentParameter(shift);
	}elsif (/isRestricted/){
		return $config{Catalog}{isRestricted};
	}elsif (/isService/){
		return $config{Catalog}{isService};
	}
	
	return;
}
sub write{
	my ($this,$options) = @_;
	my %opt = &DecodeString($options);
	$this->module_option(\%opt);
	my $product_id = $opt{product_id};
	if ($product_id){
		$this->promotion_mode(1);
		return $this->displayProduct($product_id);
	}
	return;
}
#---------------------------------------------------------------------
# Dynamic content parameter dialog
sub getDynContentParameter{
	my ($this,$sectionid) = @_;
	return if !$sectionid;
	my $sql = <<END;
select 
	p.catalog_product_id as catalog_product_id,
	p.code as code,
	p.title as title,
	c.title as category_title
from 
	catalog_product as p
	left join catalog_product_category_join as pcj on pcj.catalog_product_id = p.catalog_product_id
	left join catalog_category as c on c.catalog_category_id = pcj.catalog_category_id
where 
	c.section_id = $sectionid and
	(p.status like 'enable%' or p.status = '' or p.status is null)
order by 
	c.title asc, p.title asc
END
	my @lists = $this->{db}->custom_query($sql);
	my $out = <<END;
<table>
<tr>
	<td>
	<b>Select an option:</b>
	</td>
	<td>
	<select name="param">
	<option value="">&lt;Default&gt;</option>
END
	foreach my $list(@lists){
		my $label = $list->{code} ? "$list->{code} " : "";
		$label .= $list->{title};
		$label = $label || "n/a";
		if ($list->{category_title}){
			$label .= " ($list->{category_title})";
		}
		$out .= "\n\t<option value=\"product_id=$list->{catalog_product_id}\">$label</option>";
	}
	$out .= <<END;
	</select>
	</td>
</tr>
</table>
END
	return $out;
}
#---------------------------------------------------------------------
# Are we in product promotion mode?
sub promotion_mode{
	my ($this,$value) = @_;
	if (defined($value)){
		$this->{promotion_mode} = $value || 0;
	}
	return $this->{promotion_mode} || 0;
}
# Output product information
sub displayProduct{
	my ($this,$productid) = @_;
	my $sectionid = $this->section_id();
	$this->product_id($productid);	
	my %product = %{$this->getProduct($productid)};
	# Get product template
	my $template = $this->html_displayProduct();
	# Gather product info
	my %data = (
		promo_title=>$config{Catalog}{promo_title},
		product_code=>($product{code}." " || ""),
		product_title=>($product{title} || "no name"),
		product_summary=>($product{summary} || ""),
		product_link=>$this->html_moreInfoLink(\%product),
		product_url=>$this->html_productUrl(\%product),
		product_image=>"<img src=\"".$this->getPrimaryImageThumbnailUrl($product{catalog_product_id},$sectionid)."\" border=\"0\" width=\"100\">",
	);
	return $this->merge($template,\%data);
}
# Generate product detail url
sub html_productUrl{
	my ($this,$data) = @_;
	return $this->link(_id=>4,detail=>1,product_id=>$data->{catalog_product_id});
}
# The 'Click here for more information' link
sub html_moreInfoLink{
	my ($this,$data) = @_;
	my $product_url = $this->html_productUrl($data);
	return <<END;
<center><a href="$product_url">Click here for more information</a></center>
END
}
# Product template
sub html_displayProduct{
	my ($this) = @_;
	return <<END;
<div class="CatalogPromotions" onclick="location.href='<!--[[product_url]]-->'">
<table border="0">
<tr>
	<td valign="middle">
	<!--[[product_image]]-->
	</td>
	<td>
	<span class="CatalogPromotionsTitle"><!--[[promo_title]]--></span>&nbsp;&nbsp;
	<span class="CatalogPromotionsProductName"><!--[[product_code]]--><!--[[product_title]]--></span><br>
	<span class="CatalogPromotionsProductSummary"><!--[[product_summary]]--></span><br>
	<span class="CatalogPromotionsProductLink"><!--[[product_link]]--></span>
	</td>
</tr>
</table>
</div>
END
}
1;
