package Modules::Classified;
use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Mail;
use ExSite::Input;
use Modules::BaseDCD;
use ExSite::ReportBuilder;
use vars qw(@ISA);
push(@ISA,"Modules::BaseDCD");

#---------------------------------------------------------------------
sub new {
	my $this = shift;
	my ($obj,$class);
	$obj = {};
	$class = ref($this) || $this;
	bless $obj, $class;
	$obj->{db} = $share{DB};
	$obj->{cgi} = $config{server}{server}.$config{server}{CGIpath};
	$obj->{html} = $config{server}{server}.$config{server}{HTMLpath};
	$obj->read;  # inherits BaseDCD::read()
	$obj->read_conf;
	return $obj;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return "Classifieds";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
        	return "Classified tools allow users to post classifieds.";
	}elsif (/isRestricted/){
		return 0;
	}
}
sub write{
	my ($this,$options) = @_;
	my (%opt,$action);
	%opt = &DecodeString($options);
	$action = $this->{input}{action} || $opt{action};

	if ($this->{input}{cmd} eq "process_new"){	
		return $this->processNew();
	# Edit a classified
	}elsif ($this->{input}{cmd} eq "edit" && $this->{input}{classified_id}){
		return $this->edit($this->{input}{classified_id});
	# Delete a classified
	}elsif ($this->{input}{cmd} eq "del" && $this->{input}{classified_id}){
		$this->deleteClassified($this->{input}{classified_id});
		$this->redirect($this->reply());	
	# Send an email to the advertiser
	}elsif ($action eq "contact_post"){
		return $this->processContactForm();
	# Post a new classified
	}elsif ($this->{input}{section_id} && $action eq "new"){
		return $this->newClassified($this->{input}{section_id});
	# Contact form
	}elsif ($action eq "contact" && $this->{input}{classified_id}){
		return $this->contactForm($this->{input}{classified_id});
	# Show all classifieds in a category
	}elsif ($this->{input}{section_id} && $this->{input}{category}){
		return $this->showClassified($this->{input}{section_id},$this->{input}{category});
	# Show categories
	}else{
		return $this->showCategoryList($share{Page}->id("section"));
	}
	return undef;
}
#---------------------------------------------------------------------
# Control panel
sub control_panel{
	my $this = shift;
	my $out;
	# admin access only
	if (!$this->authorize()){
		$out = "<h3 class=\"error\">Permission denied</h3><p>You do not have sufficient system priveleges to use this feature.</p>\n";
	}else {
		# Form action
		my $action = $this->{input}{action};
		# Admin command
		my $cmd = $this->{input}{cmd};
		# Set the classified to active
		if ($cmd eq "set_active" && $this->{input}{classified_id}){
			$this->setActive($this->{input}{classified_id});
			$this->redirect($this->reply());
		# Create a new classified
		}elsif ($cmd eq "new" && $this->{input}{section_id}){
			return $this->newClassified($this->{input}{section_id});
		# Remove all expired classifieds
		}elsif ($cmd eq "removeallexpired"){
			$this->removeAllExpired();
		# Run public accessable function through write()
		}elsif ($this->{input}{category} || $this->{input}{action} eq "contact_post" || $cmd =~ /edit|del|process_new/i){
			return $this->write();
		# View current ads
		}elsif ($cmd eq "current" && $this->{input}{section_id}){		
			return $this->showCategoryList($this->{input}{section_id});
		}
		# Show the control panel
		if ($this->{input}{section_id}){
			$out = $this->classified_control_panel($this->{input}{section_id});
		# Section selector
		}else{
			$out = $this->set_section_id();
		}
	}
	return $out;
}
# Control panel
sub classified_control_panel{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	# Expire classifieds
	$this->expireClassified();
	# Fetch 'inactive' classifieds
	my @classifieds = @{$this->pendingClassified($sectionid)};
	my $table = "classified";
	my @data;
	# Report header
	my @header = ("Options","Posted By","Category","Scope","Title","Email","Location","Expiry Date");
	foreach my $classified(@classifieds){
		# Report options for each record
		my @options = (
		{url=>"javascript:activate($classified->{classified_id})",image=>"active",image_attr=>{label=>"Activate $classified->{title}"}},
		{url=>$this->link(cmd=>"edit",classified_id=>$classified->{classified_id}),image=>"edit",image_attr=>{height=>$config{db_ops}{edit}{img_height},width=>$config{db_ops}{edit}{img_width}}},
		{url=>"javascript:delete_record($classified->{classified_id})",image=>"delete",image_attr=>{height=>$config{db_ops}{delete}{img_height},width=>$config{db_ops}{delete}{img_width}}},
		);
		# Report row data
		my @rdata;
		push(@rdata,$this->recordOption(\@options));
		foreach my $column("member_id","category","scope","title","email","location","expiry_date"){
			push(@rdata,$this->show_data($table,$column,$classified->{$column},$classified->{classified_id}));
		}
		push(@data,\@rdata);
	}
	my %report = (
		title=>"Pending approval classifieds",
		headers=>\@header,
		data=>\@data,
		tools=>$this->recordOption([{url=>$this->link(cmd=>"new"),image=>"new",image_attr=>{height=>$config{db_ops}{new}{img_height},width=>$config{db_ops}{new}{img_width}}}]),
	);
	# Create the report
	my %data = (
		report=>$this->reportbuilder(\%report),
		report_option=>$this->adminReportOption(),
	);
	return $this->merge($this->html_controlpanel_template(),\%data);
}
# Expire classifieds
sub expireClassified{
	my ($this) = @_;
	# Expire classifieds
	my $sql = "update classified set status = 'expired' where expiry_date < '".$this->getCurrentDate()."'";
	$this->{db}->custom_query($sql);
	return;
}
# Admin report option
sub adminReportOption{
	my ($this) = @_;
	# Section id
	my $sectionid = $this->{input}{section_id};
	# Admin options
	my @options = (
	{url=>$this->link(cmd=>"current"),label=>"View Current Ads"},
	{url=>"$this->{cgi}/webdb.cgi?action=list&tab=classified",label=>"View All Ads"},
	{url=>$this->link(cmd=>"removeallexpired",section_id=>$sectionid),label=>"Remove Expired Ads"},	
	);
	return $this->commandOption($this->html_optionBulletStyleTemplate(),\@options);
}
# Pending for approval classifieds
sub pendingClassified{
	my ($this,$sectionid) = @_;
	my @records = @{$this->{db}->fetch_match("classified",{section_id=>$sectionid,status=>"inactive"})};
	return \@records
}

# Record options
sub recordOption{
	# $options: command options(array of hash references)
	my ($this,$options) = @_;
	return $this->commandOption($this->html_optionSimpleStyleTemplate(),$options);
}
# Take command option data and turn them into links
sub commandOption{
	# $template: template
	# $options: commands options(array of hash references)
	my ($this,$template,$options) = @_;
	my $out;
	foreach my $rec(@$options){
		my %data = (
			record_url=>$rec->{url},
			# Use icon / text for the link
			record_anchor=>(exists($rec->{image}) ? $this->getIcon($rec->{image},$rec->{image_attr}) : $rec->{label}),
		);
		$out .= $this->merge($template,\%data)
	}
	return $out;
}
# Get a predefined icon
sub getIcon{
	# $name: name of the icon
	# $opt: optional parameters
	my ($this,$name,$opt) = @_;
	my $path = "$config{server}{HTMLpath}/_Modules/Classified";
	# Define icons
	my %icons = (
		edit=>{url=>"$path/edit12.gif",height=>13,width=>12,label=>"Edit"},
		delete=>{url=>"$path/rm12.gif\" height=\"$config{db_ops}{delete}{img_height}",height=>13,width=>12,label=>"Delete"},
		new=>{url=>"$path/new12.gif",height=>13,width=>12,label=>"New"},
		active=>{url=>"$path/approve12.gif",height=>13,width=>12,label=>"Active"},
	);
	my %data = (
		icon_url=>$icons{$name}{url},
		icon_width=>($opt->{width} || $icons{$name}{width}),
		icon_height=>($opt->{height} || $icons{$name}{height}),
		icon_label=>($opt->{label} || $icons{$name}{label}),
	);
	my $template = $this->html_iconTemplate();
	return $this->merge($template,\%data);	
}
# Delete a classified
sub deleteClassified{
	# $classifiedid: classified id
	my ($this,$classifiedid) = @_;
	$this->delete("classified",$classifiedid) if $this->authorize();
	return;
}
# New classified
sub newClassified{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	my %data = (
		section_id=>$sectionid,
		category=>$this->{input}{category},
	);
	# Access level
	my $access = $this->access();
	# Hide form fields
	my @hide = ("section_id","member_id","expiry_date","status");
	# Member/admin
	if ($access){
		# Set classified status
		$data{status} = $this->setNewStatus();
		# Member id
		$data{member_id} = $this->{db}->my_uid();
		# Email address
		$data{email} = $share{DB}->my_email();
	}else{
		$data{status} = "inactive";
	}
	# If allow_scope setting isn't set to both local and global
	if ($this->isNotLocalGlobal()){
		# Set the scope to global
		if ($this->isGlobal()){
			$data{scope} = "global";
		# Set the scope to local
		}else{
			$data{scope} = "local";
		}
		# Hide scope input field
		push(@hide,"scope");
	}
	# Set default expiry date
	$data{expiry_date} = $this->getExpiryDate();
	# input_exsite handler
	$this->{db}->handler("input_exsite",\&custom_input_exsite);
	# Create the new classified form
	return $this->{db}->make(table=>"classified",data=>\%data,extra=>{reply=>$this->reply()},hide=>\@hide,action=>&relink({cmd=>"process_new",category=>undef}));
}
# Process new classified
sub processNew{
	my ($this) = @_;
	my %rec;
	foreach my $key (keys(%{$this->{input}})){
		# Gather classified info
		if ($key =~ /classified##(.+)/i){
			my $column = $1;
			$rec{$column} = $this->{input}{$key};
		}
	}
	# If a non-admin user tries to add a global ad, its status is set to 'inactive'
	if ($rec{scope} =~ /global/i && !$this->authorize()){
		$rec{status} = "inactive";
	}
	$this->{db}->insert("classified",\%rec);
	$this->redirect($this->{input}{reply});
	return;
}
# input_exsite handler
sub custom_input_exsite{
	my ($this,%opt) = @_;
	return undef if $this->is_admin();
	# Create drop list fields with default value selected
	if ($opt{table} eq "classified" && $opt{column} =~ /category|scope/i){
		# Get datatype
		my $dtype = $this->{map}->get_datatype_attr($opt{datatype},"regexp");
		return &exsite_list(\%opt,$dtype);
	}
}

# Create the drop down list input field
sub exsite_list{
	my ($data,$datatype) = @_;
	my $out = "<select name=\"$data->{name}\">";
	my @dtype = split(/\|/,$datatype);
	foreach my $type(@dtype){
		my $val = &html_escape($type);
		$out .= "<option value=\"$val\"";
		$out .= $val eq $data->{value} ? " selected" : "";
		$out .= ">$val</option>";
	}
	$out .= "</select>";
	return $out;
}

# Set status for new classified
sub setNewStatus{
	my ($this) = @_;
	# Access level
	my $access = $this->access();
	# Set the status to 'active', if this user is an admin
	if ($this->authorize()){
		return "active";
	}
	# Set the status to 'active', if this user is a member and the allow_scope is set to 'local'
	if ($access && $this->isLocal()){
		return "active";
	}
	return "inactive";
}
# Is the scope setting local?
sub isLocal{
	my ($this) = @_;
	return 1 if $config{Classified}{allow_scope} =~ /local/i;
	return 0;
}
# Is the scope setting global?
sub isGlobal{
	my ($this) = @_;
	return 1 if $config{Classified}{allow_scope} =~ /global/i;
	return 0;
}
# Is the scope setting  set to either local or global?
sub isNotLocalGlobal{
	my ($this) = @_;
	return 1 if !$this->isLocal();
	return 1 if !$this->isGlobal();
	return 0;
}
# Is the scope setting set to both local and global?
sub isLocalGlobal{
	my ($this) = @_;
	return 1 if $this->isLocal() && $this->isGlobal();
	return 0;
}
# Get the ExSite::Time object
sub timeObject{
	my ($this) = @_;
	# Time object exists
	return $this->{time_object} if exists($this->{time_object});
	# Time object doesn't exist
	require ExSite::Time;
	$this->{time_object} = new ExSite::Time;
	return $this->{time_object};
}
# Get current date
sub getCurrentDate{
	my ($this) = @_;
	my $tobj = $this->timeObject();
	return $tobj->GetCurrentSqlDate();	
}
# Get expiry date
sub getExpiryDate{
	my ($this) = @_;
	my $tobj = $this->timeObject();
	my $lifetime = $config{Classified}{ad_lifetime} || 0;
	# Calculate the expiry date based on the ad_lifetime setting
	return $tobj->GetAdvanceDate($lifetime);
}
# Edit a classified
sub edit{
	# $classifiedid: classified id
	my ($this,$classifiedid) = @_;
	# Fetch classified record
	my %rec = $this->{db}->fetch("classified",$classifiedid);
	# Hide these input fields
	my @hide = ("section_id","member_id","expiry_date","status");
	return $this->{db}->make(table=>"classified",data=>\%rec,extra=>{reply=>$this->reply()},hide=>\@hide);
}
# Set a classified to active
sub setActive{
	# $classifiedid: classified id
	my ($this,$classifiedid) = @_;
	return if !$this->authorize();
	$this->setStatus($classifiedid,"active");
	return;
}
# Set the status of a classified
sub setStatus{
	# $id: classified id
	# $status: status
	my ($this,$id,$status) = @_;
	$this->{db}->update("classified",{status=>$status},{classified_id=>$id});
	return;
}
# Delete all expired classifieds
sub removeAllExpired{
	my ($this) = @_;
	return if !$this->authorize();
	# Delete all 'expired' classifieds
	my $sql = "select * from classified where status = 'expired'";
	my @records = $this->{db}->custom_query($sql);
	foreach my $rec(@records){
		$this->delete("classified",$rec->{classified_id});
	}
	return;	
}
# Fetch active classifieds
sub activeClassified{
	# $sectionid: section id
	# $category: category
	my ($this,$sectionid,$category) = @_;
	# Get current date
	my $today = $this->getCurrentDate();
	# By default show local ads
	my $sql = "select * from classified where status = 'active' && expiry_date >= '$today' ";
	$sql .= $category ? "&& category = '$category' " : "";
	my @records = $this->{db}->custom_query($sql);
	return\@records;
}
# Counting the number of classifieds in a category
sub groupCategory{
	# $records: records
	my ($this,$records) = @_;
	my %category;
	foreach my $rec(@$records){
		$category{$rec->{category}}++;
	}
	return \%category;
}
# Show category list
sub showCategoryList{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	# Classifieds
	my @classifieds = @{$this->activeClassified($sectionid)};
	# Count the number of classifieds in each category
	my %category = %{$this->groupCategory(\@classifieds)};
	my @category;
	my %data;
	foreach my $cat(sort {$a cmp $b} keys(%category)){
		my $category = &html_escape($cat);
		my %rdata = (
			category_url=>&relink({section_id=>$sectionid,category=>$category}),
			category_name=>$category,
			category_count=>$category{$cat},
		);
		# Report rows
		my $template = $this->html_categoryStyleTemplate();
		$data{category_list} .= $this->merge($this->html_categoryStyleTemplate,\%rdata);
	}
	return $this->merge($this->html_categoryListTemplate(),\%data);
}
# Sort classifed by section id, date and title
sub sortClassified{
	# $data: classifieds
	# $sectionid: section id
	my ($this,$data,$sectionid) = @_;
	# Change all local global ads' scope to 'local'
	for(my $i = 0; $i < @$data; $i++){
		if ($data->[$i]{section_id} == $sectionid){
			$data->[$i]{scope} = "local";
		}
	}
	# Sort by:
	# 1. scope descending
	# 2. section id accending
	# 3. creation date descending
	my @sorted = sort { $b->{scope} cmp $a->{scope} || $a->{section_id} <=> $b->{section_id} || $b->{date} cmp $a->{date}} @$data;
	return \@sorted;
}
# Show classifieds
sub showClassified{
	# $sectionid: section id
	# $category: category
	my ($this,$sectionid,$category) = @_;
	# Fetch active classifieds
	my @classifieds = @{$this->activeClassified($sectionid,$category)};
	# Sort classifieds
	@classifieds = @{$this->sortClassified(\@classifieds,$sectionid)};
	my %data = (
		classified_category=>$category,
	);
	foreach my $classified(@classifieds){
		# Classified id
		my $clfdid = $classified->{classified_id};
		# Contact form url
		$classified->{classified_url} = &relink({action=>"contact",classified_id=>$clfdid});
		$classified->{classified_title} = $classified->{title};
		$classified->{classified_description} = $this->{db}->show_data("classified","description",$classified->{description},$clfdid);
		$classified->{classified_location} = $classified->{location};
		$classified->{classified_creationdate} = $this->{db}->show_data("classified","date",$classified->{date},$clfdid);
		$classified->{classified_edit_url} = &relink({cmd=>"edit",classified_id=>$clfdid});
		$classified->{classified_delete_url} = "javascript:delete_record($clfdid)";
		my $template = $this->authorize() ? $this->html_classifiedStyleTemplateAdmin() : $this->html_classifiedStyleTemplatePublic();
		$data{classified_list} .= $this->merge($template,$classified);
		# If posting new classified is allowed
		if ($this->allowNewPost()){		
			$data{classified_add_new_text} = "Post a classified ad";
			$data{classified_add_new_url} = &relink({action=>"new"});
		}
	}
	my $template = $this->authorize() ? $this->html_classifiedListTemplateAdmin() : $this->html_classifiedListTemplatePublic();
	return $this->merge($template,\%data);
}
# Alow the user to post new classifieds?
sub allowNewPost{
	my ($this) = @_;
	# Access level
	my $access = $this->access();
	# Members are allowed to post new classifieds
	return 1 if $access;
	# Config setting allows anyone to post new classifieds
	return 1 if $config{Classified}{allow_anonymous};
	return 0;
}
# Send the contact an email
sub processContactForm{
	my ($this) = @_;
	# Post data
	my %input = %{$this->{input}};
	# Email address is required
	if (!$input{email}){
		return "<p>Please enter your email address.<br><br><input type=\"button\" value=\"&lt; Back\" onClick=\"history.back()\"></p>";
	}
	# Classified data
	my %classified = $this->{db}->fetch("classified",$input{classified_id});
	my $subject = $input{subject};
	my $body = $input{message};
	my $to = $classified{email};
	my $from = $input{email};
	if ($to && $from){	
		&SendMail($to,$from,$subject,$body);
	}
	return "<p>Your message has been sent to the advertiser.<br><a href=\"$input{reply}\">Click here to go back to the listing.</a></p>";
}
# Contact email form
sub contactForm{
	my ($this,$classifiedid) = @_;
	my %classified = $this->{db}->fetch("classified",$classifiedid);
	my %form = (
		contact_reply=>&relink({action=>undef,classified_id=>undef}),
		contact_post_url=>&relink({action=>"contact_post",classified_id=>undef,category=>undef,section_id=>undef}),
		contact_email=>$classified{email},
		contact_subject=>&html_escape("Re: $classified{title}"),
		contact_message=>&html_escape($classified{description}),
		contact_classified_id=>$classified{classified_id},
	);
	return $this->merge($this->html_contactFormTemplate(),\%form);
}
# Control panel template
sub html_controlpanel_template{
	my ($this) = @_;
	my $out = $this->javascript();
	$out .= "
<!--[[report]]-->
<!--[[report_option]]-->
	";
	return $out;
}
# Contact form template
sub html_contactFormTemplate{
	my ($this) = @_;
	my $out = "
<table cellspacing=\"5\">
<form method=\"post\" action=\"<!--[[contact_post_url]]-->\">
<input type=\"hidden\" name=\"reply\" value=\"<!--[[contact_reply]]-->\">
<input type=\"hidden\" name=\"classified_id\" value=\"<!--[[contact_classified_id]]-->\">
<tr><td valign=\"top\"><b>Your email address:(required)</b></td><td><input type=\"text\" name=\"email\" value=\"<!--[[contact_email]]-->\"></td></tr>
<tr><td valign=\"top\">Subject:</td><td><input type=\"text\" name=\"subject\" value=\"<!--[[contact_subject]]-->\"></td></tr>
<tr><td valign=\"top\">Message:</td><td><textarea cols=\"80\" rows=\"15\" name=\"message\"><!--[[contact_message]]--></textarea></td></tr>
<tr><td valign=\"top\" colspan=\"2\">
<hr>
<i>
Buyer beware! We have not confirmed that the goods or services offered by this ad are legitimate or of the stated condition/quality.  
The buyer is solely responsible for evaluating the purchase, and negotiating a fair exchange with the seller.  
If you are unable to view the goods directly (becuase you are in a different city, for instance), 
then the risk of buying sight unseen is entirely yours.
</i>
<hr>
</td></tr>
<tr><td colspan=\"2\"><input type=\"submit\" value=\"Submit\">&nbsp;&nbsp;&nbsp;&nbsp;<input type=\"reset\" value=\"Reset\"></td></tr>
</form>
</table>
	";
	return $out;
}
# Classified template - admin
sub html_classifiedStyleTemplateAdmin{
	my ($this) = @_;
	my $out = "
<p>
section: <!--[[section_id]]--><br>
[<a href=\"<!--[[classified_edit_url]]-->\">Edit</a> | <a href=\"<!--[[classified_delete_url]]-->\">Delete</a>] <a href=\"<!--[[classified_url]]-->\"><!--[[classified_title]]--></a> <br>
<!--[[classified_description]]--><br>
<!--[[classified_location]]--><br>
<i>posted <!--[[classified_creationdate]]--></i>
</p>
	";
	return $out;
}
# Classified template - public
sub html_classifiedStyleTemplatePublic{
	my ($this) = @_;
	my $out = "
<p>
<a href=\"<!--[[classified_url]]-->\"><!--[[classified_title]]--></a><br>
<!--[[classified_description]]--><br>
<!--[[classified_location]]--><br>
<i>posted <!--[[classified_creationdate]]--></i>
</p>
	";
	return $out;
}
# Classified list template - admin
sub html_classifiedListTemplateAdmin{
	my ($this) = @_;
	my $out = $this->javascript();
	$out .= "
<h1><!--[[classified_category]]--></h1>
<p><a href=\"<!--[[classified_add_new_url]]-->\"><!--[[classified_add_new_text]]--></a></p>
<!--[[classified_list]]-->
<p><a href=\"<!--[[classified_add_new_url]]-->\"><!--[[classified_add_new_text]]--></a></p>
	";
	return $out;
}
# Classified list template - public
sub html_classifiedListTemplatePublic{
	my ($this) = @_;
	my $out = "
<h1><!--[[classified_category]]--></h1>
<p><a href=\"<!--[[classified_add_new_url]]-->\"><!--[[classified_add_new_text]]--></a></p>
<!--[[classified_list]]-->
<p><a href=\"<!--[[classified_add_new_url]]-->\"><!--[[classified_add_new_text]]--></a></p>
	";
	return $out;
}
# Category list template
sub html_categoryListTemplate{
	my ($this) = @_;
	my $out = "
<!--[[category_list]]-->
	";
	return $out;
}
# Category template
sub html_categoryStyleTemplate{
	my ($this) = @_;
	my $out = "<p><a href=\"<!--[[category_url]]-->\"><!--[[category_name]]--></a> (<!--[[category_count]]-->)</p>";
	return $out;
}
# Member report javascript
sub javascript{
	my ($this) = @_;
	my $out = "
<script language=\"javascript\">
<!--
function delete_record(id){
	if(confirm('Are you sure you want to delete this classified?')){
		location.href = '".&relink({cmd=>"del",classified_id=>undef})."&classified_id=' + id;
	}
}
function activate(id){
	name = escape(name);
	if(confirm('Are you sure you want to approve this classified?')){
		location.href = '".&relink({cmd=>"set_active",classified_id=>undef})."&classified_id=' + id;
	}
}
// -->
</script>
	";
	return $out;
}
# Icon template
sub html_iconTemplate{
	my ($this) = @_;
	return "<img src=\"<!--[[icon_url]]-->\" align=\"middle\" hspace=\"3\" border=\"0\" width=\"<!--[[icon_width]]-->\" height=\"<!--[[icon_height]]-->\" title=\"<!--[[icon_label]]-->\" alt=\"<!--[[icon_label]]-->\">";
}
# Bulleted style template
sub html_optionBulletStyleTemplate{
	my ($this) = @_;
	return "<li><a href=\"<!--[[record_url]]-->\"><!--[[record_anchor]]--></a></li>";
}
# Simple style template
sub html_optionSimpleStyleTemplate{
	my ($this) = @_;
	return "<a href='<!--[[record_url]]-->'><!--[[record_anchor]]--></a>";
}
#----------------------------------------------------------------------
# Get member access level
sub access{
	my ($this) = @_;
	return $this->{db}->authorize();
}
# authorization check
sub authorize{
	my ($this) = @_;
	my $access = $this->access();
	return $access if $this->{db}->is_admin();
	return $access if $this->{db}->owns();
	return 0;
}
# Build reports using ExSite::ReportBuilder
sub reportbuilder{
	# $opt: option(hash reference)
	# 	title: report title
	# 	data: report data
	# 	tools: report tool option
	# 	headers: report header
	# 	foot: report footer
	# 	width: report width
	my ($this,$opt) = @_;
	require ExSite::ReportBuilder;
	$this->{rb} = new ExSite::ReportBuilder() if !$this->{rb};
	my $rb = $this->{rb};
	$rb->init(
		title=>$opt->{title},
		data=>$opt->{data},
		tools=>$opt->{tools},
		headers=>$opt->{headers},
		foot=>$opt->{foot},
		width=>$opt->{width},
	);
 	return $rb->make();
}
sub reply{
	my ($this) = @_;
	return "$config{server}{server}".&relink({classified_id=>undef,cmd=>undef,action=>undef});
}
# Page redirection
sub redirect{
	# $reply: reply url
        my ($this,$reply) = @_;
	$reply = $reply || &relink({action=>undef});
        my $p = $share{Page} || $share{PageBuf};
        if ($p){
                $p->redirect($reply);
        }else{
                print "location: $reply\n\n";
        }
        return;
}
# Data merge
sub merge{
	# $view: template
	# $data: merge data
        my ($this,$view,$data) = @_;
	return if !$view || !$data;
        $view =~ s/\<\!\-\-\[\[((\w+(\_| )?)+\w*)\]\]\-\-\>/$data->{$1}/g;
        return $view;
}
# Delete a record
sub delete{
	# $table: name of the table
	# $id: record id
	my ($this,$table,$id) = @_;
	return if !$id || !$table;
	$share{DB}->trash_r($table,$id);
	#$this->redirect(&relink({action=>undef,classified_id=>undef,cmd=>undef}));
	return;
}
# is admin?
sub is_admin{
	my ($this) = @_;
	return $share{DB}->is_admin();
}
# show data
sub show_data { 
	# $column: table column
	my ($this,$table,$column,$data,$id) = @_;
    	my $db = $share{DB};
    	return $db->show_data($table,$column,$data,$id);
}
#----------------------------------------------------------------------
1;
