#------------------------------------------------------------------------------------------------------
# Ecommerce Module
#------------------------------------------------------------------------------------------------------
#
# Usage:
#
# <!--&Ecommerce(parameters)-->
# 
# Ecommerce module requires you to pass in a parameter consisting of an ampersand delimited
# string of "key=value" pairs.
#
# Required key values:
# - name : name
# - first : first name
# - last : last name
# - invoice_number : invoice number
# - section_id : section id
# - amount : purchase amount
# - return_url : return url
# 	e.g. http://xxxx.xxxxx.xxxx/cgi/page.cgi?_id=xxx
#
# Optional key values:
# - member_id : member id
# - note : notes
#
#------------------------------------------------------------------------------------------------------

package Modules::Ecommerce;
use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Input;
use Modules::BaseDCD;
use ExSite::ReportBuilder;
use vars qw(@ISA);
push(@ISA,"Modules::BaseDCD");

#------------------------------------------------------------------------------------------------------

sub new {
	my $this = shift;
	my ($obj,$class);
	$obj = {};
	$class = ref($this) || $this;
	bless $obj, $class;
	$class =~ /^(.+?::)+?(.+)$/;
	$obj->{class_name} = $2;
	$obj->{db} = $share{DB};
	$obj->{cgi} = $config{server}{server}.$config{server}{CGIpath};
	$obj->{html} = $config{server}{server}.$config{server}{HTMLpath};
	$obj->read;  # inherits BaseDCD::read()
	$obj->read_conf;
	return $obj;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return "Ecommerce";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
        	return "Ecommerce tools allow credit card payments through payment gateways.";
	}elsif (/isRestricted/){
		return 1;
	}elsif (/isService/){
		return 1;
	}
}
sub write{
	my ($this,$options) = @_;
	my (%opt,$action);
	%opt = &DecodeString($options);
	$action = $this->{input}{ec_action} || $opt{ec_action};
	# Process payment result
	if ($action eq "result"){
		return $this->paymentResult();
	# Send payment to the gateway
	}elsif ($action eq "transaction"){
		return $this->processPayment();
	# Create a paynow link
	}else{
		return $this->payNow(\%opt);		
	}
	return undef;
}
#---------------------------------------------------------------------
# Control panel
sub control_panel{
	my $this = shift;
	my $out;
	# Admin access only
	if (!$this->authorize($this->{input}{section_id})){
		$out = "<h3 class=\"error\">Permission denied</h3><p>You do not have sufficient system priveleges to use this feature.</p>\n";
	}else {
		my $action = $this->{input}{action};
		# View transaction report
		if ($action eq "view_report" && $this->{input}{section_id}){
			$out .= $this->transactionReport($this->{input}{section_id},$this->{input}{yearmonth});
		# Create search form
		}elsif ($action eq "search_form"){
			$out .= $this->searchForm($this->{input}{section_id});
		# Get search results
		}elsif ($action eq "search"){
			$out .= $this->{db}->do();
		# Allows the user to view transaction reports based by selecting the website
		}else{
			$out .= $this->selectSection();
		}
	}
	return $out;
}
#-------------------------------------------------------------------
# Process ecommerce query
sub payNow{
	my ($this,$in) = @_;
	my $out;
	# Inform the user that required form parameters are missing
	if (!$this->checkRequiredParameter($in)){
		my @missing_param = @{$this->missingParameter($in)};
		my $admin_email = $this->getAdminEmail();
		$out = "<p style=\"font-size: 12px; border-color: red; border-width:2px; border-style: dashed;\">Ecommerce module error:<br>The following payment specific parameters were missing:<br><br>";
		foreach my $param(@missing_param){
			$out .= "&#149;$param->{label}<br>";
		}
		$out .= "<br>Please report these errors to the site administrator(<a href=\"mailto:$admin_email\">$admin_email</a>).";
		$out .= "</p>";
		return $out;
	}
	# Areate the 'paynow' link or button
	my %input = %{$this->requiredParameter($in)};
	$input{member_id} = $in->{member_id} if $in->{member_id};
	$input{note} = $in->{note} if $in->{note};
	$input{return_url} = $in->{return_url};
	# Add gateway specific parameters
	%input = (%input,%{$this->getGatewayParameter()});
	if ($in->{type} eq "link"){
		# Create a pay now link
		return $this->payNowLink(\%input);
	}else{
		# Create a pay now button
		return $this->payNowButton(\%input);
	}
	return;
}
# Get admin email address
sub getAdminEmail{
	my ($this) = @_;
	return $config{site}{administrator};
}
# Get the required form parameters
sub requiredParameter{
	my ($this,$params) = @_;
	# Get required parameters
	my @required = @{$this->getRequiredParameter()};
	my %required;
	# Remove parameters that aren't recognized by the Ecommerce module
	foreach my $req(@required){
		$required{$req->{id}} = $params->{$req->{id}};
	}
	return \%required;
}
# Figure out the missing form fields
sub missingParameter{
	my ($this,$params) = @_;
	# Get required parameters
	my @required = @{$this->getRequiredParameter()};
	my @missing;
	# Check to see if there are any missing parameters
	foreach my $req(@required){
		push(@missing,$req) if !exists($params->{$req->{id}});
	}
	return \@missing;
}
# Verify that the required form parameters are provided
sub checkRequiredParameter{
	my ($this,$params) = @_;
	my @required = @{$this->getRequiredParameter()};
	foreach my $req(@required){
		return 0 if !exists($params->{$req->{id}});
	}
	return 1;
}
# Required form fields
sub getRequiredParameter{
	my ($this) = @_;	
	return [
		{id=>"name",label=>"Full name"},
		{id=>"first",label=>"First name"},
		{id=>"last",label=>"Last name"},
		{id=>"invoice_number",label=>"Invoice number"},
		{id=>"section_id",label=>"Website"},
		{id=>"amount",label=>"Amount"},
	];
}
# Form fields(gateway specific)
sub getGatewayParameter{
	my ($this) = @_;
	return $this->beanstreamParameter();
}
# Merge data into a template
sub merge{
        my ($this,$view,$data) = @_;
	return if !$view || !$data;
        $view =~ s/\<\!\-\-\[\[((\w+(\_| )?)+\w*)\]\]\-\-\>/$data->{$1}/g;
        return $view;
}
# Paynow link(gateway specific)
sub payNowLink{
	my ($this,$in) = @_;
	return $this->beanstreamLink($in);
}
# Paynow button(gateway specific)
sub payNowButton{
	my ($this,$in) = @_;
	return $this->beanstreamButton($in);
}
# Form hidden fields
sub defaultFormHidden{
	my ($this,$in) = @_;
	my %hidden = (
		ec_action=>"transaction",
		name=>$in->{name},
		first=>$in->{first},
		last=>$in->{last},
		invoice_number=>$in->{invoice_number},
		amount=>$in->{amount},
	);
	my $out;
	while(my($key,$val) = each(%hidden)){
		$out .= "\n<input type=\"hidden\" name=\"$key\" value=\"$val\">";
	}
	return $out;
}
# Process payment
sub processPayment{
	my ($this) = @_;
	my %input = %{$this->{input}};

	delete $input{ec_action};
	
	# Insert a transaction record and set the status to 'incomplete'
	$this->insertTransactionRecord(\%input);
	
	# Flag the user as paid when the purchase amount is 0
	# (don't need to post the payment to the gateway)
	if ($input{amount} <= 0){
		$this->complementaryPurchase(\%input);
		return;
	}
	
	my $querystring = &EncodeHash(%input);
	my $url = $this->getGatewayUrl($querystring);
	# Post the transaction to the payment gateway
	print "location: $url\n\n";
	return;
}
# Process complementary purchases(purchases with amount of $0)
sub complementaryPurchase{
	my ($this,$input) = @_;
	return $this->beanstreamComplementaryPurchase($input);
}
# Generate url to post transaction to the payment gateway
sub getGatewayUrl{
	my ($this,$querystring) = @_;
	return $this->beanstreamGatewayUrl($querystring);
}
# insert transaction record
sub insertTransactionRecord{
	my ($this,$in) = @_;
	my %metahash = (name=>$in->{name},invoice_number=>$in->{invoice_number},amount=>$in->{amount});
	$metahash{note} = $in->{note} if $in->{note};
	# Prepare meta data field data for the transaction record
	my $metadata = &EncodeHash(%metahash);
	my %insert = (
		section_id=>$in->{section_id},
		member_id=>$in->{member_id},
		amount=>$in->{amount},
		status=>"incomplete",
		metadata=>$metadata,
		invoice=>$in->{invoice_number},
	);
	# Insert a new transaction record
	my $trans_id = $this->{db}->insert("ecommerce_transaction",\%insert);
	return;	
}
# Process payment results
sub paymentResult{
	my ($this) = @_;
	my %input = %{$this->{input}};
	# Approved transaction
	if ($this->checkTransactionStatus(\%input)){
		$this->setCompletedStatus(\%input);
		# Print out the receipt
		return "your receipt...";
	# Failed transaction
	}else{
		$this->setFailedStatus(\%input);
		# Print out error messages
		return "failed transaction...";
	}
	return;

}
# Set the transaction status to 'completed'
sub setCompletedStatus{
	my ($this,$in) = @_;
	$this->setStatus($in,"completed");
	# Save the transaction status to $share
	$share{Ecommerce}{completed} = 1;
	return;
}
# Set the transaction status to 'failed'
sub setFailedStatus{
	my ($this,$in) = @_;
	$this->setStatus($in,"failed");
	return;
}
# Set transaction status
sub setStatus{
	my ($this,$in,$status) = @_;
	# Fetch a transaction record
	my @recs = $this->{db}->fetch_match("ecommerce_transaction",{invoice=>$in->{invoice_number}});
	return if @recs < 1;
	my $rec = @recs[0];
	# Set status
	$rec->{status} = $status;
	# Update transaction status
	$this->{db}->update("ecommerce_transaction",$rec,{invoice=>$rec->{invoice}});
	return;
}
# Check transaction status
sub checkTransactionStatus{
	my ($this,$in) = @_;
	# Run gateway specific status checking subroutine
	return $this->beanstreamTransactionStatus($in);
}
# Fetch transaction records
sub fetch_transaction{
	my ($this,$sectionid,$yearmonth) = @_;
	if ($yearmonth){
		# Selecting transactions occur in a specific period
		my $sql = "select * from ecommerce_transaction where section_id = $sectionid and stamp like '${yearmonth}%' order by stamp desc";
		return $this->{db}->custom_query($sql);
	}else{
		# Fetch all transactions
		return $this->{db}->fetch_match("ecommerce_transaction",{section_id=>$sectionid},"stamp desc");
	}
	return [];
}
# Transaction report
sub transactionReport{
	my ($this,$sectionid) = @_;	
	# Fetch transaction records
	my @recs = @{$this->fetch_transaction($sectionid,$this->{input}{yearmonth})};
	# Report header
	my @header = ("Transaction Date","Member","Invoice","Status","Amount","Additional Info");
	my $out;
	my @data;
	my %total;
	# Report title
	my $title = $this->{input}{yearmonth} ? "Transactions for ".$this->fixTimestamp($this->{db}->show_data("ecommerce_transaction","stamp","$this->{input}{yearmonth}00000000")) : "All Transactions";
	foreach my $rec(@recs){
		# Format report row data
		push(@data,[
			$this->{db}->show_data("ecommerce_transaction","stamp",$rec->{stamp},$rec->{id}),
			($this->{db}->show_data("ecommerce_transaction","member_id",$rec->{member_id},$rec->{id}) || "n/a"),
			$rec->{invoice},
			$rec->{status},
			sprintf("\$%.2f",$rec->{amount}),
			$this->format_metadata($rec->{metadata}),
		]);
		# Summarize transactions		
		$total{$rec->{status}} += $rec->{amount};
	}
	# Tool button
	my $searchimg = "$config{server}{HTMLpath}/_Modules/WebDB/search12.gif";
	my $toollink = "<a href=\"".$this->link(action=>"search_form",section_id=>$sectionid)."\">";
	$toollink .= "<img src=\"$searchimg\" border=\"0\" title=\"Search\" alt=\"Search\">";
	$toollink .= "</a>";
	# Create the transaction report
	my %report = (
		title=>$title,
		data=>\@data,
		headers=>\@header,
		tools=>$toollink,
		# Summary report
		foot=>$this->reportbuilder({title=>"Transaction Summary",headers=>["Approved","Incomplete","Failed"],data=>[sprintf("\$%.2f",$total{completed}),sprintf("\$%.2f",$total{incomplete}),sprintf("\$%.2f",$total{failed})],width=>"100%"}),
	);
	my %report_date = $this->getReportYearMonthSelect($sectionid);
	# Form field for selecting to view transactions occur in a specific month
	my $post_action = $this->link(action=>undef,section_id=>undef);
	$out .= "\n<form method=\"post\" action=\"$post_action\">";
	$out .= "\n<input type=\"hidden\" name=\"action\" value=\"view_report\">";
	$out .= "\n<input type=\"hidden\" name=\"section_id\" value=\"$sectionid\">";
	$out .= "\n<p>\nView transaction for: ";
	$out .= "\n<select name=\"yearmonth\">";
	$out .= "\n<option value=\"\">All Transactions</option>";
	# Sort year(decendant order)
	foreach my $year(sort {$b cmp $a} (keys(%report_date))){
		my %year = %{$report_date{$year}};
		# Sort months(decendant order)
		foreach my $month(sort {$b <=> $a} (keys(%year))){
			my $yearmon = $this->fixTimestamp($this->{db}->show_data("ecommerce_transaction","stamp","${year}${month}00000000"));
			$out .= "\n<option value=\"${year}${month}\"";
			$out .= "${year}${month}" eq $this->{input}{yearmonth} ? " selected" : "";
			$out .= ">$yearmon</option>";
		}
	}
	$out .= "\n</select>";
	$out .= "\n&nbsp;&nbsp;<input type=\"submit\" value=\"Submit\">";
	$out .= "\n</p>";
	$out .= "\n</form>";
	$out .= "\n";
	$out .= $this->reportbuilder(\%report);	
	return $out;
}
# Get the data for creating the select field for the transaction report
sub getReportYearMonthSelect{
	my ($this,$sectionid) = @_;
	# Fetch records based on the year and the month
	my @recs = $this->{db}->custom_query("select distinct(substring(stamp,1,6)) as stamp from ecommerce_transaction where section_id = $sectionid");
	my %result;
	foreach my $rec(@recs){
		my $yearmonth = $rec->{stamp};
		# Extract year and month
		$yearmonth =~ /^(\d{4})(\d{2}).*/i;
		my $year = $1;
		my $mon = $2;
		# Index data by year then month
		$result{$year}{$mon} = 1;
	}
	return %result;
}
# Extract year and month from the timestamp
sub fixTimestamp{
	my ($this,$timestamp) = @_;
	# Extra year and month
	$timestamp =~ /^(\d{4})-(\w{3})/i;
	# Year
	my $year = $1;
	# Month
	my $month = $2;
	return "$year $month";
}
# Select the website that you want to view the report
sub selectSection{
	my ($this) = @_;
	my @sites = $this->{db}->my_real_sites();
	my $out;
	$out .= "\n<h1>Select a website</h1>";
	$out .= "\n<form method=\"post\">";
	$out .= "\n<input type=\"hidden\" name=\"action\" value=\"view_report\">";
	$out .= "\n<select name=\"section_id\">";
	foreach my $site(@sites){
		$out .= "\n<option value=\"$site->{section_id}\">$site->{title}</option>";
	}
	$out .= "\n</select>";
	$out .= "&nbsp;&nbsp;";
	$out .= "\n<input type=\"submit\" value=\"Select\">";
	$out .= "\n</form>";
	return $out;
}
# Format metadata(ampersand delimited key value pair string)
sub format_metadata{
	my ($this,$mdata) = @_;
	my %data = &DecodeString($mdata);
	my $out;
	foreach my $key(keys(%data)){
		$out .= "$key: $data{$key}<br>";
	}
	return $out;
}
# Fransaction search form
sub searchForm{
	my ($this,$sectionid) = @_;	
	$this->{db}->handler("select_foreign_key",\&custom_select_foreign_key);
	# Form action url
	my $actionurl = $this->link(action=>"search",section_id=>undef);
	my $out = "<h2>Search all transactions</h2>";
	$out .= $this->{db}->make(table=>"ecommerce_transaction",method=>"post",action=>$actionurl,search=>1,data=>{section_id=>$sectionid},hide=>["section_id"]);
	return $out;
}
# Select foreign key handler
sub custom_select_foreign_key{
	my ($this, $table, $rec, $col, $ftable) = @_;
	if ($table eq "ecommerce_transaction" && $col eq "section_id"){
		my @sites = $this->my_real_sites();
		return \@sites;
	}
	return;
}
#-----------------------------------------------------
# Report builder using ExSite::ReportBuilder
sub reportbuilder{
	my ($this,$opt) = @_;
	require ExSite::ReportBuilder;
	$this->{rb} = new ExSite::ReportBuilder() if !$this->{rb};
	my $rb = $this->{rb};
	$rb->init(
		title=>$opt->{title},
		data=>$opt->{data},
		tools=>$opt->{tools},
		headers=>$opt->{headers},
		foot=>$opt->{foot},
		width=>$opt->{width},
	);
	return $rb->make();
}
# Redirect
sub redirect{
        my ($this,$reply) = @_;
        my $p = $share{Page} || $share{PageBuf};
        if ($p){
                $p->redirect($reply);
        }else{
                print "location: $reply\n\n";
        }
        return;
}
# Check and see if the user has sufficient access to use control panel
sub authorize{
	my ($this,$sectionid) = @_;
	return $this->{db}->owns($sectionid);
}
#-----------------------------------------------------
# beanstream payment gateway
#-----------------------------------------------------
# Complementary purchase
sub beanstreamComplementaryPurchase{
	my ($this,$input) = @_;
	my %res = (
		messageText=>"approved",
		trnApproved=>1,
		section_id=>$input->{section_id},
		member_id=>$input->{member_id},
		invoice_number=>$input->{invoice_number},
		ec_action=>"result",
	);
	my $url = $input->{return_url}."&".&EncodeHash(%res);
	$this->redirect($url);
	return;
}
# Gateway url
sub beanstreamGatewayUrl{
	my ($this,$querystring) = @_;
	return "$config{server}{server}$config{server}{CGIpath}/gateway.cgi?ec_action=result&".$querystring;
}
# Check transaction status
sub beanstreamTransactionStatus{
	my ($this,$in) = @_;
	if ($in->{messageText} =~ /approved/i){
		return 1;
	}
	return 0;
}
# Get merchant id
sub beanstreamMerchantID{
	my ($this) = @_;
	return $config{$this->{class_name}}{config}{merchant_id};
}
# Get gateway parameters
sub beanstreamParameter{
	my ($this) = @_;
	my %param  = (
		transaction_type=>"P",
		merchant_id=>$this->beanstreamMerchantID(),
	);
	return \%param;
}
# Payment button
sub beanstreamButton{
	my ($this,$in) = @_;
	return $this->beanstreamForm_HTML($in,{type=>"button"});
}
# Payment link
sub beanstreamLink{
	my ($this,$in) = @_;
	return $this->beanstreamForm_HTML($in,{type=>"link"});
}
# Payment form
sub beanstreamForm_HTML{
	my ($this,$in,$opt) = @_;
	my $form_hidden = $this->defaultFormHidden($in);
	$form_hidden .= "\n<input type=\"hidden\" name=\"section_id\" value=\"<!--[[section_id]]-->\">
<input type=\"hidden\" name=\"return_url\" value=\"<!--[[return_url]]-->\">
<input type=\"hidden\" name=\"note\" value=\"<!--[[note]]-->\">
<input type=\"hidden\" name=\"member_id\" value=\"<!--[[member_id]]-->\">
<input type=\"hidden\" name=\"trnType\" value=\"<!--[[transaction_type]]-->\">
<input type=\"hidden\" name=\"merchant_id\" value=\"<!--[[merchant_id]]-->\">
<input type=\"hidden\" name=\"trnAmount\" value=\"<!--[[amount]]-->\">
<input type=\"hidden\" name=\"trnOrderNumber\" value=\"<!--[[invoice_number]]-->\">
<input type=\"hidden\" name=\"ordName\" value=\"<!--[[name]]-->\">";
	$form_hidden = $this->merge($form_hidden,$in);
	my $url = $this->link;
	my $out = "\n<form method=\"post\" action=\"$url\">\n";
	$out .= $form_hidden;
	if ($opt->{type} eq "link"){
		$out .= "\n<a href=\"javascript:document.forms[0].submit()\">Proceed to check-out</a>";
	}else{
		$out .= "\n<input type=\"Submit\" value=\"Proceed to check-out\">";
	}
	$out .= "\n</form>\n";
	return $out;
}
#-----------------------------------------------------
1;
