package Modules::Email;

#-----------------------------------------------------------------------
# Email.pm - email notifications
#
# this module takes no options
#
# NB. this module simply creates the email records.  Sending of the 
# emails is performed by a cron job.
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Form;
use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

use vars qw($db $level %input %member $done);

# write : this module produces no content

sub write {
    my ($this,$options) = @_;
    return ""; 
}

# ioctl

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "Emails";
    }
    elsif (/ModuleInfo/) {
	return "The Email system automates email communications with your
membership.  Email blasts can be configured to go out on certain dates, 
to the currently active membership list.";
    }
    elsif (/isRestricted/) {
	# accessibility is controlled by config file
	return $config{Email}{isRestricted};
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
}

sub ctrl_panel {
    my $this = shift;
    my $in = $this->{input};
    my $action = $in->{emcmd} || "list";
    my $db = $share{DB};
    my $level = $db->authorize();

    my $out;
    if ($level < 2) {
	return "<H2>Access Denied</H2>\n<P CLASS=error>Admin-level access required for email functions.</P>\n";
    }
    $out .= "<h1>Email Notifications</h1>\n";
    if ($action eq 'edit') {
	$out .= $this->edit($in->{id});
    }
    elsif ($action eq 'delete') {
	$out .= $this->del($in->{id});
    }
    elsif ($action eq 'add') {
	$out .= $this->add();
    } 
    elsif ($action eq 'do_add') {
	$out .= $this->do_add();
    } 
    elsif ($action eq 'process') {
	$out .= $this->process; 
	$out .= $this->list_email;
    } 
    elsif ($action eq 'list') {
	$out .= $this->list_email;
    } 
    return $out;
}

sub list_email {
    my $this = shift;
    my $db = $share{DB};
    my $out;
    foreach my $site ($db->my_real_sites) {
	next if (! $db->license("Email",$site->{section_id}));
	$out .= "<p>\n";
	$out .= $db->report({title=>$site->{title},
			     table=>'email',
			     section_id=>$site->{section_id},
			     link=>0,
			     edit=>&relink(emcmd=>"edit",tab=>undef,id=>undef),
			     delete=>&relink(emcmd=>"delete",tab=>undef,id=>undef),
			     new=>&relink(emcmd=>"add",tab=>undef,id=>undef),
			     copy=>0,
			     sort=>['sent_date']
			     });
	$out .= "</p>\n";
    }
    return $out;
}

sub add {
    my ($this,%data) = @_;
    my $db = $share{DB};
    # new message form
    my $url = &relink(emcmd=>"do_add");

    # prebuild form with HTMLeditor
    $db->{action} = "insert";
    $config{form}{style} = "table";
    my $email_form = $db->input_record({table=>"email",
					data=>{
					    reply_address=>$db->my_user_record()->{email},
					    section_id=>$db->this_site()->{section_id},
					    status=>"active",
					},
					record=>"_1"});

    my $out = "<h3>New Email</h3>\n";
    $out .= $db->editor_javascript;
    $out .= "<form METHOD=POST ENCTYPE=\"multipart/form-data\" ACTION=\"$url\" onSubmit=\"validate()\">\n<table CLASS=\"form\">\n";
    if ($db->{htmleditor}) {
	$out .= "<input type=\"hidden\" name=\"$db->{htmleditor}\">\n";
    }
    $out .= $email_form;
    $out .= $db->input_html(name=>"reply",type=>"hidden",value=>$this->link(emcmd=>"list"));
    $out .= "</table>\n<h4>Attachments:</h4>
<table>
<tr><th>File</th><th>Description</th></tr>
<tr><td><input type=file name=attachment#_2#file#filename></td>
<td><input type=text size=40 name=attachment#_2#name></td></tr>
<tr><td><input type=file name=attachment#_3#file#filename></td>
<td><input type=text size=40 name=attachment#_3#name></td></tr>
<tr><td><input type=file name=attachment#_4#file#filename></td>
<td><input type=text size=40 name=attachment#_4#name></td></tr>
</table>
<input type=HIDDEN name=attachment#_2#email_id value=\"_1\">
<input type=HIDDEN name=attachment#_3#email_id value=\"_1\">
<input type=HIDDEN name=attachment#_4#email_id value=\"_1\">
<br>
<input TYPE=HIDDEN NAME=action VALUE=\"insert\">
<input TYPE=submit NAME=submit_action VALUE=\"Submit\">&nbsp;&nbsp;
<input TYPE=submit NAME=submit_action VALUE=\"Cancel\">&nbsp;&nbsp;
<input TYPE=reset>&nbsp;&nbsp;
</form>\n";
    return $out;
}

sub edit {
    my ($this,$id) = @_;
    my $db = $share{DB};
    my $out = "<h1>Edit Email</h1>
<p>You may edit the email contents in the form below.  You may add, modify,
or delete attachments, using the links at bottom.</p>\n";
    # new message form
    my $url = &relink(emcmd=>undef,tab=>undef,id=>undef);
    $out .= $db->make(table=>"email",
		      record=>$id,
		      extra=>{reply=>$url});
    $out .= "<h3>Attachments:</h3>\n";
    my $newurl = "form.cgi?ptab=email&pid=$id";
    $out .= $db->report({table=>"attachment",data=>{email_id=>$id},
			 edit=>1,new=>$newurl,delete=>1});
    # set the db_ops urls in previous calls
    return $out;
}

sub del {
    my ($this,$id) = @_;
    my $db = $share{DB};
    my $e = $db->fetch("email",$id);
    if ($db->owns($e->{section_id})) {
	$db->trash_r("email",$id);
	my $p = $share{Page} || $share{PageBuf};
	my $url = &relink(emcmd=>undef,tab=>undef,id=>undef);
	if ($p) {
	    $p->redirect($url);
	    return "";
	}
	else {
	    return "<h1>Deletion Complete</h1>\n<p><a href=$url>Back to control panel</a></p>\n";
	}
    }
    else {
	return "<h1>Problem</h1>\n<p class=error>Permission denied: invalid website.</p>\n";
    }
}
		
sub do_add {
    my $this = shift;
    my $db = $share{DB};
    $db->get_data;
    if (exists $db->{form}{attachment}) {
	# remove empty attachments from form
	foreach my $att_r ("_2", "_3", "_4") {
	    if (! $db->{form}{attachment}{$att_r}{file}) {
		$db->delete_record("attachment",$att_r);
	    }
	}
    }
    my @keys = keys %{$db->{form}{email}};
    my $email_id = $keys[0];
    if (! $db->{form}{email}{$email_id}{status}) {
	$db->{form}{email}{$email_id}{status} = "active";
    }
    my $out = $db->do;
    return $out;
}

1;
