package Modules::Feedback;

# Feedback : send feedback to site admin(s)
#
# Usage:
# <!--&Feedback(foo@bar.com,foo2@bar.com)-->
# <!--&Feedback(to=foo@bar.com&subject=hello&message=default+message+body)-->

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Mail;
use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$options) = @_;
    if ($options =~ /[=\&]/) {
	my %opt = &DecodeString($options);
	$this->{options} = \%opt;
    }
    else {
	$this->{options} = { to=>$options };
    }
    my $in = $this->{input};
    $this->{site} = $share{DB}->this_site;
    if ($in->{from} || $in->{subject} || $in->{message}) {
	# we have some form input
	return $this->send;
    }
    else {
	return $this->mailform($options);
    }
    return;
}

sub mailform {
    my ($this, $options) = @_;
    my $url = $this->link;
    my $db = $share{DB};
    my %data = %{$this->{options}};
    if ($db) {
	# see if we can find an appropriate from address
	if ($share{Page} && ! $share{Page}->is_publishing) {
	    $db->authorize;
	    my %user = $db->my_user_record();
	    $data{from} = $user{email};
	}
	if (!$data{subject}) {
	    # set default subject to the website name
	    $data{subject} = "Re: ".$this->{site}{title};
	    $data{subject} =~ s/\"/\&quot\;/;
	}
    }
    return "
<div class=\"Feedback\">
<form action=\"$url\" method=\"POST\">
<table>
<tr><td>Your&nbsp;email&nbsp;address:</td><td><input type=\"text\" name=\"from\" size=\"40\" value=\"$data{from}\"></td></tr>
<tr><td>Subject:</td><td><input type=\"text\" size=\"40\" name=\"subject\" value=\"$data{subject}\"></td></tr>
<tr><td>Message:</td><td><textarea name=\"message\" rows=\"6\" cols=\"50\">$data{message}</textarea></td></tr>
<tr><td></td><td><input type=\"submit\" value=\"Send\"></td></tr>
</table>
</form>
</div>\n";
}

sub send {
    my $this = shift;
    my $out;
    my $in = $this->{input};
    my $addr = $share{DB}->owner_email();
    my @err;
    foreach (split /,/,$addr) {
	if (!/^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	    push @err, "$_: invalid email address";
	}
    }
    if ($in->{from} !~ /^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	push @err, "invalid from address";
    }
    if (! $in->{message}) {
	push @err, "no message to send";
    }
    if (! $in->{subject}) { 
	push @err, "no subject";
    }
    if (@err > 0) {
	$out = "<p class=\"error\">".join("<br>\n",@err)."</p>\n";
    }
    else {
	my $subject = "$this->{site}{title}: $in->{subject}";
	my $message = "Feedback posted from $this->{site}{title}:\n\n".$in->{message};
	if (require ExSite::Mail) {
	    my $stat &SendMail($addr,$in->{from},$subject,$message);
	    if ($stat) {
		$out .= "<p class=\"error\">Problem sending message: error code $stat.</p>\n";
	    }
	    else {
		$out .= "<h3>Message sent successfully - thank you!</h3>\n";
	    }
	}
    }
    if ($this->{options}{reply}) {
	$out .= "<p><a href=\"$this->{options}{reply}\">Back</a></p>\n";
    }
#    else {
#	$out .= $this->mailform();
#    }
    return $out;
}

1;

