#!/usr/bin/perl

package Modules::News;

#==========================================================================
# News : a ZineMachine implementation of a news system
#
# News implements an e-zine for presenting news articles, with indexing,
# automatic archiving, and optional reader commentary.
#
# In fact, it's a general-purpose e-zine manager;  News is simply the most
# common usage.
#==========================================================================

=pod

TODO:

- search
- make news
- alternate index, with full view of first article
-? flag to determine which articles get priority (full) view
- topstories (like AHN)
- automatic archiving?

=cut


use strict;
use ExSite::Config;
use ExSite::Report;
use ExSite::Misc;
use ExSite::Util;
use Modules::Zine;

use vars qw(@ISA);
@ISA = qw(Modules::Zine);

sub write {
    my ($this,$options) = @_;
    my (%opt,$in,$id,$cmd);
    %opt = &DecodeString($options);
    $in = $this->{input};
    $cmd = $in->{zine} || $opt{zine};
    $id = $in->{aid} || $opt{aid};
    if ($id) { $this->fetch($id); }
    if ($cmd eq "submit") {
	my @err = $this->doform();
	if (@err) {
	    return "<h3>Problem:</h3>\n<p>The form could not be processed because:<br>".join("<br>\n",@err)."</p><p>[<a href=\"javascript:history.back()\">Back</a>]</p>\n";
	}
    }
    elsif ($cmd && $cmd ne "show") {
	return &Modules::Zine::write;
    }
    return $this->show_news;
}

sub ioctl {
    my ($this) = shift;
    $_ = shift;

    if (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/Publish/) {
	return \&Modules::Zine::publish;
    }
    elsif (/isRestricted/) {
	return 1;
    }
    elsif (/isService/) {
	return 1;
    }
}

# return array of e-zines belonging to current site

sub my_news {
    my $this = shift;
    if ($this->{my_news}) { return @{$this->{my_news}}; }
    my ($db,@site,@z);
    $db = $share{DB};
    @z = $db->fetch_match("article",{
	section_id=>0,   # public (shareable)
	reference=>0,    # top-level zine
	type=>"e-zine",  # only e-zine types
    });
    if ($this->{admin}) {
	@site = $db->my_real_sites;
	foreach my $site (@site) {
	    push @z, $db->fetch_match("article",{
		section_id=>$site->{section_id},   # belongs to this site
		reference=>0,                      # top-level zine
		type=>"e-zine",                    # only e-zine types
	    });
	}
    }
    elsif ($share{Page}) {
	my $site = $share{Page}->get("section");
	push @z, $db->fetch_match("article",{
	    section_id=>$site->{section_id},   # belongs to this site
	    reference=>0,                      # top-level zine
	    type=>"e-zine",                    # only e-zine types
	});
    }
    $this->{my_news} = \@z;
    return @z;
}

# do we own the requested article?

sub owner {
    my $this = shift;
    # $a is the article whose ownership needs to be tested
    my $a = $this->{reference} ? 
	$this->{reference}{article_id} : $this->{article_id};
    foreach my $news ($this->my_news) {
	return 1 if ($a == $news->{article_id});
    }
    return 0;
}

sub ctrl_panel {
    my $this = shift;
    my $out;
    # insert some style rules for better formatting in control panels
    $out .= "<link REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$config{server}{HTMLpath}/_Modules/Zine/zine.css\">\n";
    my $in = $this->{input};
    $this->{admin} = 1;  # set admin flag to add admin tools
    if ($in->{zine}) { 
	$out .= $this->write; 
	return $out;
    }
    $_ = $in->{zadm};

    if ($this->my_news == 0) {
	$out .= "<p>This appears to be the first time you've run News.
Please take a minute to perform some basic configuration.</p>
<span class=\"help\">
<p>These configuration parameters help define how the News index
is formatted.</p>
<ul>
<li> In principle, you can leave all fields blank, although 
a title is recommended.
<li> The subtitle, if present, is displayed just below the title at a 
slightly smaller size.  Example: \"Events and Announcements for Members\"
<li> The summary text is displayed like an abstract, below the subtitle.  
Example: \"Please confirm all dates and times with our office.\"
<li>The footnotes are displayed underneath all of the above.
Example: \"&copy; 2005 Acme Inc.\"
</ul>

</span>
<h1>Configure Your News System</h1>\n<dl>\n";
	$out .= $this->makeform({type=>"e-zine",
				 title=>"News",
			     });
    }
    else {
	$out .= $this->show_news;
    }

    return $out;
}

sub show_news {
    my $this = shift;
    my $out;
    
    if (! $this->{article_id}) {

	# no article specified

	my @news = $this->my_news;
	if (@news == 1) {
	    # one news zine - show it
	    $this->set($news[0]);
	}
	else {
	    # multiple zines;  let user select one
	    foreach my $news (@news) {
		$this->set($news);
		$out .= $this->summarize;
	    }
	    return $out || "<p class=\"error\">No news found.</p>\n";
	}
    }
    elsif (! $this->owner) {
	# validate the article
	# (prevents snooping in unrelated Zines)
	return "<p class=\"error\">News article mismatch - permission denied.</p>\n";
    }
    if (! $this->{article}) {
	return "<p class=\"error\">No news found.</p>\n";
    }

    $out .= $this->show;

    return $out;
}

1;

