package Modules::PhotoAlbum;

=pod

=head1 Modules::PhotoAlbum - photo and image management

The PhotoAlbum toil provides a simple interface to managing libraries of
images and photos.

=over 4

=item * simple indexing of available libraries/albums and their contents

=item * previewing of any album image

=item * uploading of new images, singly or in groups

=item * managing image captions

=item * thumbnailing new images (square or original dimensions)

=item * re-thumbnailing existing images (eg. to change aspect)

=item * publishing

=item + shortcuts to website manager

=back

=cut

###  TODO:
###  
### handle CANCEL button on all forms

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Util;
use ExSite::HTML;
use ExSite::Image;
use ExSite::Content;
use ExSite::Page;
use Modules::BaseDCD;
use MIME::Base64;

use vars qw(@ISA %dim);
@ISA = qw(Modules::BaseDCD);

%dim = (
	small => 100,
	medium => 320,
	large => 640,
	x_large => 1024,
	custom => 500,
	);

sub write {
    my ($this,$options) = @_;
    return "<!--&Gallery($options)-->";
}

# ioctl : all operations that don't qualify as reads or writes
#         invoked as <!--BaseDCD.ioctl(request)-->

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "Photo Albums";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/ModuleInfo/) {
	return "The Photo Album gives you a set of tools for managing 
collections of images.";
    }
    elsif (/DynContentParam/){
    	return $this->getDynContentParameter(shift);
    }
}

sub getDynContentParameter{
	my ($this,$sectionid) = @_;
	return undef if !$sectionid;
	require ExSite::Section;
	my $s = new ExSite::Section(id=>$sectionid);
	my @libs = $s->fetch_libraries();
	my $out = "<p><b>Select a library:</b>
<select name=\"param\">
<option value=\"\">&lt;Default&gt;</option>\n";
	foreach my $lib(@libs){
		$out .= "<option value=\"lib=$lib->{filename}\">$lib->{title}</option>";
	}
	$out .= "</select></p>\n";
	return $out;
}

sub ctrl_panel {
    my $this = shift;
    my ($css,$selector,$menu,$workarea,$action);
    $this->{site} = $this->{input}{section_id};
    $this->{album} = $this->{input}{album};

    if ($config{PhotoAlbum}{small}) {
	# load default sizes from config
	%dim = %{$config{PhotoAlbum}};
    }

    $css = $this->css;
    $selector = $this->site_selector();
    $menu = $this->album_menu();
    $action = $this->{input}{cmd};
    if ($action) {
	if ($action =~ /^(newalbum|update|caption|thumb|delete|newimg|bulkimg|bulkthumb)$/) {
	    $workarea = $this->$action;
	}
    }
    if (! $workarea) {
	if ($this->{input}{album}) {
	    $workarea = $this->album_view;
	}
	else  {
	    $workarea = "<p>Select an album from those at left.</p>\n";
	}
    }

    my $out = "<script language='javascript' src='$config{server}{HTMLpath}/_ExSite/js/misc.js'></script>
$css
<div class=\"SiteSelector\">\n$selector\n</div>\n";
    if ($menu) {
	$out .= "<table class=\"PhotoAlbumCtrlPanel\"><tr><td valign=\"top\" class=\"AlbumMenu\">
$menu
</td><td valign=\"top\" class=\"WorkArea\">
$workarea
</td></tr>
</table>\n";
    }
    return $out;
}

sub site_selector {
    my $this = shift;
    my $out;
    if (! $share{Page}) {
	# only output a site selector on control panels
	my @sites = $share{DB}->my_real_sites;
	$out .= "<p><form>Select website: <select name=\"section_id\" onchange=\"document.forms[0].submit()\">\n";
	foreach my $s (@sites) {
	    $out .= "<option value=\"$s->{section_id}\"";
	    if ($this->{site} == $s->{section_id}) { 
		$out .= " selected";
	    }
	    $out .= ">$s->{title}</option>\n";
	}
	$out .= "</select><input type=\"submit\" value=\"Submit\"></form></p>\n";
    }
    return $out;
}

sub album_menu {
    my $this = shift;
    return undef if (! $this->{site});
    my $out;
    my @album = $share{DB}->fetch_match("page",
					{section_id=>$this->{site},
					 type=>"library",
					 status=>"active"},
					"title");
    if (@album > 0) {
	foreach my $a (@album) {
	    my $url = $this->link(album=>$a->{page_id},cmd=>undef);
	    $out .= "<a class=\"PhotoAlbumLink\" href=\"$url\"";
	    if ($a->{page_id} == $this->{album}) {
		$out .= " id=\"thistab\"";
	    }
	    $out .= "><i>$a->{title}</i> ($a->{filename})</a>\n";
	}
    }
    else {
	$out .= "<p>No albums found.</p>\n";
    }

    # new album link
    my $url = $this->link(cmd=>"newalbum");
    $out .= "<p><a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/PhotoAlbum/add16.gif\" height=\"16\" width=\"16\" border=\"0\" class=\"ToolIcon\"> New Album</a></p>\n";
    return $out;
}

sub album_view {
    my $this = shift;
    return undef if (! $this->{album});
    my $album = new ExSite::Page(id=>$this->{album});
    my $title = $album->title;
    my $description = $album->description;
    my @img = $album->fetch_all_content_objs(version=>"newest");
    my $imgpath = "$config{server}{HTMLpath}/_Modules/PhotoAlbum";
    my $iconattr = "height=\"16\" width=\"16\" border=\"0\" class=\"ToolIcon\"";
    my $out = "<h3>$title</h3>\n<p>$description</p>\n<table class=\"PhotoAlbumView\">\n";
    my $n_non_img = 0;
    foreach my $img (@img) {
	$img->set_context("child");
	my $type = $img->content_type;
	if ($type !~ /^image/) {
	    $n_non_img++;
	    next;
	}
	my $update_url = $this->link(cmd=>"update",img=>$img->id);
	my $thumb_url = $this->link(cmd=>"thumb",img=>$img->id);
	my $caption_url = $this->link(cmd=>"caption",img=>$img->id);
	my $delete_url = $this->link(cmd=>"delete",img=>$img->id);
	my $thumb = $this->show_thumb($img);
	if (! $img->get("content_data","thumb")) {
	    $thumb .= "<br><span class=\"error\">No thumbnail.</span>";
	}
	my ($fname,$size,$mime) = $img->get_fileinfo;
	my $description = "<p><b>".
	    $img->get("content","name").
	    "</b><br>\n".
	    ($img->get("content","description") || "<span class=\"error\">no caption</span>").
	    "<br>\n<i>($fname, $size K, $mime)</p>\n";
	$out .= "<tr><td>
<table cellspacing=\"5\"><tr><td>
<a href=\"$update_url\"><img src=\"$imgpath/add16.gif\" $iconattr alt=\"update\" title=\"update\"></a><br>
<a href=\"$caption_url\"><img src=\"$imgpath/caption16.gif\" $iconattr alt=\"caption\" title=\"caption\"></a><br>
<a href=\"$thumb_url\"><img src=\"$imgpath/thumb16.gif\" $iconattr alt=\"thumbnail\" title=\"thumbnail\"></a><br>
<a href=\"$delete_url\"><img src=\"$imgpath/del16.gif\" $iconattr alt=\"delete\" title=\"delete\"></a>
</td><td>
$thumb
</td></tr></table>
</td><td>
$description
</td></tr>\n";
    }
    $out .= "</table>\n";

    # footer notes
    my $n_img = (scalar @img) - $n_non_img;
    $out .= "$n_img images total.<br>\n";
    if ($n_non_img) {
	$out .= "This album also contains $n_non_img content item(s) that are not images. (Use <a href=\"$config{server}{CGIpath}/ctrl-panel.cgi/CMS?path=/$this->{site}/$this->{album}\">Website Manager</a> or <a href=\"$config{server}{CGIpath}/ctrl-panel.cgi/Document?section_id=$this->{site}\">Document Manager</a> to manage these.)<br>\n";
    }

    # footer links
    my $newimg_url = $this->link(cmd=>"newimg");
    my $bulk_url = $this->link(cmd=>"bulkimg");
    my $bulkthumb_url = $this->link(cmd=>"bulkthumb");
    my $publish_url = "$config{server}{CGIpath}/publish.cgi?page=$this->{album}";
    $out .= "<br>
<a href=\"$newimg_url\"><img src=\"$imgpath/add16.gif\" height=\"16\" width=\"16\" border=\"0\" class=\"ToolIcon\" alt=\"new image\" title=\"new image\"> new image</a><br>
<a href=\"$bulk_url\"><img src=\"$imgpath/bulkupdate16.gif\" height=\"16\" width=\"16\" border=\"0\" class=\"ToolIcon\" alt=\"bulk upload\" title=\"bulk upload\"> bulk upload</a><br>
<a href=\"$bulkthumb_url\"><img src=\"$imgpath/thumb16.gif\" height=\"16\" width=\"16\" border=\"0\" class=\"ToolIcon\" alt=\"thumb all\" title=\"thumb all\"> thumbnail all images</a><br>
<a href=\"$publish_url\" target=\"publish\"><img src=\"$imgpath/pub16.gif\" height=\"16\" width=\"16\" border=\"0\" class=\"ToolIcon\" alt=\"publish\" title=\"publish\"> publish album</a><br>\n";

    return $out;
}

sub show_thumb {
    my ($this,$img) = @_;
    if ((ref $img) =~ /Content/) {
	$img->set_context("child");
	my $thumb = $img->get_thumb_html;
	my $url = $img->get_url;
	my $attr = $img->get("content_data","attributes");
	$attr =~ /width=\"?(\d+)\"?/i;
	my $width = $1+20;
	$attr =~ /height=\"?(\d+)\"?/i;
	my $height = $1+20;
	if ($width == 20 || $height == 20) {
	    # bad dimeensions; extract dimensions from the original image
	    my $image = new ExSite::Image($img->get("content_data","fdata"));
	    ($width,$height) = $image->dim;
	    $height += 20;
	    $width += 20;
	}
	return "<a href=\"javascript:popup_custom($width,$height,'$url')\">$thumb</a>";
    }
    return undef;
}

sub newalbum {
    my $this = shift;
    my $in = $this->{input};
    my $out;
    if ($in->{'library##type'}) {
	# looks like we have some input
	my $results = $share{DB}->do;
	if ($share{DB}->status) {
	    # bad results
	    return $results;
	}
	# on a good result we should be redirected back to the album view
	### FIXME: we should redirect to the new album
    }
    else {
	# no input; throw up a form
	my $form_action = $this->link();
	$out = "<h3>New Photo Album</h3>
<p>A Photo Album is just a library that is intended to hold images.  
It needs a unique directory  name to publish into, but the title and 
description can be anything you choose.</p>\n";
	$out .= $share{DB}->make(table=>"library",
				 show=>["title","description","filename"],
				 action=>$form_action,
				 data=>{
				     section_id=>$this->{site},
				     status=>"active",
				     access=>"public",
				 },
				 extra=>{
				     reply=>$this->link(cmd=>undef),
				 });
	return $out;
    }
}

sub update {
    my $this = shift;
    my $in = $this->{input};
    my $out;
    my $img = new ExSite::Content(id=>$in->{img});
    if ($in->{name}) {
	# looks like we have some input
	my $input = new ExSite::Input;
	my $fdata = $input->fetch_file("file");
	if (length $fdata < 20) {
	    # looks like there is no real file data;
	    # Let's assume that we are just reconfiguring the current image.
	    $this->warn("No valid image uploaded - reconfiguring current image instead.");
	    $fdata = $img->get("content_data","fdata");
	}
	if ($in->{name} ne $in->{oldname}) {
	    # update content name
	    my $c = $img->get();
	    $c->{name} = $in->{name};
	    $share{DB}->update("content",$c);
	}
	if ($in->{scale}) {
	    my $image = new ExSite::Image($fdata);
	    my $size;
	    if ($in->{scale} =~ /^(tiny|small|medium|(x_)?large)$/) {
		$size = $dim{$in->{scale}};
	    }
	    elsif ($in->{scale} eq "custom") {
		$size = $in->{custom_scale};
	    }
	    else {
		$size = 0;
	    }
	    if ($size > 0 && $size < $config{max_image_size}) {
		$image->scale($size,$size,-1);
		$fdata = $image->encode;
	    }
	    else {
		$this->warn("Invalid resize parameter - ignored.");
	    }
	}

	my $option = "attribute,".$in->{thumb};
	$img->update({content_id=>$in->{img},
		      fdata=>$fdata,
		  },$option);

	my $url = $this->link(cmd=>undef);
	if ($this->errorcheck("warn") || $this->errorcheck("error")) {
	    $out .= $this->show_diagnostics("warn","html");
	    $out .= $this->show_diagnostics("error","html");
	    $out .= "<p><a href=\"$url\">Back to album view</a></p>\n";
	}
	else {
	    &redirect($url);
	}
    }
    else {
	# no input; throw up a form
	$img->set_context;
	my $name = $img->get_my("name");
	#$out .= "<p>Currently:<br>\n".(ref $img)."</p>\n";
	$out .= "<p>Currently:<br>\n".$this->show_thumb($img)."</p>\n";
	my $form_action = $this->link();
	my $resize_form = $this->resize_form;
	my $thumb_form = $this->thumbnail_form;
	$out .= "<h3>Update Image</h3>
<form method=\"POST\" enctype=\"multipart/form-data\" action=\"$form_action\">
<p>Image Name<br>
<input type=\"text\" name=\"name\" value=\"$name\"></p>
<input type=\"hidden\" name=\"oldname\" value=\"$name\">
<p>Upload Image<br>
<input type=\"file\" name=\"file\"></p>
$resize_form
$thumb_form\n";
	$out .= $this->end_form;
	return $out;
    }
}

sub thumbnail_form {
    my $this = shift;
    my $inputs = "<p><input type=\"radio\" name=\"thumb\" value=\"thumb\" checked>normal - preserves aspect ratio of original image<br>
<input type=\"radio\" name=\"thumb\" value=\"square-thumb\">square - may produce nicer thumbnail grids, but some of the image will be cropped<br>
<input type=\"radio\" name=\"thumb\" value=\"\">do not make a thumbnail - use this if your image is already a small size</p>\n";
    return &ExSite::HTML::FrameBox(
				   title=>"Choose Thumbnail Type",
				   pane=>$inputs,
				   bgcolor=>"#eee",
				   color=>"#999",
				   );
}

sub resize_form {
    my $this = shift;
    my $inputs = "<p>Many digital photos and other computer images are too large for display
inside a web page.<br>
Use these options to resize your image for the Web:</p>
<p>
<input type=\"radio\" name=\"scale\" value=\"small\">small ($dim{small}px) - use for logos<br>
<input type=\"radio\" name=\"scale\" value=\"medium\">medium ($dim{medium}px) - use for banners and images that are embedded into articles and text<br>
<input type=\"radio\" name=\"scale\" value=\"large\">large ($dim{large}px) - use for photo galleries or stand-alone images that are embedded into a web page<br>
<input type=\"radio\" name=\"scale\" value=\"x_large\">x-large ($dim{x_large}px) - use for high resolution photo galleries<br>
<input type=\"radio\" name=\"scale\" value=\"custom\">custom: 
<input type=\"text\" name=\"custom_scale\" size=\"6\" value=\"$dim{custom}\">px<br>
<input type=\"radio\" name=\"scale\" value=\"\" checked>do not resize
</p>
<p>Your image will only be resized if it is larger than the selected size.</p>
\n";
    return &ExSite::HTML::FrameBox(
				   title=>"Resize Image",
				   pane=>$inputs,
				   bgcolor=>"#eee",
				   color=>"#999",
				   );
}

sub end_form {
    my $this = shift;
    my $out = "<INPUT TYPE=submit NAME=submit_action VALUE=\"Submit\">&nbsp;&nbsp;\n<INPUT TYPE=button VALUE=\"Cancel\" onclick=\"javascript:history.back()\">&nbsp;&nbsp;\n<INPUT TYPE=reset>&nbsp;&nbsp;\n";
    $out .= "</form>\n";
    return $out;
}

sub caption {
    my $this = shift;
    my $in = $this->{input};
    my $out;
    my $img = new ExSite::Content(id=>$in->{img});
    if ($in->{caption}) {
	# looks like we have some input
	my $stat = $share{DB}->update("content",{
	    content_id=>$img->id,
	    description=>$in->{caption},
	});
	my $url = $this->link(cmd=>undef,img=>undef);
	if ($stat) {
	    $out .= "<p class=\"error\">$stat</p>\n";
	    $out .= "<p><a href=\"$url\">Back to album view</a></p>\n";
	}
	else {
	    &redirect($url);
	}
    }
    else {
	# no input; throw up a form
	$img->set_context;
	my $caption = $img->get_my("description");
	my $form_action = $this->link();
	my $thumb = $this->show_thumb($img);
	$out .= "<h3>Update Caption</h3>
<p>$thumb</p>
<form method=\"POST\" action=\"$form_action\">
<p>Caption<br>
<textarea rows=\"6\" cols=\"60\" name=\"caption\">
$caption
</textarea>
</p>\n";
	$out .= $this->end_form;
	return $out;
    }
}

sub thumb {
    my $this = shift;
    my $in = $this->{input};
    my $out;
    my $img = new ExSite::Content(id=>$in->{img});
    if ($in->{thumb}) {
	# looks like we have some input
	$img->set_context("child");
	my $fdata = $img->get("content_data","fdata");
	if ($fdata) {
	    my $image = new ExSite::Image($fdata);
	    my $stat = ($in->{thumb} =~ /square-thumb/) ? 
		$image->square_thumb : 
		$image->thumb;
	    if ($stat) {
		my $cdata = {};
		$cdata->{thumb} = "th_".$image->encode;
		my ($x,$y) = $image->dim;
		if ($x && $y) {
		    $cdata->{thumb_attributes} = "width=\"$x\" height=\"$y\" border=\"0\"";
		}
		$cdata->{content_data_id} = $img->id("content_data");
		my $url = $this->link(cmd=>undef,img=>undef);
		my $stat = $share{DB}->update("content_data",$cdata);
		if ($stat) {
		    $out .= "<p class=\"error\">$stat</p>\n";
		    $out .= "<p><a href=\"$url\">Back to album view</a></p>\n";
		}
		else {
		    &redirect($url);
		    return undef;
		}
	    }
	    else {
		$out .= "<p class=\"error\">Error: could not make thumbnail.</p>\n";
	    }
	}
	else {
	    $out .= "<p class=\"error\">Error: no image data found.</p>\n";
	}
	my $url = $this->link(cmd=>undef,img=>undef);
	$out .= "<p><a href=\"$url\">Back to album view</a></p>\n";
	return $out;
    }
    ### FIXME: what if the user selects "no thumbnanil" ???
    else {
	# no input; throw up a form
	$img->set_context;
	my $name = $img->get_my("name");
	my $thumb_description;
	if ($img->get("content_data","thumb")) {
	    $thumb_description = "This is the original thumbnail.";
	}
	else {
	    $thumb_description = "<span class=\"error\">No thumbnail - this is a scaled version of the original image.</span>";
	}
	my $thumb = $this->show_thumb($img);
	my $form_action = $this->link();
	my $thumb_form = $this->thumbnail_form;
	$out .= "<h3>Update Thumbnail ($name)</h3>
<p>$thumb<br>
$thumb_description</p>
<form method=\"POST\" action=\"$form_action\">
$thumb_form\n";
	$out .= $this->end_form;
	return $out;
    }
}

sub delete {
    my $this = shift;
    my $in = $this->{input};
    my $out;
    if ($in->{img}) {
	$share{DB}->trash_r("content",$in->{img});
	my $url = $this->link(cmd=>undef,img=>undef);
	&redirect($url);
    }
    else {
	$out .= $this->error("Nothing to delete!");
    }
    return $out;
}

sub newimg {
    my $this = shift;
    my $in = $this->{input};
    my $out;
    if ($in->{file}) {
	# looks like we have some input
	my $input = new ExSite::Input;
	my $filename = $input->fetch_file("file","name");
	my $mimetype = &MimeType($filename);
	if ($mimetype =~ /^image\/(jpe?g|gif|png)$/) {
	    my $fdata = $input->fetch_file("file");
	    if (length $fdata > 20) {
		if ($in->{scale}) {
		    my $image = new ExSite::Image($fdata);
		    my $size;
		    if ($in->{scale} =~ /^(tiny|small|medium|(x_)?large)$/) {
			$size = $dim{$in->{scale}};
		    }
		    elsif ($in->{scale} eq "custom") {
			$size = $in->{custom_scale};
		    }
		    else {
			$size = 0;
		    }
		    if ($size > 0 && $size < $config{max_image_size}) {
			$image->scale($size,$size,-1);
			$fdata = $image->encode;
		    }
		    else {
			$this->warn("Invalid resize parameter - ignored.");
		    }
		}
		my $option = "attribute,".$in->{thumb};
		my $album = new ExSite::Page(id=>$this->{album});
		my $content = new ExSite::Content();
		$content->make({
		    name=>($in->{name} || &clean_filename($filename)),
		    description=>$in->{caption},
		    type=>"editorial",
		    allow_mime=>"image",
		    page_id=>$album->id(),
		},
		{
		    mime_type=>$mimetype,
		    fdata=>$fdata,
		},
		    $option);

		my $url = $this->link(cmd=>undef);
		if ($this->errorcheck("warn") || $this->errorcheck("error")) {
		    $out .= $this->show_diagnostics("warn","html");
		    $out .= $this->show_diagnostics("error","html");
		    $out .= "<p><a href=\"$url\">Back to album view</a></p>\n";
		}
		else {
		    &redirect($url);
		}
	    }
	    else {
		$out .= $this->error("Error: file data is missing.");
	    }
	}
	else {
	    $out .= $this->error("Invalid image type!  Please upload JPEG, GIF, or PNG format images only. $mimetype $filename");
	}
	my $url = $this->link(cmd=>undef,img=>undef);
	$out .= "<p><a href=\"$url\">Back to album view</a></p>\n";
	return $out;
    }
    else {
	# no input; throw up a form
	my $form_action = $this->link();
	my $resize_form = $this->resize_form;
	my $thumb_form = $this->thumbnail_form;
	$out .= "<h3>New Image</h3>
<form method=\"POST\" enctype=\"multipart/form-data\" action=\"$form_action\">
<p>Upload Image<br>
<input type=\"file\" name=\"file\"></p>
<p>Image Name (optional)<br>
<input type=\"text\" name=\"name\"></p>
<p>Caption<br>
<textarea rows=\"6\" cols=\"60\" name=\"caption\"></textarea></p>
$resize_form
$thumb_form\n";
	$out .= $this->end_form;
	return $out;
    }
}


sub bulkimg {
    my $this = shift;
    my $in = $this->{input};
    my $out;
    if (exists $in->{thumb} || exists $in->{scale}) {
	# looks like we have some input
	for (my $ipic=0; $ipic<10; $ipic++) {
	    my $fileinput = "file$ipic";
	    my $capinput = "caption$ipic";
	    next if (! $in->{$fileinput});
	    my $input = new ExSite::Input;
	    my $filename = &clean_filename($input->fetch_file($fileinput,"name"));
	    my $mimetype = &MimeType($filename);
	    if ($mimetype =~ /^image\/(jpe?g|gif|png)$/) {
		my $fdata = $input->fetch_file($fileinput);
		if (length $fdata > 20) {
		    if ($in->{scale}) {
			my $image = new ExSite::Image($fdata);
			my $size;
			if ($in->{scale} =~ /^(tiny|small|medium|(x_)?large)$/) {
			    $size = $dim{$in->{scale}};
			}
			elsif ($in->{scale} eq "custom") {
			    $size = $in->{custom_scale};
			}
			else {
			    $size = 0;
			}
			if ($size > 0 && $size < $config{max_image_size}) {
			    $image->scale($size,$size,-1);
			    $fdata = $image->encode;
			}
			else {
			    $this->warn("Invalid resize parameter - ignored.");
			}
		    }
		    my $option = "attribute,".$in->{thumb};
		    my $content = new ExSite::Content();
		    my $cid = $content->make(
					     {
						 name=>$filename,
						 description=>$in->{$capinput},
						 type=>"editorial",
						 allow_mime=>"image",
						 page_id=>$this->{album},
					     },
					     {
						 mime_type=>$mimetype,
						 fdata=>$fdata,
					     },
					     $option);
		    if ($cid) {
			$out .= $this->info("$filename imported successfully.");
		    }
		    else {
			$out .= $content->show_diagnostics("warn","html");
			$out .= $content->show_diagnostics("error","html");
		    }
		}
		else {
		    $out .= $this->error("$filename file data is missing.");
		}
	    }
	    else {
		$out .= $this->error("Invalid image type!  Please upload JPEG, GIF, or PNG format images only. $mimetype $filename");
	    }
	}
	my $url = $this->link(cmd=>undef,img=>undef);
	$out .= "<p><a href=\"$url\">Back to album view</a></p>\n";
	return $out;
    }
    else {
	# no input; throw up a form
	my $form_action = $this->link();
	my $resize_form = $this->resize_form;
	my $thumb_form = $this->thumbnail_form;
	$out .= "<h3>Bulk Upload</h3>
<p>This form allows you up upload up to 10 images at a time, and apply
the same scaling and thumbnailing instructions to all of them.</p>
<form method=\"POST\" enctype=\"multipart/form-data\" action=\"$form_action\">
<p>
<table>
<tr><th>File</th><th>Caption</th></tr>
<tr><td><input type=\"file\" name=\"file0\"></td><td><input name=\"caption0\" size=\"40\"></td></tr>
<tr><td><input type=\"file\" name=\"file1\"></td><td><input name=\"caption1\" size=\"40\"></td></tr>
<tr><td><input type=\"file\" name=\"file2\"></td><td><input name=\"caption2\" size=\"40\"></td></tr>
<tr><td><input type=\"file\" name=\"file3\"></td><td><input name=\"caption3\" size=\"40\"></td></tr>
<tr><td><input type=\"file\" name=\"file4\"></td><td><input name=\"caption4\" size=\"40\"></td></tr>
<tr><td><input type=\"file\" name=\"file5\"></td><td><input name=\"caption5\" size=\"40\"></td></tr>
<tr><td><input type=\"file\" name=\"file6\"></td><td><input name=\"caption6\" size=\"40\"></td></tr>
<tr><td><input type=\"file\" name=\"file7\"></td><td><input name=\"caption7\" size=\"40\"></td></tr>
<tr><td><input type=\"file\" name=\"file8\"></td><td><input name=\"caption8\" size=\"40\"></td></tr>
<tr><td><input type=\"file\" name=\"file9\"></td><td><input name=\"caption9\" size=\"40\"></td></tr>
</table>
</p>
<p>Resize and thumbnailing options will be applied to all images you have
uploaded.</p>
$resize_form
$thumb_form\n";
	$out .= $this->end_form;
	return $out;
    }
}

sub bulkthumb {
    my $this = shift;
    my $in = $this->{input};
    my $out;
    if ($in->{thumb}) {
	# looks like we have some input
	my $album = new ExSite::Page(id=>$this->{album});
	my @img = $album->fetch_all_content_objs(version=>"newest");
	foreach my $img (@img) {
	    $img->set_context("child");
	    my $type = $img->content_type;
	    next if ($type !~ /^image/);
	    my $name = $img->get_my("name");
	    my $fdata = $img->get("content_data","fdata");
	    if ($fdata) {
		if ($in->{thumb}) {
		    my $image = new ExSite::Image($fdata);
		    my $stat = ($in->{thumb} =~ /square-thumb/) ? 
			$image->square_thumb : 
			$image->thumb;
		    if ($stat) {
			# non-zero stat means thumbnailing worked
			my $cdata = $img->get("content_data");
			$cdata->{thumb} = "th_".$image->encode;
			my ($x,$y) = $image->dim;
			if ($x && $y) {
			    $cdata->{thumb_attributes} = "width=\"$x\" height=\"$y\" border=\"0\"";
			}
			my $stat = $share{DB}->update("content_data",$cdata);
			if ($stat) {
			    # non-zero stat means update failed
			    $out .= "<p class=\"error\">$name: $stat</p>\n";
			}
			else {
			    $out .= "<p class=\"help\">$name: thumbnailed successfully.</p>\n";
			}
		    }
		    else {
			$out .= "<p class=\"error\">$name: skipped thumbnail due to image size.</p>\n";
		    }
		}
		else {
		    my $cdata = $img->get("content_data");
		    if ($cdata->{thumb}) {
			$cdata->{thumb} = $cdata->{thumb_attributes} = "";
			my $stat = $share{DB}->update("content_data",$cdata);
			if ($stat) {
			    # non-zero stat means update failed
			    $out .= "<p class=\"error\">$name: $stat</p>\n";
			}
			else {
			    $out .= "<p class=\"help\">$name: removed thumbnail.</p>\n";
			}
		    }
		}		    
	    }
	    else {
		$out .= "<p class=\"error\">Error: no image data found.</p>\n";
	    }
	}
	my $url = $this->link(cmd=>undef,img=>undef);
	$out .= "<p><a href=\"$url\">Back to album view</a></p>\n";
	return $out;
    }
    else {
	# no input; throw up a form
	my $form_action = $this->link();
	my $thumb_form = $this->thumbnail_form;
	$out .= "<h3>Update All Thumbnails</h3>
<p>This function will regenerate all thumbnails in this album.</p>
<form method=\"POST\" action=\"$form_action\">
$thumb_form\n";
	$out .= $this->end_form;
	return $out;
    }
}

sub css {
    my $this = shift;
    return "
<style>
<!--
table.PhotoAlbumCtrlPanel {
    border:1px solid #666;
    background-color:#ccc;
    border-collapse:collapse;
}
table.PhotoAlbumView td {
    padding:5px;
    background-color:#fff;
}
td.AlbumMenu {
    border:1px solid #666;
    padding:0px;
    width:150px;
    overflow:hidden;
}
td.WorkArea {
    border:1px solid #666;
    padding:20px;
    background-color:#eee;
}
a.PhotoAlbumLink {
    display:block;
    width:100%;
    padding:8px;
    padding-right:20px;
    text-decoration:none;
}
a.PhotoAlbumLink:hover {
    background-color:#999999;
}
a#thistab {
    background-color:#666666;
    color:#cccccc;
}
td.AlbumMenu p {
    padding-left:8px;
}
img.ToolIcon {
    padding-top:1px;
}
th {
    background-color:#999;
}

-->
</style>
";
}

1;

