package Modules::Questionnaire::Answer;

use strict;
use ExSite::Config;
use ExSite::Misc;

use base	qw(ExSite::Base);

sub new($) {
	my $type = shift @_;
	my $id = shift @_;

	my $this = {};
	my $class = ref($type) || $type || "Modules::Questionnaire::Answer";
	bless $this, $class;

	if(ref($id) =~ /HASH/) {
		$this->set('q_answer', $id);
		$this->set('q_response', new Modules::Questionnaire::Response($id->{'response_id'}));
	} elsif($id =~ /^\d+$/) {
		$this->fetch($id);
	}
	return $this;
}

# Fetch a q_answer record
sub fetch($) {
	my $this = shift @_;
	my $id = shift @_;

	return undef unless $id;

	my $record = $share{'DB'}->fetch('q_answer', $id);
	if($record) {
		$this->set('q_answer', $record);
		$this->set('q_response', new Modules::Questionnaire::Response($record->{'response_id'}));
	}
	return $record;
}

#### ACCESSORS
sub id {
	my $this = shift @_;

	return undef unless $this->get('q_answer');
	return $this->get('q_answer')->{'answer_id'};
}

sub value($) {
	my $this = shift @_;

	return undef unless $this->get('q_answer');
	my $currentVal = $this->get('currentValue');
	return $currentVal if $currentVal;

	my $value = $this->get('q_answer')->{'value'};
	my @values = split /\s*;\s*/, $value;
=pod
	if($values[1] && !$values[0]) {
		$this->set('currentValue', $values[1]);
		return $values[1];
	} elsif($values[0]) {
		$this->set('currentValue', $values[0]);
		return $values[0];
	}
=cut
	if($values[1] && !$values[0]) {
	    $value = $values[1];
	} 
	elsif($values[0]) {
	    $value = $values[0];
	}
	else {
	    $value = undef;
	}
	if ($value) {
	    $value =~ s/[\s\r\n]*$//;  # trim trailing whitespace
	    $this->set('currentValue', $value);
	}
	return $value;
}

sub rawValue($) {
	my $this = shift @_;

	return undef unless $this->get('q_answer');
	return $this->get('q_answer')->{'value'};
}

sub printValues($) {
	my $this = shift @_;
	return undef unless $this->get('q_answer');
	my @values = split /\s*\;\s*/, $this->get('q_answer')->{'value'};
	my $start = 0;

	while($start < @values) {
		if($values[$start]) {
			last;
		}
		$start++;
	}
	my $vString = $values[$start];
	for(my $i = $start+1; $i < @values; $i++) {
		$vString .= ", $values[$i]" if $values[$i];
	}
	return $vString;
}

sub question_id {
	my $this = shift @_;

	return undef unless $this->get('q_answer');
	return $this->get('q_answer')->{'question_id'};
}

sub response($) {
	my $this = shift @_;

	return undef unless $this->get('q_response');
	return $this->get('q_response');
}

# Sometimes multiple values for the same answer will be stored
# as a string delimited by semicolons.
# This method returns an array of each of these values.
sub allValues($) {
	my $this = shift @_;

	return undef unless $this->get('q_answer');
	my $value = $this->rawValue;

	# Grep exists so that a value of "; value" won't
	# cause the resulting array to hold (undef, value)
	# instead it'll simply hold (value)
	my @values = grep { /./ } split /\s*\;\s*/, $value;
	return @values;
}

# Gives us a count of the number of seperate values
# have been provided for this answer;
sub count($) {
	my $this = shift @_;

	my @values = $this->allValues();
	return scalar(@values);
}

sub make($%) {
	my $this = shift @_;
	my %record = @_;

	return 0 unless $record{'question_id'} && $record{'response_id'};

	my $newId = 0;
	if($record{'answer_id'}) {
		$newId = $share{'DB'}->update('q_answer', \%record);
	} else {
		$newId = $share{'DB'}->insert('q_answer', \%record);
		$record{'answer_id'} = $newId;
	}
	$this->set('q_answer', \%record);
	return $newId;
}

1;

__END__

