package Modules::Questionnaire::QuestionnaireHTML;

# Basic View class.
# This package should only be full of template methods
# that take canned merge hashes

use strict;
use ExSite::Config;

# For use on the question base editing page (preview box)
sub questionBasePreviewTable($$) {
	my %data = @_;

	return <<"DONE";
	<table style='border-collapse: collapse; border-width: 1px; border-style: solid; border-color: black'>
		<tr>
		<th style='border-width: 1px; border-style: solid; border-color: black'>
		Preview
		</th>
		</tr>
		$data{preview}
	<tr>
		<td>
		$data{versions}
	</td>
		</tr>
		</table>
DONE
}

# Questionnaire editing form
sub questionnaireForm($) {
	my %data = @_;
	return <<"DONE";
	<h2>New Questionnaire:</h2>
	<form method='post'>
		<input type='hidden' name='section_id' value='$data{section_id}'>
		<input type='hidden' name='qid' value='$data{qid}'>
		<input type='hidden' name='qcmd' value='$data{qcmd}'>
		<table>
			<tr>
				<td>
					Title:
				</td>
				<td>
					<input type='text' name='title' size='40' value='$data{title}'>
				</td>
			</tr>
			<tr>
				<td>
					Versions:
				</td>
				<td>
					<input type='text' name='versions' size='40' value='$data{versions}'>
				</td>
			</tr>
			<tr>
				<td colspan='2'>
					$data{buttons}
				</td>
			</tr>
		</table>
	</form>
DONE
}

# Questionnaire view -> overview section
sub questionnaireOverview($) {
	my %data = @_;
	return <<"DONE";
	$data{icon_key}
	$data{title}
	$data{linked_questions}
	<br>
		$data{final_message}
	<br>
		$data{unused_questions}
DONE
}

# Tabs at the top of the questionnaire view
sub questionnaireViewTabs($$) {
	my %data = @_;

	return <<"DONE";
	<table class='questionnaiewView'>
		<tr>
			<td class='tag'>$data{overviewLink}</td>
			<td class='tag'>$data{previewLink}</td>
			<td class='tag'>$data{reportLink}</td>
			<td class='tag'>$data{eviteLink}</td>
			<td style='text-align: right; width: 100%'>&nbsp;$data{deleteIcon} - Delete questionnaire, sub questions and answers</td>
		</tr>
		<tr>
			<td class='tagBody' colspan='5'>	
DONE
}

# Base question editing page
sub editQuestionBase() {
	my %data = @_;
	return <<"DONE";
<h2>$data{title}</h2>
$data{js}
<form method='post'>
	$data{hiddens}
	<table>
		<tr>
			<td>
				Label:
			</td>
			<td>
				$data{label_input}
			</td.
		</tr>
		<tr>
			<td>
				Version:
			</td>
			<td>
				$data{version_input}
			</td>
		</tr>
		<tr>
			<td>
				Format:
			</td>
			<td>
				$data{format_input}
			</td>
		</tr>
		<tr>
			<td>
				Answers:	
			</td>
			<td>
				$data{value_input}
			</td>
		</tr>
		<tr>
			<td colspan='2'>
				$data{buttons}
			</td>
		</tr>
	</table>
	$data{link_hiddens}
</form>
DONE
}

# Base Question version editing page
sub editQuestionVersion() {
	my %data = @_;

	return <<"DONE";
	<p>The master version of the question and its answers is shown (if applicable). Please enter the corresponding text for the <i>$data{version}</i> version.
	<form method='post'>
	$data{master_label}
	<br>
	$data{version_label}

	$data{version_input}
	$data{version_values}
	$data{hiddens}
	<br>
	<br>
	$data{buttons}
	</form>
DONE
}

# Generic error page with a back button
sub errorScreen() {
	my %data = @_;

	return <<"DONE";
$data{error_msg}
<br>
<a href='#' onClick='history.back()'>Back</a>
DONE
}

# Form for editing the final screen page
sub finalScreenForm($) {
	my %data = @_;

	return <<"DONE";
	$data{js}
	<form method='post'>
	$data{hiddens}
	<table>
		<tr>
			<td>
				Final Screen Header:
			</td>
			<td>
					$data{label_input}
			</td>
		</tr>
		<tr>
			<td>
				Final Screen Version:
			</td>
			<td>
					$data{version_input}
			</td>
		</tr>
		<tr>
			<td>
				Final Screen Text
			</td>
			<td>
					$data{value_input}
			</td>
		</tr>
		<tr>
			<td colspan='2'>
				$data{buttons}
			</td>
		</tr>
	</table>
	</form>

DONE
}

# Form used by the popup windows on the report page
sub reportForm() {
	my %data = @_;

	return <<"DONE";
<form method='post' name='reportForm'>
	<input type='hidden' name='reply' value='$data{reply}'>
	$data{qcmd}
	$data{question_hiddens}
	$data{report_name_input}
	$data{report_questionnaire_input}
	$data{submit_input}
</form>
DONE
}

sub reportEditForm() {
	my %data = @_;

	return <<"DONE";
	<script language='javascript'>
<!-- 
		function submitReport() {
			var form = document.reportForm
			if(!form.name.value) {
				alert('Please provide a name for the saved report.');
				return null;
			}
			if(form.report_id && form.report_id.value) {
				var input = document.createElement('input');
				input.setAttribute('type', 'hidden');
				input.setAttribute('name', 'report_id');
				input.setAttribute('value', form.report_id.value);
				opener.document.reportForm.appendChild(input);
			}

			opener.document.reportForm.name.value = form.name.value;
			opener.document.reportForm.reply.value = '$data{reply}';
			opener.document.reportForm.qcmd.value = '$data{qcmd}';
			opener.document.reportForm.submit();
			window.close();
		}
-->
	</script>
	<form name='reportForm' onSubmit='submitReport()'>
		$data{hiddens}
		<input type='hidden' name='qcmd' value='edit_report'>
		<table>
			<tr>
				<td>
					Report Name:
				</td>
				<td>
					<input type='text' name='name' value='$data{name}'>
				</td>
			</tr>
			<tr>
				<td colspan='2'>
					<input type='button' value='Submit' onClick='submitReport(this);'>&nbsp;
					<input type='button' value='Cancel' onClick='window.close()'>
				</td>
			</tr>
		</table>
		$data{question_filter_report}
	</form>
DONE
}

sub reportView() {
	my %data = @_;

	return <<"DONE";
	$data{js_include}
	$data{icon_key}
	<b>Reports</b>
	<br>
	$data{summary_link}
	<br>
	<table>
		$data{saved_reports}
		$data{response_filters}
	</table>
	$data{new_filter_link}
	<br>
	<br>
	<table>
		<tr>
			<td>
				$data{current_report}
			</td>
			<td>
				$data{reset_link}
			</td>
		</tr>
	</table>
	$data{questions_report}

	$data{save_report_link}
	<br>
	$data{export_report_link}
	<br>
	<br>
	<b>Add to Report:</b>
	<br>
	$data{available_questions}
	$data{hidden_form}
DONE
}

sub responseSummaryView() {
	my %data = @_;

	return <<"DONE";
$data{css}
$data{return_link}
$data{report}
DONE
}

sub responseFilterForm() {
	my %data = @_;

	return <<"DONE";
		<script language='javascript'>
<!-- 
		$data{js_question_list}

		var filters = new Array();
		$data{filters_init}

		function submitReport() {
			var form = document.filterForm
			if(!form.name.value) {
				alert('Please provide a name for the new filter.');
				return null;
			}

			var count = 0;
			for (var i in filters) {
				// It's possible that we removed a filter and set the
				// filter array record to 'null'. We want to skip the
				// input-creation process for this item if that's
				// the case
				if(filters[i]) {
					var filterId = filters[i][3] || -1;
					var input = document.createElement('input');
					input.setAttribute('type', 'hidden');
					input.setAttribute('name', 'response_match_'+count);
					input.setAttribute('value', filterId+':'+filters[i][0]+':'+filters[i][2]);
					opener.document.reportForm.appendChild(input);
					count++;
				}
			}
			if(document.filterForm.response_filter_id) {
				var input = document.createElement('input');
				input.setAttribute('type', 'hidden');
				input.setAttribute('name', 'response_filter_id');
				input.setAttribute('value', document.filterForm.response_filter_id.value);
				opener.document.reportForm.appendChild(input);
			}

			opener.document.reportForm.name.value = form.name.value;
			opener.document.reportForm.reply.value = '$data{reply}';
			opener.document.reportForm.qcmd.value = '$data{qcmd}';
			opener.document.reportForm.submit();
			window.close();
		}

		function addFilter(question_id, filter) {
			if(!question_id) {
				alert('Error: you must indicate which question is to be filtered.');
				return null;
			}

			if(!filter) {
				alert('Error: you must supply a regular expression match for the filter.');
				return null;
			}
			var newPos = filters.length;
			var filterArray = new Array(question_id, questionList[question_id], filter);
			filters[newPos] = filterArray;
			var addition = addFilterToList(filterArray, newPos);
			rmFilterFromSelect(question_id);
		}

		function addFilterToList(filter, pos) {
			if(!filter) {
				return null;
			}
			var listDiv = document.getElementById("filterList");
			var newDiv = document.createElement('div');
			newDiv.setAttribute('id', pos);
			if(!listDiv.innerHTML) {
				listDiv.innerHTML = "<table style=\'width: 400px;\'>\\n<tr><th style=\'width: 300px\'>Question</th><th style=\'width: 100px;\'>Filter</th></tr></table>\\n";
			} 
			//var deleteLink = "<a href=\'#\' onClick='removeFilter("+listDiv.childNodes.length+", "+filter[0]+")'><img src=\'$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>\\n";
			var deleteLink = createDeleteLink(filter[0], pos);
			newDiv.innerHTML = "<table style=\'width:400px;\'><tr><td style=\'width: 300px;\'>"+deleteLink+filter[1]+"</td><td style=\'width: 100px\'>"+filter[2]+"</td></tr></table>";
			var appendRes = listDiv.appendChild(newDiv);
			return appendRes;
		}

		function createDeleteLink(question_id, position) {
			var deleteLink = "<a href=\'#\' onClick='removeFilter("+position+", "+question_id+")'><img src=\'$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>\\n";
			return deleteLink;
		}


		function rmFilterFromSelect(question_id) {
			if(!question_id) {
				return -1;
			}

			var select = document.getElementById('questionList');

			if(!select) {
				return -1;
			}

			for(var i = 0; i < select.options.length; i++) {
				if(select.options[i].value == question_id) {
					select.options[i] = null;
					return i;
				}
			}
			return -1;
		}

		// remove the filter whose divList position is 'position' and whose
		// question_id is 'question_id'
		function removeFilter(position, question_id) {
			var divList = document.getElementById('filterList');
			var selectList = document.getElementById('questionList');
			divList.childNodes[position].innerHTML = '';
			divList.childNodes[position] = null;
			filters[question_id] = null;
			selectList.options[selectList.options.length] = new Option(questionList[question_id], question_id);
			return null;
		}
-->
	</script>
	<form method='post' name='filterForm'> 
		$data{id_input}
		<table>
			<tr>
				<td>Filter Name</td>
				<td>$data{name_input}</td>
			</tr>
			<tr>
				<td colspan='2'>
					<input type='button' value='Submit' onClick='submitReport(this);'>&nbsp;
					<input type='button' value='Cancel' onClick='window.close()'>
				</td>
			</tr>
		</table>
		<table>
			<tr>
				<td colspan='2'>
					&nbsp;
				</td>
			</tr>
			<tr>
				<td>
					New Filter Question:
				</td>
				<td>
					$data{question_list}
				</td>
			</tr>
			<tr>
				<td>
					Filter expression:
				</td>
				<td>
					$data{filter_input}
				</td>
			</tr>
			<tr>
				<td>
					$data{add_filter_button}
				</td>
			</tr>
		</table>
		<div id='filterList'>$data{filter_tables}</div>
	</form>
DONE
}

sub responeMatchForm() {
	my %data = @_;

	return <<"DONE";
<form method='post'>
	<input type='hidden' name='qid' value='$data{questionnaire_id}'>
	<input type='hidden' name='response_filter_id' value='$data{response_filter_id}'>
	<input type='hidden' name='qcmd' value='$data{action}'>
	Filter question:
	$data{question_select}
	<br>
	Filter rule (regexp): <input type='text' name='filter' value='$data{filter}'>
	<br>
	<br>
	$data{buttons}
</form>
DONE
}

sub eviteForm() {
	my %data = @_;

	return <<"DONE";
$data{js}
$data{notes}
<form name='eviteForm' method='post'>
<input type='hidden' name='submit_action' value='Submit'>
<input type='hidden' name='qcmd' value='evite'>
<input type='hidden' name='reply' value='$data{reply}'>
$data{from_email_hidden}
<table>
	<tr>
		<td>E-vite sender:</td>
		<td><input type='text' name='evite_from' size='60' value='$data{from_email}'></td>
	</tr>
	<tr>
		<td>E-vite Recipients:</td>
		<td><textarea name='evite_recipients' cols='60' rows='15'>$data{recipients}</textarea></td>
	</tr>
	<tr>
		<td>E-vite Subject:</tD>
		<td><input type='text' name='evite_subject' size='60' value='$data{evite_subject}'></td>
	</tr>
	<tr>
		<td>E-vite Message:</td>
		<td><textarea cols='60' name='evite_message' rows='12'>$data{evite_message}</textarea></td>
	</tr>
	<tr>
		<td colspan='2'>
			<!--<input type='button' value='Submit' onClick='submitForm(this);'>&nbsp;-->
			<input type='button' value='Submit' onClick='$data{submit_function};'>&nbsp;
			<input type='button' value='Cancel' onClick='window.close()'>
		</td>
	</tr>
</table>
</form>
DONE
}

sub previewAddForm() {
	my %data = @_;

	return <<DONE
	<br>
	<h2>Available Questions</h2>
	<form method='post'>
		$data{select}
		$data{buttons}
	</form>
DONE
}

sub baseQuestionForm($) {
	my %data = @_;

	return <<"DONE"
	$data{js}
	$data{top_link}
	<form method='post'>
	<input type='hidden' name='qcmd' value='$data{qcmd}'>
	<input type='hidden' name='q_submit' value='1'>
	<input type='hidden' name='section_id' value='$data{section_id}'>
	<input type='hidden' name='master_id' value='$data{master_id}'>
	<input type='hidden' name='questionnaire_id' value='$data{questionnaire_id}'>
	<input type='hidden' name='qbid' value='$data{qbid}'>
	<table>
		<tr>
			<td>
				Question Label:
			</td>
			<td>
				<input type='text' name='label' value='$data{label}' size='30'>
			</td>
		</tr>
		<tr>
			<td>
				Question Version:
			</td>
			<td>
				<input type='text' name='version' value='$data{version}'>
			</td>
		</tr>
		<tr>
			<td>
				Question Format: 
			</td>
			<td>
				$data{format}
			</td>
		</tr>
		<tr>
			<td>
				Possible answers (seperated by the pipe [|] character):
			</td>
			<td>
				<textarea name='value' rows='15' cols='30'>$data{value}</textarea>
			</td>
		</tr>
		<tr>
			<td colspan='2'>
				$data{buttons}
			</td>
		</tr>
	</table>
	</form>
DONE
}

sub hiddenEviteForm() {
	my %data = @_;
	return "" unless $share{'DB'}->is_manager;
	return <<"DONE";
<form name='eviteForm' method='post'>
<input type='hidden' name='submit_action' value='Submit'>
<input type='hidden' name='reply' value='$data{reply}'>
<input type='hidden' name='qcmd' value='evite'>
<input type='hidden' name='evite_recipients' value=''>
<input type='hidden' name='evite_from' value=''>
<input type='hidden' name='evite_subject' value=''>
<input type='hidden' name='evite_message' value=''>
</form>
DONE
}

sub openQuestionnaire() {
	my %data = @_;
	return <<"DONE";
<link rel="stylesheet" type="text/css" href="$config{server}->{HTMLpath}/_Modules/Questionnaire/Questionnaire.css">
$data{eviteForm}
<script language='javascript' src='$config{server}->{HTMLpath}/_ExSite/js/misc.js'></script>
$data{js}
$data{eviteLink}
<form name='questionnaireForm' $data{target} method='post'>
	<input type='hidden' name='qid' value='$data{qid}'>
	<input type='hidden' name='version' value='$data{version}'>
	<input type='hidden' name='response_id' value='$data{response_id}'>
	<input type='hidden' name='qcmd' value='submit_questionnaire'>
	<input type='hidden' name='q_submit' value='1'>
$data{hiddens}
$data{buttons}
	<table $data{previewStyle}>
DONE
}

# These javascript and css headers needs to be placed before the survey form
sub sliderJavascript() {
	my %data = @_;

	return <<"DONE";
	<script type=\"text/javascript\" src=\"$config{server}->{HTMLpath}/_Modules/Questionnaire/js/range.js\"></script>
	<script type=\"text/javascript\" src=\"$config{server}->{HTMLpath}/_Modules/Questionnaire/js/timer.js\"></script>
	<script type=\"text/javascript\" src=\"$config{server}->{HTMLpath}/_Modules/Questionnaire/js/slider.js\"></script>
	<link type=\"text/css\" rel=\"StyleSheet\" href=\"$config{server}->{HTMLpath}/_Modules/Questionnaire/css/winclassic.css\">
	<script language=\"javascript\">
		function GetSliderValue() {
			var ratingId = 'rating_'+this.input.id;
			// update the slider value on the screen
			document.getElementById(ratingId).innerHTML = this.getValue();
			// update the slider value for the input field;
			this.input.value = this.getValue();
		}
	</script>
DONE
}

sub sectionView() {
	my %data = @_;

	return <<"DONE";
	$data{css}
	$data{js}
	$data{questionnaires}
	$data{new_questionnaire}
	$data{questions}
	$data{new_question}
DONE
}

1;
__END__
