package Modules::Questionnaire::ResponseMatch;

use strict;
use ExSite::Config;
use ExSite::Misc;

use base	qw(ExSite::Base);

sub new($) {
	my $type = shift @_;
	my $id = shift @_;

	my $this = {};
	my $class = ref($type) || $type || "Modules::Questionnaire::ResponseMatch";
	bless $this, $class;
	$this->fetch($id) if $id =~ /^\d+$/;
	$this->set('q_response_match', $id) if ($id =~ /HASH/i);
	return $this;
}

sub fetch($) {
	my $this = shift @_;
	my $id = shift @_;

	my $match = $this->get('q_response_match');
	return $match if $match;

	$match = $share{'DB'}->fetch('q_response_match', $id);
	$this->set('q_response_match', $match) if scalar(keys %{$match});
	return $match;
}

# Fetch the question associated with this filter match
sub fetchQuestion($) {
	my $this = shift @_;

	return undef unless $this->id;

	my $questionId = $this->question_id;
	my $question = $this->get('question');
	return $question if $question && ($question->id == $questionId);

	$question = new Modules::Questionnaire::Question($questionId);

	$this->set('question', $question);
	return $question;
}

# Accessors
sub id($) {
	my $this = shift @_;

	return undef unless $this->get('q_response_match');
	return $this->get('q_response_match')->{'response_match_id'};
}

sub question_id($) {
	my $this = shift @_;

	return undef unless $this->get('q_response_match');
	return $this->get('q_response_match')->{'question_id'};
}

sub filter($) {
	my $this = shift @_;

	return undef unless $this->get('q_response_match');
	return $this->get('q_response_match')->{'filter'};
}

sub response_filter_id($) {
	my $this = shift @_;

	return undef unless $this->get('q_response_match');
	return $this->get('q_response_match')->{'response_filter_id'};
}

# Database manipulation
sub make($) {
	my $this = shift @_;
	my %record = @_;

	return 0 unless $record{'question_id'} && $record{'response_filter_id'};
	my $resId = 0;
	if($record{'response_match_id'}) {
		$resId = $share{'DB'}->update('q_response_match', \%record);
	} else {
		$resId = $share{'DB'}->insert('q_response_match', \%record);
		$record{'response_match_id'} = $resId if $resId;
	}
	$this->set('q_response_match', \%record) if $resId;
	return $resId;
}

sub update($) {
	my $this = shift @_;
	my %record = @_;

	return 0 unless $this->id;
	return 0 unless $record{'question_id'} && $record{'response_filter_id'};

	$record{'response_match_id'} = $this->id;
	my $resId = $share{'DB'}->update('q_response_match', \%record);
	$this->set('q_response_match', \%record) if $resId;
	return $resId;
}

sub delete($) {
	my $this = shift @_;
	
	my $id = $this->id;
	return undef unless $id;
	return $share{'DB'}->delete('q_response_match', {'response_match_id' => $id});
}

# Printable Links
sub newLink($) {
	my $responseFilterId = shift @_;

	return "" unless $responseFilterId;

	return "<a href='".&relink('qcmd' => 'new_response_match', 'response_filter_id' => $responseFilterId)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/new12.gif' border='0'></a>\n";
}

sub editLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	return "<a href='".&relink('qcmd' => 'edit_response_match', 'response_match_id' => $this->id)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/edit12.gif' border='0'></a>\n";
}

sub deleteLink($) {
	my $this = shift @_;

	return "" unless $this->id;

	return "<a href='".&relink('qcmd' => 'delete_response_match', 'response_match_id' => $this->id)."'><img src='$config{server}->{HTMLpath}/_Modules/Questionnaire/rm12.gif' border='0'></a>\n";
}

1;

__END__
