package Modules::RequestQuote;
#---------------------------------------------------------------------
# RequestQuote module is built based on the Survey module.
#
# Usage:
# 	<!--&RequestQuote(parameters)-->
#
# Parameters:
#	product_id=<product_id>
#		- product id
#	product=<product id or code>
#		- product id or product code
#---------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use Modules::Survey;
use vars qw(@ISA);
push(@ISA,"Modules::Survey");

#---------------------------------------------------------------------
sub moduleInit{
	my ($this) = @_;
	$this->{name} = "RequestQuote";
	$this->{form_type} = "requestquote";
	$this->{registrant_list_excel_filename} = $config{$this->{class_name}}{config}{registrant_list}{excel_filename} || "registrantlist.xls";
	return;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return $this->getModuleName."s";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
		return $this->getModuleName." module.";
	}elsif (/isRestricted/){
        	return $config{RequestQuote}{isRestricted};
	}elsif (/isService/){
		return $config{RequestQuote}{isService};
#	}elsif (/DynContentParam/){
#		return $this->getDynContentParameter(shift);
	}
}
sub write{
	my ($this,$options) = @_;
	my (%opt,$action);
	%opt = &DecodeString($options);
	$action = $this->{input}{action} || $opt{action};
	my $product = $this->{input}{product} || $opt{product};
	if ($product){
		return $this->product_link($product);
	# process form data
	}elsif ($action eq "form_submit"){
		my $surveyid = $this->{input}{sid} || $opt{sid};
		return $this->validateForm;
	# public survey list
	}else{
		return $this->surveyForm();
	}
	return undef;
}
#---------------------------------------------------------------------
sub getDynContentParameter{
	my ($this,$sectionid) = @_;
	return if !$sectionid;
	my (@surveys,$out);
	@surveys = $this->{db}->fetch_match("survey",{section_id=>$sectionid,type=>"requestquote"});
	$out .= "\n<table>";
	$out .= "\n<tr><td>";
	$out .= "<b>Select a request a quote form:</b>";
	$out .= "</td><td>";
	$out .= "<select name=\"param\">";
	$out .= "<option value=\"\">&lt;Default&gt;</option>";
	foreach my $survey(@surveys){
		$out .= "<option value=\"sid=$survey->{id}&action=do\">$survey->{name}</option>";
	}
	$out .= "</select>";
	$out .= "</td></tr>";
	$out .= "\n</table>";
	return $out;
}
# control panel
sub control_panel{
	my $this = shift;
	my $out;
	my $admin_level = $config{RequestQuote}{admin_level} || $config{auth}{root_auth_level} - 1;
	my $sectionid = $this->{input}{section_id};
	# admin access only
	#if ($this->{db}->authorize < $config{auth}{root_auth_level} - 1) {
	if ($this->{db}->authorize < $admin_level){
		$out = "<h3 class=\"error\">Permission denied</h3><p>You do not have sufficient system priveleges to use this feature.</p>\n";
	}else {
		my $action = $this->{input}{action};
		if ($action =~ /^(insert|update)$/i){
			$out .= $this->exsiteFormPost();
		}elsif ($action eq "excel_user"){
			$out .= $this->exportRegistrantList($this->{input}{sid});
		# view registrant/participant list
		}elsif ($action eq "view_reg_list"){
			$out .= $this->viewRegistrantList($this->{input}{sid});
		# edit a child recrod
		}elsif ($action eq "edit_form_record"){
			$out .= $this->adminChildRecordForm($this->{input}{tab},$this->{input}{id},$this->{input}{ptab},$this->{input}{pid});
		# add new child records to survey
		}elsif ($action eq "new_form_record"){
			$out .= $this->adminChildRecordForm($this->{input}{tab},undef,$this->{input}{ptab},$this->{input}{pid});
		# edit survey info
		}elsif ($action eq "edit_mainform"){
			my $surveyid = $this->{input}{sid};
			$out .= $this->adminForm(undef,$surveyid);
		# main survey edit form
		}elsif ($action eq "main_form"){
			my $surveyid = $this->{input}{sid};
			$out .= $this->adminFormMain($surveyid);
		# process form data
		}elsif ($action eq "submit_form"){
			$out .= $this->adminFormProcess("survey",$this->{input});
		# create edit a survey
		}elsif ($action eq "new_mainform"){
			my $sectionid = $this->{input}{section_id};
			$out .= $this->adminForm($sectionid);
		# export the registrant list to excel spreadsheet
		}elsif ($action eq "excel_user"){
			my $surveyid = $this->{input}{sid};
			$out .= $this->exportRegistrantList($surveyid);
		# view individual's survey responses
		}elsif ($action eq "view_user"){
			my $surveyid = $this->{input}{sid};
			$out = $this->viewSingleSurveyResponse($surveyid,$this->{input}{uid});		
		# admin survey list
		}elsif ($action eq "admin_list"){
			$out = $this->showAllSurveyList;
		# view reports
		}elsif ($action eq "view_report"){
			my $surveyid = $this->{input}{sid};
			$out = $this->showReportList($surveyid);
		}elsif ($action eq "view_summary"){
			my $surveyid = $this->{input}{sid};
			$out = $this->viewSurveyReport($surveyid);
		# view question summaries
		}elsif ($action eq "view_question"){
			my ($questionid,$surveyid);
			$questionid = $this->{input}{qid};
			$surveyid = $this->{input}{sid};
			$out = $this->viewQuestionReport($questionid,$surveyid);
		# start survey wizard
		}elsif ($action eq "wizard"){
			my ($sectionid);
			$sectionid = $this->{input}{section_id};
			$out = $this->surveyWizard($sectionid);
		# data filter
		}elsif ($action eq "dds"){
			my ($sectionid);
			$sectionid = $this->{input}{section_id};
			$out = $this->surveyDDS($sectionid);
		# create a new data filter
		}elsif ($action eq "dds_action"){
			$out = $this->newDDSReport;
		# delete a data filter
		}elsif ($action eq "rec_delete"){
			$out = $this->deleteRecord($this->{input}{tab},$this->{input}{id},$this->{input}{reply});
		}elsif ($action){
			$out = $this->write;
		}elsif ($sectionid){
			my %section = $this->{db}->fetch("section",$sectionid);
			$out = $this->adminSurveyList(\%section);
		# display admin survey list by default
		}else{
			$out = $this->set_section_id();
		}
	}
	return $out;
}
#------------------------------------------------------------------------------------------------------------------------
# get module name
sub getModuleName{
	my ($this) = @_;
	return "Request a quote form";
}

#------------------------------------------------------------------------------------------------------------------------
sub find_product_id{
	my ($this,$product) = @_;
	my $productid = 0;
	# Check and see if $product is a product code
	my @found = $this->{db}->fetch_match("catalog_product",{code=>$product});
	# If a product is found, we return the product id
	if (@found > 0){
		$productid = $found[0]->{catalog_product_id};
		return $productid;
	}
	# If $product isn't a product code, we check product id for it
	my %product = $this->{db}->fetch("catalog_product",$product);
	# We found the product by checking product id
	$productid = $product{catalog_product_id} if $product{catalog_product_id};
	return $productid;
}
sub product_link{
	my ($this,$product) = @_;
	my $productid = $this->find_product_id($product);
	return if !$productid;
	my %option = (
		product_id=>$productid,
		product=>undef,
	);
	# Add a list of parameters to undef
	foreach my $field("path","next_page","search_field","detail","action"){
		$option{$field} = undef;
	}
	my $link = $this->link(%option);
	my $link_text = $this->link_text();
	return <<END;
<a href="$link">$link_text</a>
END
}

# Link text
sub link_text{
	my ($this) = @_;
	return $config{RequestQuote}{link_text} if $config{RequestQuote}{link_text};
	return "Request a quote";
}
#------------------------------------------------------------------------------------------------------------------------
# Insert a request a quote survey
sub add_survey{
	my ($this,$sectionid) = @_;
	my %data = (
		section_id=>$sectionid,
		type=>"requestquote",
		use_require=>"No",
		use_group=>"No",
		use_number=>"No",
		show_contact_title=>"No",
		name=>"Request a quote",
		save_result=>"Yes",
		header_html=>"&nbsp;",
	);
	my $surveyid = $this->{db}->insert("survey",\%data);
	my @fields = (
		{label=>"Product(s)",type=>"Short answer",mandatory=>"Yes"},
		{label=>"Additional Requirements or Information",type=>"Long answer",mandatory=>"No"},
		{label=>"Name",type=>"Short answer",mandatory=>"No"},
		{label=>"Title",type=>"Short answer",mandatory=>"No"},
		{label=>"Company",type=>"Short answer",mandatory=>"No"},
		{label=>"Address",type=>"Short answer",mandatory=>"No"},
		{label=>"City",type=>"Short answer",mandatory=>"No"},
		{label=>"State/Province",type=>"Short answer",mandatory=>"No"},
		{label=>"Postal Code",type=>"Short answer",mandatory=>"No"},
		{label=>"Country",type=>"Short answer",mandatory=>"No"},
		{label=>"Telephone",type=>"Short answer",mandatory=>"No"},
		{label=>"Email",type=>"Short answer",mandatory=>"Yes"},
	);
	foreach my $field(@fields){
		$field->{survey_id} = $surveyid;
		$this->{db}->insert("survey_contact_question",$field);
	}
	return $surveyid;
}
# Get requrest a quote survey id
sub get_survey_id{
	my ($this) = @_;
	my $sectionid = $this->{input}{section_id} || $share{Page}->id("section");
	my @surveys = @{$this->fetchSurvey($sectionid)};
	if (@surveys < 1){
		return $this->add_survey($sectionid);
	}
	return $surveys[0]->{id};	
}
# Get default product info
sub getProductInfo{
	my ($this) = @_;
	my $productid = $this->{input}{product_id};
	my %product = $this->{db}->fetch("catalog_product",$productid);
	return "$product{title} ($product{code})";
}
# get previously entered question answers from the query string
# (overloaded function)
sub getDefaultQuestionValue{
	# $question: question hash
	my ($this,$question) = @_;
	my $key = $question->{id};
	if (!exists($this->{input}{$key})){
		# Prefill 'Product(s)' field
		if ($question->{label} =~ /^product/i){
			return $this->getProductInfo();
		}
		return "";
	}
	my $val = $this->{input}{$key};
	# remove null inputs for checkboxes
	$val = $this->removeEmptyInput($val);
	return $val;
}
# get previously entered contact info answers from the query string
# (overloaded function)
sub getDefaultContactInfoValue{
	# $contact: contact question hash
	my ($this,$contact) = @_;
	my $productid = $this->{input}{product_id};
	my %product = $this->{db}->fetch("catalog_product",$productid);
	my $key = "cc#$contact->{id}";
	if ($contact->{label} =~ /^product/i && $product{title}){
		return $this->getProductInfo();
	}
	return $this->{input}{$key} if (exists($this->{input}{$key}));
	return "";
}
# fetch surveys(type is requestquote)
# (overloaded function)
sub fetchSurvey{
	# $sectionid : section id
	my ($this,$sectionid) = @_;
	my @list = $this->{db}->fetch_match("survey",{section_id=>$sectionid,type=>"requestquote"});
	return \@list;
}

# build survey form
# (overloaded function)
sub surveyForm{
	my ($this) = @_;	
	my $surveyid = $this->get_survey_id();
	return $this->error("The system is unable to find a request form! Please contact the webmaster for more information.") if !$surveyid;
	# fetch survey
	my $survey = $this->{db}->fetch("survey",$surveyid);
	# run survey init routine
	$this->surveyInit($survey);
	# get form validation errors
	my $error = $this->html_formError;	
	# build contact info form
	my $contactinfo = $this->surveyContactInfo($survey->{id});
	# build survey question form
	my $questions = $this->surveyQuestion($survey->{id});
	return $this->html_surveyForm($survey,$contactinfo,$questions,$error);
}

# admin table row links
# (overloaded function)
sub getAdminSurveyListRowLink{
	# $survey: survey hash
	my ($this,$survey) = @_;
	my (@links,$module);
	$module = $this->getModuleName;
	@links = (
	{id=>"view",label=>"View Reports",url=>$this->myRelink({action=>"view_reg_list",sid=>$survey->{id}}),img=>"view"},
	{id=>"edit",label=>"Reconfigure",url=>$this->myRelink({action=>"main_form",sid=>$survey->{id}}),img=>"edit"},
	);
	return \@links;
}
# build admin survey list
# (overloaded function)
sub adminSurveyList{
	# $section: section hash
	my ($this,$section) = @_;
	# missing section id
	return $this->error("Section id is missing.") if (!$section->{section_id});
	my ($surveylist,$out);
	
	# get all surveys
	$surveylist = $this->fetchSurvey($section->{section_id});
	# insert option commands into each record
	for(my $i = 0; $i < @$surveylist; $i++){
		$surveylist->[$i]{option_command} = $this->html_adminSurveyRowLink($surveylist->[$i]);
	}
	my ($header,$data) = $this->getReportbuilderData($this->getAdminSurveyHeading(),$surveylist);
	my %report = (
		title=>$this->html_adminSurveyListTitle($section),
		headers=>$header,
		data=>$data,
	);
	my $out .= "\n<div class=\"$this->{div_class}\">";
	$out .= $this->html_javascript("adminsurveylist");
	$out .= "\n".$this->reportbuilder(\%report);
	$out .= "\n</div>\n";
	$out .= "<br>";
	return $out;
}
# 'reconfigure' screen
# (overloaded function)
sub html_adminFormMain{
	my ($this,$opt) = @_;
	my $surveyid = $opt->{surveyid};
	my %survey = $this->{db}->fetch("survey",$surveyid);
	my $editurl = $opt->{edit};
	my $newurl = $opt->{new};
	my $delurl = $opt->{delete};
	$this->surveyInit;
	my $module = $this->getModuleName;	
	my @data;
	my @colmap = @{$this->getColumnMap("survey")};
	my @header;
	my $foot;
	foreach my $col (@colmap){
		next if ($this->{db}->authorize() < $col->{read});
		push(@header,$col->{label});
		push(@data,$this->{db}->show_data("survey",$col->{column},$survey{$col->{column}},$surveyid));
	}		
	# build the report
	my %report = (
		title=>"$module Form Information",
		data=>\@data,
		tools=>$this->getAdminOption($surveyid),
		headers=>\@header,
		foot=>$foot,
	);
	return $this->reportbuilder(\%report);
}
# survey editing options
# (overloaded function)
sub getAdminOption{
	my ($this,$surveyid) = @_;
	my (@commands);
	@commands = (
	{id=>"edit",label=>"Edit",url=>$this->myRelink({action=>"edit_mainform",sid=>$surveyid}),target=>"",img=>"edit"},
	);
	return $this->html_adminOption(\@commands);
}
# fetch report columns
# (overloaded function)
sub getColumnMap{
	my ($this,$table) = @_;
	my $ncol = $this->{db}->{map}->get_ncol($table);
	my @colmap;
      	for (my $icol = 0; $icol < $ncol; $icol++) {
		my %colmap = $this->{db}->{map}->get_column($table,$icol);
		if ($table eq "survey"){
			my @columns = (
				"section_id","type","use_require","use_generic_css","save_result","header_html","footer_html",
				"thankyou_message","email","from_email","return_url","return_message"
			);
			my $col_list = join("|",@columns);
			if  ($colmap{column} !~ /^($col_list)$/i){
				next;
			}
		}
		push @colmap, \%colmap;
	}
	return \@colmap;	
}
# (overloaded function)
sub html_adminForm{
	# $surveyid: survey id
	my ($this,$sectionid,$surveyid) = @_;	
	return $this->error("Missing inputs") if (!$sectionid && !$surveyid);
	my $table = "survey";
	my %survey = $this->{db}->fetch($table,$surveyid) if $surveyid;
	$sectionid = $survey{section_id} || $sectionid;
	$this->adminFormInit;
	my $formtype = $this->getFormType;
	my $module = $this->getModuleName;
	my $contact = $this->{contact};
	my $pkey = $surveyid || "_";
	my $out = "\n<form method=\"get\" action=\"".$this->link({action=>undef,sid=>undef})."\">";
	$out .= "\n<h2>Part 1: $module Settings</h2>";
	$out .= "\n<p>Title of $module:".$this->helplink($table,"name")."<br><input type=\"text\" name=\"survey#$pkey#name\" size=\"60\" value=\"$survey{name}\"></p>";	
	$out .= "\n<p>$module Description:".$this->helplink($table,"description")."<br><textarea name=\"survey#$pkey#description\" rows=\"4\" cols=\"60\">$survey{description}</textarea></p>";
	$out .= "\n<p>Display '<sub><font color=\"red\" size=\"+1\">*</font></sub>' in required questions?".$this->helplink($table,"use_require");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_require\" value=\"Yes\"";
	$out .= $survey{use_require} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_require\" value=\"No\"";
	$out .= $survey{use_require} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	
	$out .= "\n<p>Do you want to use the generic style sheet?".$this->helplink($table,"use_generic_css");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_generic_css\" value=\"Yes\"";
	$out .= $survey{use_generic_css} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_generic_css\" value=\"No\"";
	$out .= $survey{use_generic_css} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want to save responses to the database?".$this->helplink($table,"save_result");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#save_result\" value=\"Yes\"";
	$out .= $survey{save_result} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#save_result\" value=\"No\"";
	$out .= $survey{save_result} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	# Turn contact info section title on/off
	# New functionality: by default, show the title

	$out .= "\n<p>Custom HTML header:".$this->helplink($table,"header_html")."<br>";
	$out .= "<textarea name=\"survey#$pkey#header_html\" rows=\"4\" COLS=\"60\">$survey{header_html}</textarea></p>";
	$out .= "\n<p>Custom HTML footer:".$this->helplink($table,"footer_html")."<br>";
	$out .= "<textarea name=\"survey#$pkey#footer_html\" rows=\"4\" cols=\"60\">$survey{footer_html}</textarea></p>";
	$out .= "\n<p>Thank you message:".$this->helplink($table,"thankyou_message")."<br>";
	$out .= "<textarea name=\"survey#$pkey#thankyou_message\" rows=\"4\" cols=\"60\">$survey{thankyou_message}</textarea></p>";	
	$out .= "\n<P>Organization Email Address:".$this->helplink($table,"from_email")."<br>";
	$out .= "<input type=\"text\" name=\"survey#$pkey#from_email\" size=\"60\" value=\"$survey{from_email}\"></p>";
	$out .= "\n<p>A list of email address to send $module results to:".$this->helplink($table,"email")."<br>";
	$out .= "<input type=\"text\" name=\"survey#$pkey#email\" size=\"60\" value=\"$survey{email}\"></p>";
	$out .= "\n<P>Redirect URL:".$this->helplink($table,"return_url")."<br>";
	$out .= "<input type=\"text\" name=\"survey#$pkey#return_url\" size=\"60\" value=\"$survey{return_url}\"></p>";
	$out .= "\n<P>Redirect message:".$this->helplink($table,"return_message")."<br>";
	$out .= "<textarea name=\"survey#$pkey#return_message\" rows=\"4\" cols=\"60\">$survey{return_message}</textarea></p>";
	
	if (!$surveyid){
		$out .= "\n<h2>Part 2: Contact Information</h2>";
		$out .= $this->html_wizardContactInfo($contact);
	}
	#$out .= "\n<p>Do you want to add custom contact information?<br>";
	#$out .= "<input type=\"radio\" name=\"customq\" value=\"Yes\">Yes&nbsp;&nbsp;";
	#$out .= "<input type=\"radio\" name=\"customq\" value=\"No\">No&nbsp;&nbsp</p>";
	
	$out .= "\n<p>Do you want Contact Information displayed at the beginning or end of the $module?".$this->helplink($table,"contactinfo_pos")."<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#contactinfo_pos\" value=\"beginning\"";
	$out .= $survey{contactinfo_pos} !~ /end/i ? " checked" : "";
	$out .= ">Beginning&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#contactinfo_pos\" value=\"end\"";
	$out .= $survey{contactinfo_pos} =~ /end/i ? " checked" : "";
	$out .= ">End&nbsp;&nbsp</p>";

	if ($surveyid){
		$out .= "\n<input type=\"hidden\" name=\"survey#$pkey#id\" value=\"$surveyid\">";
	}
	$out .= "\n<input type=\"hidden\" name=\"survey#$pkey#section_id\" value=\"$sectionid\">";	
	$out .= "\n<input type=\"hidden\" name=\"survey#$pkey#type\" value=\"$formtype\">";
	$out .= "\n<input type=\"hidden\" name=\"action\" value=\"submit_form\">";
	$out .= "\n<input type=\"submit\" value=\"Submit\">&nbsp;&nbsp;&nbsp;&nbsp;";
	$out .= "\n<input type=\"reset\" value=\"Reset\">";
	$out .= "\n</form>";
    	return $out;
}
# create 'view report' screen
# (overloaded function)
sub reportList{
	my ($this,$surveyid) = @_;
	return $this->error("Invalid ".$this->getModuleName." id") if !$surveyid;
	my %survey = $this->{db}->fetch("survey",$surveyid);
	my $out = $this->html_javascript("reportlist",{sid=>$surveyid,action=>"view_report",table=>"survey_dds"});
	my %report = (
		data=>$this->listAllReport($surveyid),
	);
	$out .= "\n<h1>$survey{name} Reports</h1>\n";
	$out .= $this->reportbuilder(\%report);
	return $out;
}
# list all reports associated to a survey
# (overloaded function)
sub listAllReport{
	my ($this,$surveyid) = @_;
	my (@custom_reports,@rlist,$out);
	my @rlist = @{$this->getDefaultReport($surveyid)};
	$out .= "\n<p>\n";
	$out .= $this->html_listAllReport(\@rlist,{title=>"Standard Reports"});
	$out .= "\n</p>\n";
	return $out;
}
# get default reports
# (overloaded function)
sub getDefaultReport{
	my ($this,$surveyid) = @_;
	my @list = (
		{label=>"Response",description=>"List Responses",url=>$this->myRelink({action=>"view_reg_list",sid=>$surveyid})},
	);
	return \@list;
}
# participant/registrant list report title
# (overloaded function)
sub getRegistrantListTitle{
	my ($this) = @_;
	return "Responses";
}
# report top link
# (overloaded function)
sub reportTopLink{
	my ($this,$surveyid) = @_;
	my $url = $this->myRelink({sid=>$surveyid,action=>"view_report"});
	my @links = (
	#{label=>"Go back to report menu",url=>$url},
	#{label=>"Go back to report menu",url=>"javascript:history.back()"},
	);		
	return \@links;
}
1;
