package Modules::SimpleRegistration;
use strict;
use CGI::Carp qw(fatalsToBrowser);
use ExSite::Config;
use ExSite::Misc;
use Modules::Survey;
use ExSite::Mail;
use vars qw(@ISA);
push(@ISA,"Modules::Survey");

my $debug = 0;
print "content-type: text/html\n\n<p><u>debug</u></p>" if $debug;

#---------------------------------------------------------------------
sub moduleInit{
	my ($this) = @_;
	$this->{name} = "SimpleRegistration";
	$this->{form_type} = "registration";
	$this->{registrant_list_excel_filename} = $config{$this->{class_name}}{config}{registrant_list}{excel_filename} || "registrantlist.xls";
	return;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return $this->getModuleName."s";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
		return $this->getModuleName." tools allow users to create forms, modify forms, remove forms and view reports.";
	}elsif (/isRestricted/){
        	return $config{SimpleRegistration}{isRestricted};
	}elsif (/isService/){
		return $config{SimpleRegistration}{isService};
	}elsif (/DynContentParam/){
		return $this->getDynContentParameter(shift);
	}
}
sub write{
	my ($this,$options) = @_;
	my (%opt,$action);
	%opt = &DecodeString($options);
	$action = $this->{input}{action} || $opt{action};
	# create forms
	if ($action eq "do"){
		my $surveyid = $this->{input}{sid} || $opt{sid};
		my $eventid = $this->{input}{evtid} || $opt{evtid};
		if ($eventid){
			$this->setEventData($eventid);
		}
		return $this->surveyForm({surveyid=>$surveyid});
	# process form data
	}elsif ($action eq "form_submit"){
		my $surveyid = $this->{input}{sid} || $opt{sid};
		my $eventid = $this->{input}{evtid} || $opt{evtid};
		if ($eventid){
			$this->setEventInfo($eventid);
		}
		return $this->validateForm;
	# public survey list
	}else{
		return $this->surveyList($share{Page}->get("section")->{section_id});
	}
	return undef;
}
#---------------------------------------------------------------------
# Replace form header with event info
sub presetFormField{
	my ($this) = @_;
	my $eventid = $this->{input}{evtid};
	return if !$eventid;
	$this->setEventInfo($eventid);
	$this->setEventData($eventid);
	return;
}
sub setEventInfo{
	my ($this,$eventid) = @_;
	$this->eventMode(1);
	$this->event_id($eventid);
	return;
}
sub setEventData{
	my ($this,$eventid) = @_;
	my $db = $this->{db};
	my $e = $db->fetch("event",$eventid);
	$this->setEventInfo($eventid);
	my $out = "<h2>".$db->show_data("event","name",$e->{name},$eventid)."</h2>\n";    	
   	$out .= "<h4>Date: <i>".$this->show_date($e->{start},$e->{end})."</i>\n";
    	my $time = $this->show_time($e->{tstart},$e->{tend});
    	if ($time){
		$out .= "<br>Time: <i>$time</i>\n";
    	}
    	if ($e->{place}){
		$out .= "<br>Place: <i>".$db->show_data("event","place",$e->{place},$eventid)."</i>";
    	}
    	if ($e->{host}){
		$out .= "<br>Host: <i>".$db->show_data("event","host",$e->{host},$eventid)."</i>";
    	}
    	$out .= "</h4>\n";
    	$out .= "<p>".$db->show_data("event","description",$e->{description},$eventid)."</p>\n";
	$this->formHeader($out);
	return;
}
sub show_date {
    	my ($this,$date1,$date2) = @_;
    	return "" if (!$date1);
	my @wdy = qw(Sun Mon Tue Wed Thu Fri Sat);
	my @month = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
	my @dayinmonth = qw(31 28 31 30 31 30 31 31 30 31 30 31);
    	my ($year1,$mon1,$day1) = split /-/,$date1;
    	if ($date2 && $date2 ne $date1) {
		my ($year2,$mon2,$day2) = split /-/,$date2;
		if ($year1 != $year2 || $mon1 != $mon2) {
	    		return "$month[$mon1-1] $day1, $year1 - $month[$mon2-1] $day2, $year2";
		} else {
	    		return "$month[$mon1-1] $day1-$day2, $year1";
		}
    	} else {
		return "$month[$mon1-1] $day1, $year1";
    	}
	return;
}

sub show_time {
    	my ($this,$time1,$time2) = @_;
    	my $out;
    	if ($time1 != /^[\s0]0:00/) {
		$time1 =~ s/^0//;
		my ($hour,$min,undef) = split /:/,$time1;
		my $am;
		if ($hour < 12) { 
			$am = 1; 
		}elsif ($hour > 12) { 
			$hour -= 12; 
		}
		my $tstr = "$hour:$min ";
		$tstr .= $am ? "am" : "pm";
		$out = "$tstr";
    	}
    	if ($time2 != $time1 && $time2 != /^[\s0]0:00/) {
		$time2 =~ s/^0//;
		my ($hour,$min,undef) = split /:/,$time2;
		my $am;
		if ($hour < 12) { 
			$am = 1; 
		}elsif ($hour > 12) { 
			$hour -= 12; 
		}
		my $tstr = "$hour:$min ";
		$tstr .= $am ? "am" : "pm";
		$out .= " - $tstr";
    	}
    	return $out;
}
sub getDynContentParameter{
	my ($this,$sectionid) = @_;
	return if !$sectionid;
	my (@surveys,$out);
	@surveys = $this->{db}->fetch_match("survey",{section_id=>$sectionid,type=>"registration"});
	$out .= "\n<table>";
	$out .= "\n<tr><td>";
	$out .= "<b>Select a registration form:</b>";
	$out .= "</td><td>";
	$out .= "<select name=\"param\">";
	$out .= "<option value=\"\">&lt;Default&gt;</option>";
	foreach my $survey(@surveys){
		$out .= "<option value=\"sid=$survey->{id}&action=do\">$survey->{name}</option>";
	}
	$out .= "</select>";
	$out .= "</td></tr>";
	$out .= "\n</table>";
	return $out;
}
# control panel
sub control_panel{
	my $this = shift;
	my $out;
	my $admin_level = $config{SimpleRegistration}{admin_level} || $config{auth}{root_auth_level} - 1;
	my $sectionid = $this->{input}{section_id};
	# admin access only
	#if ($this->{db}->authorize < $config{auth}{root_auth_level} - 1) {
	if ($this->{db}->authorize < $admin_level){
		$out = "<h3 class=\"error\">Permission denied</h3><p>You do not have sufficient system priveleges to use this feature.</p>\n";
	}else {
		my $action = $this->{input}{action};
		if ($action =~ /^(insert|update)$/i){
			$out .= $this->exsiteFormPost();
		}elsif ($action eq "excel_user"){
			$out .= $this->exportRegistrantList($this->{input}{sid});
		# view registrant/participant list
		}elsif ($action eq "view_reg_list"){
			$out .= $this->viewRegistrantList($this->{input}{sid});
		# edit a child recrod
		}elsif ($action eq "edit_form_record"){
			$out .= $this->adminChildRecordForm($this->{input}{tab},$this->{input}{id},$this->{input}{ptab},$this->{input}{pid});
		# add new child records to survey
		}elsif ($action eq "new_form_record"){
			$out .= $this->adminChildRecordForm($this->{input}{tab},undef,$this->{input}{ptab},$this->{input}{pid});
		# edit survey info
		}elsif ($action eq "edit_mainform"){
			my $surveyid = $this->{input}{sid};
			$out .= $this->adminForm(undef,$surveyid);
		# main survey edit form
		}elsif ($action eq "main_form"){
			my $surveyid = $this->{input}{sid};
			$out .= $this->adminFormMain($surveyid);
		# process form data
		}elsif ($action eq "submit_form"){
			$out .= $this->adminFormProcess("survey",$this->{input});
		# create edit a survey
		}elsif ($action eq "new_mainform"){
			my $sectionid = $this->{input}{section_id};
			$out .= $this->adminForm($sectionid);
		# export the registrant list to excel spreadsheet
		}elsif ($action eq "excel_user"){
			my $surveyid = $this->{input}{sid};
			$out .= $this->exportRegistrantList($surveyid);
		# view individual's survey responses
		}elsif ($action eq "view_user"){
			my $surveyid = $this->{input}{sid};
			$out = $this->viewSingleSurveyResponse($surveyid,$this->{input}{uid});		
		# admin survey list
		}elsif ($action eq "admin_list"){
			$out = $this->showAllSurveyList;
		# view reports
		}elsif ($action eq "view_report"){
			my $surveyid = $this->{input}{sid};
			$out = $this->showReportList($surveyid);
		}elsif ($action eq "view_summary"){
			my $surveyid = $this->{input}{sid};
			$out = $this->viewSurveyReport($surveyid);
		# view question summaries
		}elsif ($action eq "view_question"){
			my ($questionid,$surveyid);
			$questionid = $this->{input}{qid};
			$surveyid = $this->{input}{sid};
			$out = $this->viewQuestionReport($questionid,$surveyid);
		# start survey wizard
		}elsif ($action eq "wizard"){
			my ($sectionid);
			$sectionid = $this->{input}{section_id};
			$out = $this->surveyWizard($sectionid);
		# data filter
		}elsif ($action eq "dds"){
			my ($sectionid);
			$sectionid = $this->{input}{section_id};
			$out = $this->surveyDDS($sectionid);
		# create a new data filter
		}elsif ($action eq "dds_action"){
			$out = $this->newDDSReport;
		# delete a data filter
		}elsif ($action eq "rec_delete"){
			$out = $this->deleteRecord($this->{input}{tab},$this->{input}{id},$this->{input}{reply});
		}elsif ($action){
			$out = $this->write;
		}elsif ($sectionid){
			my %section = $this->{db}->fetch("section",$sectionid);
			$out = $this->adminSurveyList(\%section);
		# display admin survey list by default
		}else{
			$out = $this->set_section_id();
		}
	}
	return $out;
}
#------------------------------------------------------------------------------------------------------------------------
# get module name
sub getModuleName{
	my ($this) = @_;
	return "Registration Form";
	$this->{name} =~ /^simple(.+)/i;
	return ucfirst($1);
}
#  list registrants
# (overloaded function)
sub getRegistrantListTitle{
	my ($this) = @_;
	return "Registrants";
}
# fetch surveys(type is registration)
# (overloaded function)
sub fetchSurvey{
	# $sectionid : section id
	my ($this,$sectionid) = @_;
	my @list = $this->{db}->fetch_match("survey",{section_id=>$sectionid,type=>"registration"});
	return \@list;
}
# wizard init routine
# (overloaded function)
sub wizardInit{
	my ($this) = @_;
	$this->{screen} = 0;
	$this->{wizard_error} = [];
	# default contact info fields
	$this->{contact} = [
		{id=>1,name=>"First Name",type=>"Short answer",checked=>1,mandatory=>1},
		{id=>2,name=>"Last Name",type=>"Short answer",checked=>1,mandatory=>1},
		{id=>3,name=>"Title",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>4,name=>"Company",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>5,name=>"Street Address",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>6,name=>"City",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>7,name=>"State/Province",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>8,name=>"Country",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>9,name=>"Telephone",type=>"Short answer",checked=>1,mandatory=>1},
		{id=>10,name=>"Fax",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>11,name=>"Email",type=>"Short answer",checked=>1,mandatory=>1},
	];
	return;
}
# wizard init routine
# (overloaded function)
sub adminFormInit{
	my ($this) = @_;
	# default contact info fields
	$this->{contact} = [
		{id=>1,name=>"First Name",type=>"Short answer",checked=>1,mandatory=>1},
		{id=>2,name=>"Last Name",type=>"Short answer",checked=>1,mandatory=>1},
		{id=>3,name=>"Title",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>4,name=>"Company",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>5,name=>"Street Address",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>6,name=>"City",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>7,name=>"State/Province",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>8,name=>"Country",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>9,name=>"Telephone",type=>"Short answer",checked=>1,mandatory=>1},
		{id=>10,name=>"Fax",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>11,name=>"Email",type=>"Short answer",checked=>1,mandatory=>1},
	];
	return;
}
# survey wizard screen 1
# (overloaded function)
sub html_wizardPage{
	# $opt: option hash
	my ($this,$opt) = @_;	
	my ($out,@types,$contact,$module,$formtype);
	$formtype = $this->getFormType;
	$module = $this->getModuleName;
	$contact = $opt->{contact};
	$out .= "<h2>Part 1: $module Settings</h2>";
	$out .= "<p>Title of $module:<br>\n
	<input type=\"text\" name=\"survey#_#name\" size=\"60\" value=\"\"></p>\n
	<p>$module Description:<br>\n
	<textarea name=\"survey#_#description\" rows=\"4\" cols=\"60\"></textarea></p>\n";
	$out .= "<p>Display '<sub><font color=\"red\" size=\"+1\">*</font></sub>' in required questions?<br>\n
	<input type=\"radio\" name=\"survey#_#use_require\" value=\"Yes\">Yes&nbsp;&nbsp;
	<input type=\"radio\" name=\"survey#_#use_require\" value=\"No\">No&nbsp;&nbsp</p>\n";
	$out .= "<input type=\"hidden\" name=\"survey#_#use_group\" value=\"No\">\n";
	$out .= "<input type=\"hidden\" name=\"survey#_#use_number\" value=\"No\">\n";
	$out .= "<p>Custom HTML header:<br>\n
	<textarea name=\"survey#_#header_html\" rows=\"4\" COLS=\"60\"></textarea></p>\n";
	$out .= "<p>Custom HTML footer:<br>\n
	<textarea name=\"survey#_#footer_html\" rows=\"4\" cols=\"60\"></textarea></p>\n";
	$out .= "<p>Thank you message:<br>\n
	<textarea name=\"survey#_#thankyou_message\" rows=\"4\" cols=\"60\"></textarea></p>\n";
	$out .= "<p>A list of email address to send $module results to:<br>\n
	<input type=\"text\" name=\"survey#_#email\" size=\"60\" value=\"\"></p>\n";
	$out .= "<P>Redirect message:<br>\n
	<textarea name=\"survey#_#return_message\" rows=\"4\" cols=\"60\"></textarea></P>\n";
	$out .= "<P>Redirect Url:<br>\n
	<input type=\"text\" name=\"survey#_#return_url\" size=\"60\" value=\"\"></P>\n";
	$out .= "<h2>Part 2: Contact Information</h2>";
	$out .= $this->html_wizardContactInfo($contact);
	$out .= "<input type=\"hidden\" name=\"action\" value=\"wizard\">";
	$out .= "<p>Do you want to add custom contact information?<br>\n
	<input type=\"radio\" name=\"customq\" value=\"Yes\">Yes&nbsp;&nbsp;
	<input type=\"radio\" name=\"customq\" value=\"No\">No&nbsp;&nbsp</p>\n";
	$out .= "<input type=\"hidden\" name=\"survey#_#contactinfo_pos\" value=\"beginning\">
	<input type=\"hidden\" name=\"survey#_#section_id\" value=\"$opt->{section_id}\">\n";
	$out .= "<input type=\"hidden\" name=\"survey#_#type\" value=\"$formtype\">\n";
	############# reserved input for event id
	# $out .= "<input type=\"hidden\" name=\"survey#_#event_id=\" value=\"\">";
    	return $out;
}
# get default reports
# (overloaded function)
sub getDefaultReport{
	my ($this,$surveyid) = @_;
	my @list = (
		{label=>"Registration",description=>"List Registrations",url=>$this->myRelink({action=>"view_reg_list",sid=>$surveyid})},
	);
	return \@list;
}
# admin table row links
# (overloaded function)
sub getAdminSurveyListRowLink{
	# $survey: survey hash
	my ($this,$survey) = @_;
	my (@links,$module);
	$module = $this->getModuleName;
	@links = (
	{id=>"id",label=>"Do Registration",url=>&relink({action=>"do",sid=>$survey->{id}}),img=>"do"},
	{id=>"view",label=>"View Reports",url=>&relink({action=>"view_report",sid=>$survey->{id}}),img=>"view"},
	{id=>"edit",label=>"Reconfigure",url=>$this->myRelink({action=>"main_form",sid=>$survey->{id}}),img=>"edit"},
	{id=>"delete",label=>"Delete",url=>"javascript:deleteRecord($survey->{id})",img=>"delete"},
	);
	return \@links;
}
# gather brief summary data
sub constructBriefSummaryData{
	my ($this,$reg_count,$question_count) = @_;
	my @list = (
		{label=>"Total Registrants",value=>"$reg_count"},
		{label=>"Total Questions",value=>"$question_count"},
	);
	return \@list;
}
sub addAdditionalColumn{
	my ($this,$surveyid) = @_;
	my ($append_string,$records,$additional_column,$loop,@addcol);
	$additional_column = $this->{registrant_list_additional_column} || undef;
	$append_string = "question_";
	$records = $this->getSurveyData($surveyid,"survey_question",$additional_column);
	$loop = !$additional_column || $additional_column > @$records ? @$records : $additional_column;
	for(my $i = 0; $i < $loop; $i++){
		my $colname = $append_string.$records->[$i]{id};
		push(@addcol,{label=>$records->[$i]{label},column=>$colname});
	}
	return @addcol;
}
sub getRegistrantListRecord{
	my ($this,$surveyid) = @_;
	my ($records,$registrants);
	$records = $this->{db}->fetch_match("survey_question",{survey_id=>$surveyid});
	$registrants = $this->fetchRegistrant($surveyid,[],$records);
	return $registrants;
}
# detect event mode
sub detectEventMode{
	my ($this,$regs) = @_;
	foreach my $reg(@$regs){
		if ($reg->{event_id}){
			$this->eventMode(1);
			return;
		}
	}
	return;
}
# survey form start and end tags
sub html_surveyFormTag{
	# $option: options(open,close)
	my ($this,$option,$survey) = @_;
	my ($out,$surveyid);
	$surveyid = $this->{input}{sid} || $survey->{id};
	if ($option eq "open"){
		$out .= "\n<form method=\"post\" action=\"".&relink({action=>undef,sid=>undef})."\">";
		$out .= "\n<input type=\"hidden\" name=\"action\" value=\"form_submit\">";
#		$out .= "\n<input type=\"hidden\" name=\"sid\" value=\"$this->{input}{sid}\">";
		$out .= "\n<input type=\"hidden\" name=\"sid\" value=\"$surveyid\">";
		if ($this->eventMode()){
			$out .= "\n<input type=\"hidden\" name=\"evtid\" value=\"".$this->event_id()."\">";
		}
		$out .= "\n";
	}elsif($option eq "close"){
		$out .= "\n</form>";
	}
	return $out;
}
# event registration form completion screen
sub html_eventFinalScreen{
	my ($this) = @_;
	my $out .= "\n<div class=\"$this->{div_class}\">";
	$out .= "\n<p>";
	$out .= "Thank you for your registration.";
	$out .= "</p>";
	$out .= "<p><a href=\"".$this->myRelink({evtid=>$this->{input}{evtid}})."\">Click here to return to the event calendar.</a></p>";
	return $out;
}
# get form footer
sub getFormFooter{
	my ($this,$survey) = @_;
	my $out;
	# using the event header
	if ($this->eventMode()){
		$out = $this->formFooter();
	}
	if ($survey->{footer_html}){
		$out .= "<br>".$survey->{footer_html};
	}
	return $out;
}
# get form header
sub getFormHeader{
	my ($this,$survey) = @_;
	my $out;
	# using the event header
	if ($this->eventMode()){
		$out = $this->formHeader();
	}
	#$out .= $survey->{header_html} || "\n<h2>$survey->{name}</h2>\n<p>$survey->{description}</p>\n";
	$out .= $survey->{header_html};
	return $out;
}
# set form footer
sub formFooter{
	my ($this,$content) = @_;
	return $this->{survey_footer} if !$content;
	$this->{survey_footer} = $content;
	return $this->{survey_footer};
}
# set form header
sub formHeader{
	my ($this,$content) = @_;
	return $this->{survey_header} if !$content;
	$this->{survey_header} = $content;
	return $this->{survey_header};
}
# form completion screen
sub html_formFinalScreen{
	# $survey: survey hash
	my ($this,$survey,$contact,$qdata) = @_;
	if ($this->eventMode()){
		return $this->html_eventFinalScreen();
	}else{
		return $this->html_surveyFinalScreen($survey,$contact,$qdata);
	}
	return $this->html_surveyFinalScreen($survey,$contact,$qdata);
}
# using event mode ?
sub eventMode{
	my ($this,$flag) = @_;
	if (!defined($flag)){
		return 1 if $this->{event_enable};
		return 0;
	}
	$this->{event_enable} = $flag || 0;
	return $this->{event_enable};
}
sub event_id{
	my ($this,$eventid) = @_;
	if (!defined($eventid)){
		return $this->{event_id} || 0;
	}
	$this->{event_id} = $eventid;
	return $this->{event_id};	
}
sub addRegistrantData{
	my ($this) = @_;
	my %in;
	$in{event_id} = $this->event_id() if $this->eventMode();
	return %in;
}
sub getEventList{
	my ($this) = @_;
	my $sql = "select * from survey where type = 'registration' ";
	my @sites = $this->{db}->my_real_sites();
	my @siteid;
	foreach my $site(@sites){
		push(@siteid,$site->{section_id});
	}
	$sql .= " and (section_id = ".join(" or section_id = ",@siteid).")";
	# fetch registration forms
	my @forms = $this->{db}->custom_query($sql);
	my @formid;
	foreach my $form(@forms){
		push(@formid,$form->{id});
	}
	$sql = "select * from event where (survey_id = ".join(" or survey_id = ",@formid).")";
	return $this->{db}->custom_query($sql);
}
sub selectEvent{
	my ($this,$surveyid) = @_;
	return if !$this->eventMode();
	my @events = $this->getEventList();
	my $out;
	my $selevent = $this->{input}{evtid};
	$out .= "\n<script language=\"javascript\">";
	$out .= "\n<!--";
	$out .= "\n
	function changeReport(){
		var sel = document.getElementById('evtid');
		location = '".$this->myRelink({sid=>$surveyid,action=>"view_reg_list"})."&evtid=' + sel.value;
	}
		";
	$out .= "\n-->";
	$out .= "\n</script>";
	$out .= "<form>";
	$out .= "Select an event: ";
	$out .= "<select id=\"evtid\" name=\"evtid\" onChange=\"changeReport()\">";
	$out .= "<option value=\"\">All</option>";
	$out .= "<option value=\"na\"";
	$out .= $selevent eq "na" ? " selected" : "";
	$out .= ">SimpleRegistration(no event association)</option>";
	foreach my $event(@events){
		$out .= "<option value=\"$event->{event_id}\"";
		$out .= $selevent == $event->{event_id} ? " selected" : "";
		$out .= ">$event->{name}</option>";
	}
	$out .= "</select>";
	$out .= "</form>";
	return $out;
}
sub filterRegistrant{
	my ($this,$regs) = @_;
	return $regs if !$this->eventMode();
	my $selevent = $this->{input}{evtid};
	$regs = $this->attachEventInfo($regs);
	return $regs if !$selevent;
	my @newlist;
	foreach my $reg(@$regs){
		if ($reg->{event_id} == $selevent || (!$reg->{event_id} && ($selevent eq "na"))){			
			push(@newlist,$reg);
		}
	}
	return \@newlist;
}
sub attachEventInfo{
	my ($this,$regs) = @_;
	my %events;
	my @events = $this->{db}->fetch_all("event");
	foreach my $ev (@events){
		$events{$ev->{event_id}} = $ev->{name};
	}
	for(my $i = 0;$i < @$regs;$i++){
		$regs->[$i]{contact_0} = $events{$regs->[$i]{event_id}} || "n/a";
		$regs->[$i]{event_name} = $events{$regs->[$i]{event_id}} || "n/a";
	}
	return $regs;
}
# view list of registrant
sub viewRegistrantList{
	my ($this,$surveyid) = @_;	
	return $this->error("permission denied. invalid id.") if !$surveyid;	
	my $registrants = $this->getRegistrantListRecord($surveyid);
	$this->detectEventMode($registrants);
	$registrants = $this->filterRegistrant($registrants);
	my @cols = (	
		{label=>"Option",column=>"option_command"},
		{label=>"No",column=>"no"},
		{label=>"Name",column=>"fullname"},
		{label=>"Email",column=>"email"},	
	);
	if ($this->eventMode()){
		push(@cols,{label=>"Event",column=>"event_name"});
	}
	@cols = (@cols,$this->addAdditionalColumn($surveyid));
	
	my $out = $this->html_javascript("reportlist",{action=>"view_reg_list",sid=>$surveyid,table=>"survey_registrant"});
	$out .= $this->reportMenu($surveyid);
	$out .= "<p>".$this->selectEvent($surveyid)."</p>";
			
	return "<p>Sorry, no records found.</p><input type=\"button\" value=\"&lt; Back\" onClick=\"history.back();\">" if (@$registrants < 1);	
	for(my $i = 0; $i < @$registrants; $i++){		
		my ($name,$email,$link,%myHash,$reg);
		$reg = $registrants->[$i];		
		$name = $reg->{first}." ".$reg->{last};
		$email = $reg->{email} || "n/a";
		# create email link
		$email = $this->validateEmail($email) ? $this->displayData("email",$email) : $email;
		%myHash = (action=>"view_user",uid=>$reg->{id},sid=>$surveyid);
		# view registrant's detilas
		$link = "<a href=\"".&relink(\%myHash)."\">".$this->getIcon("view","View ${name}'s info")."</a>";
		# delete registrant
		$link .= "<a href=\"javascript:deleteRecord($reg->{id})\">".$this->getIcon("delete","Delete ${name}")."</a>";		
		$reg->{option_command} = $link;
		$reg->{no} = $i + 1;
		$reg->{fullname} = $name;
		$reg->{email} = $email;
	}
	my ($header,$data) = $this->getReportbuilderData(\@cols,$registrants);
	my $path = $this->myRelink({sid=>$surveyid,action=>"excel_user",evtid=>$this->{input}{evtid}});
	my %report = (
		title=>$this->getRegistrantListTitle(),
		tools=>"\n<a href=\"".$path."\">".$this->getIcon("excel","Export to Excel")."</a>",
		headers=>$header,
		data=>$data,
	);
	$out .= $this->reportbuilder(\%report);
	$out .= "\n";
	return $out;
}
# export registrant list to excel
sub exportRegistrantList{
	my ($this,$surveyid) = @_;
	return $this->error("permission denied. invalid id.") if !$surveyid;
	my ($contactinfo,$questions,$registrants,$filename,@column,%heading,@completelist);
	# fetch all survey questions
	$questions = $this->getSurveyData($surveyid,"survey_question");
	# fetch all contact info 
	$contactinfo = $this->getSurveyData($surveyid,"survey_contact_question");
	# insert survey answer and contact info into the registrant records
	$registrants = $this->fetchRegistrant($surveyid,$contactinfo,$questions);
	# Convert date
	for(my $i = 0; $i < @$registrants; $i++){
		$registrants->[$i]{last_update} = $this->{db}->show_data("survey_registrant","last_update",$registrants->[$i]{last_update});
	}
	$this->detectEventMode($registrants);
	$registrants = $this->filterRegistrant($registrants);
	if ($this->eventMode()){
		push(@{$contactinfo},{id=>0,label=>"Event"});
	}
	# filename for exporting spreadsheet
	$filename = $this->{registrant_list_excel_filename};

	push(@column,"last_update");
	$heading{last_update} = "Date";	

	foreach my $section({table=>"contact",data=>$contactinfo},{table=>"question",data=>$questions}){
		foreach my $list(@{$section->{data}}){
			my $col = $section->{table}."_".$list->{id};
			# spreadsheet columns
			push(@column,$col);
			# spreadsheet column labels
			$heading{$col} = $list->{label};
		}
	}
	print "content-type: application/vnd.ms-excel\ncontent-disposition: attachment; filename=$filename\n\n";
	# export!
	print $this->{db}->export({table=>"survey_registrant",format=>"excel",data=>$registrants,column=>\@column,heading=>\%heading});
	return;
}

# overloaded functions

sub getColumnMap{
	my ($this,$table) = @_;
	my $ncol = $this->{db}->{map}->get_ncol($table);
	my @colmap;
      	for (my $icol = 0; $icol < $ncol; $icol++) {
		my %colmap = $this->{db}->{map}->get_column($table,$icol);
		push @colmap, \%colmap;
	}
	return \@colmap;	
}

# sub validate form data
sub validateForm{
	my ($this) = @_;
	my $surveyid = $this->{input}{sid};
	# missing survey id
	return $this->error("Missing survey id.") if (!$surveyid);
	my ($clist,%clist,$qlist,%qlist,%contact,%question,%data);
	# run survey init routine
	$this->surveyInit;
	# fetch all questions
	$qlist = $this->{db}->fetch_match("survey_question",{survey_id=>$surveyid},"survey_section_id asc, id asc");
	# index questions
	foreach my $q (@$qlist){
		$qlist{$q->{id}} = $q;
		$this->{input}{$q->{id}} = "" if !exists($this->{input}{$q->{id}});
	}
	# fetch all contact info
	$clist = $this->{db}->fetch_match("survey_contact_question",{survey_id=>$surveyid});
	# index contact info
	foreach my $c (@$clist){
		$clist{$c->{id}} = $c;
	}
	# get form data
	%data = %{$this->{input}};
	foreach my $key (keys(%data)){
		next if $key =~ /^(sid|action|)$/i;
		# remove return characters
		$data{$key} = $this->stripChr($data{$key});
		# remove blank input data
		$data{$key} = $this->removeEmptyInput($data{$key});
		if ($key =~ /^cc/i){
			my ($keyid);
			$keyid = $key;
			$keyid =~ s/cc#(.*)/$1/g;
			# index input contact info data
			$contact{$keyid} = $data{$key};
		}elsif ($key =~ /^\d*$/){
			# index input question data
			$question{$key} = $data{$key};
		}
	}
	foreach my $c (keys(%clist)){
		# validate mandatory contact info questions
		if (!$this->validateContactInfo($contact{$c},$clist{$c})){
			push(@{$this->{form_error}},"Please enter your ".$clist{$c}->{label}.".");		
		}
	}
	foreach my $q (keys(%qlist)){
		# validate mandatory survey questions
		$this->validateQuestion($question{$q},$qlist{$q});
	}
	# preset form fields
	$this->presetFormField();
	if (@{$this->{form_error}} > 0){		
		# show survey form if there are missing inputs on the mandatory questions
		return $this->surveyForm({surveyid=>$surveyid});
	}else{
		my ($registrantid,$registrant,%survey,$saveresponse);
		# fetch survey data
		%survey = $this->{db}->fetch("survey",$surveyid);
		# create registrant data based on the form inputs
		$registrant = $this->getRegistrantData(\%contact,\%clist);
		$registrant->{survey_id} = $surveyid;
		%$registrant = (%$registrant,$this->addRegistrantData());		
		# insert registrant data into the database
		$saveresponse = $this->saveResponse(\%survey);
		$registrantid = $this->{db}->insert("survey_registrant",$registrant) if $saveresponse;		
		my %mailcontactdata;
		foreach my $c (sort {$a <=> $b} keys(%contact)){
			my %cdata;
			$clist{$c}{label} =~ /^(first|last|email)( name)?$/i;
			if (exists($registrant->{lc($1)})){
				$contact{$c} = $registrant->{lc($1)};
			}
			# gather contact info inputs from form data
			$cdata{survey_registrant_id} = $registrantid;
			$cdata{survey_contact_question_id} = $c;
			$cdata{value} = $contact{$c};
			# info to be emailed
			my $info = "$clist{$c}{label}: ";
			$info .= $contact{$c} ? $contact{$c} : "n/a";
			# save registrant responses for email purposes		
			$mailcontactdata{$c} = $info;
			# insert contact info responses into the database
			$this->{db}->insert("survey_contact_answer",\%cdata) if $saveresponse;
		}		
		my %mailquestiondata;
		foreach my $que (@$qlist){
			my %qdata;
			# question id
			my $questionid = $que->{id};
			# gather survey question inputs from form data
			$qdata{survey_registrant_id} = $registrantid;
			$qdata{survey_question_id} = $questionid;
			$qdata{value} = $question{$questionid};
			# info to be emailed
			my $info = "$qlist{$questionid}{label}: ";
			$info .= $question{$questionid} ? join(", ",split(/; /,$question{$questionid})) : "n/a";
			# save survey response
			$mailquestiondata{$questionid} = $info;
			# insert contact info responses into the database
			$this->{db}->insert("survey_answer",\%qdata) if $saveresponse;
		}
		$this->mailSurveyResult(\%survey,\%mailcontactdata,\%mailquestiondata);
			
		# build survey completion screen
		return $this->html_formFinalScreen(\%survey,\%mailcontactdata,\%mailquestiondata);
	}
	return;
}


# email survey responses to email addresses listed on the emaillist(survey setup)
sub mailResponse{
	# $emaillist: email list
	my ($this,$survey,$mailcontent,$qdata) = @_;
	my ($from,$subject,$body,@emails,$emaillist);
	$emaillist = $survey->{email};
	# from address(from config file)
#	$from = $survey->{from_email} || $config{$this->{class_name}}{config}{mail}{from};
	$from = $this->mailFrom($survey,$qdata);
	# email subject(from config file)
	$subject = $config{$this->{class_name}}{config}{mail}{subject};
	# survey responses
	$body = join("\n",@$mailcontent);
	@emails = split(/,/,$emaillist);
	# send emails
	foreach my $to (@emails){
		next if !$this->validateEmail($to);
		&SendMail($to,$from,$subject,$body);
	}
	return;
}


# email survey response
sub mailSurveyResult{
	my ($this,$survey,$contact,$qdata) = @_;
	my @email;
	# Survey info
	push(@email,"\n".&html_to_plaintext($this->getFormHeader($survey)));
	push(@email,"\n");
	# save registrant responses for email purposes
	push(@email,"$config{$this->{class_name}}{config}{contact_info}{label}:\n") if keys(%$contact) > 0;
	foreach my $contactid(sort {$a <=> $b} keys(%$contact)){
		push(@email,$contact->{$contactid});		
	}
	# save registrant responses for email purposes
	push(@email,"\n") if keys(%$qdata) > 0;
	push(@email,"Response:\n") if keys(%$qdata) > 0;
	my $questions = $this->fetchSurveyQuestion($survey->{id});
	my $prev_section = 0;
	foreach my $question(@$questions){
		my $sec = $question->{survey_section_id} || 0;
		if ($sec != $prev_section){
			my %section = $this->{db}->fetch("survey_section",$sec);
			push(@email,"\n\n$section{label}:\n");
			$prev_section = $section{id};
		}
		# save registrant responses for email purposes
		push(@email,$qdata->{$question->{id}});
	}
	# Create a participant response data hash
	my %combine_data;
	my %combine = (%$contact,%$qdata);
	foreach my $k (keys(%combine)){
		my $a_data = $combine{$k};
		# We know that the data is present in this format:
		#	<question label>: <answer>
		# Therefore we split on ': ' to get the label and the answer
		my @data_pair = split(/: /,$a_data,2);
		$combine_data{$data_pair[0]} = $data_pair[1];
	}
	#	
	# send survey repsonses to every email address on the emaillist(from the survey setup)	
	$this->mailResponse($survey,\@email,\%combine_data);
	return;
}
sub mailFrom{
	my ($this,$survey,$answerdata) = @_;
	my $from = $survey->{from_email} || $config{$this->{class_name}}{config}{mail}{from};
	# Get a list of possible email field names from the conf file
	my $conf_email_search_field = $config{$this->{class_name}}{email_search_field};
	if (!$config{$this->{class_name}}{use_participant_email} || !$conf_email_search_field){
		return $from;
	}
	# Exit loop flag - This flag is used to exit multiple for loops
	my $exit_loop = 0;
	my @search_fields;
	if ($conf_email_search_field =~ /,/){
		@search_fields = split(/,/,$conf_email_search_field);
	}else{
		@search_fields = ($conf_email_search_field);
	}
	# Search would stop when it finds the email that matches the search field listed
	# in the conf file
	foreach my $field(@search_fields){
		foreach my $key(keys(%$answerdata)){
			my $answer_value = $answerdata->{$key};
			if ($key =~ /$field/i && $this->validateEmail($answer_value)){
				$from = $answer_value;
				$exit_loop = 1;
			}
			if ($exit_loop){
				last;
			}
		}
		if ($exit_loop){
			last;
		}
	}
	# Search for registrant's name
	my @name;
	my @name_fields;
	# Get a list of possible name fields from the conf file
	my $conf_name_search_field = $config{$this->{class_name}}{name_search_field};
	if ($conf_name_search_field =~ /,/){
		@name_fields = split(/,/,$conf_name_search_field);
	}else{
		@name_fields = ($conf_name_search_field);
	}
	foreach my $field(@name_fields){
		foreach my $key(keys(%$answerdata)){
			my $answer_value = $answerdata->{$key};
			# If the field matches and the field isn't null we save the value in an array
			if ($key =~ /$field/i && $answer_value){
				$answer_value =~ s/"/'/g;
				push(@name,$answer_value);
			}
		}
	}
	return "\"".join(" ",@name)."\" <$from>";
}


sub html_adminForm{
	# $surveyid: survey id
	my ($this,$sectionid,$surveyid) = @_;	
	return $this->error("Missing inputs") if (!$sectionid && !$surveyid);
	my $table = "survey";
	my %survey = $this->{db}->fetch($table,$surveyid) if $surveyid;
	$sectionid = $survey{section_id} || $sectionid;
	$this->adminFormInit;
	my $formtype = $this->getFormType;
	my $module = $this->getModuleName;
	my $contact = $this->{contact};
	my $pkey = $surveyid || "_";
	my $out = "\n<form method=\"get\" action=\"".&relink({action=>undef,sid=>undef})."\">";
	$out .= "\n<h2>Part 1: $module Settings</h2>";
	$out .= "\n<p>Title of $module:".$this->helplink($table,"name")."<br><input type=\"text\" name=\"survey#$pkey#name\" size=\"60\" value=\"$survey{name}\"></p>";	
	$out .= "\n<p>$module Description:".$this->helplink($table,"description")."<br><textarea name=\"survey#$pkey#description\" rows=\"4\" cols=\"60\">$survey{description}</textarea></p>";
	$out .= "\n<p>Display '<sub><font color=\"red\" size=\"+1\">*</font></sub>' in required questions?".$this->helplink($table,"use_require");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_require\" value=\"Yes\"";
	$out .= $survey{use_require} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_require\" value=\"No\"";
	$out .= $survey{use_require} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want to group questions into sections?".$this->helplink($table,"use_group");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_group\" value=\"Yes\"";
	$out .= $survey{use_group} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_group\" value=\"No\"";
	$out .= $survey{use_group} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want to number every single question?".$this->helplink($table,"use_number");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_number\" value=\"Yes\"";
	$out .= $survey{use_number} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_number\" value=\"No\"";
	$out .= $survey{use_number} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want to use the generic style sheet?".$this->helplink($table,"use_generic_css");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_generic_css\" value=\"Yes\"";
	$out .= $survey{use_generic_css} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_generic_css\" value=\"No\"";
	$out .= $survey{use_generic_css} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want to save responses to the database?".$this->helplink($table,"save_result");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#save_result\" value=\"Yes\"";
	$out .= $survey{save_result} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#save_result\" value=\"No\"";
	$out .= $survey{save_result} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	
	$out .= "\n<p>Do you want to return a confirmation page after the user submits the form?".$this->helplink($table,"use_confirmation");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_confirmation\" value=\"Yes\"";
	$out .= $survey{use_confirmation} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_confirmation\" value=\"No\"";
	$out .= $survey{use_confirmation} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";	
	
	# Turn contact info section title on/off
	# New functionality: by default, show the title
	$out .= "\n<p>Do you want to show the contact information section title on the form?".$this->helplink($table,"show_contact_title");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#show_contact_title\" value=\"Yes\"";
	$out .= $survey{show_contact_title} !~ /no/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#show_contact_title\" value=\"No\"";
	$out .= $survey{save_result} =~ /no/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp;</p>";	
	
	$out .= "\n<p>Custom HTML header:".$this->helplink($table,"header_html")."<br>";
	$out .= "<textarea name=\"survey#$pkey#header_html\" rows=\"4\" COLS=\"60\">$survey{header_html}</textarea></p>";
	$out .= "\n<p>Custom HTML footer:".$this->helplink($table,"footer_html")."<br>";
	$out .= "<textarea name=\"survey#$pkey#footer_html\" rows=\"4\" cols=\"60\">$survey{footer_html}</textarea></p>";
	$out .= "\n<p>Thank you message:".$this->helplink($table,"thankyou_message")."<br>";
	$out .= "<textarea name=\"survey#$pkey#thankyou_message\" rows=\"4\" cols=\"60\">$survey{thankyou_message}</textarea></p>";	
	$out .= "\n<P>Organization Email Address:".$this->helplink($table,"from_email")."<br>";
	$out .= "<input type=\"text\" name=\"survey#$pkey#from_email\" size=\"60\" value=\"$survey{from_email}\"></p>";
	$out .= "\n<p>A list of email address to send $module results to:".$this->helplink($table,"email")."<br>";
	$out .= "<input type=\"text\" name=\"survey#$pkey#email\" size=\"60\" value=\"$survey{email}\"></p>";
	$out .= "\n<P>Redirect URL:".$this->helplink($table,"return_url")."<br>";
	$out .= "<input type=\"text\" name=\"survey#$pkey#return_url\" size=\"60\" value=\"$survey{return_url}\"></p>";
	$out .= "\n<P>Redirect message:".$this->helplink($table,"return_message")."<br>";
	$out .= "<textarea name=\"survey#$pkey#return_message\" rows=\"4\" cols=\"60\">$survey{return_message}</textarea></p>";
	
	if (!$surveyid){
		$out .= "\n<h2>Part 2: Contact Information</h2>";
		$out .= $this->html_wizardContactInfo($contact);
	}
	#$out .= "\n<p>Do you want to add custom contact information?<br>";
	#$out .= "<input type=\"radio\" name=\"customq\" value=\"Yes\">Yes&nbsp;&nbsp;";
	#$out .= "<input type=\"radio\" name=\"customq\" value=\"No\">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want Contact Information displayed at the beginning or end of the $module?".$this->helplink($table,"contactinfo_pos")."<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#contactinfo_pos\" value=\"beginning\"";
	$out .= $survey{contactinfo_pos} !~ /end/i ? " checked" : "";
	$out .= ">Beginning&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#contactinfo_pos\" value=\"end\"";
	$out .= $survey{contactinfo_pos} =~ /end/i ? " checked" : "";
	$out .= ">End&nbsp;&nbsp</p>";
	if ($surveyid){
		$out .= "\n<input type=\"hidden\" name=\"survey#$pkey#id\" value=\"$surveyid\">";
	}
	$out .= "\n<input type=\"hidden\" name=\"survey#$pkey#section_id\" value=\"$sectionid\">";	
	$out .= "\n<input type=\"hidden\" name=\"survey#$pkey#type\" value=\"$formtype\">";
	$out .= "\n<input type=\"hidden\" name=\"action\" value=\"submit_form\">";
	$out .= "\n<input type=\"submit\" value=\"Submit\">&nbsp;&nbsp;&nbsp;&nbsp;";
	$out .= "\n<input type=\"reset\" value=\"Reset\">";
	$out .= "\n</form>";
    	return $out;
}

sub html_surveyFinalScreen{
	# $survey: survey hash
	my ($this,$survey,$contact,$qdata) = @_;
	my ($out,$url,$urlmessage);
	$out .= "\n<div class=\"$this->{div_class}\">";
	$out .= "\n<p>";
	
	if ( $survey->{use_confirmation} =~ /yes/i ) {
		$out .= $this->buildConfirmationPage($survey,$contact,$qdata);		
	}
	else {
		$out .= $survey->{thankyou_message};
	}
	
	$out .= "</p>";
	return $out if !$survey->{return_url};
	$url = $survey->{return_url} ? $survey->{return_url} : &relink();
	# append http://
	if ($url !~ /^\/.*/){
		$url = $url =~ /^http:\/\// ? $url : "http://".$url;
	}
	$urlmessage = $survey->{return_message} ? $survey->{return_message} : $url;
	$out .= "<a href=\"$url\">$urlmessage</a>";
	$out .= "\n";
	return $out;
}

sub buildConfirmationPage() {
	my ($this,$survey,$contact,$qdata) = @_;
	my (@info,$body);
	# Survey info
	push (@info,$survey->{thankyou_message});
	foreach my $contactid(sort {$a <=> $b} keys(%$contact)) {
		push(@info,$contact->{$contactid});		
	}
	# save registrant responses for email purposes
	my $questions = $this->fetchSurveyQuestion($survey->{id});
	my $prev_section = 0;
	foreach my $question(@$questions) {
		my $sec = $question->{survey_section_id} || 0;
		if ($sec != $prev_section){
			my %section = $this->{db}->fetch("survey_section",$sec);
			push(@info,"<br>$section{label}<br>");
			$prev_section = $section{id};
		}
		push(@info,$qdata->{$question->{id}});
	}
	$body = join("<br>",@info);
	return $body;
}

1;
