package Modules::SiteBuilder;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#   SiteBuilder is a tool that lets members of the public create their 
#   own websites.
#
#   - allows for MySite basic, regular, advanced site types
#   - uses a shopping cart and e-commerce system to pay for sites
#
#----------------------------------------------------------------------------
#
#   TODO
#
#----------------------------------------------------------------------------

=pod


wiz1 - commercial

    1. Site Type
    2. Site Plan
    3. Theme
    4. Identification
    5. Payment

wiz2 - non-commercial

    1. Site Plan
    2. Theme
    3. Identification





=cut


use strict;
use MIME::Base64;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use ExSite::Input;
use ExSite::Section;
use Modules::BaseDCD;
use Modules::CMS;

use vars qw(@ISA $SB);
@ISA = qw(Modules::BaseDCD);

# write : redirect to user control panel, or prompt for login

sub write {
    my ($this,$options) = @_;
    # save object in a global so that we can call it from wizard
    $SB = $this;
    if ($config{SiteBuilder}{mode} eq "ecommerce") {
	return $this->newsite_ecom;
    }
    else {
	return $this->newsite_simple;
    }
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "Site Builder";
    }
    elsif (/ControlPanel/) {
	return \&write;
    }
    elsif (/isRestricted/) {
	return $config{SiteBuilder}{isRestricted};
    }
    elsif (/ModuleInfo/) {
	return "Site Builder is a tool for creating websites.  It can be
used by system administrators, or by members of the public.  Websites
created by the public can optionally have fees associated with them,
and SiteBuilder will automatically invoke a shopping cart/e-commerce 
tool to collect payment.";
    }
    return undef;
}

# simplified version of CMS::list_template

sub list_template {
    my ($this,$id) = @_;
    my ($out,@ptempl,@options);
    my $site = $this->mysite()->{section_id};
    $out .= "<p>Select a template:</p>\n<blockquote>\n";

    # a little javascript to control the target window

    $out .= "<script LANGUAGE=\"javascript\">
<!--
function NewWindow(target){
	var NewWin = window.open(target,'','width=800,height=500,scrollbars,resizable')
	}
-->
</script>\n";

    my $db = $share{DB};

    # fetch public templates
    @ptempl = $db->fetch_match("page",{type=>"template",access=>"public"});
    # ... plus the member-only templates from the current section
    push @ptempl, $db->fetch_match("page",{type=>"template",access=>"members",section_id=>$site});
    my $sec = $db->fetch("section",$site);
    if ($sec && $sec->{parent_id}) {
	# ... plus the member-only templates from the parent section
	push @ptempl, $db->fetch_match("page",{type=>"template",access=>"members",section_id=>$sec->{parent_id}});
    }
    # ...plus the private templates from this section
    push @ptempl, $db->fetch_match("page",{type=>"template",access=>"administrators",section_id=>$site});

    if (@ptempl > 0) {
	foreach my $pt (sort { lc($a->{title}) cmp lc($b->{title}) } @ptempl) {
	    $out .= "<p><dl><dt><input type=\"radio\" name=\"template\" value=\"$pt->{page_id}\"";
	    $out .= ($pt->{page_id} == $id) ? " checked" : "";
	    $out .= "> <b>$pt->{title}</b></dt>\n";
	    my $preview_url = "javascript:NewWindow('$config{server}{CGIpath}/page.cgi?_id=$pt->{page_id}')";
	    $out .= "<dd>$pt->{description}  [<A HREF=\"$preview_url\">preview</A>]</dd></dl>\n";
	}
    }
    else {
	$out .= "<p>Sorry, there don't seem to be any templates to choose from.</p>\n";
    }
    $out .= "</blockquote>\n";
    return $out;
}

sub publish {
    my ($this,$page) = @_;
    my $db = $share{DB};
    my $site = $this->mysite;
    my $out;
    if ($page) {
	my $p = $db->fetch("page",$page);
	if ($p->{section_id} == $site->{section_id}) {
	    # spawn the publish program, so it runs setuid
	    $ENV{QUERY_STRING} = "page=$page";
	    $out .= `$config{CGIroot}/publish.cgi`;
	}
	else {
	    $out .= "<p class=\"error\">Permission denied: invalid page.</p>\n";
	}
    }
    else {
	# spawn the publish program, so it runs setuid
	$ENV{QUERY_STRING} = "section=$site->{section_id}";
	$out .= `$config{server}{CGIroot}/publish.cgi`;
    }
    return $out;
}

# -------- site builder wizard
#
# 1. choose site type
# 2. choose site plan (sitemap)
# 3. choose graphic design
# 4. identification
# 5. checkout
# (6) payment -> handled by Ecommerce plug-in
# (7) receipt/instructions -> handled by newsite_done() method

sub newsite_ecom {
    my $this = shift;
    if (exists $this->{input}{trnApproved}) {
	return $this->confirm_build_site;
    }
    my $action = &relink();
    my $wiz = new ExSite::Wizard(title=>"New Website",
				 (action=>$action));
    $wiz->add_step("Web Site Type",\&wiz_site_type,\&do_wiz_site_type);
    $wiz->add_step("Web Site Plan",\&wiz_site_plan,\&do_wiz_site_plan);
    $wiz->add_step("Pick a Theme",\&wiz_theme,\&do_wiz_theme);
    $wiz->add_step("Identification",\&wiz_id,\&do_wiz_id);
    $wiz->add_step("Payment",\&wiz_final_ecom);
    return $wiz->go();
}

sub newsite_simple {
    my $this = shift;
    my $action = &relink();
    my $wiz = new ExSite::Wizard(title=>"New Website",
				 (action=>$action));
    $wiz->add_step("Web Site Plan",\&wiz_site_plan_simple,\&do_wiz_site_plan);
    $wiz->add_step("Pick a Theme",\&wiz_theme,\&do_wiz_theme);
    $wiz->add_step("Identification",\&wiz_id,\&do_wiz_id);
    $wiz->add_step("Instructions",\&wiz_final_simple);
    return $wiz->go();
}

sub wiz_site_type {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my ($out,$price,$nopt);

    $out = "<p>Select the type of site you want from the available site
types, below.</p>

<table width=100% cellpadding=10>\n";

    if ($config{SiteBuilder}{basic}{on}) {
	$nopt++;
	$price = $config{SiteBuilder}{basic}{price} ?
	    "\$$config{SiteBuilder}{basic}{price}/$config{SiteBuilder}{price_term}" :
	    "FREE";
	$out .= "<tr>
  <td><input type=radio name=\"site_type\" value=\"basic\">&nbsp;$price</td>
  <td><p><b>Basic Brochure Website</b><br>
      Limited regular webpages.  You can add and remove pages, edit your 
      webpage content, choose new site designs, and republish at any time.</p>
  </td>
</tr>\n";
    }

    if ($config{SiteBuilder}{regular}{on}) {
	$nopt++;
	$price = $config{SiteBuilder}{regular}{price} ?
	    "\$$config{SiteBuilder}{regular}{price}/$config{SiteBuilder}{price_term}" :
	    "FREE";
	$out .= "<tr>
  <td><input type=radio name=\"site_type\" value=\"regular\">&nbsp;$price</td>
  <td><p><b>Deluxe Brochure Website</b><br>
      More webpages, and you can upload your own images and documents.
      You have the option of setting up your own private web domain, and 
      getting your own custom graphic design installed.</p>
  </td>
</tr>\n";
    }

    if ($config{SiteBuilder}{advanced}{on}) {
	$nopt++;
	$price = $config{SiteBuilder}{advanced}{price} ?
	    "\$$config{SiteBuilder}{advanced}{price}/$config{SiteBuilder}{price_term}" :
	    "FREE";
	$out .= "<tr>
  <td><input type=radio name=\"site_type\" value=\"advanced\">&nbsp;$price</td>
  <td><p><b>Power Website</b><br>
      Unlimited web pages, graphics, and documents, plus a full suite of
      web applications and syndicated content, for a fully-loaded
      dynamic website.</p>
  </td>
</tr>\n";
    }

    $out .= "</table>\n";

    if ($nopt > 1) {
	$out .= "<p>Note:  It is possible to upgrade your site type later, 
if you change your mind.</p>\n";
    }
    elsif ($nopt == 1) {
	$out .= "<p>Note:  There is only one configuration available at this time.</p>\n";
    }
    else {
	$out .= $this->error("SiteBuilder: no web site configurations enabled");
    }

    return $out;
}

sub do_wiz_site_type {
    my $this = shift;
    my $in = shift;
    my @err;
    if ($in->{"site_type"} !~ /^(basic|regular|advanced)/) {
	push @err, "Please select a site type!";
    }
    return @err;
}

sub wiz_site_plan {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my $out;

    $out .= "<p>You can choose from a variety of site plans.  These will
give you preconfigured pages and services on your site, reducing the amount
of work you have to do when setting things up.</p>
<p><span class=help><img src=$config{server}{HTMLpath}/_Modules/SiteBuilder/check.gif><b>Recommended Plans</b></span> are well-suited to the type
of site you chose on the previous screen.  <span class=error><b>Other Plans</b></span>
may not work as well, because your site may not include all the features you
need for these plans, or the plan may not implement all the features you have
selected.  (However you can always upgrade or manually add the missing 
pieces later.)</p>
<style>
.help img { background-color:#009900; }
.error img { background-color:#ffffff; }
</style>
\n";
    $out .= $SB->select_site_plan($in->{site_type});
    return $out;
}

sub do_wiz_site_plan {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my @err = &do_wiz_site_type($this,$in);
    if (! $in->{website_template}) {
	push @err, "Please select a web site plan!";
    }
    return @err;
}

sub wiz_site_plan_simple {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my $out = "<p>You can choose from a variety of site plans.  These will
give you preconfigured pages and services on your site, reducing the amount
of work you have to do when setting things up.</p>\n";
    $out .= $db->input_html(type=>"hidden", name=>"site_type", value=>"basic");
    $out .= $SB->select_site_plan($in->{site_type});
    return $out;
}

sub wiz_theme {
    my $this = shift;
    my $in = shift;
    my $out;
    my $db = $share{DB};
    $out .= "<p>Now you can select the visual theme of your site, from the
graphic designs provided below.  Use the <b>preview</b> links to view 
samples of each design.</p>\n";
    require Modules::CMS;
    my $cms = new Modules::CMS();
    $out .= $cms->list_template("page_template",undef,2);
    return $out;
}

sub do_wiz_theme {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my @err = &do_wiz_site_plan($this,$in);
    if (! $in->{page_template}) {
	push @err, "Please select a graphic design template!";
    }
    return @err;
}

sub wiz_id {
    my $this = shift;
    return $SB->identity_form;
}

sub do_wiz_id {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my @err = &do_wiz_theme($this,$in);
    if ($in->{"email"} !~ /^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	push @err, "Invalid e-mail address.";
    }
    if ($in->{"sitename"} !~ /^[A-Za-z0-9_\-]{3,32}$/) {
	push @err, "Invalid website ID.  The site ID must be between
3 and 32 characters long, containing only letters, numerals, underscores, 
and hyphens.";
    }
    if (! $in->{"title"}) {
	# not an error
	$in->{title} = $in->{sitename};
    }

    if ($in->{"sitename"}) {
	my @match = $db->fetch_match("section",{publish_to=>$in->{sitename}});
	if (@match > 0) {
	    push @err, "Sorry, a website with that ID already exists in the system.  Please try another.";
	}
	else {
	    @match = $db->fetch_match("member",{login=>$in->{sitename}});
	    if (@match > 0) {
		push @err, "Sorry, a website with that ID already exists in the system.  Please try another.";
	    }
	}
    }

    return @err;
}

sub wiz_final_ecom {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my $out;

    # build preliminary site (not yet activated)

    my @err = $SB->build_site;
    if (@err > 0) {
	return "<p class=error>\n".join("<br>\n",@err)."</p>\n";
    }

    # read pricing
    #&read_exsite_conf("MySite.conf","MySite");
    
    # manual pricing for now

    $out = "<p>Thank you, we have collected all the information we need to 
build your website.</p>\n";

    my %ecopt;

    my $type = $in->{site_type} || "basic";
    $ecopt{amount} = $config{SiteBuilder}{$type}{price}; 

    my @name = split / /,$in->{name};

    my (undef,undef,undef,$mday,$mon,$year,undef,undef,undef) = localtime(time);
    $mon++;
    $year += 1900;

    my $invoice = sprintf "%4d%02d%02d-%d",$year,$mon,$mday,$SB->{section_id};

    $ecopt{name} = $in->{name};
    $ecopt{first} = $name[0];
    $ecopt{last} = $name[-1];
    $ecopt{invoice_number} = $invoice,
    $ecopt{section_id} = $SB->{section_id};
    $ecopt{member_id} = $SB->{member_id};
    $ecopt{return_url} = &relink;

    my $options = &EncodeHash(%ecopt);

    $out .= "<table><tr><th>Purchase</th><th>Amount</th></tr>
<tr><td>".ucfirst($in->{site_type})." Web Site</td><td>\$$ecopt{amount}</td></tr>
<tr><td>TOTAL</td><td>\$$ecopt{amount}</td></tr>
</table>\n";

    if (! $ecopt{amount} || $ecopt{amount} <= 0) {
	my $url = $SB->link(trnApproved=>1);
	$out .= "<p>The web site you have chosen is free.  
<a href=\"$url\">Click here</a> to proceed to your confirmation page.</p>\n";
    }
    else {
	$out .= "<p>test <!--&Ecommerce($options)--></p>";
    }

    return $out;
}

sub wiz_final_simple {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my $out;

    # build preliminary site (not yet activated)

    my @err = $SB->build_site;
    if (@err > 0) {
	return "<p class=error>\n".join("<br>\n",@err)."</p>\n";
    }
    return $SB->confirm_build_site;
}

sub select_site_plan {
    my ($this,$type) = @_;
    my ($out,$label,$class);
    my $db = $share{DB};
    my @wtempl = $db->fetch_match("section","type=\"website template\" and (parent_id=0 or parent_id is NULL)");
    if (@wtempl > 0) {
	$out .= "<p><dl>";
	foreach my $wt (@wtempl) {
	    if ($type) { 
		$class = ($wt->{note} =~ /$type/) ? "help" : "error";
	    }
	    else {
		$class = "SiteBuilder";
	    }
	    $label = "<span class=\"$class\"><img src=$config{server}{HTMLpath}/_Modules/SiteBuilder/check.gif> <b>$wt->{title}</b></span></dt>\n<dd><span class=\"$class\">$wt->{description}</span></dd>\n";
	    $out .= "<dt>".
		$db->input_html(type=>"radio",name=>"website_template",
				value=>$wt->{section_id},
				label=>$label);
	}
	$out .= "</dl></p>\n";
    }
    
    return $out;
}

sub identity_form {
    my $this = shift;
    my $db = $share{DB};
    my $out = "<p>Now we need to collect some basic identification info to
help us and you manage your site.</p>
<ol>
<li><p>Your full name:<br>\n";
    $out .= $db->input_html(type=>"text", name=>"name",size=>60);
    $out .= "</p></li>
<li><p>We need a working e-mail address to communicate with you on the status 
of your web site, and send password and login instructions.  It is kept 
strictly confidential.  ";
    if ($share{Page} && $config{SiteBuilder}{privacy_url}) {
	$out .= "(<a href=\"$config{SiteBuilder}{privacy_url}\" target=\"other\">View privacy policy</a>.)";
    }
    $out .= "</p>\n<p>Email address:<br>\n";
    $out .= $db->input_html(type=>"text", name=>"email",size=>60);

    $out .= "</p></li>
<li><p>Next, give us the title of your web site.  Typically, this is the
full name of your organization.</p>
<p>Full organization name OR website title:<br>\n";
    $out .= $db->input_html(type=>"text", name=>"title", size=>60);

    $out .= "</p></li>
<li><p>You can optionally provide a longer web site description.
Some graphic designs may display this information as a subtitle, while 
others will simply use it as a hidden description for search engines.</p>
<p>Description/subtitle (optional):<br>\n";
    $out .= $db->input_html(type=>"text", name=>"description", size=>60);

    $out .= "</p></li>
<li><p>You also need to define a short web site ID.  
This must be a short text string with no spaces or punctuation.
It will be used as part of your web site URL, and also as a login ID for 
you to access your site and
edit it.  You may use up to 32 letters, numerals, underscores, and
hyphens.</p>
<p>Short web site ID:<br>\n";
    $out .= $db->input_html(type=>"text", name=>"sitename");
    $out .= "</p></li></ol>\n";
    return $out;
}

sub build_site {
    my $this = shift;
    my $in = $this->{input};
    my $db = $share{DB};

    # we need to create:
    # 1) a user (but only access level 0 for now)
    # 2) a site (section)
    # 3) pages and content objects

    my %section = (title=>$in->{title},
		   type=>"native website",
		   description=>$in->{description},
		   publish_to=>$in->{sitename},
		   note=>$in->{site_type},
		   template_id=>$in->{page_template});

    my $s = new ExSite::Section;
    my $sid = $s->install(\%section,$in->{website_template});
    if (! $sid) {
	# bad install
	return ("failed to install site \"$section{title}\"");
    }

    my $pwd = &randtext(7,'abcdefghijklmnopqrstuvwxyz0123456789');
    my %user = (login=>$in->{sitename},
		password=>$pwd,
		name=>$in->{name},
		email=>$in->{email},
		section_id=>$sid,
		access=>0);

    # add user
    my $uid = $db->insert("member",\%user);
    if (! $uid) {
	# undo section insert
	$db->delete("member",$uid);
	return ("failed to create new user account!");
    }

    # add full-priv administrator ticket
    my $tid = $db->insert("member_site_link",{section_id=>$sid,
					      member_id=>$uid,
					      priveleges=>"editor; designer; administrator"});
    if (! $tid) {
	return ("failed to create new administrator key!");
    }

    $this->{section_id} = $sid;
    $this->{member_id} = $uid;
    return ();
}

sub confirm_build_site {
    my ($this) = @_;
    my $in = $this->{input};
    my $db = $share{DB};
    my $out;

    #### NEED SECURE VALIDATION !

    #### $this->{(member|section)_id} are set by build_site()
    #### How are these set via ecom method?

    my %member = $db->fetch("member",$this->{member_id});
    my %section = $db->fetch("section",$this->{section_id});
    my @key = $db->fetch_match("member_site_link",
			       {member_id=>$this->{member_id},
				section_id=>$this->{section_id}});
			       
    # validate inputs, and activate the user

    if ($member{login} && 
	($member{access} == 0) && 
	($member{section_id} == $this->{section_id}) &&
	(@key > 0)) {
	$member{access} = 2;
	$db->update("member",\%member);
    }
    else {
	return "<p class=error>Invalid member data found while trying to activate new web site.</p>\n";
    }

    # pre-publish the site - need to log the member in temporarily to gain
    # the priveleges to do this, however.

    #if ($db->do_login_temp($member{login})) { # fails if logged in to another acct
    if ($db->do_login_temp(\%member)) {
	# pre-publish the site
	$ENV{QUERY_STRING} = "section=$section{section_id}";
	my $pub .= `$config{server}{CGIroot}/publish.cgi`;
    }
    else {
	$out .= $this->error("Failed to pre-publish new website due to login failure.");
    }

    # parent site name

    my $parent_site = $config{SiteBuilder}{name} || $config{site}{name};
    my $login_url = $config{SiteBuilder}{login_url} || "$config{server}{server}$config{server}{HTMLpath}";

    # send email notification to user 

    my $instructions = &get_file("$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/SiteBuilder/instructions.txt");

    require ExSite::Mail;

    my $msg = "This message has been sent to you because someone created
a new website at $parent_site and provided 
this email address for contact information.

To access the administrator functions for this website, direct your
browser to $login_url
$instructions

Your login information:

Login ID: $member{login}
Password: $member{password}

Enjoy your new website!
";
    # mail to user
    &ExSite::Mail::SendMail($in->{email},$config{site}{administrator},
			    "Your $parent_site login information",
			    $msg);
    # notify administrator
    &ExSite::Mail::SendMail($config{site}{administrator},$config{site}{administrator},
			    "$parent_site account created",
			    "The website $section{title} was created by $member{email}");

    $out .= "<h1>Your web site is ready</h1>
<p>Your website has been created, and can be viewed in its preliminary state at
<a href=\"$config{server}{server}$config{server}{HTMLpath}/$member{login}\" target=\"other\">$config{server}{server}$config{server}{HTMLpath}/$member{login}</a>.</p>
<p>Instructions have been e-mailed to <tt>$member{email}</tt>, explaining
how to log in to the administrator section of $parent_site to make 
changes and updates to your website.</p>
\n";

#<div style=\"overflow:auto;height:100;width=400;border:1px solid black;\">
#$pub
#</div>
    return $out;
}			  

1;
