package Modules::Survey;
use strict;	
use ExSite::Config;
use ExSite::Misc;
use ExSite::Input;
use ExSite::Mail;
use Modules::BaseDCD;
use ExSite::ReportBuilder;
use vars qw(@ISA);
push(@ISA,"Modules::BaseDCD");


#---------------------------------------------------------------------
sub new {
	my $this = shift;
	my ($obj,$class);
	$obj = {};
	$class = ref($this) || $this;
	bless $obj, $class;
	$class =~ /^(.+?::)+?(.+)$/;
	$obj->{name} = "survey";
	$obj->{form_type} = "survey";
	$obj->{class_name} = $2;
	$obj->{db} = $share{DB};
	$obj->{cgi} = $config{server}{server}.$config{server}{CGIpath};
	$obj->{html} = $config{server}{server}.$config{server}{HTMLpath};
	$obj->{survey_script} = "survey.cgi";
	$obj->{dbadm_script} = "webdb.cgi";
	$obj->{div_class} = "survey";
	$obj->{form_error} = [];
	$obj->read;  # inherits BaseDCD::read()
	$obj->read_conf;
	$obj->{registrant_list_excel_filename} = $config{$obj->{class_name}}{config}{registrant_list}{excel_filename} || "participantlist.xls";
	$obj->{registrant_list_additional_column} = $config{$obj->{class_name}}{config}{registrant_list}{number_of_additional_column};
	$obj->moduleInit;
	return $obj;
}
sub moduleInit{
	my ($this) = @_;	
	return;
}
sub ioctl{
	my $this = shift;
	$_ = shift;   # $_ is the ioctl request
	if (/ModuleName/){
		return $this->getModuleName."s";
	}elsif (/ControlPanel/){
		return \&control_panel;
	}elsif (/ModuleInfo/){
        	return $this->getModuleName." tools allow users to create surveys, modify surveys, remove surveys and view reports.";
	}elsif (/DynContentParam/){
		return $this->getDynContentParameter(shift);
	}elsif (/isRestricted/){
		return $config{Survey}{isRestricted};
	}elsif (/isService/){
		return $config{Survey}{isService};
	}
}
sub write{
	my ($this,$options) = @_;
	my (%opt,$action);
	%opt = &DecodeString($options);
	$action = $this->{input}{action} || $opt{action};
	# create forms
	if ($action eq "do"){
		my $surveyid = $this->{input}{sid} || $opt{sid};
		return $this->surveyForm({surveyid=>$surveyid});
	# process form data
	}elsif ($action eq "form_submit"){
		my $surveyid = $this->{input}{sid} || $opt{sid};
		return $this->validateForm;
	# public survey list
	}else{
		return $this->surveyList($share{Page}->get("section")->{section_id});
	}
	return undef;
}
#---------------------------------------------------------------------
sub getDynContentParameter{
	my ($this,$sectionid) = @_;
	return if !$sectionid;
	my (@surveys,$out);
	@surveys = $this->{db}->fetch_match("survey",{section_id=>$sectionid,type=>"survey"});
	$out .= "\n<table>";
	$out .= "\n<tr><td>";
	$out .= "<b>Select a survey:</b>";
	$out .= "</td><td>";
	$out .= "<select name=\"param\">";
	$out .= "<option value=\"\">&lt;Default&gt;</option>";
	foreach my $survey(@surveys){
		$out .= "<option value=\"sid=$survey->{id}&action=do\">$survey->{name}</option>";
	}
	$out .= "</select>";
	$out .= "</td></tr>";
	$out .= "\n</table>";
	return $out;
}

# control panel
sub control_panel{
	my $this = shift;
	my $out;	
	my $admin_level = $config{$this->{class_name}}{admin_level} || $config{auth}{root_auth_level} - 1;
	my $sectionid = $this->{input}{section_id};
	# admin access only
	if ($this->{db}->authorize < $admin_level){
		$out = "<h3 class=\"error\">Permission denied</h3><p>You do not have sufficient system priveleges to use this feature.</p>\n";
	}else {
		my $action = $this->{input}{action};
		if ($action eq "view_dds_summary" && $this->{input}{dds_id} && $this->{input}{sid}){
			$out .= $this->viewDDSSummaryReport($this->{input}{dds_id},$this->{input}{sid});
		}elsif ($action =~ /^(insert|update)$/i){
			$out .= $this->exsiteFormPost();
		# export registrant info and responses to excel spreadsheet
		}elsif ($action eq "excel_user"){
			$out .= $this->exportRegistrantList($this->{input}{sid});
		# view registrant/participant list
		}elsif ($action eq "view_reg_list"){
			$out .= $this->viewRegistrantList($this->{input}{sid});
		# edit a child recrod
		}elsif ($action eq "edit_form_record"){
			$out .= $this->adminChildRecordForm($this->{input}{tab},$this->{input}{id},$this->{input}{ptab},$this->{input}{pid});
		# add new child records to survey
		}elsif ($action eq "new_form_record"){
			$out .= $this->adminChildRecordForm($this->{input}{tab},undef,$this->{input}{ptab},$this->{input}{pid});
		# edit survey info
		}elsif ($action eq "edit_mainform"){
			my $surveyid = $this->{input}{sid};
			$out .= $this->adminForm(undef,$surveyid);
		# main survey edit form
		}elsif ($action eq "main_form"){
			my $surveyid = $this->{input}{sid};
			$out .= $this->adminFormMain($surveyid);
		# process form data
		}elsif ($action eq "submit_form"){
			$out .= $this->adminFormProcess("survey",$this->{input});
		# create edit a survey
		}elsif ($action eq "new_mainform"){
			my $sectionid = $this->{input}{section_id};
			$out .= $this->adminForm($sectionid);
		# view individual's survey responses
		}elsif ($action eq "view_user"){
			my $surveyid = $this->{input}{sid};
			$out = $this->viewSingleSurveyResponse($surveyid,$this->{input}{uid});
		# admin survey list
		}elsif ($action eq "admin_list"){
			$out = $this->showAllSurveyList;
		# view reports
		}elsif ($action eq "view_report"){
			my $surveyid = $this->{input}{sid};
			$out = $this->showReportList($surveyid);
		}elsif ($action eq "view_summary"){
			my $surveyid = $this->{input}{sid};
			$out = $this->viewSurveyReport($surveyid);
		# view question summaries
		}elsif ($action eq "view_question"){
			my ($questionid,$surveyid);
			$questionid = $this->{input}{qid};
			$surveyid = $this->{input}{sid};
			$out = $this->viewQuestionReport($questionid,$surveyid);
		# start survey wizard
		}elsif ($action eq "wizard"){
			my ($sectionid);
			$sectionid = $this->{input}{section_id};
			$out = $this->surveyWizard($sectionid);
		# data filter
		}elsif ($action eq "dds"){
			my ($sectionid);
			$sectionid = $this->{input}{section_id};
			$out = $this->surveyDDS($sectionid);
		# create a new data filter
		}elsif ($action eq "dds_action"){
			$out = $this->newDDSReport;
		# delete a data filter
		}elsif ($action eq "rec_delete"){
			$out = $this->deleteRecord($this->{input}{tab},$this->{input}{id},$this->{input}{reply});
		}elsif ($action){
			$out = $this->write;
		}elsif ($sectionid){
			my %section = $this->{db}->fetch("section",$sectionid);
			$out = $this->adminSurveyList(\%section);
		# display admin survey list by default
		}else{
			$out = $this->set_section_id();
		}
	}
	return $out;
}
# delete a record
sub deleteRecord{
	# $tab: table
	# $id: id
	# $reply: reply url
	my ($this,$tab,$id,$reply) = @_;
	return $this->error("Missing record table.") if !$tab;
	return $this->error("Missing record id.") if !$id;
	return $this->denied() if !$this->{db}->approve("delete",$tab,$id);
	return $this->denied() if !$this->validateAccess($tab,$id);
	# call &relink if no reply url is specified
	$reply = $reply ?  $reply : &relink();
	my $del = $this->{db}->find_descendants($tab,$id);
	foreach my $deltab(keys(%$del)){
		foreach my $delrow(@{$del->{$deltab}}){
			# delete the record
			$this->{db}->delete_key($deltab,$delrow);                        
		}
	}
	$this->redirect($reply);
	return;
}
sub redirect{
	my ($this,$reply) = @_;
	my $p = $share{Page} || $share{PageBuf};
	if ($p){
		$p->redirect($reply);
	}else{
		print "location: $reply\n\n";
	}
	return;
}
# admin survey list
sub showAllSurveyList{
	my ($this) = @_;
	my (@sites,$out);
	# get websites
	@sites = $this->{db}->my_real_sites;
	# diplay survey lists for each of the website separately
	foreach my $s (@sites){
		$out .= $this->adminSurveyList($s);
	}
	return $out;
}
# fetch surveys
sub fetchSurvey{
	my ($this,$sectionid) = @_;
	my @list = $this->{db}->fetch_match("survey",{section_id=>$sectionid,type=>"survey"});
	return \@list;
}
# admin survey list bootn menu links
sub getAdminSurveyListBottomMenuLink{
	my ($this,$section) = @_;
	my ($module,@commands);
	$module = $this->getModuleName;
	@commands = (
	{label=>"Add new $module form to $section->{title}",url=>$this->myRelink({action=>"new_mainform",section_id=>$section->{section_id}}),img=>"new"},
	);
	return \@commands;	
}
# get icons
sub getIcon{
	my ($this,$name,$label,$attr) = @_;
	my (%icons,%default_attribute,$img);
	%default_attribute = (
	height=>16,
	width=>16,
	border=>0,
	align=>"middle",
	hspace=>3,
	);
	%icons = (
	new=>{img=>"new16.gif",label=>"Add new record"},
	edit=>{img=>"conf16.gif",label=>"Edit"},
	delete=>{img=>"del16.gif",label=>"Delete"},
	do=>{img=>"do16.gif",label=>"Do"},
	view=>{img=>"report16.gif",label=>"View"},
	new_report=>{img=>"newreport16.gif",label=>"Add new report"},
	excel=>{img=>"excel16.gif",label=>"Export to excel"},
	);
	$label = $label || "click me";
	$img = "$config{server}{HTMLpath}/_Modules/".ucfirst($this->{name})."/".$icons{$name}{img};
	return "[$label]" if (!exists($icons{$name}));
	my (@attribute,$out);
	foreach my $def_attr(keys(%default_attribute)){
		next if $attr->{$def_attr};
		push(@attribute,"$def_attr=\"$default_attribute{$def_attr}\"");
	}
	foreach my $atr(keys(%$attr)){
		push(@attribute,"$atr=\"$attr->{$atr}\"");
	}	
	$out = "<img src=\"$img\" alt=\"$label\" title=\"$label\"";
	$out .= @attribute > 0 ? " ".join(" ",@attribute).">" : "";
	return $out;
}
# logics for showing admin menu links
sub getAdminSurveyListRowLinkFix{
	my ($this,$survey) = @_;
	my (@links,@newlinks,%hide_link);
	
	# %hide_link define a list of link id's(check getAdminSurveyListRowLink for the id's) with a boolean id 
	# e.g. 	view=>1 <-- show the link with id view
	#	view=>0 <-- hide the link with id view
	%hide_link = (
		view=>$this->saveResponse($survey),
	);
	@links = @{$this->getAdminSurveyListRowLink($survey)};
	foreach my $link(@links){
		if (exists($hide_link{$link->{id}})){			
			push(@newlinks,$link) if ($hide_link{$link->{id}});
		}else{
			push(@newlinks,$link);		
		}	
	}
	return \@newlinks;
}

# admin table row links
sub getAdminSurveyListRowLink{
	# $survey: survey hash
	my ($this,$survey) = @_;
	my (@links,$module);
	$module = $this->getModuleName;
	@links = (
	{id=>"do",label=>"Do $module",url=>&relink({action=>"do",sid=>$survey->{id}}),img=>"do"},
	{id=>"view",label=>"View Reports",url=>&relink({action=>"view_report",sid=>$survey->{id}}),img=>"view"},
	{id=>"edit",label=>"Reconfigure",url=>$this->myRelink({action=>"main_form",sid=>$survey->{id}}),img=>"edit"},
	{id=>"delete",label=>"Delete",url=>"javascript:deleteRecord($survey->{id})",img=>"delete"},
	);
	return \@links;
}
# admin table headings
sub getAdminSurveyHeading{
	my ($this) = @_;
	my @cols = (
	{label=>$this->getModuleName." form",column=>"name"},
	{label=>"Description",column=>"description"},
	{label=>"Options",column=>"option_command"},
	);
	return \@cols;
}
# build reports using ExSite::ReportBuilder
sub reportbuilder{
	my ($this,$opt) = @_;
	my $rb = $this->{rb} || new ExSite::ReportBuilder();
	$rb->init(
		title=>$opt->{title},
		data=>$opt->{data},
		tools=>$opt->{tools},
		headers=>$opt->{headers},
		foot=>$opt->{foot},
		width=>$opt->{width},
	);
	return $rb->make();
}
# build admin survey list
sub adminSurveyList{
	# $section: section hash
	my ($this,$section) = @_;
	# missing section id
	return $this->error("Missing section id.") if (!$section->{section_id});
	my ($surveylist,$out);
	
	# get all surveys
	$surveylist = $this->fetchSurvey($section->{section_id});
	# insert option commands into each record
	for(my $i = 0; $i < @$surveylist; $i++){
		$surveylist->[$i]{option_command} = $this->html_adminSurveyRowLink($surveylist->[$i]);
	}
	my ($header,$data) = $this->getReportbuilderData($this->getAdminSurveyHeading(),$surveylist);
	my %report = (
		title=>$this->html_adminSurveyListTitle($section),
		tools=>$this->html_adminSurveyListBottomMenu($section),
		headers=>$header,
		data=>$data,
	);
	my $out .= "\n<div class=\"$this->{div_class}\">";
	$out .= $this->html_javascript("adminsurveylist");
	$out .= "\n".$this->reportbuilder(\%report);
	$out .= "\n</div>\n";
	$out .= "<br>";
	return $out;
}
sub getReportbuilderData(){
	my ($this,$header,$records) = @_;
	my @header;
	my @data;
	foreach my $head(@$header){			
		push(@header,$head->{label});
	}
	foreach my $rec (@$records){
		my @data_row;
		foreach my $head(@$header){
			push(@data_row,$rec->{$head->{column}});
		}
		push(@data,\@data_row);
	}
	return (\@header,\@data);
}

#===================================================================================
# build public survey list
sub surveyList{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	# missing section id
	return $this->error("Missing section id.") if (!$sectionid);
	my ($db,$surveylist,$out);
	$db = $this->{db};
	$surveylist = $this->fetchSurvey($sectionid);
	return $this->surveyForm({surveyid=>$surveylist->[0]{id}}) if @$surveylist == 1;
	foreach my $s (@$surveylist){
		# display surveys that the user has access to
		if ($this->accessSurvey){
			$out .= $this->html_surveyListRow($s);
		}
	}
	return $this->html_surveyList($out);
}
# build survey form
sub surveyForm{
	# $opt: option hash
	#	{surveyid}: survey id
	my ($this,$opt) = @_;
	# missing survey id
	return $this->error("Missing survey id.") if (!$opt->{surveyid});
	# disallow public to view member only survey
	return $this->error("Sorry, this is a member-only survey. :-(") if (!$this->accessSurvey);
	my ($survey,$contactinfo,$questions,$error);
	# fetch survey
	$survey = $this->{db}->fetch("survey",$opt->{surveyid});
	# run survey init routine
	$this->surveyInit($survey);
	# get form validation errors
	$error = $this->html_formError;	
	# build contact info form
	$contactinfo = $this->surveyContactInfo($survey->{id});
	# build survey question form
	$questions = $this->surveyQuestion($survey->{id});
	return $this->html_surveyForm($survey,$contactinfo,$questions,$error);
}
# build contact info form
sub surveyContactInfo{
	# $surveyid: survey id
	my ($this,$surveyid) = @_;
	my ($clist,$body);
	$clist = $this->fetchContactQuestion($surveyid);
	return if (@$clist < 1);
	foreach my $c (@$clist){
		# run contact info form init routine
		$this->contactinfoInit($c);
		$body .= $this->html_surveyContactInfoRow($c);
	}
	return $this->html_surveyContactInfo($body,$surveyid);
}
sub fetchContactQuestion{
	my ($this,$surveyid) = @_;
	my @questions = $this->{db}->fetch_match("survey_contact_question",{survey_id=>$surveyid},"id asc");
	return \@questions;
}
# fetch survey questions
sub fetchSurveyQuestion{
	my ($this,$surveyid) = @_;
	my @questions = $this->{db}->fetch_match("survey_question",{survey_id=>$surveyid},"survey_section_id asc, id asc");
	return \@questions;
}
# build survey question
sub surveyQuestion{
	# $surveyid: survey id
	my ($this,$surveyid) = @_;
	my ($qlist,@rdata,$prevID); #$prevTitle);
	# Store output question data
	my @questions_list;
	# fetch all questions
	$qlist = $this->fetchSurveyQuestion($surveyid);
	for(my $i = 0; $i < @$qlist; $i++){
		my $num = $i + 1;
		my $idata;
		my $group_flag = 0;
		# run survey question init routine
		$this->questionInit($qlist->[$i]);
		# check to see if we can group several questions into one form(for multiple checkboxes and radio button questions only)
		# we only group questions if the next question's answer choices are the same as the current one
		if($this->useAnswerStack() && $this->stackAnswerChoice($qlist,$i) && $qlist->[$i]{type} =~ /multiple|radio/i){
			# build forms for questions that can be grouped
			$idata .= $this->html_questionStack("open",$qlist->[$i],$num);
			$idata .= $this->html_questionStack("input",$qlist->[$i],$num);
			while($this->stackAnswerChoice($qlist,$i)){
				$i++;
				$num++;
				$idata .= $this->html_questionStack("input",$qlist->[$i],$num);				
			}
			$idata .= $this->html_questionStack("close",$qlist->[$i],$num);
			$group_flag = 1;
		}else{
			$idata .= $this->html_surveyQuestionInputData($qlist->[$i],$num);
			$idata = $this->html_questionInputLabel($qlist->[$i]{type},$this->html_questionInputNumber($qlist->[$i],$num),$this->getQuestionLabel($qlist->[$i]),$idata);
		}
		my $body;
		# display section label
		## if ($prevTitle ne $this->{section}{$qlist->[$i]{survey_section_id}}{label} && $qlist->[$i]{survey_section_id} > 0){
		if (($prevID != $qlist->[$i]{survey_section_id}) && ($qlist->[$i]{survey_section_id} > 0)){
			my $label = $this->{section}{$qlist->[$i]{survey_section_id}}{label};
			if ($this->{section}{$qlist->[$i]{survey_section_id}}{css_type} =~ /question/i){				
				$body .= $this->getQuestionRegularLabel($label);
				if (!$group_flag){
					$body .= "<br>";
				}
			}else{
				$body .= $this->getSectionLabel($label);
			}
		}
		if ($idata){
			push(@questions_list,{label=>$body,content=>$idata});
		}
		## $prevTitle = exists($this->{section}{$qlist->[$i]{survey_section_id}}) ? $this->{section}{$qlist->[$i]{survey_section_id}}{label} : "";
		$prevID = $qlist->[$i]{survey_section_id};
	}
	# Output every questions
	my $out;
	my $numbering = $this->{use_number} || 0;
	if ($numbering){
		$out = "<ol>";
	}
	for(my $i = 0; $i < @questions_list; $i++){
		my $qlist = $questions_list[$i];
		my $count = $i + 1;
		if ($numbering){
			$out .= "<li value=\"$count\">";
		}else{
			$out .= "<p>";
		}
		if (!$qlist->{label}){
			if ($qlist->{content} =~ /<table/s && $numbering){
				# The space is used to format the li content properly when there is a table present.
				$out .= "&nbsp;";
			}
		}else{
			$out .= $qlist->{label};
		}
		$out .= $qlist->{content};
		if ($numbering){
			$out .= "</li>";
		}else{
			$out .= "</p>";
		}
	}
	if ($numbering){
		$out .= "</ol>\n";
	}
	return $this->html_surveyQuestion($out);
}
# Check and see if we are stacking questions
sub useAnswerStack{
	my ($this) = @_;
	if ($config{$this->{class_name}}{disable_answer_stack}){
		return 0;
	}
	return 1;
}
# get survey section label
sub getSectionLabel{
	my ($this,$label) = @_;
	return "<p class=\"SurveySectionHead\">$label</p>";
}
sub addRegistrantData{
	my ($this) = @_;
	return ();
}
# sub validate form data
sub validateForm{
	my ($this) = @_;
	my $surveyid = $this->{input}{sid};
	# missing survey id
	return $this->error("Missing survey id.") if (!$surveyid);
	my ($clist,%clist,$qlist,%qlist,%contact,%question,%data);
	# run survey init routine
	$this->surveyInit;
	# fetch all questions
	$qlist = $this->{db}->fetch_match("survey_question",{survey_id=>$surveyid},"survey_section_id asc, id asc");
	# index questions
	foreach my $q (@$qlist){
		$qlist{$q->{id}} = $q;
		$this->{input}{$q->{id}} = "" if !exists($this->{input}{$q->{id}});
	}
	# fetch all contact info
	$clist = $this->{db}->fetch_match("survey_contact_question",{survey_id=>$surveyid});
	# index contact info
	foreach my $c (@$clist){
		$clist{$c->{id}} = $c;
	}
	# get form data
	%data = %{$this->{input}};
	foreach my $key (keys(%data)){
		next if $key =~ /^(sid|action|)$/i;
		# remove return characters
		$data{$key} = $this->stripChr($data{$key});
		# remove blank input data
		$data{$key} = $this->removeEmptyInput($data{$key});
		if ($key =~ /^cc/i){
			my ($keyid);
			$keyid = $key;
			$keyid =~ s/cc#(.*)/$1/g;
			# index input contact info data
			$contact{$keyid} = $data{$key};
		}elsif ($key =~ /^\d*$/){
			# index input question data
			$question{$key} = $data{$key};
		}
	}
	foreach my $c (keys(%clist)){
		# validate mandatory contact info questions
		if (!$this->validateContactInfo($contact{$c},$clist{$c})){
			push(@{$this->{form_error}},"Please enter your ".$clist{$c}->{label}.".");		
		}
	}
	foreach my $q (keys(%qlist)){
		# validate mandatory survey questions
		$this->validateQuestion($question{$q},$qlist{$q});
	}
	# preset form fields
	$this->presetFormField();
	if (@{$this->{form_error}} > 0){		
		# show survey form if there are missing inputs on the mandatory questions
		return $this->surveyForm({surveyid=>$surveyid});
	}else{
		my ($registrantid,$registrant,%survey,$saveresponse);
		# fetch survey data
		%survey = $this->{db}->fetch("survey",$surveyid);
		# create registrant data based on the form inputs
		$registrant = $this->getRegistrantData(\%contact,\%clist);
		$registrant->{survey_id} = $surveyid;
		%$registrant = (%$registrant,$this->addRegistrantData());		
		# insert registrant data into the database
		$saveresponse = $this->saveResponse(\%survey);
		$registrantid = $this->{db}->insert("survey_registrant",$registrant) if $saveresponse;		
		my @mailcontactdata;
		foreach my $c (sort {$a <=> $b} keys(%contact)){
			my %cdata;
			$clist{$c}{label} =~ /^(first|last|email)( name)?$/i;
			if (exists($registrant->{lc($1)})){
				$contact{$c} = $registrant->{lc($1)};
			}
			# gather contact info inputs from form data
			$cdata{survey_registrant_id} = $registrantid;
			$cdata{survey_contact_question_id} = $c;
			$cdata{value} = $contact{$c};
			# info to be emailed
			push(@mailcontactdata,{id=>$c,label=>$clist{$c}{label},value=>($contact{$c} ? $contact{$c} : "n/a")});
			# insert contact info responses into the database
			$this->{db}->insert("survey_contact_answer",\%cdata) if $saveresponse;
		}		
		my @mailquestiondata;
		foreach my $que (@$qlist){
			my %qdata;
			# question id
			my $questionid = $que->{id};
			# gather survey question inputs from form data
			$qdata{survey_registrant_id} = $registrantid;
			$qdata{survey_question_id} = $questionid;
			$qdata{value} = $question{$questionid};
			# info to be emailed
			push(@mailquestiondata,{id=>$questionid,label=>$qlist{$questionid}{label},value=>($question{$questionid} ? join(", ",split(/; /,$question{$questionid})) : "n/a")});
			# insert contact info responses into the database
			$this->{db}->insert("survey_answer",\%qdata) if $saveresponse;
		}
		$this->mailSurveyResult(\%survey,\@mailcontactdata,\@mailquestiondata);
			
		# build survey completion screen
		return $this->html_formFinalScreen(\%survey);
	}
	return;
}

# Data merge
sub merge{
	# $view: template with merge fields
	# $data: data hash contains data to be merged into the template
        my ($this,$view,$data) = @_;
	return if !$view || !$data;
	# Merge field format: <!--[[<name>]]-->
        $view =~ s/\<\!\-\-\[\[((\w+(\_| )?)+\w*)\]\]\-\-\>/$data->{$1}/g;
        return $view;
}
sub formatEmailData{
	# $edata: survey response in this data structure({id=><primary key value>,label=><question label>,value=><answer>})
	my ($this,$edata) = @_;
	my $template = $config{$this->{class_name}}{mail_answer_template};
	if ($template){
		$template =~ s/\\n/\n/g;
		return $this->merge($template,$edata);
	}
	return "$edata->{label}: $edata->{value}";
}

# email survey response
sub mailSurveyResult{
	my ($this,$survey,$contact,$qdata) = @_;
	my @email;
	# Survey info
	push(@email,"\n".&html_to_plaintext($this->getFormHeader($survey)));
	push(@email,"\n");
	# save registrant responses for email purposes
	push(@email,"$config{$this->{class_name}}{config}{contact_info}{label}:\n") if scalar(@$contact) > 0;
	foreach my $c_info(@$contact){
		push(@email,$this->formatEmailData($c_info));
	}
	# save registrant responses for email purposes
	push(@email,"\n") if scalar(@$qdata) > 0;
	push(@email,"Response:\n") if scalar(@$qdata) > 0;
	my $questions = $this->fetchSurveyQuestion($survey->{id});
	my $prev_section = 0;
	my %question_data;
	foreach my $qdat(@$qdata){
		$question_data{$qdat->{id}} = $this->formatEmailData($qdat);
	}
	foreach my $question(@$questions){
		my $sec = $question->{survey_section_id} || 0;
		if ($sec != $prev_section){
			my %section = $this->{db}->fetch("survey_section",$sec);
			push(@email,"\n\n$section{label}:\n");
			$prev_section = $section{id};
		}
		# save registrant responses for email purposes
		push(@email,$question_data{$question->{id}});
	}
	# Create a participant response data hash
	my %combine_data;
	foreach my $dat (@$contact,@$qdata){
		$combine_data{$dat->{label}} = $dat->{value};
	}
	#	
	# send survey repsonses to every email address on the emaillist(from the survey setup)	
	$this->mailResponse($survey,\@email,\%combine_data);
	return;
}
sub getNameSearchField{
	my ($this) = @_;
	return $config{$this->{class_name}}{name_search_field};
}
sub getEmailSearchField{
	my ($this) = @_;
	return $config{$this->{class_name}}{email_search_field};
}
sub mailFrom{
	my ($this,$survey,$answerdata) = @_;
	my $from = $survey->{from_email} || $config{$this->{class_name}}{config}{mail}{from};
	# Get a list of possible email field names from the conf file
	my $conf_email_search_field = $this->getEmailSearchField();
	if (!$config{$this->{class_name}}{use_participant_email} || !$conf_email_search_field){
		return $from;
	}
	# Exit loop flag - This flag is used to exit multiple for loops
	my $exit_loop = 0;
	my @search_fields;
	if ($conf_email_search_field =~ /,/){
		@search_fields = split(/,/,$conf_email_search_field);
	}else{
		@search_fields = ($conf_email_search_field);
	}
	# Search would stop when it finds the email that matches the search field listed
	# in the conf file
	foreach my $field(@search_fields){
		foreach my $key(keys(%$answerdata)){
			my $answer_value = $answerdata->{$key};
			if ($key =~ /$field/i && $this->validateEmail($answer_value)){
				$from = $answer_value;
				$exit_loop = 1;
			}
			if ($exit_loop){
				last;
			}
		}
		if ($exit_loop){
			last;
		}
	}
	# Search for registrant's name
	my @name;
	my @name_fields;
	# Get a list of possible name fields from the conf file
	my $conf_name_search_field = $this->getNameSearchField();
	if ($conf_name_search_field =~ /,/){
		@name_fields = split(/,/,$conf_name_search_field);
	}else{
		@name_fields = ($conf_name_search_field);
	}
	foreach my $field(@name_fields){
		foreach my $key(keys(%$answerdata)){
			my $answer_value = $answerdata->{$key};
			# If the field matches and the field isn't null we save the value in an array
			if ($key =~ /$field/i && $answer_value){
				$answer_value =~ s/"/'/g;
				push(@name,$answer_value);
			}
		}
	}
	return "\"".join(" ",@name)."\" <$from>";
}
# email survey responses to email addresses listed on the emaillist(survey setup)
sub mailResponse{
	# $emaillist: email list
	my ($this,$survey,$mailcontent,$qdata) = @_;
	my ($from,$subject,$body,@emails,$emaillist);
	$emaillist = $survey->{email};
	# from address(from config file)
#	$from = $survey->{from_email} || $config{$this->{class_name}}{config}{mail}{from};
	$from = $this->mailFrom($survey,$qdata);
	# email subject(from config file)
	$subject = $config{$this->{class_name}}{config}{mail}{subject};
	# survey responses
	$body = join("\n",@$mailcontent);
	@emails = split(/,/,$emaillist);
	# send emails
	foreach my $to (@emails){
		next if !$this->validateEmail($to);
		&SendMail($to,$from,$subject,$body);
	}
	return;
}
# preset form fields(mainly used by the SimpleRegistration module)
sub presetFormField{
	my ($this) = @_;
	return;
}
# save registrant data?
sub saveResponse{
	my ($this,$survey) = @_;
	return $this->{save_registrant_data} if !$survey;
	return 1 if $survey->{save_result} =~ /yes/i;
	return 0 
}
# add section info to question
sub addQuestionSectionInfo{
	my ($this,$surveyid,$questions) = @_;
	my (@sections,%sections);
	@sections = $this->{db}->fetch_match("survey_section",{survey_id=>$surveyid});
	foreach my $section(@sections){
		$sections{$section->{id}} = $section;
	}
	for(my $i = 0; $i < @$questions; $i++){
		next if !$questions->[$i]{survey_section_id};
		$questions->[$i]{section_label} = $sections{$questions->[$i]{survey_section_id}}{label};
	}
	return $questions;
}
# build survey summary report
sub viewSurveyReport{
	# $surveyid: survey id
	my ($this,$surveyid) = @_;
	return $this->viewSummaryReport($surveyid,'all');
}
sub viewDDSSummaryReport{
	my ($this,$ddsid,$surveyid) = @_;
	my (%input,$result,@ids,%dds_rec);
	%dds_rec = $this->{db}->fetch("survey_dds",$ddsid);
	%input = $this->getFilterData($ddsid);
	($result,undef,undef) = $this->getDDSResult(\%input);
	foreach my $r(@$result){
		push(@ids,$r->{survey_registrant_id});
	}
	return $this->viewSummaryReport($surveyid,join("|",@ids),$dds_rec{name});
}
# build survey question summary report
sub viewQuestionReport{
	# $questionid: question id
	# $surveyid: survey id
	my ($this,$questionid,$surveyid) = @_;
	my ($question,@registrants,@answers,%registrants,$body);
	# fetch question data
	$question = $this->{db}->fetch("survey_question",$questionid);
	# fetch registrants
	@registrants = $this->{db}->fetch_match("survey_registrant",{survey_id=>$surveyid});
	# fetch registrants' reponses
	@answers = $this->{db}->fetch_match("survey_answer",{survey_question_id=>$questionid});
	# index registrant
	foreach my $r (@registrants){
		$registrants{$r->{id}} = $r;
	}
	# populate answer list with registrant info
	for(my $i = 0; $i < @answers; $i++){
		$answers[$i]{first} = $registrants{$answers[$i]{survey_registrant_id}}{first};
		$answers[$i]{last} = $registrants{$answers[$i]{survey_registrant_id}}{last};
		$answers[$i]{email} = $registrants{$answers[$i]{survey_registrant_id}}{email};
	}
	$body = $this->html_questionRegistrantReport(\@answers);
	return $this->html_surveyQuestionReport($question,$body,$surveyid);
}
# get survey/registration data
sub getSurveyData{
	# $surveyid: survey id
	# $table: table name
	# $limit: no of record to return
	my ($this,$surveyid,$table,$limit) = @_;
	my @list = $this->{db}->fetch_match($table,{survey_id=>$surveyid},"id asc",$limit);
	return \@list;
}
# survey wizard
sub surveyWizard{
	# $section_id: section id
	my ($this,$section_id) = @_;
	my (@contact,@step_name,@show_step,$wiz,@do_step,$step_menu,%input,$next,$another,$wtitle,$out,$module);
	# run wizard init routine
	$this->wizardInit;
	# default contact info fields
	@contact = @{$this->{contact}};
	$module = $this->getModuleName;
	$wtitle = "Create a new $module form";
	# names of steps
	@step_name = ("$module Settings","Custom Contact Information","$module Sections","$module Questions");
	# routines to display each step in the wizard
	@show_step = ("html_wizardPage1","html_wizardPage2","html_wizardPage3","html_wizardPage4");
	# routines to process the user input from each step in the wizard
	@do_step = ("do_wizardPage1","do_wizardPage2","do_wizardPage3","do_wizardPage4");

	# wizard URL
	$wiz = &relink({action=>undef,section_id=>undef});

	# wizard step menu - off by default
	$step_menu = 0;
	
	# form inputs
	%input = %{$this->{input}};
	# wizard screen no.
	$this->{screen} = $input{screen};
	$next = $this->{screen} + 1;
	$another = 0;
	# return to the same wizard screen
	if ($input{anotherone} =~ /y/i){
		$another = 1;
		$next--;
	}
	# abort wizard
	if ($input{submit_action} eq "Cancel") { 
		$out = $this->error("$wtitle aborted");
	}elsif ($this->{screen} =~ /^\d+$/){
		# validate input
		if (defined $do_step[$this->{screen}]){
			my $subname = $do_step[$this->{screen}];
			$this->$subname({contact=>\@contact});
			if (@{$this->{wizard_error}} > 0){
				my $message = "<h3>Problem:</h3>\n<p class=\"error\">\n";
				$message .= join("<br>",@{$this->{wizard_error}});
				$message .= "<p><form><input type=\"button\" value=\"&lt; Back\" onclick=\"history.back()\"></form></p>\n";
				# input error				
				return $this->error($message);
			}
		}
		if (@{$this->{wizard_error}} == 0){
			# goto next screen
			if ($this->{screen} == $#show_step && !$another){
				# no more screens!
				$out = $this->html_wizardFinishScreen($wtitle);
				return $out;
			}else{				
				# build wizard form
				delete	$this->{input}{anotherone};
				$this->{screen} = $next;
				$out = $this->html_wizardStartForm($wtitle,$wiz);
				my $subname = $show_step[$this->{screen}];
				$out .= $this->$subname({contact=>\@contact,screen=>$this->{screen}});
				$out .= $this->html_wizardFinishForm($this->{screen},\@show_step);
				return $out;
			}
		}
	}else{		
		delete $this->{input}{anotherone};
		# start the wizard sequence
		$this->{screen} = 0;
		$out = $this->html_wizardStartForm($wtitle,$wiz);
		my $subname = $show_step[$this->{screen}];
		$out .= $this->$subname({contact=>\@contact,screen=>$this->{screen},section_id=>$section_id});
		$out .= $this->html_wizardFinishForm($this->{screen},\@show_step);
		return $out;
	}
	return $out;
}
# build a list of custom reports
sub surveyDDS{
	# $sectionid: section id
	my ($this,$sectionid) = @_;	
	my ($out,$surveys,$dds);
	# run data filter init routine
	$out = $this->ddsInit;
	# fetch survey data
	$surveys = $this->{db}->fetch_match("survey",{section_id=>$sectionid},"name asc");
	$out .= $this->html_ddsSurveySelect($surveys);
	$dds = $this->{db}->fetch_match("survey_dds",{section_id=>$sectionid},"survey_id asc");
	# build custom reports
	$out .= $this->html_ddsSurveyDDSReport($dds);
	return  $out;
}
# build new custom report form
sub newDDSReport{
	my ($this) = @_;
	my ($out);
	# run data filter init routine
	$out = $this->ddsInit;
	# remove steps from the data filter
	if ($this->{input}{dds_action} eq "rollback"){
		$this->ddsRollback;
	}
	# save custom reports
	if ($this->{input}{submit_action} =~ /^save$/i){
		$this->ddsSaveReport;
		#return $this->surveyDDS($this->{input}{section_id});
		return $this->reportList($this->{saved_survey_id});
	}
	if ($this->{input}{submit_action} =~ /over/i){
		$this->ddsCleanData;
	}
	if ($this->{input}{dds_action} eq "view"){
		# view custom reports
		$out .= $this->html_ddsViewReport;
	}else{
		# go to the next step
		$out .= $this->html_ddsFormStep($this->{input}{survey_id});
	}
	return $out;
}
# admin form
sub adminForm{
	my ($this,$sectionid,$surveyid) = @_;
	# Validate user access for using this function
	my $va_tab = $sectionid ? "section" : "survey";
	my $va_id =  $sectionid ? $sectionid : $surveyid;
	return $this->denied() if !$this->validateAccess($va_tab,$va_id);
	return $this->html_adminForm($sectionid,$surveyid);
}
# Global access denied message
sub denied{
	my ($this) = @_;
	return $this->error("You do not have sufficient system priveleges to use this feature.");
}
# Validate user access for using admin functions
sub validateAccess{
	my ($this,$table,$id) = @_;
	my $sectionid;

	if ($table =~ /^section$/i){
		$sectionid = $id;
	# Get section_id from a survey record
	}elsif ($table =~ /^survey$/i){
		my %survey = $this->{db}->fetch("survey",$id);
		$sectionid = $survey{section_id};
	# Need to do a look up on the parent survey record's section id
	}elsif ($table =~ /^survey_(question|contact_question|section|registrant)$/i){
		my $sql = "select b.section_id as section_id from ${table} a left join survey b on b.id = a.survey_id where a.id = $id";
		my @recs = $this->{db}->custom_query($sql);
		$sectionid = $recs[0]->{section_id};
	}
	return 0 if !$sectionid;
	# Verify that the user has the access key for the section
	return $this->{db}->owns($sectionid);
}
#------------------------------------------------------------------
# remove dds data from the query string(start over command)
sub ddsCleanData{
	my ($this) = @_;
	my %skip = (
		action=>"dds_action",
		dds_action=>"step",
		step=>0,
		survey_id=>$this->{input}{survey_id},
	);
	foreach my $in(keys(%{$this->{input}})){
		if (exists($skip{$in})){
			$this->{input}{$in} = $skip{$in};
		}else{
			delete $this->{input}{$in};
		}
	}
	return;
}
# get module name
sub getModuleName{
	my ($this) = @_;
	return ucfirst($this->{name});
}
# data filter init routine
sub ddsInit{
	my ($this) = @_;
	#return $this->html_ddsJavascript;
	return $this->html_javascript("dds");
}
# modified version of &relink
sub myRelink{
	my ($this,$data) = @_;
	my (@list,$path,@systemQuery);
	$path = $ENV{SCRIPT_NAME};
	$path .= $ENV{PATH_INFO} ?  $ENV{PATH_INFO} : "";
	$path .= "?";
	# ?????????
	#$path .= $this->{input}{_id} ? "_id=$this->{input}{_id}" : "";
	# ?????????
	foreach my $key (keys(%{$this->{input}})){
		next if $key !~ /^_/i;
		push(@systemQuery,"$key=$this->{input}{$key}");
	}
	$path .= join("&",@systemQuery);
	$path .= @systemQuery > 0 ? "&" : "";
	foreach my $key (keys(%$data)){
		push(@list,"$key=$data->{$key}");
	}
	return $path.join("&",@list);
}
sub encodeString{
	my ($this,$data) = @_;
	return &url_escape($data);
}
# rollback database filter steps
sub ddsRollback{
	my ($this) = @_;
	my ($step,$survey_id);
	# steps to rollback
	$step = $this->{input}{step};
	# survey id
	$survey_id = $this->{input}{survey_id};
	# nothing to rollback
	return %{$this->{input}} if (keys(%{$this->{input}}) < 1 || !$step);
	# remove data filter steps
	foreach my $key (keys(%{$this->{input}})){
		next if ($key !~ /^step#/);
		my (undef,$currstep,$qid) = split(/#/,$key);
		if ($currstep >= $step){delete $this->{input}{$key};}
	}
	$this->{input}{action} = "dds_action";
	$this->{input}{step} = $step - 1;
	$this->{input}{survey_id} = $survey_id;
	delete $this->{input}{dds_action};
	return;
}
# verify that the user owns the site
sub verifySectionID{
	my ($this,$sectionid) = @_;
	my @sites = $this->{db}->my_real_sites;
	foreach my $site(@sites){
		return 1 if ($site->{section_id} == $sectionid);
	}
	return 0;
}
########################################################################################
# exsite handlers
########################################################################################
#
# select foreign key
sub handler_select_foreign_key{
	my ($this,$table,$rec,$col,$ftable) = @_;
	my ($fid);
	# select all sections associated to a survey
	if ($table eq "survey_question" && $col eq "survey_section_id"){
		# create a new record
		if (!$rec){
			my ($query,$in);
			$in = new ExSite::Input;
			$query = $in->combine;
			$fid = $query->{pid};
		# edit a record
		}else{
			my %record = $this->fetch($table,$rec);
			$fid = $record{survey_id};
		}
		return $this->fetch_match("survey_section",{survey_id=>$fid});
	}
	return undef;
}
# pre_insert
sub handler_pre_insert{
	my ($this,$table,$data) = @_;
	if ($table eq "survey_question"){
		return if !$data->{list};
		# reformat answer choice list
		$data->{list} = &stripChr(undef,$data->{list});
		$data->{list} = &stripChrQuestionAnswerChoice(undef,$data->{list});
	}
	return;
}
# pre_update
sub handler_pre_update{
	my ($this,$table,$data,$cond) = @_;
	if ($table eq "survey_question"){
		return if !$data->{list};
		# reformat answer choice list
		$data->{list} = &stripChr(undef,$data->{list});
		$data->{list} = &stripChrQuestionAnswerChoice(undef,$data->{list});
	}
	return;
}
########################################################################################
# insert/post exsite post
sub exsiteFormPost{
	my ($this) = @_;
	$this->{db}->handler("pre_insert",\&handler_pre_insert);
	$this->{db}->handler("pre_update",\&handler_pre_update);
	my $out = $this->{db}->do();
	return $out if $out;
	return;
}
# add/edit child record
sub adminChildRecordForm{
	my ($this,$tab,$id,$ptab,$pid) = @_;
	# Validate user access for using this function
	my $va_id = $id ? $id : $pid;
	my $va_tab = $id ? $tab : $ptab;
	return $this->denied() if !$this->validateAccess($va_tab,$va_id);
	my ($module,%c_record);
	$this->surveyInit;
	$module = $this->getModuleName;
	if (!$id){
		# invalid pid
		return $this->error("Invalid $module id.") if (!$pid);
		# invalid ptab
		return $this->error("Invalid $module information.") if (!$ptab);
	}else{
		# figure out parent table and id
		%c_record = $this->{db}->fetch($tab,$id);
		$ptab = "survey";
		$pid = $c_record{survey_id};
	}
	# invalid table
	return $this->error("Invalid $module information.") if (!$tab);
	my (%table,$reply,$out,%survey,%record);
	# install handler
	$this->{db}->handler("select_foreign_key",\&handler_select_foreign_key);	
	%survey = $this->{db}->fetch("survey",$pid);
	$reply = $this->myRelink({action=>"main_form",sid=>$pid});
	# define form creation input for different child tables
	%table = (
	survey_contact_question=>{
		label=>"$module Contact Question",
		form_data=>{
			table=>$tab,
			hide=>["id","survey_id"],
			extra=>{reply=>$reply},
		},
	},
	survey_section=>{
		label=>"$module Section",
		form_data=>{
			table=>$tab,
			hide=>["id","survey_id"],
			extra=>{reply=>$reply},
		},
	},
	survey_question=>{
		label=>"$module Question",
		form_data=>{
			table=>$tab,
			hide=>["id","survey_id"],
			extra=>{reply=>$reply},
		},
	},
	);
	# survey_question table: hide survey section field
	if ($survey{use_group} !~ /$this->{yes}/i){
		push(@{$table{survey_question}{form_data}{hide}},"survey_section_id");
	}
	%record = %{$table{$tab}{form_data}};
	if ($id){
		# for editing a record
		$record{record} = $id;
	}else{
		# for inserting a record
		$record{data}{survey_id} = $pid;
	}
	$record{action} = $this->myRelink();
	$out .= "<h2>$table{$tab}{label}</h2>";
	$out .= $this->{db}->make(%record);
	return $out;
}
# 'reconfigure' screen for administrators
sub adminFormMain{
	my ($this,$surveyid) = @_;
	return $this->error("Missing survey id. Please verify that you've set up Survey dbmaps and tables correctly.") if !$surveyid;
	return $this->denied() if !$this->validateAccess("survey",$surveyid);
	my (%options,$reply);
	$options{surveyid} = $surveyid;
	# form reply url
	$reply = $this->encodeString($this->myRelink({action=>"main_form",sid=>$surveyid}));
	# edit url
	$options{edit} = $this->myRelink({action=>"edit_form_record"});
	# delete url
	$options{delete} = $this->myRelink({action=>"rec_delete",reply=>$reply});
	# add new url
	$options{new} = $this->myRelink({action=>"new_form_record",ptab=>"survey",pid=>$surveyid});
	return $this->html_adminFormMain(\%options);
}
# survey editing options
sub getAdminOption{
	my ($this,$surveyid) = @_;
	my (@commands);
	@commands = (
	{id=>"edit",label=>"Edit",url=>$this->myRelink({action=>"edit_mainform",sid=>$surveyid}),target=>"",img=>"edit"},
	{id=>"preview",label=>"Preview",url=>$this->myRelink({action=>"do",sid=>$surveyid}),target=>"_blank",img=>"view"},
	);
	return $this->html_adminOption(\@commands);
}
# process admin form data
sub adminFormProcess{
	my ($this,$type,$formdata) = @_;
	my (%table_data,%contact_data,%form_data,$surveyid);
	foreach my $formkey(keys(%$formdata)){
		if ($formkey =~ /^(.+?)#(.+?)#(.+)$/){
			$table_data{$1}{$2}{$3} = $formdata->{$formkey};
		}elsif ($formkey =~ /^(.+?)_(.+)$/){
			$contact_data{$2}{$1} = $formdata->{$formkey};
		}else{
			$form_data{$formkey} = $formdata->{$formkey};
		}
	}
	# insert records into the table
	foreach my $tab (keys(%table_data)){
		foreach my $pkey(keys(%{$table_data{$tab}})){
			my %insert = %{$table_data{$tab}{$pkey}};
			if($pkey =~ /\d+/i){
				$surveyid = $tab eq "survey" ? $pkey : 0;
				$this->{db}->update($tab,\%insert,{id=>$pkey});
			}else{
				if (!$this->verifySectionID($insert{section_id}) && $tab eq "survey"){
					return $this->error("Invalid website.");
				}
				my $newid = $this->{db}->insert($tab,\%insert);
				$surveyid = ($tab eq "survey") ? $newid : $surveyid;
			}
		}
	}
	# insert contact information
	$this->adminFormInit;
	foreach my $vkey (sort {$a <=> $b} keys(%contact_data)){
		next if !$contact_data{$vkey}{checkbox};
		next if !$surveyid;
		my (%insert,$arrayindex);
		$arrayindex = $vkey - 1;
		%insert = (
			survey_id=>$surveyid,
			label=>$this->{contact}[$arrayindex]{name},
			type=>$this->{contact}[$arrayindex]{type},
			mandatory=>$contact_data{$vkey}{mandatory},
		);
		$this->{db}->insert("survey_contact_question",\%insert);
	}
	return $this->adminFormMain($surveyid);
}
# validate wizard step 4 - survey question
sub do_wizardPage4 {
	# $opt: option hash
	my ($this,$opt) = @_;
	my ($val,%input);
	# form data
	%input = %{$this->{input}};
	# validate previous steps
    	$this->do_wizardPage3;
	# get question no.
    	$val = $this->getRecNo('survey_question');
	# validate required fields
	if (
		!$input{"survey_question#_$val#label"} ||
		!$input{"survey_question#_$val#type"} ||
		!$input{"survey_question#_$val#mandatory"}
	){
		push(@{$this->{wizard_error}},"Missing data - please fill out all values.");
	}
	# set default values for other fields
	if (!$input{"survey_question#_$val#other"}){
		$input{"survey_question#_$val#other"} = "N";
	}
	# escape characters
	if ($input{"survey_question#_$val#list"}){
		$input{"survey_question#_$val#list"} = $this->stripChr($input{"survey_question#_$val#list"});
		$input{"survey_question#_$val#list"} = $this->stripChrQuestionAnswerChoice($input{"survey_question#_$val#list"});
		#$input{"survey_question#_$val#list"} =~ s/\s*\|\s*/\|/g;
		#$input{"survey_question#_$val#list"} =~ s/\"/&quote;/g;
	}
	if (!$this->verifySectionID($input{"survey#_#section_id"})){
		push(@{$this->{wizard_error}},"Permission denied: Invalid website.");
	}
	# return the user back to the wizard if there are input errors
    	if (@{$this->{wizard_error}} || $input{anotherone} =~ /y/i){
		return;
	}
	my ($customq,$form,%insert,$surveyid,%section,%surveyq,$id,$db);
	$db = $this->{db};
	$customq = $input{'customq'};
	
	# remove form inputs
	delete $this->{input}{anotherone};
	delete $this->{input}{customq};
	delete $this->{input}{screen};
	delete $this->{input}{submit_action};

	# parse input data
	$form = $this->myParse($this->{input});

	# get survey input data
	%insert = %{$form->{'survey'}->{_}};
	# insert survey data into the database
	$id = $db->insert('survey',\%insert);

	# new survey id
	$surveyid = $id;
	$this->{new_surveyid} = $surveyid;
	
	# insert contact info data
	if (exists($form->{survey_contact_question})){
		my %contactinfo = %{$form->{survey_contact_question}};
		if (keys(%contactinfo) > 0){
			%contactinfo = $this->fixVirtualKey(\%contactinfo);
			foreach my $rec (sort {$a <=> $b} keys(%contactinfo)){
				%insert = %{$contactinfo{$rec}};
				$insert{survey_id} = $surveyid;
				$id = $db->insert("survey_contact_question", \%insert);
			}
		}
	}
	
	# insert survey section data
	if ($input{"survey#_#use_group"} =~ /y/i){
		my %surveysection = %{$form->{survey_section}};
		if (keys(%surveysection) > 0){
			%surveysection = $this->fixVirtualKey(\%surveysection);
			foreach my $rec (sort {$a <=> $b} keys(%surveysection)){
				%insert = %{$surveysection{$rec}};
				$insert{survey_id} = $surveyid;
				$id = $db->insert("survey_section", \%insert);
				$section{$rec} = $id;
			}
		}
	}
	#insert survey questiondata
	%surveyq = %{$form->{survey_question}};
	if (keys(%surveyq) > 0){
		%surveyq = $this->fixVirtualKey(\%surveyq);
		foreach my $rec (sort {$a <=> $b} keys(%surveyq)){
			%insert = %{$surveyq{$rec}};
			$insert{survey_id} = $surveyid;
			if ($input{"survey#_#use_group"} =~ /y/i){
				$insert{survey_section_id} = $section{$insert{survey_section_id}};
			}
			$id = $db->insert("survey_question",\%insert);
		}
	}	
	return;
}
# reindex the hash 
# from _xxxxx keys to xxxx keys
sub fixVirtualKey{
	my ($this,$myHash) = @_;
	my %newHash;
	foreach my $key (keys(%$myHash)){
		$key =~ /^_(.+)$/;
		$newHash{$1} = $myHash->{$key};
	}
	return %newHash;
}
# validate wizard step 3 - survey section
sub do_wizardPage3 {
	# $opt: option hash
	my ($this,$opt) = @_;
	my ($val,$table,%input);
	# form data
	%input = %{$this->{input}};
	$table = "survey_section";
	# validate previous steps
    	$this->do_wizardPage2;
	$val = $this->getRecNo($table); 
	if ($input{'survey#_#use_group'} =~ /y/i){
		# validate required fields
    		if (!$input{"$table#_$val#label"}){
			push(@{$this->{wizard_error}},"Missing data - please fill out all values.");
		}
		# escape quotes
		if ($input{"$table#_$val#label"} =~ /\"/){
			$input{"$table#_$val#label"} =~ s/\"/&quote;/g;
		}
	}
    	return;
}
# validate wizard step 2 - contact info
sub do_wizardPage2{
	# $opt: option hash
	my ($this,$opt) = @_;
	my ($table,%input,$val);
	%input = %{$this->{input}};
	$table = "survey_contact_question";
	# validate previous steps
    	$this->do_wizardPage1;
	$val = $this->getRecNo($table); 
	if ($input{'customq'} =~ /y/i){
		# contact info label, type and mandatory fields are quired
    		if (
			!$input{"$table#_$val#label"} ||
			!$input{"$table#_$val#type"} ||
			!$input{"$table#_$val#mandatory"}
		){
			push(@{$this->{wizard_error}},"Missing data - please fill out all values.");
		}
		# escape quotes
		if ($input{"$table#_$val#label"} =~ /\"/){
			$input{"$table#_$val#label"} =~ s/\"/&quote;/g;
		}
	}
    	return;
}
# validate wizard step 1 - survey info
sub do_wizardPage1{
	# $opt: option hash
	my ($this,$opt) = @_;
	my (%input,@contact,$count);
	# get default contact info fields
	@contact = @{$this->{contact}};
	# get form data
	%input = %{$this->{input}};
	# validate required fields
    	if (!$input{"survey#_#name"} || 
#		!$input{"survey#_#survey_type"} || 
		!$input{"survey#_#use_require"} ||
		!$input{"survey#_#use_group"} ||
		!$input{"survey#_#use_number"} ||
		!$input{"survey#_#thankyou_message"} ||
		!$input{"customq"}) {
		#!$input{"survey#_#survey_return"}) {
		push(@{$this->{wizard_error}},"Missing data - please fill out all values.");
    	}
	$count = 1;
	# reorganize contact info data
	for(my $i = 0; $i < @contact; $i++){
		my ($checkbox,$mandatory,$realkey);
		$checkbox = "checkbox_".($i + 1);
		$mandatory = "mandatory_".($i + 1);
		$realkey = "survey_contact_question#_".($count)."#";
		if (exists($input{$checkbox})){
			$count++;
			$this->{input}{$realkey."label"} = $contact[$i]->{name};
			$this->{input}{$realkey."type"} = $contact[$i]->{type};
			$this->{input}{$realkey."mandatory"} = $input{$mandatory};			
		}
		delete $this->{input}{$checkbox} if (exists($this->{input}{$checkbox}));
		delete $this->{input}{$mandatory} if (exists($this->{input}{$mandatory}));
	}
	# escape quotes
    	foreach my $key (keys %{$this->{input}}){
    		if ($this->{nput}{$key} =~ /\"/){
    			$this->{input}{$key} =~ s/\"/&#34;/g; # quotation mark
		}
    	}
    	return;
}
# wizard init routine
sub wizardInit{
	my ($this) = @_;
	$this->{screen} = 0;
	$this->{wizard_error} = [];
	# default contact info fields
	$this->{contact} = [
		{id=>1,name=>"First Name",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>2,name=>"Last Name",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>3,name=>"Title",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>4,name=>"Company",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>5,name=>"Street Address",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>6,name=>"City",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>7,name=>"State/Province",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>8,name=>"Country",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>9,name=>"Telephone",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>10,name=>"Fax",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>11,name=>"Email",type=>"Short answer",checked=>1,mandatory=>0},
	];
	return;
}
# admin form init routine
sub adminFormInit{
	my ($this) = @_;
	# default contact info fields
	$this->{contact} = [
		{id=>1,name=>"First Name",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>2,name=>"Last Name",type=>"Short answer",checked=>1,mandatory=>0},
		{id=>3,name=>"Title",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>4,name=>"Company",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>5,name=>"Street Address",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>6,name=>"City",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>7,name=>"State/Province",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>8,name=>"Country",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>9,name=>"Telephone",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>10,name=>"Fax",type=>"Short answer",checked=>0,mandatory=>0},
		{id=>11,name=>"Email",type=>"Short answer",checked=>1,mandatory=>0},
	];
	return;
}
# wizard hidden inputs
sub wizardIncludeInput {
	my ($this) = @_;
	my ($out,%input);
	# form data
	%input = %{$this->{input}};
	# delete input keys
	delete $input{screen};
	delete $input{submit_action};
	while (my ($key, $val) = each %input){
		$out .= $this->html_wizardIncludeInput($key,$val);
	}
	return $out;
}
# calculate percentage
sub calcPercent{
	# $count: count
	# $total: total
	my ($this,$count,$total) = @_;
	return sprintf("%.2f %",($count/$total) * 100);
}
# calculate sum
sub calcSum{
	# $myHash: custom answer hash
	my ($this,$myHash) = @_;
	my $count = 0;
	foreach my $key(keys(%$myHash)){
		# sum counts
		$count += $myHash->{$key}{count};
	}
	return $count;
}
# calculate stats
sub calcStat{
	# $myHash: custom answer hash
	my ($this,$myHash) = @_;
	# sum responses
	my $count = $this->calcSum($myHash);
	foreach my $key(keys(%$myHash)){
		# calculate percentage
		$myHash->{$key}{percent} = !$count ? "0.00 %" : $this->calcPercent($myHash->{$key}{count},$count);
	}
	return %$myHash;
}
sub viewSummaryReport{
	my ($this,$surveyid,$option,$title) = @_;
	my ($questions,%survey,%stats,$body);	
	# fetch survey data
	%survey = $this->{db}->fetch("survey",$surveyid);
	# fetch all questions
	$questions = $this->{db}->fetch_match("survey_question",{survey_id=>$surveyid});
	$questions = $this->addQuestionSectionInfo($surveyid,$questions);
	foreach my $q (@$questions){
		# build question label
		my $title = $this->html_summaryQuestionTitle($q,$surveyid);
		my $tool = $this->html_summaryQuestionViewDetailLink($q,$surveyid);
		my $data;
		my %report = (
			title=>$title,
			width=>"100%",
		);
		# no summaries are shown for long/short answer questions
		if ($q->{type} =~ /long|short/i){
			$data = $this->html_noSummary;
			$report{no_data} = $data;
		}else{
			# fetch answers
			my $answers = $this->getResponse($q->{id},$option);
			# calculate stats
			my %stats = $this->summaryStats($q,$answers);
			my @header = ("Answer Choice","Count","Percent");
			my @stat = $this->summaryConvertHashToArray(\%stats);
			@stat = sort {$b->{percent} <=> $a->{percent}} @stat;
			foreach my $s (@stat){
				my @stat_row = ($s->{label},$s->{count},$s->{percent});
				push(@$data,\@stat_row);
			}
			$report{headers} = \@header;
			$report{tools} = $tool;
		}
		$report{data} = $data;
		$body .= "<p>".$this->reportbuilder(\%report)."</p>";
	}
	$body .= $this->html_surveyBottomSummary($surveyid,$option);
	return $this->html_surveySummaryReport(\%survey,$body,$title);
}

sub getResponse{
	my ($this,$questionid,$option) = @_;
	my $answers;
	if ($option =~ /^all$/i){
		$answers = $this->{db}->fetch_match("survey_answer",{survey_question_id=>$questionid});
	}else{
		my @regs = split(/\|/,$option);
		my $sql = "select * from survey_answer where survey_question_id = $questionid and ( survey_registrant_id = ".join(" or survey_registrant_id = ",@regs).")";
		$answers = $this->{db}->custom_query($sql);
	}
	return $answers;	
}
# build summary report
sub summaryStats{
	# $question: question hash
	my ($this,$question,$answers) = @_;
	if ($question->{type} =~ /^check/i){
		# build summary reports for single checkbox questions
		return $this->singleCheckboxSummaryStats($answers);
	}elsif ($question->{type} =~ /multi|radio|list/i){
		# build summary reports for multiple answer choice questions(multiple checkboxes, radio button and drop-down list)
		return $this->multipleAnswerSummaryStats($question,$answers);
	}
	return;
}
# build summary reports for multiple answer question
sub multipleAnswerSummaryStats{
	# $question: question hash
	# $answers: answer array
	my ($this,$question,$answers) = @_;
	my (%result);
	# index answer choices
	foreach my $q (split(/\|/,$question->{list})){
		$result{$q} = {percent=>0,count=>0};
	}
	$result{"Other"} = {percent=>0,count=>0};
	# sum answers
	foreach my $a (@$answers){
		foreach my $v (split(/; /,$a->{value})){
			if (exists($result{$v})){
				$result{$v}{count}++;
			}else{
				$result{"Other"}{count}++;
			}
		}
	}
	# calculate stats on the summaries
	%result = $this->calcStat(\%result);
	return %result;
}
# build summary reports for single checkbox answer question
sub singleCheckboxSummaryStats{
	# $answers: answer array
	my ($this,$answers) = @_;
	my (%result,$total);
	%result = ("Yes"=>{percent=>0,count=>0},"No"=>{percent=>0,count=>0});
	# sum answers
	foreach my $a (@$answers){
		$result{"Yes"}{count}++ if $a;
		$result{"No"}{count}++ if !$a;
	}
	%result = $this->calcStat(\%result);
	return %result;
}
# validate email address
sub validateEmail{
	# $email: email address
	my ($this,$email) = @_;
	return 1 if ($email =~ /^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/);
	return 0;
}
# get registrant data
sub getRegistrantData{
        # $fdata: form data
        # $contact: contact question hash
        my ($this,$fdata,$contact) = @_;
        my (%rdata);
	my $email;
        my $name;
	# Put $fdata(form data) and $contact(contact info question data)
	# into this format:
	#	my %hash = ( <question>=><answer>...etc);
	my %contact_data;
	foreach my $info_id(keys(%$fdata)){
		$contact_data{$contact->{$info_id}{label}} = $fdata->{$info_id};
	}
        # populate first name, last name and email with the contact info data
        foreach my $label (keys(%contact_data)){
                if ($label =~ /first name/i){
                        $rdata{first} = $contact_data{$label};
                }elsif($label =~ /last name/i){
                        $rdata{last} = $contact_data{$label};
                }elsif($label =~ /email/i){
                        $email = $contact_data{$label};
                }
                if ($label =~ /^name$/i){
                        $name = $contact_data{$label};
                }
		my $conf_name = $this->getNameSearchField();
		if ($conf_name){
			if ($label =~ /$conf_name/i){
				$name = $contact_data{$label};
			}
		}
		my $conf_email = $this->getEmailSearchField();
		if ($conf_email){
			if ($label =~ /$conf_email/i){
				$email = $contact_data{$label};
			}
		}
        }
        # set the last name to anonymous if the participant didn't provide a first name and a last name
        if (!$rdata{first} && !$rdata{last}){
                if ($name) {
                        my ($firstname,$lastname) = split(/ /,$name,2);
                        $rdata{first} = $firstname;
                        $rdata{last} = $lastname;
                }else{
                        $rdata{last} = $config{$this->{class_name}}{config}{registrant}{last};
                }
        }
	if (!$email){
		$email = $config{$this->{class_name}}{config}{registrant}{email};
	}
	$rdata{email} = $email;
        return \%rdata;	
}
# validate question
sub validateQuestion{
	# $fdata: form data
	# $question: question hash
	my ($this,$fdata,$question) = @_;
	# check the selection limit(for multiple checkboxes questions only)
	if ($question->{type} =~ /multiple/i){
		my $limit = $question->{multicheck};
		if ($limit < split(/; /,$fdata) && $limit){
			push(@{$this->{form_error}},"Please limit your selected answer choices to $limit or less for question: \"$question->{label}\".");
			return 0;
		}
	}
	# this question isn't mandatory
	return 1 if !$this->isMandatory($question);
	# particpant didn't answer this question
	if (!$fdata){
		push(@{$this->{form_error}},"Please answer question: \"$question->{label}\".");
		return 0;
	}
	return 1;
}
# validate contact info
sub validateContactInfo{
	# $fdata: form data
	# $contact: contact info hash
	my ($this,$fdata,$contact) = @_;
	# this question isn't mandatory
	return 1 if !$this->isMandatory($contact);
	# particpant didn't answer this question
	return 1 if $fdata;
	return 0;
}
# check to see if questions should be grouped or not
sub stackAnswerChoice{
	# $list: question array
	# $count: array index
	my ($this,$list,$count) = @_;
	# reached the last element in the array
	return 0 if scalar(@$list) <= $count + 1;
	# questions are in different sections
	return 0 if $list->[$count]{survey_section_id} != $list->[($count + 1)]{survey_section_id};
	# current question has the same answer choices as the next one
	return 1 if $list->[$count]{list} eq $list->[($count + 1)]{list};
	return 0;
}
# survey question init routine
sub questionInit{
	# $question: question hash
	my ($this,$question) = @_;
	# set values mandatory question indexes
	if ($question->{mandatory}){
		$this->{question}{$question->{id}} = 1;
	}
	return;
}
# contact info init routine
sub contactinfoInit{
	# $contact: contact info hash
	my ($this,$contact) = @_;
	# set values mandatory question indexes
	if ($contact->{mandatory}){
		$this->{contactinfo}{$contact->{id}} = 1;
	}
	return;
}
# survey init routine
sub surveyInit{
	# $survey: survey hash
	my ($this,$survey) = @_;
	my ($yes);
	$yes = $config{$this->{class_name}}{config}{yes} || "y";
	# use default style sheet
	$this->{use_default_css} = $survey->{use_generic_css} =~ /$yes/i ? 1 : 0;
	# use config variable(old)
	#$this->{save_registrant_data} = $config{$this->{class_name}}{config}{save};
	$this->{save_registrant_data} = $survey->{save_result} =~ /$yes/i ? 1 : 0;
	# number questions?
	$this->{use_number} = $survey->{use_number} =~ /$yes/i ? 1 : 0;
	# group questions?
	$this->{use_group} = $survey->{use_group} =~ /$yes/i ? 1 : 0;
	# highlight required mandatory questions?
	$this->{use_require} = $survey->{use_require} =~ /$yes/i ? 1 : 0;
	# where to place the contact info questions
	$this->{contactinfo_pos} = $survey->{contactinfo_pos} =~ /^begin/i ? 1 : 0;
	$this->{yes} = $yes;
	# survey section lookup table
	##$this->{section} = $this->getSurveySection($this->{input}{sid});
	$this->{section} = $this->getSurveySection($survey->{id});
	return;
}
# get survey sections
sub getSurveySection{
	# $surveyid: survey id
	my ($this,$surveyid) = @_;
	return {} if !$surveyid;
	my ($list,%list);
	$list = $this->{db}->fetch_match("survey_section",{survey_id=>$surveyid});
	# index sections by section id
	foreach	my $l (@$list){
		$list{$l->{id}} = $l;
	}
	return \%list;
}
# check to see if a question is mandatory or not
sub isMandatory{
	# $dat: data hash(could be a question/contact question)
	my ($this,$dat) = @_;
	my $yes = $this->{yes};
	return 1 if $dat->{mandatory} =~ /$yes/i;
	return 0;
}
# check to see if the user has access to the survey or not
sub accessSurvey{
	# $survey: survey hash
	my ($this,$survey) = @_;
	# the user has sufficient access to view the survey
	return 1 if $this->{db}->authorize >= $config{$this->{class_name}}{access}{member_level};
	return 0;
}
# print out error messages
sub error{
	# $message: error message
	my ($this,$message) = @_;
	return "<p>$message</p>";
}
# highlight text
sub highlightText{
	# $text: string
	my ($this,$text) = @_;
	return "<span class=\"SurveyQuestionRequired\">$text</span>";
}
# bold face text
sub boldText{
	# $text: string
	my ($this,$text) = @_;
	return $text;
#	return "<b>$text</b>";
}
# get the asterisk for required questions
sub asterisk{
	# $color: could be a color/hex code
	my ($this,$color) = @_;
	return "*";
	# red is the default color
	$color = $color || "red";
	return "<font color=\"$color\">*</font>";
}
# display survey data
sub displayData{
	my ($this,$type,$value) = @_;
	if ($type eq "answer"){
		return join(", ",split(/; /,$value));
	}elsif ($type eq "email"){
		return "<a href=\"mailto:$value\">$value</a>";
	}
	return;
}
# get labels for contact info questions
sub getContactInfoLabel{
	# $contact: contact question hash
	my ($this,$contact) = @_;
	my ($mandatory,$label,$key);
	$key = "cc#$contact->{id}";
	# find out if the question is mandatory
	$mandatory = $this->isMandatory($contact);
	$label = $contact->{label};
	$label .= ($this->{use_require} && $mandatory) ? $this->asterisk : "";
	# if the survey participant didn't answer a mandatory field, we highlight the label
	if ($mandatory && exists($this->{input}{$key})){
		if (!$this->validateContactInfo($this->{input}{$key},$contact)){
			return $this->highlightText($label);
		}
	}
	return $label;
}
# get regular question label
sub getQuestionRegularLabel{
	# $label: label
	# $stack_flag: are we stacking the question?(accept value 1 or 0)
	my ($this,$label,$stack_flag) = @_;
	# don't apply css style if we are stacking questions
	if ($stack_flag){
		return $label;
	}
	return "<span class=\"SurveyQuestion\">$label</span>";
}
# get question label
sub getQuestionLabel{
	# $question: question hash
	# $stack: # $stack_flag: are we stacking the question? (accept value 1 or 0)
	my ($this,$question,$stack_flag) = @_;
	my ($key,$mandatory,$label,$inputvalue);	
	$key = $question->{id};
	$inputvalue = $this->removeEmptyInput($this->{input}{$key});
	# find out if the question is mandatory
	$mandatory = $this->isMandatory($question);
	# insert an asterisk beside the label
	$label = $mandatory ? $this->boldText($question->{label}) : $question->{label};
	$label .= ($this->{use_require} && $mandatory) ? " ".$this->asterisk : "";
	if ($question->{type} =~ /multiple/i){
		if ($this->{input}{action} =~ /form_submit/ && !$this->validateQuestion($inputvalue,$question)){
			return $this->highlightText($label);
		}else{
			return $this->getQuestionRegularLabel($label,$stack_flag);
		}
	}else{	
		if ($this->{input}{action} =~ /form_submit/ && $mandatory && !$inputvalue){
			return $this->highlightText($label);
		}else{
			return $this->getQuestionRegularLabel($label,$stack_flag);
		}
	}
	return $label;
}
# get previously entered contact info answers from the query string
sub getDefaultContactInfoValue{
	# $contact: contact question hash
	my ($this,$contact) = @_;
	my $key = "cc#$contact->{id}";
	return $this->{input}{$key} if (exists($this->{input}{$key}));
	return "";
}
# insert an asterisk before question label
sub getQuestionAsterisk{
	# $question: question hash
	my ($this,$question) = @_;
	my ($mandatory);
	$mandatory = $this->isMandatory($question);
	return ($this->{use_require} && $mandatory) ? $this->asterisk : "&nbsp;";
}
# get previously entered question answers from the query string
sub getDefaultQuestionValue{
	# $question: question hash
	my ($this,$question) = @_;
	my ($key,$val);
	$key = $question->{id};
	return "" if (!exists($this->{input}{$key}));
	$val = $this->{input}{$key};
	# remove null inputs for checkboxes
	$val = $this->removeEmptyInput($val);
	return $val;
}
# remove null inputs for checkboxes
sub removeEmptyInput{
	# $value: answer value
	my ($this,$value) = @_;
	my (@answers,@newanswer);
	@answers = split(/; /,$value);
	if (@answers <= 1){
		return $answers[0] if (@answers == 1);
		if (@answers == 0){
			my $temp = $value;
			$temp =~ s/; //g;
			return $temp;
		}
	}
	# reconstruct answers
	foreach $a (@answers){
		push(@newanswer,$a) if $a;
	}
	return join("; ",@newanswer);
}
# survey answer lookup tabl
sub getDefaultInputHash{
	# $value: answer value
	my ($this,$value) = @_;
	my (@answers,%myHash);
	@answers = split(/; /,$value);
	foreach my $a (@answers){
		$myHash{$a} = 1;
	}
	return %myHash;
}
# get 'other' input answers
sub getDefaultOtherInput{
	# $question: question hash
	my ($this,$question) = @_;
	my (@answers,$defaultInput,$default,%default);
	$default = $this->getDefaultQuestionValue($question);
	foreach my $d (split(/; /,$default)){
		$default{$d} = 1;
	}
	@answers = split(/\|/,$question->{list});	
	foreach my $a (@answers){
		delete $default{$a} if exists($default{$a});
	}
	return join("",keys(%default));
}
# convert hash to array
sub summaryConvertHashToArray{
	# $stat: hash data
	my ($this,$stat) = @_;
	my (@result);
	foreach my $s (keys(%$stat)){
		my %temp = %{$stat->{$s}};
		$temp{label} = $s;
		push(@result,\%temp);
	}
	return @result;
}
# parse submit form data for the wizard
sub myParse{
	# $fdata: formdata
	my ($this,$fdata) = @_;
	my (%result);
	foreach my $f (keys(%$fdata)){
		next if $f !~ /(.*?)#(.*?)#(.*)/;
		# index the hash by tablename->id->column
		$result{$1}{$2}{$3} = $fdata->{$f};
	}
	return \%result;
}
# count number of questions. this count is used to set up virtual input field name
sub getRecNo{
	# $table: table
	my ($this,$table) = @_;
	my ($form,@keys);
	$form = $this->myParse($this->{input});
	foreach my $key (keys %{$form->{$table}}){
		push(@keys,substr($key,1,length($key)));
	}
	@keys = sort @keys;
	return @keys;
}
# fetch records based on the selection logics specified by the queries
sub ddsGetData{
	# $input: input data
	my ($this,$input) = @_;
	my (%data,%sort,@keys,%compare,%query,$db);
	$db = $this->{db};
	%data = %$input;
	# sort the hash
	foreach my $key (sort(keys(%data))){
		# process the filter data and ignore all other data
		next if ($key !~ /^step#/ || !$data{$key} || $key =~ /^(id|survey_id)/);
		my (undef,$step,$qid) = split(/#/,$key);
		$sort{$step}{$qid} = $data{$key};
	}
	@keys = sort(keys(%sort));
	# organize the filter data, generate sql query and fetch survey data
	for(my $i = 0; $i < @keys; $i++){
		my $count = 1;
		foreach my $key (keys(%{$sort{$keys[$i]}})){
			my ($val,@vals,@where,$sql,$fetch,$question,$index,@tmp);
			# prepare fetch data
			$val = $sort{$keys[$i]}{$key};
			$val =~ s/\s*;\s*/;/g;
			$val =~ s/\;+/;/g;
			# escape quotes for sql query
			$val =~ s/'/\\'/g;
			@vals = split(/;/,$val);
			# generate sql queries
			push(@where,"(survey_question_id = $key and (value like '%".join("%' and value like '%",@vals)."%'))");	
			$sql = "select distinct survey_registrant_id from survey_answer where ".join(" and ",@where);
			# fetch matching survey records
			$fetch = $db->custom_query($sql);
			$index = $i + 1;
			if (@$fetch > 0){
				foreach my $f (@$fetch){$compare{$index}{$count}{$f->{survey_registrant_id}} = $f;}
			}else{$compare{$index}{$count}{0} = undef;}
			# generate description of the filter data
			$question = $db->fetch("survey_question",$key);
			# remove escape characters
			foreach my $val (@vals){
				$val =~ s/\\//g;
				push(@tmp,$val);
			}
			push(@{$query{$keys[$i]}},"$question->{label} : ".join(", ",@tmp));
			$count++;
		}
	}
	return {compare=>\%compare,query=>\%query};
}
# filter answers from one step to another
sub ddsCompareResult{
	# $opt: option hash
	#	data: posted data
	#	compare: compare data
	my ($this,$opt) = @_;
	my (@result,$cmp,$input,%compare,$firstpass);
	$input = $opt->{data};
	$cmp = $opt->{compare};
	$firstpass = $opt->{firstpass};
	# exit the function if data is avaible
	return [] if (keys(%$cmp) < 1);
	# begin comparing data
	foreach my $c (keys(%$cmp)){
		# no comparison is needed if no datum is passed
		if (scalar(@result) > 0 && @$input <= 0){
			@$input = @result;
		}
		# in the very first loop, no comparison is needed
		if (scalar(@$input) <= 0 && scalar(@result) <= 0 && $firstpass){
			foreach my $id (keys(%{$cmp->{$c}})){
				next if (!$id);
				push(@result,$cmp->{$c}{$id});
			}
		}else{
			# compare data and save the matching data to a temperary variable
			@result = ();
			foreach my $i (@$input){
				$compare{$i->{survey_registrant_id}} = $i;
			}
			foreach my $cc (keys(%{$cmp->{$c}})){
				if(exists($compare{$cc})){push(@result,$cmp->{$c}{$cc});}
			}
		}
	}
	return \@result;
}
# check to see if the user provides blank inputs
sub ddsCheckInput{
	# $step: step no.
	# $input: input data
	my ($this,$step,$input) = @_;
	return 1 if (keys(%$input) < 1);
	foreach my $key (keys(%$input)){
		next if ($key !~ /^step#/);
		my (undef,$currstep,$qid) = split(/#/,$key);
		if (($step - $currstep) == 1){
			return 0 if ($input->{$key});
		}
	}
	return 1;
}
# save the custom report
sub ddsSaveReport{
	my ($this) = @_;
	my ($survey_id);
	$survey_id = $this->{input}{survey_id};
	return if (!$survey_id);
	my ($id,%insert,%survey);
	%survey = $this->{db}->fetch("survey",$survey_id);
	$this->{saved_survey_id} = $survey_id;
	# insert report info
	$id = $this->{db}->insert("survey_dds",{section_id=>$survey{section_id},survey_id=>$survey_id,
		name=>$this->{input}{report_name},description=>$this->{input}{report_desc}});
	# prepare data for insertion
	foreach my $key (keys(%{$this->{input}})){
		# only save the filter(looking for key, step#<step no.>#<answers>)
		next if ($key !~ /^step#/ || !$this->{input}{$key});
		$insert{$key} = $this->{input}{$key};
	}
	foreach my $key (sort(keys(%insert))){
		# insert each filter one by one.
		$this->{db}->insert("survey_dds_data",{survey_dds_id=>$id,data_key=>$key,data_value=>$insert{$key}});
	}
	foreach my $key (keys(%{$this->{input}})){
		next if $key =~ /^_(.*)/;
		delete $this->{input}{$key};
	}
	$this->{input}{action} = "dds";
	$this->{input}{section_id} = $survey{section_id};
	return;
}
# gather survey responses for a participant
sub viewSingleSurveyResponse{
	my ($this,$surveyid,$userid) = @_;
	my (@contact_answers,@answers,@contacts,%contacts,@questions,%questions,@results,%registrant);
	%registrant = $this->{db}->fetch("survey_registrant",$userid);
	@contacts = $this->{db}->fetch_match("survey_contact_question",{survey_id=>$surveyid},"id asc");
	@questions = $this->{db}->fetch_match("survey_question",{survey_id=>$surveyid},"id asc");
	@questions = @{$this->addQuestionSectionInfo($surveyid,\@questions)};
	@contact_answers = $this->{db}->fetch_match("survey_contact_answer",{survey_registrant_id=>$userid},"survey_contact_question_id asc");
	@answers = $this->{db}->fetch_match("survey_answer",{survey_registrant_id=>$userid},"survey_question_id asc");
	foreach my $contact(@contacts){
		$contacts{$contact->{id}} = $contact;
	}
	foreach my $question(@questions){
		$questions{$question->{id}} = $question;
	}
	@results = (
		{label=>"Contact Info",data=>[]},
		{label=>"Response",data=>[]},
	);
	foreach my $contact_answer(@contact_answers){
		my $value = $contact_answer->{value} || "n/a";
		if ($contacts{$contact_answer->{survey_contact_question_id}}{label} =~ /^email$/i && $this->validateEmail($value)){
			$value = $this->displayData("email",$value);
		}
		push(@{$results[0]->{data}},{label=>$contacts{$contact_answer->{survey_contact_question_id}}{label},value=>$value});
	}
	foreach my $answer(@answers){
		my ($value,$label);
		$value = $this->displayData("answer",$answer->{value}) || "n/a";		
		$label = $questions{$answer->{survey_question_id}}{section_label} ?  "$questions{$answer->{survey_question_id}}{section_label}<br>" : "";
		$label .= $questions{$answer->{survey_question_id}}{label};
		push(@{$results[1]->{data}},{label=>$label,value=>$value});
	}
	return $this->html_viewSingleSurveyResponse(\@results,$surveyid);
}
# get form type(ie. survey, registration..etc)
sub getFormType{
	my ($this) = @_;
	return $this->{form_type};
}
# gather brief summary data
sub constructBriefSummaryData{
	my ($this,$reg_count,$question_count) = @_;
	my @list = (
		{label=>"Total Participants",value=>"$reg_count"},
		{label=>"Total Questions",value=>"$question_count"},
	);
	return \@list;
}
# create brief summary
sub briefSummary{
	my ($this,$surveyid) = @_;
	my (@registrants,@questions);
	@registrants = $this->{db}->fetch_match("survey_registrant",{survey_id=>$surveyid});
	@questions = $this->{db}->fetch_match("survey_question",{survey_id=>$surveyid});	
	return $this->html_briefSummary($this->constructBriefSummaryData(scalar(@registrants),scalar(@questions)));
}
# show a list of reports
sub showReportList{
	my ($this,$surveyid) = @_;
	return $this->error("Invalid ".$this->getModuleName." id") if !$surveyid;
	my %survey = $this->{db}->fetch("survey",$surveyid);
	$this->surveyInit(\%survey);
	return $this->error("Sorry, there are no reports available. This ".$this->getModuleName." is configured not to save responses in the database.") if !$this->saveResponse();
	return $this->reportList($surveyid);
	
}
# create 'view report' screen
sub reportList{
	my ($this,$surveyid) = @_;
	return $this->error("Invalid ".$this->getModuleName." id") if !$surveyid;
	my %survey = $this->{db}->fetch("survey",$surveyid);
	my $out = $this->html_javascript("reportlist",{sid=>$surveyid,action=>"view_report",table=>"survey_dds"});
	my %report = (
		data=>$this->briefSummary($surveyid),
		foot=>$this->listAllReport($surveyid),
	);
	$out .= "\n<h1>$survey{name} Reports</h1>\n";
	$out .= $this->reportbuilder(\%report);
	return $out;
}
# get default reports
sub getDefaultReport{
	my ($this,$surveyid) = @_;
	my @list = (
		{label=>"Participant",description=>"List Participants",url=>$this->myRelink({action=>"view_reg_list",sid=>$surveyid})},
		{label=>"Summary",description=>"Summary Reports",url=>$this->myRelink({action=>"view_summary",sid=>$surveyid})},
	);
	return \@list;
}

# list all reports associated to a survey
sub listAllReport{
	my ($this,$surveyid) = @_;
	my (@custom_reports,@rlist,$out);
	my @rlist = @{$this->getDefaultReport($surveyid)};
	$out .= "\n<p>\n";
	$out .= $this->html_listAllReport(\@rlist,{title=>"Standard Reports"});
	$out .= "\n</p>\n";
	@rlist = ();
	@custom_reports = $this->{db}->fetch_match("survey_dds",{survey_id=>$surveyid});
	foreach my $report(@custom_reports){
		push(@rlist,{
			label=>($report->{name} || "unknown"),
			description=>($report->{description} || "n/a"),
			target=>"_blank",
			url=>$this->myRelink({action=>"dds_action",dds_action=>"view",dds_id=>$report->{id}}),
			delete_url=>"javascript:deleteRecord($report->{id})",
			img=>"delete",
		});
	}
	$out .= "\n<p>\n";
	$out .= $this->html_listAllReport(\@rlist,{tool=>$this->html_reportAddNewLink($surveyid),title=>"Custom Reports"});
	$out .= "\n</p>\n";
	return $out;
}
# export registrant list to excel
sub exportRegistrantList{
	my ($this,$surveyid) = @_;
	return $this->error("Invalid id.") if !$surveyid;
	my ($contactinfo,$questions,$registrants,$filename,@column,%heading,@completelist);
	# fetch all survey questions
	$questions = $this->getSurveyData($surveyid,"survey_question");
	# fetch all contact info 
	$contactinfo = $this->getSurveyData($surveyid,"survey_contact_question");
	# insert survey answer and contact info into the registrant records
	$registrants = $this->fetchRegistrant($surveyid,$contactinfo,$questions);
	# Convert date
	for(my $i = 0; $i < @$registrants; $i++){
		$registrants->[$i]{last_update} = $this->{db}->show_data("survey_registrant","last_update",$registrants->[$i]{last_update});
	}
	# filename for exporting spreadsheet
	$filename = $this->{registrant_list_excel_filename};
	foreach my $section({table=>"contact",data=>$contactinfo},{table=>"question",data=>$questions}){
		foreach my $list(@{$section->{data}}){
			my $col = $section->{table}."_".$list->{id};
			# spreadsheet columns
			push(@column,$col);
			# spreadsheet column labels
			$heading{$col} = $list->{label};
		}		
	}
	push(@column,"last_update");
	$heading{last_update} = "Date";
	print "content-type: application/vnd.ms-excel\ncontent-disposition: attachment; filename=$filename\n\n";
	# export!
	print $this->{db}->export({table=>"survey_registrant",format=>"excel",data=>$registrants,column=>\@column,heading=>\%heading});
	return;
}
# fetch all registrant related info
sub fetchRegistrant{
	my ($this,$surveyid,$contactinfo,$questions) = @_;
	my ($regs,@questions,$answers,$sql,$count,%answers,$appendkey,@contactinf,$contact_answers,%contact_answers);
	# fetch all registrant
	$regs = $this->{db}->fetch_match("survey_registrant",{survey_id=>$surveyid});
	# index survey question
	for(my $i = 0; $i < @$questions; $i++){
		push(@questions,$questions->[$i]{id});
	}
	# index survey contact question
	for(my $i = 0; $i < @$contactinfo; $i++){
		push(@contactinf,$contactinfo->[$i]{id});
	}
	if (@$questions){
		# fetch survey answers
		$sql = "select * from survey_answer where survey_question_id = ".join(" or survey_question_id = ",@questions);
		$answers = $this->{db}->custom_query($sql);
	}
	if (@$contactinfo){
		# fetch survey contact info
		$sql = "select * from survey_contact_answer where survey_contact_question_id = ".join(" or survey_contact_question_id = ",@contactinf);
		$contact_answers = $this->{db}->custom_query($sql);
	}
	# reindex survey answers
	foreach my $ans(@$answers){
		$answers{$ans->{survey_registrant_id}}{$ans->{survey_question_id}} = $ans;
	}
	# reindex survey contact answers
	foreach my $ans(@$contact_answers){
		$contact_answers{$ans->{survey_registrant_id}}{$ans->{survey_contact_question_id}} = $ans;
	}
	for(my $i = 0; $i < @$regs; $i++){
		# registrant id
		my $regid = $regs->[$i]{id};		
		foreach my $key(keys(%{$answers{$regid}})){			
			my ($ansval,$newkey);
			$ansval = $answers{$regid}{$key}{value};
			$appendkey = "question_";
			# create a survey question hash key for the registrant record
			$newkey = $appendkey.$answers{$regid}{$key}{survey_question_id};
			# insert the new key and value into the registrant recrod
			$regs->[$i]{$newkey} = exists($answers{$regid}{$key}) ? $this->displayData("answer",$ansval) : "n/a";
		}
		foreach my $key(keys(%{$contact_answers{$regid}})){			
			my ($ansval,$newkey);
			$ansval = $contact_answers{$regid}{$key}{value};
			$appendkey = "contact_";
			# create a contact info hash key for the registrant record
			$newkey = $appendkey.$contact_answers{$regid}{$key}{survey_contact_question_id};
			# insert the new key and value into the registrant recrod
			$regs->[$i]{$newkey} = exists($contact_answers{$regid}{$key}) ? $this->displayData("answer",$ansval) : "n/a";
		}
	}
	return $regs;
}
# participant/registrant list report title
sub getRegistrantListTitle{
	my ($this) = @_;
	return "Participants";
}
# show additional columns on the participant/registrant list report
sub addAdditionalColumn{
	my ($this,$surveyid) = @_;
	my ($append_string,$records,$additional_column,$loop,@addcol,@record);
	$additional_column = $this->{registrant_list_additional_column} || undef;
	$append_string = "contact_";
	$records = $this->getSurveyData($surveyid,"survey_contact_question");
	# Don't show the name field specified in the conf file
	my $skipcolumn = "";
	if ($this->getNameSearchField()){
		$skipcolumn = "|".$this->getNameSearchField();
	}
	# first name, last name and email are shown by default
	foreach my $rec (@$records){
		next if ($rec->{label} =~ /(first|last) name|email${skipcolumn}/i);
		push(@record,$rec);
	}
	# determine the number of additional column to show
	$loop = !$additional_column || $additional_column > @record ? @record : $additional_column;
	for(my $i = 0; $i < $loop; $i++){
		my $colname = $append_string.$record[$i]{id};
		push(@addcol,{label=>$record[$i]{label},column=>$colname});
	}
	return @addcol;
}
# get registrant records
sub getRegistrantListRecord{
	my ($this,$surveyid) = @_;
	my ($records,$registrants);
	$records = $this->{db}->fetch_match("survey_contact_question",{survey_id=>$surveyid});
	$registrants = $this->fetchRegistrant($surveyid,$records,[]);
	return $registrants;
}
# report top link
sub reportTopLink{
	my ($this,$surveyid) = @_;
	my $url = $this->myRelink({sid=>$surveyid,action=>"view_report"});
	my @links = (
	#{label=>"Go back to report menu",url=>$url},
	{label=>"Go back to report menu",url=>"javascript:history.back()"},
	);
	return \@links;
}
# report menu
sub reportMenu{
	my ($this,$surveyid) = @_;
	my $out;
	my @commands = @{$this->reportTopLink()};
	foreach my $command(@commands){
		$out .= "[<a href=\"$command->{url}\">$command->{label}</a>]";
	}	
	return $out;
}

# view list of registrant
sub viewRegistrantList{
	my ($this,$surveyid) = @_;	
	return $this->error("Invalid id.") if !$surveyid;
	my $registrants = $this->getRegistrantListRecord($surveyid);
	# report header columns
	my @cols = (	
		{label=>"Option",column=>"option_command"},
		{label=>"No",column=>"no"},
		{label=>"Name",column=>"fullname"},
		{label=>"Email",column=>"email"},	
	);
	@cols = (@cols,$this->addAdditionalColumn($surveyid));
	# javascript
	my $out .= $this->html_javascript("reportlist",{action=>"view_reg_list",sid=>$surveyid,table=>"survey_registrant"});
	# report menu
	$out .= $this->reportMenu($surveyid);
	
	# gather all registrant data
	for(my $i = 0; $i < @$registrants; $i++){		
		my $reg = $registrants->[$i];
		my $name = $reg->{first}." ".$reg->{last};
		my $email = $reg->{email} || "n/a";
		# create email link
		$email = $this->validateEmail($email) ? $this->displayData("email",$email) : $email;
		my %myHash = (action=>"view_user",uid=>$reg->{id},sid=>$surveyid);
		# view registrant's details
		my $link = "<a href=\"".&relink(\%myHash)."\">".$this->getIcon("view","View ${name}'s info")."</a>";
		# delete registrant
		$link .= "<a href=\"javascript:deleteRecord($reg->{id})\">".$this->getIcon("delete","Delete ${name}")."</a>";		
		$reg->{option_command} = $link;
		$reg->{no} = $i + 1;
		$reg->{fullname} = $name;
		$reg->{email} = $email;
	}

	my $path = $this->myRelink({sid=>$surveyid,action=>"excel_user"});
	my $tool = "<a href=\"".$path."\">".$this->getIcon("excel","Export to Excel")."</a>";		
	my $title = $this->getRegistrantListTitle();
	my ($header,$data) = $this->getReportbuilderData(\@cols,$registrants);
	my %report = (
		title=>$title,
		data=>$data,
		headers=>$header,
		tools=>$tool,
	);
	# build report
	$out .= $this->reportbuilder(\%report);
	$out .= "\n";
	return $out;
}
# reformat the answer choices for survey question
sub stripChrQuestionAnswerChoice{
	my ($this,$str) = @_;
	# strip spaces around the "|" character
	$str =~ s/\s*\|\s*/\|/g;
	# escape quotation marks
	$str =~ s/\"/&quote;/g;
	# remove null answer choices e.g. "|||||";
	my (@choices,@newchoices,$newstr);
	@choices = split(/\|/,$str);
	foreach my $choice(@choices){
		next if !$choice;
		push(@newchoices,$choice);
	}
	$newstr = join("|",@newchoices);
	return $newstr;
}
# strip return characters
sub stripChr{
	my ($this,$str) = @_;
	$str =~ s/(\r)?\n//g;
	return $str;
}
# get form footer
sub getFormFooter{
	my ($this,$survey) = @_;
	my $out;
	if ($survey->{footer_html}){
		$out .= "<br>".$survey->{footer_html};
	}
	return $out;
}
# get form header
sub getFormHeader{
	my ($this,$survey) = @_;
	my $out;	
	$out .= $survey->{header_html} || "\n<h2>$survey->{name}</h2>\n<p>$survey->{description}</p>\n";
	return $out;
}
# Check and see if we are using a multi column display for a vertical checkbox/radio buttonq uestion
sub useAlternativeVerticalDisplay{
	my ($this,$count) = @_;
	my $limit = $config{$this->{class_name}}{config}{question}{special_format_vertical_record} || 10;
	return 1 if $limit <= $count;
	return 0;
}
#------------------------------------------------------------------
# all sub's names begin with html_xxxxx are mainly used to generate html outputs..

# add new custom report link
sub html_reportAddNewLink{
	my ($this,$surveyid) = @_;
	my (@links,$out);
	@links = (
		{label=>"Add new custom report",url=>$this->myRelink({action=>"dds_action",dds_action=>"step",step=>0,survey_id=>$surveyid}),img=>"new_report"}
	);
	foreach my $link (@links){
		$out .= "<a href=\"$link->{url}\">".$this->getIcon($link->{img},$link->{label})."</a>";
	}
	return $out;
}

# list all reports
sub html_listAllReport{
	# $list: records
	# $opt(hash reference)
	#	title=> report title
	#	type => "tool"(content for toolbar section of the report)
	my ($this,$list,$opt) = @_;
	my @data;
	my @header = ("Name","Description","Options");
	foreach my $list(@$list){
		# view report link
		my $links = "<a href=\"$list->{url}\"";
		$links .= $list->{target} ? " target=\"$list->{target}\"" : "";
		$links .= ">";
		$links .= $this->getIcon("view","View report: $list->{label}");
		$links .= "</a>";
		# delete link
		if ($list->{delete_url}){
			$links .= "<a href=\"$list->{delete_url}\">";
			$links .= $this->getIcon($list->{img},"Delete report: ".$list->{label});
			$links .= "</a>";
		}		
		push(@data,[$list->{label},$list->{description},$links]);
	}
	my %report = (
		title=>$opt->{title},
		width=>"100%",
		headers=>\@header,
		data=>\@data,
		tools=>$opt->{tool},
	);
	return $this->reportbuilder(\%report);
}
# brief survey summary
sub html_briefSummary{
	my ($this,$sdata) = @_;
	my @header;
	my @data;
	foreach my $sdat(@$sdata){
		push(@header,$sdat->{label});
		push(@data,$sdat->{value});
	}
	return $this->reportbuilder({headers=>\@header,data=>\@data,width=>"100%"});
}
sub getColumnMap{
	my ($this,$table) = @_;
	my $ncol = $this->{db}->{map}->get_ncol($table);
	my @colmap;
      	for (my $icol = 0; $icol < $ncol; $icol++) {
		my %colmap = $this->{db}->{map}->get_column($table,$icol);
		push @colmap, \%colmap;
	}
	return \@colmap;	
}
# 'reconfigure' screen
sub html_adminFormMain{
	my ($this,$opt) = @_;
	my $surveyid = $opt->{surveyid};
	my %survey = $this->{db}->fetch("survey",$surveyid);
	my $editurl = $opt->{edit};
	my $newurl = $opt->{new};
	my $delurl = $opt->{delete};
	$this->surveyInit;
	my $module = $this->getModuleName;	
	my @data;
	my @colmap = @{$this->getColumnMap("survey")};
	my @header;
	my $foot;
	foreach my $col (@colmap){
		next if ($this->{db}->authorize() < $col->{read});
		push(@header,$col->{label});
		push(@data,$this->{db}->show_data("survey",$col->{column},$survey{$col->{column}},$surveyid));
	}		

	# survey info
	my $contact_info_rec = $this->{db}->fetch_match("survey_contact_question",{survey_id=>$surveyid},"id asc");

	$foot .= "<p>";
	$foot .= $this->{db}->report({title=>"$module Contact Information",table=>"survey_contact_question",data=>$contact_info_rec,edit=>$editurl,delete=>$delurl,link=>1,new=>$newurl});
	$foot .= "</p>";
	
	# survey sections	
	if ($survey{use_group} =~ /$this->{yes}/i){
		my $section_rec = $this->{db}->fetch_match("survey_section",{survey_id=>$surveyid},"id asc");
		$foot .= "<p>";
		$foot .= $this->{db}->report({title=>"$module Sections",table=>"survey_section",data=>$section_rec,edit=>$editurl,delete=>$delurl,link=>1,new=>$newurl});
		$foot .= "</p>";
	}
	
	# survey questions
	my $question_rec = $this->{db}->fetch_match("survey_question",{survey_id=>$surveyid},"survey_section_id asc, id asc");
	$foot .= "<p>";
	$foot .= $this->{db}->report({title=>"$module Questions",table=>"survey_question",data=>$question_rec,edit=>$editurl,delete=>$delurl,link=>1,new=>$newurl});
	$foot .= "</p>";
	
	# build the report
	my %report = (
		title=>"$module Form Information",
		data=>\@data,
		tools=>$this->getAdminOption($surveyid),
		headers=>\@header,
		foot=>$foot,
	);
	return $this->reportbuilder(\%report);
}

# admin commands for editing a survye
sub html_adminOption{
	my ($this,$commands) = @_;
	my ($out);
	foreach my $cmd(@$commands){
		$out .= "<a href=\"$cmd->{url}\"";
		$out .= $cmd->{target} ? " target=\"$cmd->{target}\"" : "";
		$out .= ">".$this->getIcon($cmd->{img},$cmd->{label})."</a>";
	}
	return $out;
}
# view summary for individual participant
sub html_viewSingleSurveyResponse{
	my ($this,$results,$surveyid) = @_;
	my $out = $this->reportMenu($surveyid);
	my $body;
	foreach my $result (@$results){
		next if @{$result->{data}} < 1;
		my (@header,@data);
		my $title = $result->{label};
		foreach my $dat (@{$result->{data}}){
			push(@header,$dat->{label});
			push(@data,$dat->{value});			
		}
		my %report = (
			title=>$title,
			data=>\@data,
			headers=>\@header,
			width=>"100%",
		);
		$body .= $this->reportbuilder(\%report);
	}
	$out .= "\n<h1>"."View ".$this->getModuleName." responses</h1>";
	$out .= "\n<table><tr><td>$body</td></tr></table>\n";
	return $out;
}
# survey section fields on the question wizard form
sub html_getSurveySection{
	# $newval: virtual key for input fields in the wizard
	# $select: selected value
	my ($this,$newval,$select) = @_;
	my ($out,$form,%input,$module);
	$module = $this->getModuleName;
	%input = %{$this->{input}};
	$out .= "<p>$module Section(used to group ".lc($module)." questions)<br>\n";
	$out .= "<select name=\"survey_question#_$newval#survey_section_id\">\n";
	$out .= "<option value=\"\">Select $module section</option>\n";
	$form = $this->myParse($this->{input});
	foreach my $t (keys %{$form->{"survey_section"}}){
		$out .= "<option value=\"$t\"";
		$out .=  $t eq $select ? " selected" : "";
		$out .= ">".$input{"survey_section#$t#label"}."</option>\n";
	}
	$out .= "</select>\n</p>\n";
	return $out;
}
# question type drop down list
sub html_getQuestionType{
	# $question: question hash
	my ($this,$question) = @_;
	if ($question =~ /^multiple/i){
		$question .= " (multiple checkboxes)";
	}elsif ($question =~ /^checkbox/i){
		$question .= " (single checkbox)";
	}elsif ($question =~ /list/i){
		$question .= " (drop-down list)";
	}elsif ($question =~ /short/i){
		$question .= " (text input)";
	}elsif ($question =~ /^long/i){
		$question .= " (textarea)";
	}elsif ($question =~ /radio/i){
		$question .= " (radio buttons)";
	}
	return $question;
}
# survey wizard screen 4
sub html_wizardPage4{
	# $opt: option hash:
	my ($this,$opt) = @_;
	my ($out,$recno,$newval,@types,%input,$module);
	$module = $this->getModuleName;
	%input = %{$this->{input}};
	$recno = $this->getRecNo("survey_question");
	$newval =  1 + $recno;
	$out .= "<h3>New $module Question</h3>\n";
	if ($input{"survey#_#use_group"} =~ /y/i){
		$out .= $this->html_getSurveySection($newval);
	}
	$out .= "<p>Question:<br>\n
	<input type=\"text\" name=\"survey_question#_$newval#label\" size=\"60\" value=\"\"></p>\n";
	$out .= "<p>Question type:<br>\n";
    	@types = split(/\|/,$this->{db}->{map}->regexp("list:survey_questiontype"));
	$out .= "<select name=\"survey_question#_$newval#type\">\n";
	$out .= "<option value=\"\">Select question type</option>\n";
	# a list of available question types
	foreach my $t (@types){
		$out .= "<option value=\"$t\">";
		$out .= $this->html_getQuestionType($t);
		$out .= "</option>\n";
	}
	$out .= "</select></p>\n";
	$out .= "<p>Answer choices:<br>(To enter multiple choices, each choice is separated using a \"|\" character.  e.g. \"choice1|choic2|choice3\")<br>\n
	<textarea name=\"survey_question#_$newval#list\" rows=\"4\" cols=\"60\"></textarea></P>\n";
	$out .= "<p>How would you like to display the answer choices? (for multiple checkboxes and radio button questions only)<br>\n";
	$out .= "<input type=\"radio\" name=\"survey_question#_$newval#display_style\" value=\"vertical\">Vertical&nbsp;&nbsp;
	<input type=\"radio\" name=\"survey_question#_$newval#display_style\" value=\"horizontal\">Horizontal&nbsp;&nbsp;";
	$out .= "<p>Selection limits (for multiple checkboxes questions only):<br>\n
	<input type=\"text\" name=\"survey_question#_$newval#multicheck\" size=\"10\" value=\"\"></p>\n";
	$out .= "<p>Include an \"Other\" input field for multiple checkboxes or listbox question<br>\n
	<input type=\"radio\" name=\"survey_question#_$newval#other\" value=\"Y\">Y&nbsp;&nbsp;
	<input type=\"radio\" name=\"survey_question#_$newval#other\" value=\"N\">N&nbsp;&nbsp;</p>\n";
	$out .= "<p>Is this a mandatory question?<br>\n
	<input type=\"radio\" name=\"survey_question#_$newval#mandatory\" value=\"Y\">Y&nbsp;&nbsp;
	<input type=\"radio\" name=\"survey_question#_$newval#mandatory\" value=\"N\">N&nbsp;&nbsp;</p>\n";
	$out .= "<hr>\n";
	$out .= "<p><b>Do you want to add another survey question?</b><br>\n
	<input type=\"radio\" name=\"anotherone\" value=\"Y\">Y&nbsp;&nbsp;
	<input type=\"radio\" name=\"anotherone\" value=\"N\">N&nbsp;&nbsp;</p>\n";
	$out .= $this->wizardIncludeInput;
	return $out;
}
# survey wizard screen 3
sub html_wizardPage3{
	# $opt: option hash
	my ($this,$opt) = @_;
	my (%input,$module);
	$module = $this->getModuleName;
	%input = %{$this->{input}};
    	if ($input{"survey#_#use_group"} =~ /y/i){
    		my ($out,$recno,$newval,$table);
		$table = "survey_section";
		$recno = $this->getRecNo($table);
		$newval =  1 + $recno;
		$out .= "<h3>New $module Section</h3>\n";
    		$out .= "<P>Label:<br>\n
		<INPUT TYPE=\"text\" NAME=\"$table#_$newval#label\" SIZE=\"60\" VALUE=\"\"></P>\n";
		$out .= "<hr>\n";
		$out .= "<p><b>Do you want to add another ".lc($module)." section?</b><br>\n
		<input type=\"radio\" name=\"anotherone\" value=\"Y\">Y&nbsp;&nbsp;
		<input type=\"radio\" name=\"anotherone\" value=\"N\">N&nbsp;&nbsp</p>\n";
	    	$out .= $this->wizardIncludeInput;
    		return $out;
	}else{
		$this->{screen}++;
		return $this->html_wizardPage4($opt);
	}
	return;
}
# survey wizard screen 2
sub html_wizardPage2{
	# $opt: option hash
	my ($this,$opt) = @_;
	my (%input,$db);
	$db = $this->{db};
	%input = %{$this->{input}};
	if ($input{'customq'} =~ /y/i){
    		my ($out,$recno,$newval,@types,$table);
		$table = "survey_contact_question";
		$recno = $this->getRecNo($table);
		$newval =  1 + $recno;
		$out .= "<h3>New Custom Input Field</h3>\n";
    		$out .= "<p>Label for input field:<br>\n
		<input type=\"text\" name=\"$table#_$newval#label\" SIZE=\"60\" VALUE=\"\"></P>\n";
		$out .= "<p>Type of input field<br>\n";
	    	@types = split(/\|/, $db->{map}->regexp("list:survey_contacttype"));
		$out .= "<select name=\"$table#_$newval#type\">\n";
		$out .= "<option value=\"\">Select type of input field</option>\n";
		foreach my $t (@types){
			$out .= "<option value=\"$t\">$t</option>\n";
		}
		$out .= "</select></p>\n";
		$out .= "<p>Is this a mandatory field?<br>\n
		<input type=\"radio\" name=\"$table#_$newval#mandatory\" value=\"Y\">Y&nbsp;&nbsp;
		<input type=\"radio\" name=\"$table#_$newval#mandatory\" value=\"N\">N&nbsp;&nbsp</p>\n";
		$out .= "<hr>\n";
		$out .= "<p><b>Do you want to add another custom input field?</b><br>\n
		<input type=\"radio\" name=\"anotherone\" value=\"Y\">Y&nbsp;&nbsp;
		<input type=\"radio\" name=\"anotherone\" value=\"N\">N&nbsp;&nbsp</p>\n";
	    	$out .= $this->wizardIncludeInput;
    		return $out;
	}elsif ($input{"survey#_#use_group"} =~ /y/i){
		$this->{screen}++;
		return $this->html_wizardPage3($opt);
	}else{
		$this->{screen} += 2;
		return $this->html_wizardPage4($opt);
	}
	return;
}
# survey wizard screen 1
sub html_wizardPage1{
	# $opt: option hash
	my ($this,$opt) = @_;	
	my ($out,@types,$contact,$module,$formtype);
	$formtype = $this->getFormType;
	$module = $this->getModuleName;
	$contact = $opt->{contact};
	$out .= "<h2>Part 1: $module Settings</h2>";
	$out .= "<p>Title of $module:<br>\n
	<input type=\"text\" name=\"survey#_#name\" size=\"60\" value=\"\"></p>\n
	<p>$module Description:<br>\n
	<textarea name=\"survey#_#description\" rows=\"4\" cols=\"60\"></textarea></p>\n";
	$out .= "<p>Display '<sub><font color=\"red\" size=\"+1\">*</font></sub>' in required questions?<br>\n
	<input type=\"radio\" name=\"survey#_#use_require\" value=\"Yes\">Yes&nbsp;&nbsp;
	<input type=\"radio\" name=\"survey#_#use_require\" value=\"No\">No&nbsp;&nbsp</p>\n";
	$out .= "<p>Do you want to group questions into sections?<br>\n
	<input type=\"radio\" name=\"survey#_#use_group\" value=\"Yes\">Yes&nbsp;&nbsp;
	<input type=\"radio\" name=\"survey#_#use_group\" value=\"No\">No&nbsp;&nbsp</p>\n";
	$out .= "<p>Do you want to number every single question?<br>\n
	<input type=\"radio\" name=\"survey#_#use_number\" value=\"Yes\">Yes&nbsp;&nbsp;
	<input type=\"radio\" name=\"survey#_#use_number\" value=\"No\">No&nbsp;&nbsp</p>\n";
	$out .= "<p>Custom HTML header:<br>\n
	<textarea name=\"survey#_#header_html\" rows=\"4\" COLS=\"60\"></textarea></p>\n";
	$out .= "<p>Custom HTML footer:<br>\n
	<textarea name=\"survey#_#footer_html\" rows=\"4\" cols=\"60\"></textarea></p>\n";
	$out .= "<p>Thank you message:<br>\n
	<textarea name=\"survey#_#thankyou_message\" rows=\"4\" cols=\"60\"></textarea></p>\n";
	$out .= "<p>A list of email address to send $module results to:<br>\n
	<input type=\"text\" name=\"survey#_#email\" size=\"60\" value=\"\"></p>\n";
	$out .= "<P>Redirect message:<br>\n
	<textarea name=\"survey#_#return_message\" rows=\"4\" cols=\"60\"></textarea></P>\n";
	$out .= "<P>Redirect Url:<br>\n
	<input type=\"text\" name=\"survey#_#return_url\" size=\"60\" value=\"\"></P>\n";
	$out .= "<h2>Part 2: Contact Information</h2>";
	$out .= $this->html_wizardContactInfo($contact);
	$out .= "<input type=\"hidden\" name=\"action\" value=\"wizard\">";
	$out .= "<p>Do you want to add custom contact information?<br>\n
	<input type=\"radio\" name=\"customq\" value=\"Yes\">Yes&nbsp;&nbsp;
	<input type=\"radio\" name=\"customq\" value=\"No\">No&nbsp;&nbsp</p>\n";
	$out .= "<p>Do you want Contact Information displayed at the beginning or end of the $module?<br>\n
	<input type=\"radio\" name=\"survey#_#contactinfo_pos\" value=\"beginning\">Beginning&nbsp;&nbsp;
	<input type=\"radio\" name=\"survey#_#contactinfo_pos\" value=\"end\">End&nbsp;&nbsp</p>
	<input type=\"hidden\" name=\"survey#_#section_id\" value=\"$opt->{section_id}\">\n";	
	$out .= "<input type=\"hidden\" name=\"survey#_#type\" value=\"$formtype\">\n";
    	return $out;
}
# survey wizard contact form
sub html_wizardContactInfo{
	# $contact: contact info question hash
	my ($this,$contact) = @_;
	my ($out);
	$out .= "\n<table border=\"0\" cellspacing=\"5\">";
	$out .= "\n<tr><td width=\"300\">Select Contact Info  Fields</td><td colspan=\"2\" align=\"center\">Make the field mandatory?</td></tr>";
	$out .= "\n<tr><td>&nbsp;</td><td align=\"center\">Yes</td><td align=\"center\">No</td></tr>";
	for(my $i = 0; $i < @$contact; $i++){
		# virtual key for wizard input fields
		my $vkey = $i + 1;
		$out .= "\n<tr>";
		$out .= "<td><input type=\"checkbox\" name=\"checkbox_${vkey}\" value=\"$vkey\"";
		# by default first name, last name and email are checked
		$out .= $contact->[$i]{checked} ? " checked" : "";
		$out .= "> ";
		$out .= "$contact->[$i]{name}</td>";
		$out .= "<td align=\"center\"><input type=\"radio\" name=\"mandatory_${vkey}\" value=\"Yes\"";
		$out .= $contact->[$i]{mandatory} ? " checked" : "";
		$out .= "></td>";
		$out .= "<td align=\"center\"><input type=\"radio\" name=\"mandatory_${vkey}\" value=\"No\"";
		$out .= !$contact->[$i]{mandatory} ? " checked" : "";
		$out .= "></td>";
		$out .= "</tr>";
	}
	$out .= "\n</table>";
	return $out;
}
# survey wizard form header
sub html_wizardStartForm{
	# $title: title
	# $actionurl: form's action url
	my ($this,$title,$actionurl) = @_;
	my ($out,$step);
	$step = $this + 1;
	$out = "<h2>$title</h2>\n";
	$out .= "\n<form action=\"$actionurl\" method=\"post\">\n";
	return $out;
}
# survey wizard form footer
sub html_wizardFinishForm{
	# $screen: current screen number
	# $show_step: number of steps/screens in the wizard
	my ($this,$screen,$show_step) = @_;
	my ($out,$next);
	$out = "\n<input type=\"hidden\" name=\"screen\" value=\"$screen\">\n"; 
	$next = $screen == @$show_step ? "Next &gt; " : "Next &gt;";
	$out .= $screen > 0 ? "\n<input type=\"button\" value=\"&lt; Back\" onclick=\"history.back()\">&nbsp;&nbsp;" : "";
	$out .= "\n<input type=\"submit\" name=\"submit_action\" value=\"$next\">&nbsp;&nbsp;";
	$out .= "\n<input type=\"submit\" name=\"submit_action\" value=\"Cancel\">&nbsp;&nbsp;";
	$out .= "\n<input type=\"reset\">&nbsp;&nbsp;";
	$out .= "\n</form>\n";
	return $out;
}
# survey wizard menu
sub html_wizardMenu {
	# $screen: screen number/step number
	# $step_name: name of the steps
	my ($this,$screen,$step_name) = @_;
	my ($out);
	$out = "<ol>\n";
	for (my $istep = 0; $istep <= @$step_name; $istep++) {
	    $out .= "<li> ";
	    $out .= $istep == $screen ? "<b>" : "";
	    $out .= "$step_name->[$istep]";
	    $out .= $istep == $screen ? "</b>" : "";
	    $out .= "</li>\n";
	}
	$out .= "</ol>\n";
	return $out;
}
# survey wizard hidden inputs
sub html_wizardIncludeInput{
	# $key: key
	# $val: value
	my ($this,$key,$val) = @_;
	return "\n<input type=\"hidden\" name=\"$key\" value=\"$val\">";
}
# survey wizard finish screen
sub html_wizardFinishScreen{
	# $title: title
	my ($this,$title) = @_;
	my ($out,$module);
	$module = $this->getModuleName;
	$out = "\n<H2>$title Complete!</H2>\n";
	$out .= "\n<script langauge=\"javascript\">";
	$out .= "\n<!--";
	$out .= "\nfunction openWindow(){";
	$out .= "\nvar NewWindow = window.open(\"".&relink({action=>"do",sid=>$this->{new_surveyid}})."\",'','height=300,width=500,scrollbars,resizable');";
	$out .= "\nreturn;";
	$out .= "\n}";
	$out .= "\nfunction controlpanel(){";
	$out .= "\ntop.location.href = '$config{server}{CGIpath}/ctrl-panel.cgi/$this->{class_name}';";
	$out .= "\nreturn;";
	$out .= "\n}";
	$out .= "\n//-->";
	$out .= "\n</script>";
	$out .= "\n<form>";
	$out .= "<input type=\"button\" value=\"Preview $module\" onClick=\"openWindow();\">";
	$out .= "&nbsp;&nbsp;&nbsp;";
	$out .= "<input type=\"button\" value=\"Back to control panel\" onClick=\"controlpanel();\">";
	$out .= "</form>";
	return $out;
}
# survey wizard screen 1
sub html_adminForm{
	# $surveyid: survey id
	my ($this,$sectionid,$surveyid) = @_;	
	return $this->error("Missing inputs") if (!$sectionid && !$surveyid);
	my $table = "survey";
	my %survey = $this->{db}->fetch($table,$surveyid) if $surveyid;
	$sectionid = $survey{section_id} || $sectionid;
	$this->adminFormInit;
	my $formtype = $this->getFormType;
	my $module = $this->getModuleName;
	my $contact = $this->{contact};
	my $pkey = $surveyid || "_";
	my $out = "\n<form method=\"get\" action=\"".&relink({action=>undef,sid=>undef})."\">";
	$out .= "\n<h2>Part 1: $module Settings</h2>";
	$out .= "\n<p>Title of $module:".$this->helplink($table,"name")."<br><input type=\"text\" name=\"survey#$pkey#name\" size=\"60\" value=\"$survey{name}\"></p>";	
	$out .= "\n<p>$module Description:".$this->helplink($table,"description")."<br><textarea name=\"survey#$pkey#description\" rows=\"4\" cols=\"60\">$survey{description}</textarea></p>";
	$out .= "\n<p>Display '<sub><font color=\"red\" size=\"+1\">*</font></sub>' in required questions?".$this->helplink($table,"use_require");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_require\" value=\"Yes\"";
	$out .= $survey{use_require} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_require\" value=\"No\"";
	$out .= $survey{use_require} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want to group questions into sections?".$this->helplink($table,"use_group");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_group\" value=\"Yes\"";
	$out .= $survey{use_group} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_group\" value=\"No\"";
	$out .= $survey{use_group} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want to number every single question?".$this->helplink($table,"use_number");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_number\" value=\"Yes\"";
	$out .= $survey{use_number} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_number\" value=\"No\"";
	$out .= $survey{use_number} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want to use the generic style sheet?".$this->helplink($table,"use_generic_css");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_generic_css\" value=\"Yes\"";
	$out .= $survey{use_generic_css} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#use_generic_css\" value=\"No\"";
	$out .= $survey{use_generic_css} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want to save responses to the database?".$this->helplink($table,"save_result");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#save_result\" value=\"Yes\"";
	$out .= $survey{save_result} =~ /yes/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#save_result\" value=\"No\"";
	$out .= $survey{save_result} !~ /yes/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp</p>";
	# Turn contact info section title on/off
	# New functionality: by default, show the title
	$out .= "\n<p>Do you want to show the contact information section title on the form?".$this->helplink($table,"show_contact_title");
	$out .= "<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#show_contact_title\" value=\"Yes\"";
	$out .= $survey{show_contact_title} !~ /no/i ? " checked" : "";
	$out .= ">Yes&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#show_contact_title\" value=\"No\"";
	$out .= $survey{save_result} =~ /no/i ? " checked" : "";
	$out .= ">No&nbsp;&nbsp;</p>";	
	
	$out .= "\n<p>Custom HTML header:".$this->helplink($table,"header_html")."<br>";
	$out .= "<textarea name=\"survey#$pkey#header_html\" rows=\"4\" COLS=\"60\">$survey{header_html}</textarea></p>";
	$out .= "\n<p>Custom HTML footer:".$this->helplink($table,"footer_html")."<br>";
	$out .= "<textarea name=\"survey#$pkey#footer_html\" rows=\"4\" cols=\"60\">$survey{footer_html}</textarea></p>";
	$out .= "\n<p>Thank you message:".$this->helplink($table,"thankyou_message")."<br>";
	$out .= "<textarea name=\"survey#$pkey#thankyou_message\" rows=\"4\" cols=\"60\">$survey{thankyou_message}</textarea></p>";	
	$out .= "\n<P>Organization Email Address:".$this->helplink($table,"from_email")."<br>";
	$out .= "<input type=\"text\" name=\"survey#$pkey#from_email\" size=\"60\" value=\"$survey{from_email}\"></p>";
	$out .= "\n<p>A list of email address to send $module results to:".$this->helplink($table,"email")."<br>";
	$out .= "<input type=\"text\" name=\"survey#$pkey#email\" size=\"60\" value=\"$survey{email}\"></p>";
	$out .= "\n<P>Redirect URL:".$this->helplink($table,"return_url")."<br>";
	$out .= "<input type=\"text\" name=\"survey#$pkey#return_url\" size=\"60\" value=\"$survey{return_url}\"></p>";
	$out .= "\n<P>Redirect message:".$this->helplink($table,"return_message")."<br>";
	$out .= "<textarea name=\"survey#$pkey#return_message\" rows=\"4\" cols=\"60\">$survey{return_message}</textarea></p>";
	
	if (!$surveyid){
		$out .= "\n<h2>Part 2: Contact Information</h2>";
		$out .= $this->html_wizardContactInfo($contact);
	}
	#$out .= "\n<p>Do you want to add custom contact information?<br>";
	#$out .= "<input type=\"radio\" name=\"customq\" value=\"Yes\">Yes&nbsp;&nbsp;";
	#$out .= "<input type=\"radio\" name=\"customq\" value=\"No\">No&nbsp;&nbsp</p>";
	$out .= "\n<p>Do you want Contact Information displayed at the beginning or end of the $module?".$this->helplink($table,"contactinfo_pos")."<br>";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#contactinfo_pos\" value=\"beginning\"";
	$out .= $survey{contactinfo_pos} !~ /end/i ? " checked" : "";
	$out .= ">Beginning&nbsp;&nbsp;";
	$out .= "<input type=\"radio\" name=\"survey#$pkey#contactinfo_pos\" value=\"end\"";
	$out .= $survey{contactinfo_pos} =~ /end/i ? " checked" : "";
	$out .= ">End&nbsp;&nbsp</p>";
	if ($surveyid){
		$out .= "\n<input type=\"hidden\" name=\"survey#$pkey#id\" value=\"$surveyid\">";
	}
	$out .= "\n<input type=\"hidden\" name=\"survey#$pkey#section_id\" value=\"$sectionid\">";	
	$out .= "\n<input type=\"hidden\" name=\"survey#$pkey#type\" value=\"$formtype\">";
	$out .= "\n<input type=\"hidden\" name=\"action\" value=\"submit_form\">";
	$out .= "\n<input type=\"submit\" value=\"Submit\">&nbsp;&nbsp;&nbsp;&nbsp;";
	$out .= "\n<input type=\"reset\" value=\"Reset\">";
	$out .= "\n</form>";
    	return $out;
}
sub helplink{
	my ($this,$table,$column) = @_;
	my $out = $this->{db}->helplink($table,$column,"<img src=\"$config{server}->{HTMLpath}/_ExSite/images/qmark.gif\" width=\"20\" height=\"20\" border=\"0\" title=\"click for help\" align=\"middle\">");
	return $out;
}

# detailed question report
sub html_questionRegistrantReport{
	# $answer: a list of survey responds for a question
	my ($this,$answers) = @_;
	@$answers = sort {uc($a->{last}) cmp uc($b->{last})} @$answers;
	my @data;
	for(my $i; $i < @$answers; $i++){
		my $answer = $this->displayData("answer",$answers->[$i]{value});
		my $email = $answers->[$i]{email};
		my $name = $answers->[$i]{first}." ".$answers->[$i]{last};
		$name .= $this->validateEmail($email) ? " (".$this->displayData("email",$email).")" : "";
		push(@data,[($i + 1),$name,($answer || "n/a")]);
	}
	
	my %report = (
		title=>scalar(@$answers)." participant(s)",
		headers=>["No.","Name","Response"],
		data=>\@data,
		width=>"100%",
	);
	return $this->reportbuilder(\%report);
}
sub html_questionChoice{
	my ($this,$question) = @_;
	my $out;
	if ($question->{type} !~ /long|short/i){
		$out .= "<b>Available responses: </b>";
		if ($question->{type} =~ /^Check/){
			$out .= "Yes, No";
		}else{
			$out .= join(", ",split(/\|/,$question->{list}));
		}
	}
	return $out;
}
# question report title
sub html_questionReportTitle{
	# $question: question hash
	my ($this,$question) = @_;
	return $question->{label};
}
# survey summary report
sub html_surveyQuestionReport{
	# $body: body content
	my ($this,$question,$body,$surveyid) = @_;
	my %report = (
		title=>$this->html_questionReportTitle($question),
		data=>$this->html_questionChoice($question),
		foot=>$body,
	);
	my $out .= "\n<div class=\"$this->{div_class}\">";
	$out .= $this->reportMenu($surveyid);	
	$out .= $this->reportbuilder(\%report);
	$out .= "\n</div>";
	$out .= "\n";
	return $out;
}
# survey summary
sub html_surveyBottomSummary{
	# surveyid: surveyid
	# registrantid: list of registrant id(in | delimited format, e.g. 1|2|3|4|5|6|7|8)
	my ($this,$surveyid,$registrantid) = @_;
	my @participants;
	my @questions = $this->{db}->fetch_match("survey_question",{survey_id=>$surveyid});
	if ($registrantid eq "all"){
		@participants = $this->{db}->fetch_match("survey_registrant",{survey_id=>$surveyid});	
	}else{
		@participants = split(/\|/,$registrantid);		
	}
	my %report = (
		headers=>["Total Questions","Total Participants"],
		data=>[scalar(@questions),scalar(@participants)],
		width=>"100%",
	);
	return $this->reportbuilder(\%report);
}

# insert an hr in the survey summary report
sub html_summaryHR{
	my ($this) = @_;
	return "\n<tr><td colspan=\"2\"><hr></td></tr>";
}
sub html_summaryQuestionViewDetailLink{
	my ($this,$question,$surveyid) = @_;
	return "<a href=\"".&relink({action=>"view_question",qid=>$question->{id},sid=>$surveyid})."\">".$this->getIcon("view","View details")."</a>";
}
# survey summary question title
sub html_summaryQuestionTitle{
	# $question: question hash
	# $surveyid: survey id
	my ($this,$question,$surveyid) = @_;
	my $out .= $question->{section_label} ? $question->{section_label}." - " : "";
	$out .= "<b>$question->{label}</b>";
	return $out;
}

# survey summary report
sub html_surveySummaryReport{
	# $survey: survey has
	# $body: body content
	my ($this,$survey,$body,$title) = @_;
	my $out = "\n<div class=\"$this->{div_class}\">";
	$out .= $this->reportMenu($survey->{id});
	$out .= "\n<h1>".($title || $survey->{name})."</h1>";
	$out .= "<table><tr><td>$body</td></tr></table>";
	$out .= "\n</div>";
	$out .= "\n";
	return $out;
}
# no summary message
sub html_noSummary{
	my ($this) = @_;
	return "There are no summaries available for this question.";
}
# contact info question(short answer)
sub html_contactInputShortAnswer{
	# $contact: contact info hash
	# $default: default input
	my ($this,$contact,$default) = @_;
	my ($out,$key);
	$key = "cc#$contact->{id}";
	$out = "<input type=\"text\" name=\"$key\" value=\"$default\" ";
	$out .= "size=\"$config{$this->{class_name}}{form}{custom_text_size}\" maxlength=\"$config{$this->{class_name}}{form}{custom_text_max}\">";
	return $out;
}
# contact info question(long answer)
sub html_contactInputLongAnswer{
	# $contact: contact info hash
	# $default: default input
	my ($this,$contact,$default) = @_;
	my ($out,$key);
	$key = "cc#$contact->{id}";
	$out = "<textarea name=\"$key\" cols=\"$config{$this->{class_name}}{form}{custom_textarea_col}\" ";
	$out .= "rows=\"$config{$this->{class_name}}{form}{custom_textarea_row}\" wrap=\"virtual\">$default</textarea>\n";
	return $out;
}
# merge questions with same answer choices into a table
sub html_questionStack{
	# $option: options(ie: open,close,input)
	# $question question hash
	# $num: question number
	my ($this,$option,$question,$num) = @_;
	my ($out);
	# create a table
	if($option eq "open"){
		my (@answer);
		$out = "\n<table border=\"0\" cellspacing=\"0\">";
		$out .= "\n<tr><td></td><td align=\"center\">";
		@answer = split(/\|/,$question->{list});
		# answer choices
		$out .= join("</td><td align=\"center\">",@answer);
		$out .= "</td></tr>";
	# close the table
	}elsif($option eq "close"){
		$out = "</table>";
	# table contents
	}elsif($option eq "input"){
		$out = $this->html_surveyQuestionInputData($question,$num,1);
		$out = $this->html_questionInputLabel($question->{type},$this->html_questionInputNumber($question,$num),$this->getQuestionLabel($question,1),$out,1);
	}
	return $out;
}
# survey question(checkbox)
sub html_questionInputSingleCheckbox{
	# $question: question hash
	# $defaultinput: default input
	my ($this,$question,$defaultinput) = @_;
	my ($out,$yes);
	$yes = $this->{yes};
	$out .= "<input type=\"checkbox\" name=\"$question->{id}\" value=\"Yes\"";
	$out .= $defaultinput =~ /$yes/i ? " checked" : "";
	$out .= ">";
	return $out;
}
# survey question(short answer)
sub html_questionInputShortAnswer{
	# $question: question hash
	# $defaultinput: default input
	my ($this,$question,$defaultinput) = @_;
	my ($out);
	$out .= "<br>";
	$out .= "<input type=\"text\" name=\"$question->{id}\" VALUE=\"$defaultinput\" ";
	$out .= "size=\"$config{$this->{class_name}}{form}{survey_text_size}\" ";
	$out .= "maxlength=\"$config{$this->{class_name}}{form}{survey_text_max}\">";
	return $out;
}
# survey quesiton(long answer)
sub html_questionInputLongAnswer{
	# $question: question hash
	# $defaultinput: default input
	my ($this,$question,$defaultinput) = @_;
	my ($out);
      	#$out .= "&nbsp;&nbsp;";
	$out .= "<br>";
	$out .= "<textarea name=\"$question->{id}\" cols=\"$config{$this->{class_name}}{form}{survey_textarea_col}\" ";
	$out .= "rows=\"$config{$this->{class_name}}{form}{survey_textarea_row}\">$defaultinput</textarea>";
	return $out;
}
# survey question(drop-down list)
sub html_questionInputList{
	# $question: question hash
	# $defaultinput: default input
	my ($this,$question,$defaultinput) = @_;
	my ($out,@answers,%default);
	%default = $this->getDefaultInputHash($defaultinput);
	@answers = split(/\|/,$question->{list});
	$out .= "\n<select name=\"$question->{id}\">";
	$out .= "\n<option value=\"\">== select ==</option>";
	foreach my $a (@answers){
		$out .= "\n<option value=\"$a\"";
		$out .= exists($default{$a}) ? " selected" : "";
		$out .= ">$a</option>";
	}
	$out .= "\n</select>";
	if ($question->{display_style} =~ /horizontal/i) {
		$out .= "\n&nbsp;&nbsp;&nbsp;";
	}
	else {
		$out .= "<br>";
	}
	$out .= $this->html_questionInputOther($question);
	return $out;
}
# survey question('other specified' field)
sub html_questionInputOther{
	# $question: question hash
	my ($this,$question) = @_;
	return if $question->{other} !~ /$this->{yes}/i;
	my ($out,$defaultinput);
	$defaultinput = $this->getDefaultOtherInput($question);
	$out .= "<span class=\"otherInput\">$config{$this->{class_name}}{form}{other_question} </span><input type=\"text\" name=\"$question->{id}\" ";
	$out .= "value=\"$defaultinput\" size=\"$config{$this->{class_name}}{form}{survey_text_size}\" ";
	$out .= "maxlength=\"$config{$this->{class_name}}{form}{survey_text_max}\">";
	return $out;
}
# display multiple checkboxes/radio buttons vertically or horizontally.
sub html_questionDisplayStyle{
	# $type: question type
	# $quesiton: question hash
	# $answers: available answers
	# $default: default avlue
	# $override: this flag is used when we stack answer choices in a table
	my ($this,$type,$question,$answers,$default,$override) = @_;
	# Create a list of available answer inputs for checkbox and radio button inputs
	my @choices;
	foreach my $a (@$answers){
		# Set default checked value
		my $checked = exists($default->{$a}) ? " checked" : "";
		my $input = <<END;
<input type="$type" name="$question->{id}" value="$a"${checked}>
END
		# Display answer choices
		if (!$override){
			$input .= "&nbsp;$a";
		}
		push(@choices,$input);
	}
	# The separator that used to concatenate answer choices data array in join()
	my $sep;
	if ($question->{display_style} =~ /vertical/i){
		$sep = "<br>";
	}
	# When displaying a stacking/grouping questions, we display them in a table format
	if ($override){
		$sep = "</td><td align=\"center\">";
	}
	my $out;
	if (!$override){
		# We only add a <br> to the question display when we aren't displaying the
		# question in a multi column format
		if (!$this->useAlternativeVerticalDisplay(scalar(@$answers))){
			$out .= "<br>";
		}
	}
	if ($override){
		$out .= "<td align=\"center\">";
	}else{
		$out .= "\n";
	}
	# Output html code for answer choices
	if ($question->{display_style} =~ /vertical/i){
		# html_specialFormatVerticalAnswerStyle will determine whether to display
		# answer choices in one long list or multiple column list display
		$out .= $this->html_specialFormatVerticalAnswerStyle(\@choices,$sep);
	}else{
		# Display the answer choices in horizontal format
		$out .= join($sep,@choices);
	}
	if ($override){
		$out .= "</td>";
	}
	# Other input field: for multiple checkboxes questions only
	if ($question->{type} =~ /multi/i){
		if (!$override){
			$out .= "<br>\n".$this->html_questionInputOther($question,$default);
		}
	}
	return $out;
}
# special format for vertical answer choices(for checkbox/radio button question only)
sub html_specialFormatVerticalAnswerStyle{
	# $choice: answer choices
	# $separator: separate each answer choices
	my ($this,$choice,$separator) = @_;
	my ($limit,$column,$noLoop,$out,@inputs,$width);
	# if the answer choices exceed the $limit, a special table format is used
	$limit = $config{$this->{class_name}}{config}{question}{special_format_vertical_record} || 10;
	# how many columns should the special table have
	$column = $config{$this->{class_name}}{config}{question}{special_format_column} || 2;
	# calculate the number of rows in the table
	$noLoop = sprintf("%i",(@$choice / $column));
	# add another row to $noLoop if the number of answer choices isn't divisible by the number of columns
	$noLoop += 1 if (@$choice / $column) =~ /\./;
	# use the default display format
	return join($separator,@$choice) if @$choice < $limit;	
	for(my $i = 0;$i < $noLoop;$i++){
		for(my $j = 0;$j < $column;$j++){
			my $current_pos = ($column * $i) + $j;
			# reached the end of an array. fill the table cell with nbsp
			if ($current_pos < @$choice){
				$inputs[$j] .= $choice->[$current_pos]."<br>";
			}
		}
	}
	$width = (100 / $column);
	$width =~ /(\d+)\.?/;
	$width = $1;
	$out .= "\n<table border=\"0\">";
	$out .= "\n<tr>";
	foreach my $input(@inputs){
		$out .= "\n<td valign=\"top\" width=\"${width}%\">\n$input</td>\n";
	}
	$out .= "</tr>";
	$out .= "\n</table>\n";
	return "<br>".$out;
}
# survey question(radio buttons)
sub html_questionInputRadioButton{
	# $quesiton: question hash
	# $answers: available answers
	# $default: default avlue
	# $override: this flag is used when we stack answer choices in a table
	my ($this,$question,$defaultinput,$override) = @_;
	my ($out,@answers,%default);
	@answers = split(/\|/,$question->{list});
	%default = $this->getDefaultInputHash($defaultinput);
	$out .= $this->html_questionDisplayStyle("radio",$question,\@answers,\%default,$override);	
	return $out;
}
# survey question(multiple checkboxes
sub html_questionInputMultipleCheckbox{
	# $quesiton: question hash
	# $answers: available answers
	# $default: default avlue
	# $override: this flag is used when we stack answer choices in a table
	my ($this,$question,$defaultinput,$override) = @_;
	my ($out,@answers,%default);
	@answers = split(/\|/,$question->{list});
	%default = $this->getDefaultInputHash($defaultinput);
	$out .= $this->html_questionDisplayStyle("checkbox",$question,\@answers,\%default,$override);
	return $out;
}
# survey question labels
sub html_questionInputLabel{
	# $type: question type
	# $number: question number
	# $label: question label
	# $input: html input
	# $override: this flag is used when we stack answer choices in a table
	my ($this,$type,$number,$label,$input,$override) = @_;
	if ($override){
		$input = "\n<tr><td>".$label."</td>".$input."</tr>";
	}elsif($type =~ /^check/i){
		$input = $input."&nbsp;&nbsp;".$label;
	}elsif($type =~ /multiple|radio|list|long/i){
		$input = $label.$input;
	}elsif($type =~ /short/i){
		$input = $label."&nbsp;&nbsp;".$input;
	}
	return $input;
}
# insert a question number beside the question label
sub html_questionInputNumber{
	# $question: question hash
	# $count: question number
	my ($this,$question,$count) = @_;
	my ($out,$asterisk,$number);
	$asterisk = $this->getQuestionAsterisk($question);
	$number = $this->{use_number} ? $count.".&nbsp;" : "";
	return $asterisk."&nbsp;".$number;	
}
# print out survey questions
sub html_surveyQuestionInputData{
	# $question: question hash
	# $count: question number 
	# $override: this flag is used when we stack answer choices in a table
	my ($this,$question,$count,$override) = @_;
	my ($input,$defaultinput,$type);
	$type = $question->{type};
	$defaultinput = $this->getDefaultQuestionValue($question);
	if ($type =~ /^check/i){
		$input = $this->html_questionInputSingleCheckbox($question,$defaultinput);
	}elsif ($type =~ /multiple/i){
		$input = $this->html_questionInputMultipleCheckbox($question,$defaultinput,$override);
	}elsif ($type =~ /list/i){
		$input = $this->html_questionInputList($question,$defaultinput);
	}elsif ($type =~ /short/i){
		$input = $this->html_questionInputShortAnswer($question,$defaultinput);
	}elsif ($type =~ /long/i){
		$input = $this->html_questionInputLongAnswer($question,$defaultinput);
	}elsif ($type =~ /radio/i){
		$input = $this->html_questionInputRadioButton($question,$defaultinput,$override);
	}
	return $input;
}
# put questions in a table row
sub html_surveyQuestionRow{
	# $input: html content
	my ($this,$input) = @_;
	return <<END;
<li>
$input
</li>
END
}
# complete survey questions
sub html_surveyQuestion{
	# $body: cody content
	my ($this,$body) = @_;
	return $body;
}
# complete survey contact info
sub html_surveyContactInfo{
	# $body: cody content
	# $surveyid: survey id
	my ($this,$body,$surveyid) = @_;
	my %survey = $this->{db}->fetch("survey",$surveyid);
	my $section_label = $config{$this->{class_name}}{config}{contact_info}{label};
	# Show "Contact Information" title on the form?
	my $out = $survey{show_contact_title} !~ /no/i ? $this->getSectionLabel($section_label) : "";
	$out .= $body;
	return $out;
}
# put contact info questions in tables rows
sub html_surveyContactInfoRow{
	# $contact: contact info question hash
	my ($this,$contact) = @_;
	my ($label,$input,$defaultinput);
	$label = $this->getContactInfoLabel($contact);
	$defaultinput = $this->getDefaultContactInfoValue($contact);
	if ($contact->{type} =~ /^short/i){
		$input = $this->html_contactInputShortAnswer($contact,$defaultinput);
	}elsif ($contact->{type} =~ /^long/i){
		$input = $this->html_contactInputLongAnswer($contact,$defaultinput);
	}
	return <<END;
<p>
<span class="SurveyQuestion">$label</span>
<br>
$input
</p>
END
}
# survey form start and end tags
sub html_surveyFormTag{
	# $option: options(open,close)
	my ($this,$option,$survey) = @_;
	my ($out,$surveyid);
	$surveyid = $this->{input}{sid} || $survey->{id};
	if ($option eq "open"){
		$out .= "\n<form method=\"post\" action=\"".&relink({action=>undef,sid=>undef})."\">";
		$out .= "\n<input type=\"hidden\" name=\"action\" value=\"form_submit\">";
		$out .= "\n<input type=\"hidden\" name=\"sid\" value=\"$surveyid\">";	
		$out .= "\n";
	}elsif($option eq "close"){
		$out .= "\n</form>";
	}
	return $out;
}
# form buttons
sub html_surveyFormButton{
	my ($this) = @_;
	my ($out,@button,@input);
	@input = (
	{type=>"submit",value=>"Submit"},
	{type=>"reset",value=>"Reset"},
	);	
	foreach my $b (@input){
		push(@button,"<input type=\"$b->{type}\" value=\"$b->{value}\">");
	}	
	$out .= "\n<p class=\"SurveyButton\">".join("&nbsp;&nbsp;",@button)."</p>";
	return $out;
}
# form validattion errors
sub html_formError{
	my ($this) = @_;
	my ($out);
	foreach my $e (@{$this->{form_error}}){
		$out .= "\n";
		$out .= "<p>";
		$out .= $this->highlightText($this->boldText($e));
		$out .= "</p>";
	}
	$out .= "\n";
	return $out;
}
# complete survey form
sub html_surveyForm{
	# $survey: survey hash
	# $contact: complete contact info questions
	# $question: complete questions
	my ($this,$survey,$contact,$question,$error) = @_;
	my ($out);
	$out .= "\n<div class=\"$this->{div_class}\">";
	$out .= "\n";
	#### default survey stylesheet ###############
	$out .= "\n<style>\n<!--\n.SurveyQuestionRequired {font-style:italic;color:red;}";
	if ($this->{use_default_css}){
		$out .= "\n.SurveySectionHead {font-size:120%;font-weight:bold;padding-top:20px;}";
	}
	$out .= "\n//-->\n</style>\n";
	##############################################
	$out .= $this->html_surveyFormTag("open",$survey);
	$out .= $this->getFormHeader($survey);
	$out .= $error;
	$out .= "\n";
	$out .= $this->{contactinfo_pos} ? $contact : "";
	$out .= "\n";
	$out .= $question;
	$out .= "\n";
	$out .= !$this->{contactinfo_pos} ? $contact : "";
	$out .= "\n";
	$out .= $this->{use_require} ? "<p class=\"SurveyQuestionRequired\">* Required inputs.</p>" : "";
	$out .= $this->html_surveyFormButton;
	$out .= "\n";
	$out .= $this->getFormFooter($survey);
	$out .= "\n";
	$out .= $this->html_surveyFormTag("close");
	$out .= "\n";
	$out .= "\n</div>";
	$out .= "\n";
	return $out;
}
# form completion screen
sub html_formFinalScreen{
	# $survey: survey hash
	my ($this,$survey) = @_;	
	return $this->html_surveyFinalScreen($survey);
}
# survey completion screen
sub html_surveyFinalScreen{
	# $survey: survey hash
	my ($this,$survey) = @_;
	my ($out,$url,$urlmessage);
	$out .= "\n<div class=\"$this->{div_class}\">";
	$out .= "\n<p>";
	$out .= $survey->{thankyou_message};
	$out .= "</p>";
	return $out if !$survey->{return_url};
	$url = $survey->{return_url} ? $survey->{return_url} : &relink();
	# append http://
	if ($url !~ /^\/.*/){
		$url = $url =~ /^http:\/\// ? $url : "http://".$url;
	}
	$urlmessage = $survey->{return_message} ? $survey->{return_message} : $url;
	$out .= "<a href=\"$url\">$urlmessage</a>";
	$out .= "\n";
	return $out;
}
# put survey in a list format
sub html_surveyListRow{
	# $surey: survey hash
	my ($this,$survey) = @_;
	my ($take,$out);
	$take = "<a href=\"".&relink({action=>"do",sid=>$survey->{id}})."\">$survey->{name}</a>";
	$out .= "\n<dl>";
	$out .= "\n<dt>$take";
	$out .= $survey->{description} ? "\n<dd>$survey->{description}" : "";
	$out .= "\n</dl>";
	return $out;
}
# complete survey list
sub html_surveyList{
	# $hdata: complete survey list
	my ($this,$hdata) = @_;
	my ($out);
	return if !$hdata;
	$out .= "\n<div class=\"$this->{div_class}\">";
	$out .= "<h2>$config{$this->{class_name}}{config}{publiclist}{title}</h2>";
	$out .= $hdata;
	$out .= "\n</div>";
	$out .= "\n";
	return $out;
}
# survey titles(admin interface)
sub html_adminSurveyListTitle{
	# $section: section hash
	my ($this,$section) = @_;
	my ($out);
	#$out .= "\n<h3>$section->{title}</h3>";
	$out .= "\n$section->{title}";
	return $out;
}
sub html_adminSurveyRowLink{
	my ($this,$survey) = @_;
	my @commands;
	my @links = @{$this->getAdminSurveyListRowLinkFix($survey)};
	# gather all menu commands
	foreach my $l (@links){
		my $link = "<a href=\"$l->{url}\">";
		$link .= $this->getIcon($l->{img},$l->{label});
		$link .= "</a>";
		push(@commands,$link);
	}
	return join("",@commands);	
}
# admin survey list bottom menu
sub html_adminSurveyListBottomMenu{
	my ($this,$section) = @_;
	my (@commands,@links);
	@commands = @{$this->getAdminSurveyListBottomMenuLink($section)};
	foreach my $command(@commands){
		push(@links,"<a href=\"$command->{url}\">".$this->getIcon($command->{img},$command->{label})."</a>");
	}
	return join(" | ",@links);
}
# create a list of surveys for creating custom reports
sub html_ddsSurveySelect{
	# $survey: survey hash
	my ($this,$surveys) = @_;
	my ($out,$module);
	$module = $this->getModuleName;
	# display a form for create a new report
	$out .= "\n<h3>Create a new report</h3>";	
	$out .= "\n<form method=\"get\" action=\"".&relink({action=>undef,section_id=>undef})."\">";
	$out .= "\n<input type=\"hidden\" name=\"action\" value=\"dds_action\">";
	$out .= "\n<input type=\"hidden\" name=\"dds_action\" value=\"step\">";
	$out .= "\n<input type=\"hidden\" name=\"step\" value=\"0\">";
	$out .= "\n<h3>Select a $module</h3>";
	$out .= "\n<select name=\"survey_id\">";
	foreach my $s (@$surveys){
		my $questions = $this->{db}->fetch_match("survey_question",{survey_id=>$s->{id}});
		# hide the survey with no questions
		if (scalar(@$questions) < 1){next;}
		$out .= "<option value=\"$s->{id}\">$s->{name}</option>";
	}
	$out .= "</select>";
	$out .= "\n<input type=\"submit\" value=\"Submit\">";
	$out .= "\n</form>";
	return $out;
}
# create a list of custom reports
sub html_ddsSurveyDDSReport{
	# $dds: a list of custom reports
	my ($this,$dds) = @_;
	my ($out);
	if (@$dds > 0){
		# display all custom reports in table formats and allow the admin to view
		# and delete reports
		my ($prev);
		$out .= "\n<h3>View saved reports</h3>";
		$out .= "\n<table border=\"0\" cellpadding=\"5\">";
		foreach my $d (@$dds){
			my ($filters,$qdata,@queries,%data,%compare,%query,%myHash);
			%data = ();
			# display survey name as the section header
			if ($prev ne $d->{survey_id}){
				my $survey = $this->{db}->fetch("survey",$d->{survey_id});
				$out .= "\n<tr><th colspan=\"2\">$survey->{name}</th></tr>";
				$out .= "\n<tr><td><b>Options</b></td><td><b>Name</b></td></tr>";
				$prev = $d->{survey_id};
			}
			# fetch filter data
			$filters = $this->{db}->fetch_match("survey_dds_data",{survey_dds_id=>$d->{id}});
			foreach my $f (@$filters){
				$data{$f->{data_key}} = $f->{data_value};
			}
			$data{survey_id} = $d->{survey_id};
			# prepare report data for viewing the report
			$qdata = $this->ddsGetData(\%data);
			%compare = %{$qdata->{compare}};
			%query = %{$qdata->{query}};
			$out .= "\n<tr><td valign=\"top\" width=\"90\">";
			$out .= "<a href=\"javascript:deleteDDSRecord($d->{id});\">Delete</a> | ";
			# remove useless keys and values from the query string
			foreach my $key(keys(%{$this->{input}})){
				if ($key =~ /^_(.*)/){
					$data{$key} = $this->{input}{$key};
				}else{
					$data{$key} = undef;
					delete $this->{input}{$key};
				}
			}
			$data{action} = "dds_action";
			$data{dds_action} = "view";
			$data{step} = 9999;
			$data{submit_action} = undef;
			$out .= "<a href=\"";
			$out .= $this->myRelink({action=>"dds_action",dds_action=>"view",dds_id=>$d->{id}});
			$out .= "\" target=\"_new\">View</a></td><td>";
			$out .= $d->{name} ? "<b>".$d->{name}."</b>" : "N/A";
			$out .= $d->{description} ? "<br><i>$d->{description}</i>" : "";
			$out .= join("<br>",@queries);
			$out .= "</td></tr>";
		}
		$out .= "\n</table>";
	}
	return $out;
}
# wizard like form for creating filter reports
sub html_ddsFormStep{
	# survey id
	my ($this,$surveyid) = @_;
	my ($out,$sid,$step,$question,$prev,$halt,%data,$module);
	$module = $this->getModuleName;
	%data = %{$this->{input}};
	$sid = $surveyid;
	$step = $data{step};
	$step++;
	delete $data{submit_action};
	delete $data{step};
	delete $data{action};
	
	if ($step > 1){
		# error message for the user that clicks 'next' without selecting any questions
		my $error = "<p>You did not select any question.  Please go 'back' and select questions.</p>";
		# display the previously specified queries
		$prev = $this->html_ddsGetPrevQuery($sid,\%data);
		# checking for black form inputs
		$halt = $this->ddsCheckInput($step,\%data);
		# display error message if the form input checks fail
		if ($halt){
			$out .= $error;
		}else{
			$out .= "<p>The selection criteria for this report are listed below.  ";
			$out .= "To view the results after applying all selection criteria, select ";
			$out .= "\"view\" for the last step shown.  To view intermediate results, after ";
			$out .= "applying only some selection crtieria, select \"view\" for an earlier ";
			$out .= "step (if you have defined multiple steps).  You can alter the crtieria ";
			$out .= "for this report by removing steps (click on the \"remove\" link), or ";
			$out .= "adding new steps (see below).</p>";
			$out .= $prev->{out};
		}
	}

	delete $data{survey_id};
	# display directions depending on the wizard steps
	if ($step == 1 && !$halt){
		$out .= "<p>A custom report allows you to define various criteria for ";
		$out .= "selecting particular ${module}s to view.  Criteria are defined by ";
		$out .= "selecting the questions that are of interest to you, and then choosing ";
		$out .= "the particular answers you want to select for</p>";
	}elsif (!$halt){
		$out .= "<p>Add New Step:  you can add additional selection criteria to this report ";
		$out .= "here.  These new criteria will select only from those reponses that have ";
		$out .= "already passed previous steps in this report.</p>";
	}
	
	# generate the input form
	$out .= "\n<form method=\"get\">";
	$out .= "\n<input type=\"hidden\" name=\"action\" value=\"dds_action\">";
	$out .= "\n<input type=\"hidden\" name=\"step\" value=\"$step\">";
	$out .= "\n<input type=\"hidden\" name=\"survey_id\" value=\"$sid\">";
	foreach my $key (keys(%data)){
		next if (!$data{$key} || $key =~ /^question_id/);
		$out .= "\n<input type=\"hidden\" name=\"$key\" value=\"$data{$key}\">";
	}
	# display survey questions
	$question = !$halt ? $this->html_ddsSurveyQuestion($sid,$step) : "";
	$out .= $question;
	$out .= $step > 1 ? "\n<input type=\"button\" value=\"Back\" onclick=\"history.back()\"> " : "";
	$out .= ($step == 1 || ($prev->{founddata} && $step > 1)) && !$halt ? "\n<input type=\"submit\" name=\"submit_action\" value=\"Next\"> " : "";
	$out .= "\n<input type=\"submit\" name=\"submit_action\" value=\"Start Over\"> ";
	if ($step != 1 && !$halt){
		# display a form for saving custom reports
		$out .= "\n<br>";
		$out .= "\n<h3>Save report</h3>";
		$out .= "<p>If you wish to revisit this report, you can save it for future reference.  ";
		$out .= "Give it a name and helpful description, and click on \"Save\".</p>";
		$out .= "\n<table border=\"0\" cellpadding=\"5\">";
		$out .= "\n<tr><td>Name:</td><td><input type=\"text\" name=\"report_name\" VALUE=\"\" size=\"40\" maxlength=\"200\"></td></tr>";
		$out .= "\n<tr><td>Description:</td><td><input type=\"text\" name=\"report_desc\" value=\"\" size=\"40\" maxlength=\"200\"></td></tr>";
		$out .= "\n</table>";
		$out .= "\n <input type=\"submit\" name=\"submit_action\" value=\"Save\">";
	}
	$out .= "\n</form>";
	return $out;
}
# get previously entered filters
sub html_ddsGetPrevQuery{
	# $surveyid: survey id
	# $input: query hash
	my ($this,$surveyid,$input) = @_;
	my $out;
	my $founddata;
	my %data = %$input;
	# get previously entered filters
	my $qdata = $this->ddsGetData(\%data);
	my %compare = %{$qdata->{compare}};
	my %query = %{$qdata->{query}};
	my $result = [];
	my $stepcount = 1;
	my $firstpass = 1;
	if (keys(%compare) > 0){
		my %report = (
			headers=>["Options","Step","Filter","Responses"],
		);
		$founddata = 1;
		foreach my $key (keys(%compare)){
			my %qstring;
			# gather query string
			foreach my $key2 (keys(%data)){
				next if (!$data{$key2} || $key2 =~ /^question_id/);
				$qstring{$key2} = $data{$key2};
			}
			# fetch and filter records with the previously entered filters
			$result = $this->ddsCompareResult({compare=>$compare{$key},data=>$result,firstpass=>$firstpass});
			$qstring{step} = $stepcount;
			$qstring{action} = "dds_action";
			$qstring{dds_action} = "rollback";
			my $link = "<a href=\"".&relink(\%qstring)."\">".$this->getIcon("delete","Remove this step")."</a>";
			$qstring{action} = "dds_action";
			$qstring{dds_action} = "view";
			$link .= "<a href=\"".&relink(\%qstring)."\" target=\"_new\">".$this->getIcon("view","View report")."</a>";
			delete $qstring{action};
			delete $qstring{step};
			push(@{$report{data}},[$link,$stepcount,join("<br>",@{$query{$key}}),scalar(@$result)]);
			$stepcount++;
			$firstpass = 0;
		}		
		$out .= $this->reportbuilder(\%report);
	}
	return {out=>$out,founddata=>$founddata};	
}
# fix question data
sub fixQuestionData{
	my ($this,$list) = @_;
	my (@newlist,$prev);
	for(my $i = 0; $i < @$list; $i++){
		if ($prev != $list->[$i]{survey_section_id}){
			push(@newlist,{sv_section=>1,label=>$list->[$i]{section_label}});
		}
		$list->[$i]{sv_section} = 0;
		push(@newlist,$list->[$i]);
		$prev = $list->[$i]{survey_section_id};
	}
	return \@newlist;
}
# get the number of characters to truncate for question labels
sub customReportQuestionTruncate{
	my ($this) = @_;
	# by default, the system truncate a question's label after 100 characters
	my $truncateNumber = $config{$this->{class_name}}{config}{custom_report}{truncate_question_label} || 100;
	return $truncateNumber;
}
# generate dhtml for survey questions
sub html_ddsSurveyQuestion{
	# $surveyid: survey id
	# $step: wizard step
	my ($this,$surveyid,$step) = @_;
	my ($out,$questions,$sid,$select,$js,$db);
	$sid = $surveyid;
	# fetch questions
	$questions = $this->{db}->fetch_match("survey_question",{survey_id=>$sid},"survey_section_id asc, label asc");
	$questions = $this->addQuestionSectionInfo($surveyid,$questions);
	$questions = $this->fixQuestionData($questions);
	# generate javascripts
	$js .= "\n<script language=\"javascript\">";
	$js .= "\n<!--";
	$js .= "\nvar selQuestion = new Object();";
	$js .= "\nselQuestion['0'] = '';";
	for(my $i = 0; $i < @$questions; $i++){
		# generate dhtml code for survey questions
		$js .= "\nselQuestion['".($i + 1)."'] = ";
		$js .= $questions->[$i]{sv_section} ? "\"\";" : "\"<h3>Input response</h3>".$this->html_ddsGetQuestion($questions->[$i],$step)."\";";
	}
	for(my $x = 1; $x < 4; $x++){	
		$select = "<select name=\"question_id$step$x\" onclick=\"display$step$x()\">";
		$select .= "<option value=\"\">== None ==</option>";
		for(my $i = 0; $i < @$questions; $i++){
			my $label = $questions->[$i]->{label};
			if ($questions->[$i]{sv_section}){
				$select .= "<option value=\"\">";
				$select .= "== ".uc($label)." ==</option>";
			}else{
				if (length($label) > $this->customReportQuestionTruncate()){
					$label = substr($label,0,$this->customReportQuestionTruncate())."...";
				}
				#if ($questions->[$i]{section_label}){
				#	$label .= " ($questions->[$i]{section_label})";
				#}
				$select .= "<option value=\"$questions->[$i]->{id}\">";
				$select .= "&nbsp;&nbsp;&nbsp;&nbsp;$label</option>";
			}
		}
		$js .= "\n\nfunction display$step$x(){";
		$js .= "\nvar i,target,selected;";
		$js .= "\ni = document.forms[0].question_id$step$x.selectedIndex;";
		$js .= "\ntarget = document.getElementById('display$step$x');";
		$js .= "\ntarget.innerHTML = selQuestion[i];";
		$js .= "\n}";
		$select .= "</select>";
		$out .= "\n<table border=\"0\">";
		$out .= "\n<tr><td><h3>Select question</h3>$select</td></tr>";	
		$out .= "\n<tr><td><div id=\"display$step$x\"></div></tr>";
		$out .= "\n</table>";
	}
	$js .= "\n//-->";
	$js .= "\n</script>";
	$out .= $js;
	return $out;
}
# generate input fields for filters
sub html_ddsGetQuestion{
	# $question: question hash
	# $step: wizard step
	my ($this,$question,$step) = @_;
	my ($out);
	# strip window return characters
	$question->{list} = $this->stripChr($question->{list});
	# generate dhtml codes for various types of survey questions
	if($question->{type} =~ /list/i){
		# list box question
		$out .= "<select name=\"step#$step#$question->{id}\">";
		foreach my $l (split(/\|/,$question->{list})){
			$out .= "<option value=\"$l\">$l</option>";
		}
		$out .= "</select>";
		if($question->{other} =~ /y/i){
			$out .= "<br>Other (Specify): ";
			$out .= "<input type=\"text\" NAME=\"step#$step#$question->{id}\" VALUE=\"\" ";
			$out .= "size=\"40\" maxlength=\"200\"> ";
		}
	}elsif($question->{type} =~ /multi|^radio/i){
		# multiple checkbox question
		foreach my $l (split(/\|/,$question->{list})){
			$out .= "<input type=\"";
			$out .= $question->{type} =~ /multi/ ? "checkbox" : "radio";
			$out .= "\" name=\"step#$step#$question->{id}\" value=\"$l\"> $l <br>";
		}
		if($question->{other} =~ /y/i){
			$out .= "Other (Specify): ";
			$out .= "<input type=\"text\" NAME=\"step#$step#$question->{id}\" VALUE=\"\" ";
			$out .= "size=\"40\" maxlength=\"200\"> ";
		}
	}elsif($question->{type} =~ /checkbox/i){
		# single checkbox question
		$out .= "<input type=\"radio\" name=\"step#$step#$question->{id}\" value=\"yes\"> Yes ";
		$out .= "<input type=\"radio\" name=\"step#$step#$question->{id}\" value=\"no\"> No ";
	}else{
		# long and short answer question
		$out .= "<input type=\"text\" name=\"step#$step#$question->{id}\" size=\"50\">";
	}
	# escape quotation marks
	$out =~ s/"/\\"/g;
	return $out;
}
# construct filter data
sub getFilterData{
	my ($this,$ddsid) = @_;
	return () if !$ddsid;
	my (@filters,%filter_table,%survey,%result);	
	%filter_table = $this->{db}->fetch("survey_dds",$ddsid);
	%survey = $this->{db}->fetch("survey",$filter_table{survey_id});
	@filters = $this->{db}->fetch_match("survey_dds_data",{survey_dds_id=>$filter_table{id}});
	
	$result{dds_name} = $filter_table{name};
	$result{survey_id} = $survey{id};
	$result{step} = 9999;
	$result{dds_action} = "view";
	$result{section_id} = $survey{section_id};
	$result{action} = "dds_action";
	foreach my $filter(@filters){
		$result{$filter->{data_key}} = $filter->{data_value};
	}
	return %result;
}
sub getDDSResult{
	my ($this,$input) = @_;
	my ($step,%input,$alldata,%compare,%query,$result,$firstpass);
	%input = %$input;
	$step = $input{step};	
	# determine the fetch data based on the steps
	foreach my $key (keys(%input)){
		next if (!$input{$key} || $key !~ /^step#/);
		my (undef,$currstep,undef) = split(/#/,$key);
		if ($currstep > $step){
			delete $input{$key};
		}
	}
	# prepare report data for viewing the report
	$alldata = $this->ddsGetData(\%input);
	%compare = %{$alldata->{compare}};
	%query = %{$alldata->{query}};
	$result = [];
	$firstpass = 1;
	# figure out the matched data
	foreach my $key (keys(%compare)){
		$result = $this->ddsCompareResult({compare=>$compare{$key},data=>$result,firstpass=>$firstpass});
		$firstpass = 0;
	}
	return ($result,\%compare,\%query);
}
# generate a custom report
sub html_ddsViewReport{
	my ($this) = @_;
	my %input = $this->{input}{dds_id} ? $this->getFilterData($this->{input}{dds_id}) : %{$this->{input}};
	my ($result,$compare,$query) = $this->getDDSResult(\%input);
	my %compare = %$compare;
	my %query = %$query;
	# generate the report
	my %survey = $this->{db}->fetch("survey",$input{survey_id});	
	my $module = $this->getModuleName();	
	my $out;
	my %report;
	
	$report{title} = "Filter";
	$report{width} = "100%";
	# display the user specified queries
	if(keys(%compare) > 0){
		my @data;
		foreach my $key (keys(%compare)){
			push(@data,[join("<br>",@{$query{$key}})]);
		}
		$report{data} = \@data;
	}else{
		$report{data} = "<p>No query defined</p>";
	}
	$out .= $this->reportbuilder(\%report);
	
	# view answer summary(for survey only)
	if ($this->{input}{dds_id} && $module =~ /survey/i){
		$out .= "<p><a href=\"".$this->myRelink({action=>"view_dds_summary",dds_id=>$this->{input}{dds_id},sid=>$input{survey_id}})."\">View Summary Report</a></p>";
	}
	
	%report = ();
	$report{title} = "These responses matched the above criteria";
	$report{width} = "100%";
	
	# display the matching records
	if (@$result > 0){		
		$report{headers} = ["No.","Name","Email","Options"];
		my $count = 1;
		foreach my $r (@$result){
			my ($reg,%myHash,$email);
			$reg = $this->{db}->fetch("survey_registrant",$r->{survey_registrant_id});
			$email = $reg->{email};
			$email = $this->validateEmail($email) ? $this->displayData("email",$email) : $email;			
			# remove useless query string
			foreach my $i (keys(%input)){
				if ($i =~ /^_(.*)/){
					$myHash{$i} = $input{$i};
				}elsif($i eq "action"){
					$myHash{$i} = "view_user";
				}else{
					$myHash{$i} = undef;
				}
			}
			$myHash{sid} = $input{survey_id};
			$myHash{uid} = $reg->{id};			
			push(@{$report{data}},[$count,"$reg->{first} $reg->{last}",$email,"<a href=\"".&relink(\%myHash)."\">".$this->getIcon("view","View $reg->{first} $reg->{last}'s responses")."</a>"]);
						$count++;
		}	
	}else{
		$report{data} = "<p>No matching records found.</p>";
	}
	$out .= $this->reportbuilder(\%report);
	my $report_out = "\n<p>This page shows the results of applying the following selection criteria to this ${module}'s results</p>";
	$report_out .= "\n<table><tr><td>$out</td></tr></table>\n";
	return $report_out;
}

sub html_javascript{
	my ($this,$type,$option) = @_;
	my ($out);
	$out .= "
<script language=\"javascript\">
<!--
	";
	if ($type eq "dds"){
		$out .= "
function deleteDDSRecord(id){
	var reply = '".$this->encodeString($this->myRelink({action=>$this->{input}{action},section_id=>$this->{input}{section_id}}))."';
	if (confirm('Are you sure you want to delete this record?')){
		location = '".$this->myRelink({action=>"rec_delete",tab=>"survey_dds"})."&id=' + id + '&reply=' + reply;
	}else{
		return;
	}
}
		";
	}
	if ($type eq "adminsurveylist"){
		$out .= "
function deleteRecord(id){
	var reply = '".$this->encodeString($this->myRelink())."';
	if (confirm('Are you sure you want to delete this record?')){
		location = '".$this->myRelink({action=>"rec_delete",tab=>"survey"})."&id=' + id + '&reply=' + reply;
	}else{
		return;
	}
}
		";
	}
	if ($type eq "mainform"){
		$out .= "
function deleteRecord(id){
	var reply = '".$this->encodeString($this->myRelink({action=>$option->{action},sid=>$option->{sid}}))."';
	if (confirm('Are you sure you want to delete this record?')){
		location = '".$this->myRelink({action=>"rec_delete",tab=>$option->{table}})."&id=' + id + '&reply=' + reply;
	}else{
		return;
	}
}
		";
	}
	if ($type eq "reportlist"){
		$out .= "
function deleteRecord(id){
	var reply = '".$this->encodeString($this->myRelink({action=>$option->{action},sid=>$option->{sid}}))."';
	if (confirm('Are you sure you want to delete this record?')){
		location = '".$this->myRelink({action=>"rec_delete",tab=>$option->{table}})."&id=' + id + '&reply=' + reply;
	}else{
		return;
	}
}
		";
	}
	$out .= "
// -->
</script>
	";
	return $out;
}
1;
