package Modules::Webalizer;

# The Webalizer module is just a shortcut to the webalizer stats pages.

#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;

use ExSite::Config;
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub read {
    # do nothing
    return;
}

sub write {
    # do nothing
    return "";
}

sub ioctl {
    my ($this,$request) = @_;
    if ($request eq "ModuleName") {
	return "Webalizer Stats";
    }
    elsif ($request eq "ControlPanel") {
	return \&ctrl_panel;
    }
    elsif ($request eq "ModuleInfo") {
	return "The Webalizer tool provides you with a convenient link to
your Webalizer web stats pages.  (If you are not running Webalizer stats,
it will link to nothing.)";
    }
}

sub ctrl_panel {
    my $this = shift;
    require ExSite::Auth;
    my $a = new ExSite::Auth;
    my @sites = $a->my_real_sites;
    my $subdir = $config{Webalizer}{subdir} || "_stats";
    my $out;
    if (@sites > 1) {
	$out .= "<p>Multiple websites found, please choose one:</p>\n";
    }
    $out .= "<dl>\n";
    require ExSite::Section;
    foreach my $site (@sites) {
	my $s = new ExSite::Section(section=>$site);
	$s->set_context;
	my $section = $s->get;
	my $path = $s->diskpath."/".$subdir;
	my $url = $s->get_url;
	if (! -e $path) {
	    $out .= "<dt>$section->{title}</dt>\n";
	    $out .= "<dd>(Webalizer stats not found on this site)</dd>\n";
	}
	elsif (@sites == 1) {
	    print "Location: $url/_stats\n\n";
	    return;
	}
	else {
	    $out .= "<dt><a href=$url/_stats>$section->{title}</a></dt>\n";
	    $out .= "<dd>$section->{description}</dd>\n";
	}
    }
    $out .= "</dl>\n";
    return $out;
}

1;
